<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f9d7b286feee $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 01:23 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_move{

	/**
	 *
	 * @param int $targetDirectoryID
	 * @param int $id
	 * @param string $table
	 * @param array $items2move
	 * @return string[]
	 * @throws we_exception_file
	 */
	private static function checkMoveItem(int $targetDirectoryID, int $id, string $table, array $items2move): array{
		// check if entry is a folder
		$row = getHash('SELECT Path,Text,IsFolder FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		if(!$row /* || $row['IsFolder'] */){
			throw new we_exception_file('', we_base_file::ERROR_NO_SUCH_FILE);
		}

		if($row['IsFolder']){
			$targetDir = $targetDirectoryID ? id2path($targetDirectoryID, $table) . '/' : '/';
			if(str_starts_with($targetDir, $row['Path'] . '/')){
				throw new we_exception_file('', we_base_file::ERROR_SAME_PARENT);
			}
		}

		$temp = explode('/', $row['Path']);
		$rootdir = (count($temp) < 2 ? '/' : '/' . $temp[1]);

		// add the item to the item names which could be moved
		$items2move[] = $row['Text'];

		foreach(DB_WE::$inst->getAllFirstq('SELECT Text,Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ParentID=' . $targetDirectoryID, false) as $text => $path){
			'@phan-var string $path';
			// check if there is a item with the same name in the target directory
			if(in_array($text, $items2move, true)){
				throw new we_exception_file('', we_base_file::ERROR_DUPLICATE_NAME);
			}

			if($table === OBJECT_FILES_TABLE){
				// check if class directory is the same
				if(!str_starts_with($path, $rootdir . '/')){
					throw new we_exception_file('', we_contents_objectFile::ERROR_NOT_SAME_CLASS);
				}
			}
		}
		return $items2move;
	}

	/**
	 *
	 * @param int $targetDirectoryID
	 * @param int[] $ids
	 * @param string $table
	 * @return array<int,array{ID:int,Text:string,Path:string,ContentType:string}>
	 */
	//FIXME: check if we need this, or we can make this better in just moving the top elements and updating the path
	private static function moveItems(int $targetDirectoryID, array $ids, string $table): array{
		if(!$ids || ($table === OBJECT_TABLE && !$targetDirectoryID)){
			throw new we_exception_file(g_l('tools', '[path_nok]'), we_base_file::ERROR_NO_SUCH_FILE);
		}
		$notMovedItems = [];
		$DB_WE = DB_WE::$inst;

		// get information about the target directory
		if($targetDirectoryID){
			$parentID = $targetDirectoryID;
			$newPath = fS('SELECT Path FROM ' . $DB_WE->escape($table) . ' WHERE IsFolder=1 AND ID=' . $parentID);
			if(!$newPath){
				throw new we_exception_file(g_l('tools', '[path_nok]'), we_base_file::ERROR_NO_SUCH_FILE);
			}
		} else {
			$newPath = '';
			$parentID = 0;
		}

		$allIds = implode(',', $ids);
		unset($ids);

		//move folders first
		$ids = $DB_WE->getAllq('SELECT ID FROM ' . $DB_WE->escape($table) . ' WHERE IsFolder=1 AND ID IN (' . $allIds . ') AND ParentID NOT IN(' . $allIds . ') ' .
			($table === OBJECT_FILES_TABLE ? ' AND IsClassFolder=0' : '')
			. ' ORDER BY Path', true);
		foreach($ids as $id){
			$folder = we_folder_base::instance($table, (int) $id);
			$folder->ParentID = $targetDirectoryID;
			$folder->Path = $folder->getPath();
			if(!$folder->save()){
				$notMovedItems[] = [
					'ID' => $folder->ID,
					'Text' => $folder->getText(),
					'Path' => $folder->Path,
					'ContentType' => $folder->ContentType
				];
			}
		}
		//if folders are unable to move, we must stop here.
		if(!empty($notMovedItems)){
			return $notMovedItems;
		}

//continue with single files
		$ids = $DB_WE->getAllq('SELECT ID FROM ' . $DB_WE->escape($table) . ' WHERE IsFolder=0 AND ID IN (' . $allIds . ') AND ParentID NOT IN(' . $allIds . ') ' .
			($table === OBJECT_FILES_TABLE ? ' AND IsClassFolder=0' : '')
			. ' ORDER BY Path', true);

		switch($table){
			case TEMPLATES_TABLE:
				// bugfix 0001643
				foreach($ids as $id){
					$template = new we_contents_template((int) $id);
					$template->ParentID = $targetDirectoryID;
					if(!$template->save()){
						$notMovedItems[] = [
							'ID' => $template->ID,
							'Text' => $template->getText(),
							'Path' => $template->Path,
							'ContentType' => $template->ContentType
						];
					}
				}
				break;
			case FILE_TABLE:
				foreach($ids as $id){
					$id = (int) $id;
					// get information about the document which has to be moved
					$row = getHash('SELECT Text,Path,IsPublished,ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $id);
					$fileName = $row['Text'];
					$oldPath = $row['Path'];
					$isPublished = (bool) ($row['IsPublished']);
					if(
					// move document file
						(file_exists(WE_DOCUMENT_ROOT . WE_SITE_DIR . $oldPath) && !we_base_file::moveFile(WE_DOCUMENT_ROOT . WE_SITE_DIR . $oldPath, WE_DOCUMENT_ROOT . WE_SITE_DIR . $newPath . '/' . $fileName)) ||
						// move published document file
						(($isPublished) && file_exists(WE_DOCUMENT_ROOT . $oldPath) && !we_base_file::moveFile(WE_DOCUMENT_ROOT . $oldPath, WE_DOCUMENT_ROOT . $newPath . '/' . $fileName))){
						$notMovedItems[] = [
							'ID' => $id,
							'Text' => $fileName,
							'Path' => $oldPath,
							'ContentType' => $row['ContentType']
						];
						continue;
					}

					if(we_versions_version::CheckPreferencesCtypes($row['ContentType'])){
						if(!we_versions_version::versionExists($id, $table)){
							$object = we_exim_contentProvider::getInstance($row['ContentType'], $id, $table);
							assert($object instanceof we_contents_root);
							$object->Path = $newPath . '/' . $fileName;
							$object->ParentID = $parentID;
							we_versions_version::saveVersion($object);
						} else {
							we_versions_version::updateLastVersionPath($id, $table, $parentID, $newPath . '/' . $fileName);
						}
					}

					// update table
					$DB_WE->query('UPDATE ' . FILE_TABLE . ' SET ' . we_database_base::arraySetter([
							'ParentID' => $parentID,
							'Path' => $newPath . '/' . $fileName
						]) . ' WHERE ID=' . $id);
					$DB_WE->query('UPDATE ' . INDEX_TABLE . ' SET WorkspaceID=' . $parentID . ' WHERE ClassID=0 AND ID=' . $id);
				}
				break;

			// move Objects
			case OBJECT_FILES_TABLE :
				foreach($ids as $id){
					$id = (int) $id;

//FIME: check no classfolder (top level element is moved)
					$row = getHash('SELECT TableID,Path,Text,ContentType FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $id);
					// get information about the object which has to be moved

					$fileName = $row['Text'];

					if(we_versions_version::CheckPreferencesCtypes($row['ContentType'])){
						if(!we_versions_version::versionExists($id, $table)){
							$object = we_exim_contentProvider::getInstance($row['ContentType'], $id, $table);
							assert($object instanceof we_contents_root);
							$object->Path = $newPath . '/' . $fileName;
							$object->ParentID = $parentID;
							we_versions_version::saveVersion($object);
						} else {
							we_versions_version::updateLastVersionPath($id, $table, $parentID, $newPath . '/' . $fileName);
						}
					}

					// update table
					$DB_WE->query('UPDATE ' . $DB_WE->escape($table) . ' SET ' . we_database_base::arraySetter([
							'ParentID' => $parentID,
							'Path' => $newPath . '/' . $fileName
						]) . ' WHERE ID=' . $id);
				}
				break;
		}

		return $notMovedItems;
	}

	private static function hasPerm(string $table): bool{
		switch($table){
			case FILE_TABLE:
				return (we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_MOVE]));
			case TEMPLATES_TABLE:
				return (we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_MOVE]));
			case OBJECT_FILES_TABLE:
				return (we_base_permission::hasPerm([we_base_perm::OBJECTFILE_MOVE]));
		}
		return false;
	}

	public static function getDialog(): void{
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 2);

		if(!self::hasPerm($table)){
			we_base_permission::noPermDialog(g_l('alert', '[no_perms]', false, true), "'exit_move','','" . $table . "'");
		}
		$jsCmd = new we_base_jsCmd();
		$cmd0 = we_base_request::string('we_cmd', '', 0);
		$message = '';
		// list of all item names which should be moved
		$items2move = [];

		switch($cmd0){
			case 'do_move':
			case 'move_single_document':
				if(($targetDirectroy = we_base_request::int('we_target', -1)) === -1){
					$jsCmd->addMsg(g_l('alert', '[move_no_dir]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if(!($selectedItems = we_base_request::intList('sel'))){
					$jsCmd->addMsg(g_l('alert', '[nothing_to_move]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				// list of the selected items
				foreach($selectedItems as $selectedItem){
					'@phan-var int $selectedItem';
					// check if user is allowed to move this item
					if(!we_base_permission::checkIfRestrictUserIsAllowed($selectedItem, $table)){
						$message = sprintf(g_l('alert', '[noRightsToMove]'), id2path($selectedItem, $table));
						break;
					}

					try{
						$items2move = self::checkMoveItem($targetDirectroy, $selectedItem, $table, $items2move);
					} catch (we_exception_file $e){
						switch($e->getCode()){
							case we_base_file::ERROR_NO_SUCH_FILE:
								$message = 'File not found';
								break 2;
							case we_base_file::ERROR_SAME_PARENT:
								$message = g_l('weEditor', '[folder_save_nok_parent_same]');
								break 2;
							case we_base_file::ERROR_DUPLICATE_NAME:
								$message = g_l('alert', '[move_duplicate]');
								break 2;
							case we_contents_objectFile::ERROR_NOT_SAME_CLASS :
								$message = g_l('alert', '[move_onlysametype]');
								break 2;
						}
					}
				}
				if($message){
					$jsCmd->addMsg($message, we_base_util::WE_MESSAGE_ERROR);
				} else {
//	move files !
					try{
						$notMovedItems = self::moveItems($targetDirectroy, $selectedItems, $table);
						if(we_editor_functions::isNormalMode()){ //	different messages in normal or seeMode
							$jsCmd->addCmd('moveTreeEntries', $table === OBJECT_FILES_TABLE); //	only update tree when in normal mode
							if($notMovedItems){
								$_SESSION['weS']['move_files_nok'] = [];
								foreach($notMovedItems as $item){
									$_SESSION['weS']['move_files_nok'][] = [
										'ContentType' => $item['ContentType'],
										'path' => $item['Path']
									];
								}
								$jsCmd->addCmd('moveInfo');
							} else {
								$jsCmd->addMsg(g_l('alert', '[move_ok]'), we_base_util::WE_MESSAGE_NOTICE);
								$jsCmd->addCmd('mv', 1, $table);
							}
						}
					} catch (we_exception_file $e){
						$jsCmd->addMsg($e->getMessage(), we_base_util::WE_MESSAGE_ERROR);
					}
				}
		}
//	in seeMode return to startDocument ...

		if(we_editor_functions::isSeeMode()){
			if(!$message){
				$jsCmd->addMsg(g_l('alert', '[move_single][return_to_start]'), we_base_util::WE_MESSAGE_NOTICE);
				$jsCmd->addCmd('start_multi_editor');
			} else {
				$jsCmd->addMsg(g_l('alert', '[move_single][no_delete]'), we_base_util::WE_MESSAGE_ERROR);
			}

			echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'move.js') . $jsCmd->getCmds(), we_html_element::body());
			return;
		}

		if($cmd0 === 'do_move'){

			echo we_html_tools::getHtmlTop('', $jsCmd->getCmds() .
				we_html_element::jsScript(JS_DIR . 'move.js'), we_html_element::body());
			return;
		}
		switch($table){
			case TEMPLATES_TABLE:
				$type = g_l('global', '[templates]');
				break;
			case OBJECT_FILES_TABLE :
				$type = g_l('global', '[objects]');
				break;
			default:
				$type = g_l('global', '[documents]');
				break;
		}

		$ws_Id = we_base_request::int('we_target', (we_users_util::get_def_ws($table) ?: 0));
		$ws_path = ($ws_Id ? id2path($ws_Id, $table) : '/');
		$weSuggest = we_gui_suggest::getInstance();

		$weSuggest->setAcId('Dir')
			->setContentType(we_base_ContentTypes::FOLDER_TYPES)
			->setInput('we_targetname', $ws_path)
			->setMaxResults(4)
			//->setRequired(true)
			->setResult('we_target', (string) $ws_Id)
			->setSelector(we_gui_suggest::DirSelector)
			->setTable($table)
			->setWidth(250)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:top.treeheader.document.we_form.elements.we_target.value,table:'" . $table . "',targetID:'we_target',targetName:'we_targetname'})"));

		echo we_html_tools::getHtmlTop('', $jsCmd->getCmds() .
			we_html_element::jsScript(JS_DIR . 'move.js', "initMove('" . $table . "');"),
														 we_html_element::body(['class' => 'weTreeHeaderMove'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post', 'onsubmit' => "return false;"], we_html_element::div([
						], we_html_element::h(1, htmlspecialchars(g_l('weClass', '[moveMode][title]')), ['class' => 'big', 'style' => "padding:0;margin:0;"]) .
						we_html_element::p(['class' => "small"], we_html_element::span(['class' => 'middlefont', 'style' => "padding-right:5px;padding-bottom:10px;"], g_l('weClass', '[moveMode][text]')) .
							we_html_element::p(['style' => "margin:0 0 10px 0;padding:0;"], $weSuggest->getHTML())) .
						we_html_element::div([], we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:press_ok_move('" . $type . "');"), '', we_html_button::create_button('quit_move', "javascript:we_cmd('exit_move','','" . $table . "')")))) . we_html_element::hidden('sel', '')
			))
		);
	}

	public static function getMoveInfo(): void{
		$table = new we_html_table(['style' => 'margin:10px;', 'class' => 'default withSpace'], 1, 2);
		if(!empty($_SESSION['weS']['move_files_nok'])){
			foreach($_SESSION['weS']['move_files_nok'] as $i => $data){
				$table->setCol($i, 0, ' ', ['class' => 'moveIcon', 'data-contenttype' => (empty($data['ContentType']) ? '' : $data['ContentType']),
						'data-extension' => ''])
					->setCol($i, 1, str_replace(WE_DOCUMENT_ROOT, '', $data['path']));
			}
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(str_replace("\\n", '', sprintf(g_l('alert', '[move_of_files_failed]'), '')), we_html_tools::TYPE_ALERT, 500))))
			->add((new we_html_multiIconBoxContentLine('', we_html_element::div(['class' => "blockWrapper", 'style' => "width: 475px; height: 350px; border:1px #dce6f2 solid;"], $table->getHtml()))));

		$buttons = we_html_element::div(['style' => 'text-align:right', 'class' => 'default'], we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"));

		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('setIconOfDocClass', 'moveIcon');

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'dialogPushCmd.js'), we_html_element::body(['class' => 'weDialogBody'], (new we_html_multiIconBox($parts))->marginLeft()->buttons($buttons)->get() . $jsCmd->getCmds()
			)
		);
	}

}
