<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 883c0197a34e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-08-03 19:59 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//NOTE: there is no need to add any variables to we_conf_global.inc.php.default anymore.
abstract class we_base_preferencesConfig{

	/** @var array<string,array<string,string,mixed,?array>> */
	public const GLOBAL = [//key => comment, type,default, changed right (default Admin)
// Variables for SEEM
		'WE_SEEM' => ['Enable seeMode', we_base_request::BOOL, 1],
// Variables for thumbnails
		'WE_THUMBNAIL_DIRECTORY' => ['Directory in which to save thumbnails', we_base_request::FILE, '/_thumbnails_'],
// Variables for error handling
		//'WE_ERROR_HANDLER' => ['Show errors that occur in webEdition', we_base_request::BOOL, true],
		'WE_ERROR_NOTICES' => ['Handle notices', we_base_request::BOOL, false],
		//'WE_ERROR_DEPRECATED' => ['Handle deprecated warnings', we_base_request::BOOL, true],
		//'WE_ERROR_WARNINGS' => ['Handle warnings', we_base_request::BOOL, true],
		//'WE_ERROR_ERRORS' => ['Handle errors', we_base_request::BOOL, true],
		'DEBUG_SLOW_QUERY' => ['Handle slow queries and save them to database', we_base_request::BOOL, false],
		'WE_ERROR_SHOW' => ['Show errors', we_base_request::BOOL, false],
		//'WE_ERROR_LOG' => ['Log errors', we_base_request::BOOL, true],
		'WE_ERROR_MAIL' => ['Mail errors', we_base_request::BOOL, false],
		'WE_ERROR_MAIL_ADDRESS' => ['E-Mail address to which to mail errors', we_base_request::EMAIL, ''],
		'ERROR_DOCUMENT_NO_OBJECTFILE' => ['Document to open when trying to open non-existing object', we_base_request::INT, 0],
		'INLINEEDIT_DEFAULT' => ['Default setting for inlineedit attribute', we_base_request::BOOL, true],
		'IMAGESTARTID_DEFAULT' => ['Default setting for attribute imagestartdir in wetextarea', we_base_request::INT, 0],
		'COMMANDS_DEFAULT' => ['Default setting for commands attribute', we_base_request::STRING, ''],
		'REMOVEFIRSTPARAGRAPH_DEFAULT' => ['Default setting for removeparagraph attribute', we_base_request::BOOL, false],
		'HIDENAMEATTRIBINWEFORM_DEFAULT' => ['Default setting for hide name attribute in weform output', we_base_request::BOOL, false],
		'REPLACEACRONYM' => ['Remove unsupported html5 acronym tag', we_base_request::BOOL, false],
// we_css
		'CSSAPPLYTO_DEFAULT' => ['Default setting for we:css attribute applyto', we_base_request::STRING, 'around'],
// hooks
		'EXECUTE_HOOKS' => ['Default setting for hook execution', we_base_request::BOOL, false],
//system
		'FILE_UPLOAD_MAX_UPLOAD_SIZE' => ['Set the maximum size a file can have', we_base_request::INT, 128],
		'FILE_UPLOAD_IMG_MAX_SIZE' => ['Set the maximum size a file can have', we_base_request::INT, 0],
		'WE_NEW_FOLDER_MOD' => ['File permissions when creating a new directory', we_base_request::INT, 755], //this should be string but deny access by user doesn't make sense
		'WE_DOCTYPE_WORKSPACE_BEHAVIOR' => ['Which Doctypes should be shown for which workspace', we_base_request::BOOL, false],
		'SCHEDULER_TRIGGER' => ['decide how the scheduler works', we_base_request::INT, we_schedule_scheduler::TRIGGER_POSTDOC],
		'SYSTEM_WE_SESSION_TIME' => ['time after which the session is killed if not active anymore', we_base_request::INT, 1440],
		'SYSTEM_WE_SESSION_CRYPT' => ['crypt we session before save', we_base_request::INT, we_base_sessionHandler::CRYPT_TRANSPARENT],
		'SYSTEM_DEBUG_DAVFS' => ['Enable debug mode for davFS', we_base_request::BOOL, false],
// accessibility
		'SHOWINPUTS_DEFAULT' => ['Default setting for showinputs attribute', we_base_request::BOOL, true],
		//'WYSIWYG_TYPE' => ['define used wysiwyg editor', we_base_request::STRING, 'tinyMCE'],
		//'WYSIWYG_TYPE_FRONTEND' => ['define used wysiwyg editor in frontend', we_base_request::STRING, 'tinyMCE'],
		'WE_MAILER' => ['mailer type; possible values are php and smtp', we_base_request::STRING, 'php'],
		'SMTP_SERVER' => ['SMTP_SERVER', we_base_request::STRING, 'localhost'],
		'SMTP_PORT' => ['SMTP server port', we_base_request::INT, 25],
		'SMTP_AUTH' => ['SMTP authentication', we_base_request::STRING, ''],
		'SMTP_MAIL_SENDER' => ['SMTP Sender Email', we_base_request::EMAIL, ''],
		'SMTP_USERNAME' => ['SMTP username', we_base_request::STRING, ''],
		'SMTP_PASSWORD' => ['SMTP password', we_base_request::RAW_CHECKED, ''],
		'SMTP_ENCRYPTION' => ['SMTP encryption', we_base_request::STRING, 0],
//formmail stuff
		'FORMMAIL_CONFIRM' => ['Flag if formmail confirm function should be work', we_base_request::BOOL, true], //this is restricted to admin
		'FORMMAIL_VIAWEDOC' => ['Flag if formmail should be send only via a webEdition document', we_base_request::BOOL, false, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_LOG' => ['Flag if formmail calls should be logged', we_base_request::BOOL, true, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_EMPTYLOG' => ['Time how long formmail calls should be logged', we_base_request::INT, we_base_util::SECONDS_WEEK, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_BLOCK' => ['Flag if formmail calls should be blocked after a time', we_base_request::BOOL, true, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_SPAN' => ['Time span in seconds', we_base_request::INT, 300, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_TRIALS' => ['Num of trials sending formmail with same ip address in span', we_base_request::INT, 3, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'FORMMAIL_BLOCKTIME' => ['Time to block ip', we_base_request::INT, we_base_util::SECONDS_DAY, 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
// sidebar stuff
		'SIDEBAR_DISABLED' => ['Sidebar is disabled', we_base_request::BOOL, false],
		'SIDEBAR_SHOW_ON_STARTUP' => ['Show Sidebar on startup', we_base_request::BOOL, true],
		'SIDEBAR_DEFAULT_DOCUMENT' => ['Default document id of the sidebar', we_base_request::INT, 0],
		'SIDEBAR_DEFAULT_WIDTH' => ['Default width of the sidebar', we_base_request::INT, 300],
// extension stuff
		'DEFAULT_STATIC_EXT' => ['Default static extension', we_base_request::STRING, '.html'],
		'DEFAULT_DYNAMIC_EXT' => ['Default dynamic extension', we_base_request::STRING, '.php'],
		'DEFAULT_HTML_EXT' => ['Default html extension', we_base_request::STRING, '.html'],
//naviagtion stuff
		'NAVIGATION_ENTRIES_FROM_DOCUMENT' => ['Flag if new NAV- entries added from Dokument should be items or folders', we_base_request::BOOL, false],
		'NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH' => ['Flag if NAV- rules should be evaluated even after a first match', we_base_request::BOOL, false],
//SEO stuff
		'NAVIGATION_DIRECTORYINDEX_HIDE' => ['Flag if directoy-index files should be hidden in Nav-output', we_base_request::BOOL, false],
		'NAVIGATION_DIRECTORYINDEX_NAMES' => ['Comma seperated list such as index.php,index.html', we_base_request::STRING, 'index.php,index.html'],
		'WYSIWYGLINKS_DIRECTORYINDEX_HIDE' => ['Flag if directoy-index files should be hidden in Wysiwyg-editor output', we_base_request::BOOL, false],
		'TAGLINKS_DIRECTORYINDEX_HIDE' => ['Flag if directoy-index files should be hidden in tag output', we_base_request::BOOL, false],
		'OBJECTSEOURLS_LOWERCASE' => ['Flag if object SEO-URLs save in lower or camelcase', we_base_request::BOOL, false],
		'NAVIGATION_OBJECTSEOURLS' => ['Flag if we_objectID should be hidden from output of navigation', we_base_request::BOOL, false],
		'WYSIWYGLINKS_OBJECTSEOURLS' => ['Flag if we_objectID should be hidden from output of wysiwyg editior', we_base_request::BOOL, false],
		'TAGLINKS_OBJECTSEOURLS' => ['Flag if we_objectID should be hidden from output of tags', we_base_request::BOOL, false],
		'URLENCODE_OBJECTSEOURLS' => ['Flag if seo-urls should be urlencoded', we_base_request::BOOL, false],
		'SUPPRESS404CODE' => ['Flag if 404 not found should be suppressed', we_base_request::BOOL, false],
		'FORCE404REDIRECT' => ['Flag if redirect to 404 instead of include', we_base_request::BOOL, false],
		'SEOINSIDE_HIDEINWEBEDITION' => ['Flag if should be displayed in webEdition ', we_base_request::BOOL, false],
		'SEOINSIDE_HIDEINEDITMODE' => ['Flag if should be displayed in Editmode', we_base_request::BOOL, false],
//mod time
		'DISABLE_MEDIA_MODTIME' => ['Flag if mod time stamp should be added to media files', we_base_request::BOOL, false],
//lang link stuff
		'LANGLINK_SUPPORT' => ['Flag if automatic LanguageLinks should be supported ', we_base_request::BOOL, true],
//countries
		'WE_COUNTRIES_TOP' => ['top countries', we_base_request::STRING_LIST, ['DE', 'AT', 'CH']],
		'WE_COUNTRIES_SHOWN' => ['other shown countries', we_base_request::STRING_LIST, ['BE', 'DK', 'FI', 'FR', 'GR', 'IE', 'IT', 'LU', 'NL', 'PT', 'SE', 'ES', 'GB', 'EE', 'LT', 'MT', 'PL', 'SK', 'SI', 'CZ', 'HU', 'CY']],
		'WE_COUNTRIES_DEFAULT' => ['shown if no coutry was choosen', we_base_request::STRING, ''],
//versions
		'VERSIONING_IMAGE' => ['Versioning status for ContentType image', we_base_request::BOOL, false],
		'VERSIONING_TEXT_HTML' => ['Versioning status for ContentType text/html', we_base_request::BOOL, false],
		'VERSIONING_TEXT_WEBEDITION' => ['Versioning status for ContentType text/webedition', we_base_request::BOOL, true],
		'VERSIONING_TEXT_HTACCESS' => ['Versioning status for ContentType text/htaccess', we_base_request::BOOL, false],
		'VERSIONING_TEXT_WETMPL' => ['Versioning status for ContentType text/weTmpl', we_base_request::BOOL, true],
		'VERSIONING_TEXT_JS' => ['Versioning status for ContentType text/js', we_base_request::BOOL, true],
		'VERSIONING_TEXT_JSON' => ['Versioning status for ContentType application/json', we_base_request::BOOL, true],
		'VERSIONING_TEXT_CSS' => ['Versioning status for ContentType text/css', we_base_request::BOOL, true],
		'VERSIONING_TEXT_PLAIN' => ['Versioning status for ContentType text/plain', we_base_request::BOOL, false],
		'VERSIONING_VIDEO' => ['Versioning status for ContentType video/*', we_base_request::BOOL, false],
		'VERSIONING_AUDIO' => ['Versioning status for ContentType audio/*', we_base_request::BOOL, false],
		'VERSIONING_PDF' => ['Versioning status for ContentType application/pdf', we_base_request::BOOL, false],
		'VERSIONING_SONSTIGE' => ['Versioning status for ContentType application/*', we_base_request::BOOL, false],
		'VERSIONING_TEXT_XML' => ['Versioning status for ContentType text/xml', we_base_request::BOOL, false],
		'VERSIONING_OBJECT' => ['Versioning status for ContentType objectFile', we_base_request::BOOL, false],
		'VERSIONS_TIME_DAYS' => ['Versioning Number of Days', we_base_request::INT, -1],
		'VERSIONS_TIME_WEEKS' => ['Versioning Number of Weeks', we_base_request::INT, -1],
		'VERSIONS_TIME_YEARS' => ['Versioning Number of Years', we_base_request::INT, -1],
		'VERSIONS_ANZAHL' => ['Versioning Number of Versions', we_base_request::INT, 3],
		'VERSIONS_CREATE' => ['Versioning Save version only if publishing', we_base_request::BOOL, false],
		'VERSIONS_CREATE_TMPL' => ['Versioning Save template version only on special request', we_base_request::BOOL, true],
		'VERSIONS_TIME_DAYS_TMPL' => ['Versioning Number of Days', we_base_request::INT, -1],
		'VERSIONS_TIME_WEEKS_TMPL' => ['Versioning Number of Weeks', we_base_request::INT, -1],
		'VERSIONS_TIME_YEARS_TMPL' => ['Versioning Number of Years', we_base_request::INT, -1],
		'VERSIONS_ANZAHL_TMPL' => ['Versioning Number of Versions', we_base_request::INT, 5],
//security
		'SECURITY_DELETE_SESSION' => ['whether to delte a session on logout of a customer', we_base_request::BOOL, true],
		'SECURITY_LIMIT_CUSTOMER_IP' => ['Limit # of failed logins comming from the same IP', we_base_request::INT, 10],
		'SECURITY_LIMIT_CUSTOMER_IP_HOURS' => ['Limit failed logins comming from same IP connections per # hours', we_base_request::INT, 3],
		'SECURITY_LIMIT_CUSTOMER_NAME' => ['Limit # of failed logins with same username', we_base_request::INT, 4],
		'SECURITY_LIMIT_CUSTOMER_NAME_HOURS' => ['Limit failed logins with same usernames per # hours', we_base_request::INT, 1],
		'SECURITY_LIMIT_CUSTOMER_REDIRECT' => ['If limit reached, redirect to page', we_base_request::INT, 0],
		'SECURITY_DELAY_FAILED_LOGIN' => ['Delay a failed login by # seconds', we_base_request::INT, 3],
		'SECURITY_ENCRYPTION_KEY' => ['This is the encryption key used for password, if set to symmetric mode', we_base_request::STRING, ''],
		'SECURITY_SESSION_PASSWORD' => ['Determine if a userpassword is allowed to be stored in current session', we_base_request::INT, we_customer_customer::REMOVE_PASSWORD],
		'SECURITY_USER_PASS_REGEX' => ['Regex used to compare user password', we_base_request::STRING, we_users_user::DEFAULT_PASS_REGEX, '((?=.*\d)(?=.*[a-zA-Z]).{10,})'],
		'SECURITY_USER_PASS_DESC' => ['Userfriendly description of how the password should be chosen', we_base_request::STRING, 'Enter a password with at least 10 and at most 20 characters containing a number, upper and lower case letters.'],
		'MAINTENANCE_MODE' => ['Set the maintenance mode (frontend,backend,fontend_backend) if you want to do maintenance', we_base_request::STRING, ''],
		'MAINTENANCE_DOCUMENT' => ['Set the (static) maintenance document shown in frontend', we_base_request::STRING, ''],
		'MAINTENANCE_IPS' => ['Set allowed IPs during maintenance', we_base_request::STRING_LIST, []],
		//Cache
		'MEMCACHE_SERVERS' => ['Servername of the memcache instance', we_base_request::STRING_LIST, []],
		'MEMCACHE_DATABASE' => ['true if memcache is used to store DB results', we_base_request::BOOL, false],
		'MEMCACHE_BACKEND' => ['Kind of backend, currently redis and memcached are supported, additionally database can be set', we_base_request::STRING, we_cache_backendNone::BACKEND_NONE],
		'MEMCACHE_BACKEND_DB' => ['Database number to select', we_base_request::INT, 0],
//internal
		'CONF_SAVED_VERSION' => ['config file version', we_base_request::INT, WE_SVNREV],
	];

	/** @var array<string,array<string,string,mixed>> */
	public const USER = [//FIXME: most defaults (currently null) are handled by remember_value! change this!
//key => type,default-val, permission. default true
		'Language' => ['*DESC*', we_base_request::STRING, 'Deutsch'],
		'default_tree_count' => ['*DESC*', we_base_request::INT, 0],
		/* 'sizeOpt' => [we_base_request::BOOL, 0],
		  'weWidth' => [we_base_request::INT, 0],
		  'weHeight' => [we_base_request::INT, 0], */
		'cockpit_amount_columns' => ['*DESC*', we_base_request::INT, 5],
		'cockpit_amount_last_documents' => ['*DESC*', we_base_request::INT, 5],
		'cockpit_dat' => ['*DESC*', we_base_request::STRING, ''],
		//all rss feeds set in cockpit
		'cockpit_rss' => ['*DESC*', we_base_request::SERIALIZED_KEEP, ''],
		'editorMode' => ['*DESC*', we_base_request::STRING, 'codemirror2'],
		'editorCodecompletion' => ['*DESC*', we_base_request::STRING, '{"WE":true,"htmlTag":true}'],
		'editorDocuintegration' => ['*DESC*', we_base_request::BOOL, true],
		'editorFont' => ['*DESC*', we_base_request::BOOL, ''],
		'editorFontname' => ['*DESC*', we_base_request::STRING, ''],
		'editorFontsize' => ['*DESC*', we_base_request::INT, 0],
		'editorHighlightCurrentLine' => ['*DESC*', we_base_request::BOOL, true],
		'editorTheme' => ['*DESC*', we_base_request::STRING, 'elegant'],
		'editorLinenumbers' => ['*DESC*', we_base_request::BOOL, true],
		'editorTooltips' => ['*DESC*', we_base_request::BOOL, false], //tags
		'editorTooltipsIDs' => ['*DESC*', we_base_request::BOOL, false], //IDS
		'editorTooltipFont' => ['*DESC*', we_base_request::BOOL, ''],
		'editorTooltipFontname' => ['*DESC*', we_base_request::STRING, ''],
		'editorTooltipFontsize' => ['*DESC*', we_base_request::INT, 0],
		'editorWrap' => ['*DESC*', we_base_request::BOOL, false],
		'message_reporting' => ['*DESC*', we_base_request::INT, we_base_util::WE_MESSAGE_WARNING | we_base_util::WE_MESSAGE_ERROR],
		'seem_start_type' => ['*DESC*', we_base_request::STRING, 'cockpit', 'CHANGE_START_DOCUMENT'],
		'seem_start_file' => ['*DESC*', we_base_request::INT, 0],
		//'seem_start_weapp' => [we_base_request::STRING, ''],
		'DefaultTemplateID' => ['*DESC*', we_base_request::INT, 0],
		'editorHeight' => ['*DESC*', we_base_request::INT, 0],
		'editorSizeOpt' => ['*DESC*', we_base_request::BOOL, false],
		'editorWidth' => ['*DESC*', we_base_request::INT, 0],
		'force_glossary_action' => ['*DESC*', we_base_request::BOOL, false],
		'force_glossary_check' => ['*DESC*', we_base_request::BOOL, false],
		'import_from' => ['*DESC*', we_base_request::FILE, ''],
		'openFolders_tblFile' => ['*DESC*', we_base_request::INTLISTA, []],
		'openFolders_tblObject' => ['*DESC*', we_base_request::INTLISTA, []],
		'openFolders_tblObjectFiles' => ['*DESC*', we_base_request::INTLISTA, []],
		'openFolders_tblTemplates' => ['*DESC*', we_base_request::INTLISTA, []],
		'openFolders_tblVFile' => ['*DESC*', we_base_request::INTLISTA, []],
		'siteImportPrefs' => ['*DESC*', we_base_request::STRING, ''],
		'editorShowTab' => ['*DESC*', we_base_request::BOOL, true],
		'editorShowSpaces' => ['*DESC*', we_base_request::BOOL, true],
		'editorTabSize' => ['*DESC*', we_base_request::INT, 2],
		'editorAutoIndent' => ['*DESC*', we_base_request::BOOL, true],
		'editorIndentSpaces' => ['*DESC*', we_base_request::BOOL, false],
	];

	/** @var array<string,array<string,string,mixed,bool>> */
	public const CONF = [
		'HTTP_USERNAME' => ['if used password protection to the webEdition directory, the username', we_base_request::STRING, '', 'encode' => true],
		'HTTP_PASSWORD' => ['if used password protection to the webEdition directory, the password', we_base_request::RAW_CHECKED, '', 'encode' => true],
		//note these settings are user-settings, not changed by request/frontend
		'DB_HOST' => ['Domain or IP address of the database server, use : if you want to add a port', we_base_request::STRING, 'localhost'],
		'DB_DATABASE' => ['Name of database used by webEdition', we_base_request::STRING, 'webedition'],
		'DB_USER' => ['Username to access the database', we_base_request::STRING, 'root', 'encode' => true],
		'DB_PASSWORD' => ['Password to access the database', we_base_request::STRING, 'root', 'encode' => true],
		'TBL_PREFIX' => ['Prefix of tables in database for this webEdition.', we_base_request::STRING, ''],
		'DB_COLLATION' => ['Collation of tables in database for this webEdition.', we_base_request::STRING, ''],
		'WE_LANGUAGE' => ['Original language of this version of webEdition, used for login-screen', we_base_request::STRING, 'English'],
		'DB_SLAVES' => ['Database read slaves', we_base_request::STRING_LIST, []],
	];

	/** @var array<string,array<string,string,mixed,?array>> */
	public const OTHER = [
		'formmail_values' => ['*DESC*', we_base_request::RAW_CHECKED, '', 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'formmail_deleted' => ['*DESC*', we_base_request::RAW, '', 'perm' => [we_base_perm::WORK_FORMMAIL_SHOW]],
		'useproxy' => ['*DESC*', we_base_request::BOOL, false],
		'proxyhost' => ['*DESC*', we_base_request::URL, ''],
		'proxyport' => ['*DESC*', we_base_request::INT, 0],
		'proxyuser' => ['*DESC*', we_base_request::STRING, ''],
		'proxypass' => ['*DESC*', we_base_request::RAW_CHECKED, ''],
		'active_integrated_modules' => ['*DESC*', we_base_request::STRING, ''],
		'useauth' => ['*DESC*', we_base_request::BOOL, false], //pseudo element
		'locale_default' => ['*DESC*', we_base_request::STRING, 'de_DE'],
		'locale_locales' => ['*DESC*', we_base_request::STRING, 'de_DE'],
	];

	/** @var array<string,array> */
	public const ALL_CONFIG = [
		'global' => self::GLOBAL,
		'user' => self::USER,
		'conf' => SELF::CONF,
		'other' => SELF::OTHER,
	];

}
