<?php

declare(strict_types=1);

/** webEdition CMS
 *
 * $Rev: f524c1d58ef0 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-11-03 11:03 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_sessionHandler implements SessionHandlerInterface, SessionUpdateTimestampHandlerInterface{

	public const CRYPT_NONE = 0;
	public const CRYPT_TRANSPARENT = 1;
	public const CRYPT_COOKIE = 2;

//prevent crashed or killed sessions to stay

	/** @var int */
	protected $execTime;

	/** @var string */
	protected $sessionName;

	/** @var we_database_base */
	private $DB;

	/** @var string */
	protected $id = '';

	/** @var string */
	private $crypt = '';

	/** @var string */
	protected $hash = '';

	/** @phan-var int */
	protected static $sessTime = 1440;

	/** @var array<string,mixed> */
	public static $releaseError = [];

	public static function inst(): we_base_sessionHandler{
		return we_cache_session::canUseCache() ?
				new we_base_sessionHandlerCache() :
				new self();
	}

	public function __construct(){//fixme public due to update! change in cardada
		if(session_status() === PHP_SESSION_ACTIVE){
			return;
		}

		/* if(!empty($_REQUEST['debug'])){
		  self::$debug = true;
		  } */
		if($this->id){
			session_start();
			return;
		}
		session_set_save_handler($this, true);
		$this->DB = new DB_WE();
		$this->execTime = max(min(60, ((int) get_cfg_var('max_execution_time'))), 5); //time might be wrong (1&1); make exectime at least 5 seconds which is quite small
		$this->id = md5(uniqid('', true));
		self::$sessTime = defined('SYSTEM_WE_SESSION_TIME') ? SYSTEM_WE_SESSION_TIME : 1440;
		$this->setCrypt(false);
		session_start();
	}

	private function setCrypt(bool $old): void{
		if(defined('SYSTEM_WE_SESSION_CRYPT') && SYSTEM_WE_SESSION_CRYPT){
			// due to IE we can't use HTTP_ACCEPT_LANGUAGE, HTTP_ACCEPT_ENCODING - they change the string on each request
			$key = ($old ? $_SERVER['DOCUMENT_ROOT'] : $_SERVER['WE_ORG_ROOT']) . (!empty($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'HTTP_USER_AGENT');
			/* if(SYSTEM_WE_SESSION_CRYPT === self::CRYPT_COOKIE){
			  if(!isset($_COOKIE['secure'])){
			  $_COOKIE['secure'] = we_users_user::getHashIV(30);
			  }
			  $key .= $_COOKIE['secure'];
			  } */
			$this->crypt = hash('haval224,4', $key);
			//double key size is needed
			$this->crypt .= $this->crypt;
		}
	}

	public function __destruct(){
		if(session_status() === PHP_SESSION_ACTIVE){
			session_write_close();
		}
	}

	#[\ReturnTypeWillChange]
	public function open(/* string */$savePath, /* string */ $sessName)/* :string */{
		$this->sessionName = $sessName;
		return true;
	}

	public function close(): bool{//FIX for php >5.5, where write is only called, if sth. in session changed
		/* if(self::$debug){
		  t_e(session_id());
		  } */
		return true;
	}

	#[\ReturnTypeWillChange]
	public function read(/* string */$sessID)/* : string */{
		//$lock = microtime(true);
		if(fB('SELECT 1 FROM ' . SESSION_TABLE . ' WHERE touch>=NOW()-INTERVAL ' . self::$sessTime . ' second AND session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"')){//session exists
			$max = $this->execTime * 6; //just wait 80% of the time to clear the lock, if not cleared, break the lock

			while(!($this->DB->query('UPDATE ' . SESSION_TABLE . ' SET
	lock_id=x\'' . $this->id . '\',
	lockTime=NOW()
WHERE
	session_id=x\'' . $sessID . '\' AND
	sessionName="' . $this->sessionName . '"' . ( --$max ? ' AND
	(lock_id IS NULL OR lock_id=x\'' . $this->id . '\' OR lockTime+INTERVAL ' . $this->execTime . ' second<NOW())' : '')
			) &&
			$this->DB->affected_rows())){
				self::$releaseError = self::$releaseError ?: getHash('SELECT sessionName,HEX(session_id) AS session_id,lockTime,HEX(lock_id) AS lock_id,touch,NOW()'/* ,debug */ . ' FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"');
				usleep(250000);
			}
			$data = fS('SELECT session_data FROM ' . SESSION_TABLE . ' WHERE	session_id=x\'' . $sessID . '\' AND	sessionName="' . $this->sessionName . '"');

			//self::$acquireLock = microtime(true) - $lock;
			if(!$max){
				self::$releaseError['myLock'] = $this->id;
			} else {
				self::$releaseError = [];
			}

			if($data){
				return $this->decryptSessData($data, $sessID);
			}
		}
		//self::$acquireLock = microtime(true) - $lock;
		//if we don't find valid data, generate a new ID because of session stealing
		$this->write($sessID, '', true);
		return '';
	}

	protected function decryptSessData(string $orgdata, string $sessID): string{
		$data = ($orgdata[0] === '$' && $this->crypt ? we_customer_customer::decryptData($orgdata, $this->crypt) : $orgdata);
		if($orgdata[0] === '$' && $this->crypt && !$data && defined('LIVEUPDATE_DIR')){
			$this->setCrypt(true);
		}
		if($data && $data[0] === 'x'){
			//valid gzip
			$data = (gzuncompress($data) ?: '');
			$this->hash = md5($sessID . $orgdata, true);
			return $data;
		}
		$this->hash = md5($sessID . $orgdata, true);
		return $data;
	}

	protected function cryptSessData(string $sessData): string{
		return (defined('SYSTEM_WE_SESSION_CRYPT') ? SYSTEM_WE_SESSION_CRYPT : we_base_sessionHandler::CRYPT_TRANSPARENT) && $this->crypt ? we_customer_customer::cryptData(gzcompress($sessData, 4) ?: '', $this->crypt, true) : (gzcompress($sessData, 4) ?: '');
	}

	public function write($sessID, $sessData, $lock = false): bool{
		if(!$sessID){
			return true;
		}
		if(!$sessData && !$lock){
			static::delSess($sessID, $this->sessionName);
			return true;
		}

		if(md5($sessID . $sessData, true) === $this->hash){//if nothing changed,we don't have to bother the db
			if($lock){
				$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET ' .
						we_database_base::arraySetter([
							'lock_id' => sql_function('x\'' . $this->id . '\''),
							'lockTime' => sql_function('NOW()'),
						]) .
						' WHERE
						session_id=x\'' . $sessID . '\' AND
						sessionName="' . $this->sessionName . '"');
				if($this->DB->affected_rows() > 0){
					return true;
				}
			} elseif($this->updateTimestamp($sessID, '')){
				return true;
			}
		}

		$this->DB->query('REPLACE INTO ' . SESSION_TABLE . ' SET ' . we_database_base::arraySetter([
					'sessionName' => $this->sessionName,
					'session_id' => sql_function('x\'' . $sessID . '\''),
					'session_data' => sql_function('x\'' . bin2hex(($data = $this->cryptSessData($sessData))) . '\''),
					'lock_id' => $lock ? sql_function('x\'' . $this->id . '\'') : null,
					'lockTime' => ($lock ? sql_function('NOW()') : null),
		]));
		$this->hash = md5($sessID . $data, true);
		return true;
	}

	public function destroy($sessID): bool{
		unset($_SESSION);
		static::delSess($sessID, $this->sessionName);
		return true;
	}

	protected static function delSess(string $sessID, string $sessName): void{
		DB_WE::$inst->query('DELETE FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $sessName . '"');
	}

	#[\ReturnTypeWillChange]
	public function gc($sessMaxLifeTime){
		return self::cleanSessions() > 0;
	}

	public static function cleanSessions(): int{
		DB_WE::$inst->query('DELETE FROM ' . SESSION_TABLE . ' WHERE touch<NOW()-INTERVAL ' . self::$sessTime . ' second');
		$ret = DB_WE::$inst->affected_rows();
		DB_WE::$inst->query('OPTIMIZE TABLE ' . SESSION_TABLE);
		return $ret;
	}

	protected static function session_create_id(): string{
		session_regenerate_id(false); // @phan-suppress-current-line PhanPluginUseReturnValueInternalKnown
		return session_id();
	}

	static function makeNewID(bool $destroy = false): void{
		if($destroy){
			session_destroy();
			session_start();
			return;
		}
		//we need a new lock on the generated id, since partial data is sent to the browser, subsequent calls with the new sessionid might happen
		self::delSess(session_id(), session_name());
		self::session_create_id();
		session_write_close();
		session_start();
	}

	public static function getCurrentHex(): string{
		$sessID = session_id();
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) ? $sessID : bin2hex($sessID);
	}

	#[\ReturnTypeWillChange]
	public function updateTimestamp($sessID, $val){
		$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET
			lock_id=NULL,
			lockTime=NULL
		WHERE
			session_id=x\'' . $sessID . '\' AND
			sessionName="' . $this->sessionName . '"' .
				//if this is from we_base_sessionHandlerCache skip
				($val != '' ? ' AND	lock_id=x\'' . $this->id . '\'' : '')
		);
		return ($this->DB->affected_rows() > 0);
	}

	#[\ReturnTypeWillChange]
	public function validateId($sessID){
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) === 1;
	}

}
