<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8e576d14bbda $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-11 18:42 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_cache_file implements we_cache_base{
	const DOMAIN_TW = 'TagWizard_';
	const DOMAIN_GLOSSAR = 'glossar_';
	const DOMAIN_NAVIGATION = 'navigation_';
	const DOMAIN_NEWSLETTER = 'nl_';
	const DOMAIN_RSS = 'rss_';
	const DOMAIN_XMLFEED = 'feed_';

	/**
	 *
	 * @param string $entry
	 * @return array
	 * @throws we_exception_cache
	 */
	public static function load(string $entry, bool $expiryCheck = true){
		$file = WE_CACHE_PATH . 'we_cache_' . $entry;

		if(!file_exists($file) ||
			($expiryCheck && fB('SELECT 1 FROM ' . CLEAN_UP_TABLE . ' WHERE Path="' . DB_WE::$inst->escape(WE_CACHE_DIR . 'we_cache_' . $entry) . '" AND Date<NOW()'))){
			throw new we_exception_cache('cache file ' . $file . ' not found');
		}

		try{
			return we_unserialize(we_base_file::load($file));
		} catch (we_exception_file $e){
			throw new we_exception_cache('cache file not loadable: ' . $e->getMessage());
		}
	}

	public static function stat(string $entry): int{
		$file = WE_CACHE_PATH . 'we_cache_' . $entry;
		if(!file_exists($file) ||
			fB('SELECT 1 FROM ' . CLEAN_UP_TABLE . ' WHERE Path="' . DB_WE::$inst->escape(WE_CACHE_DIR . 'we_cache_' . $entry) . '" AND Date<NOW()')){
			throw new we_exception_cache('cache file ' . $file . ' not found');
		}
		return filemtime($file) ?: 0;
	}

	/**
	 *
	 * @param string $entry
	 * @param array $data
	 * @param int $expiry
	 * @param string $serializer
	 * @return bool
	 */
	public static function save(string $entry, $data, int $expiry = 3600, string $serializer = SERIALIZE_PHP): bool{
		$ser = we_serialize($data, $serializer); //keep this php, since encoding etc. works better & faster in all modes
		while(($lock = we_base_file::lock('we_cache_' . $entry)) == false){
			usleep(100000);
		}
		we_base_file::save(WE_CACHE_PATH . 'we_cache_' . $entry, (strlen($ser) > 1024 ? (gzcompress($ser, 6) ?: '' ) : $ser));
		if($lock){
			we_base_file::unlock($lock);
		}
		if($expiry > 0){
			we_base_file::insertIntoCleanUp(WE_CACHE_DIR . 'we_cache_' . $entry, $expiry);
		}
		return true;
	}

	public static function touch(string $entry, int $expiry = 3600): bool{
		if($expiry > 0){
			we_base_file::insertIntoCleanUp(WE_CACHE_DIR . 'we_cache_' . $entry, $expiry);
			return true;
		}
		return false;
	}

	public static function clean(string $pattern = '', bool $exact = false): bool{
		$files = [];
		foreach(glob(WE_CACHE_PATH . 'we_cache_' . $pattern . ($exact ? '' : '*'), GLOB_NOSORT) as $file){
			we_base_file::deleteLocalFile($file);
			$files[] = DB_WE::$inst->escape($file);
		}
		if($files){
			DB_WE::$inst->query('DELETE FROM ' . CLEAN_UP_TABLE . ' WHERE Path IN ("' . implode('","', $files) . '")');
		}
		return true;
	}

}
