<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 135172fe7bdf $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2022-04-24 20:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_customer_frames extends we_modules_frame{
	/** @var array<string,string> */
	private $jsOut_fieldTypesByName;

	/** @var we_customer_view */
	protected $View;

	/** @var we_tree_customer */
	protected $Tree;

	/** @var int */
	private $advSearchFoundItems = 0;

	public function __construct(){
		$this->module = we_modules_info::CUSTOMER;
		parent::__construct();
		$this->showTreeHeader = true;
		$this->showTreeFooter = true;

		$this->Tree = new we_tree_customer($this->jsCmd);
		$this->View = new we_customer_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::CUSTOMER_EDIT, we_base_perm::CUSTOMER_NEW], ['we_customer_save']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::CUSTOMER_DELETE], ['we_customer_delete']]
						]
				]);
			case 'customer_admin':
				return $this->getHTMLCustomerAdmin();
			case 'branch_editor':
				return $this->getHTMLFieldEditor('branch', $mode);
			case 'field_editor':
				return $this->getHTMLFieldEditor('field', $mode);
			case 'sort_admin':
				return $this->View->getHTMLSortEditor($this);
			case 'settings':
				return $this->getHTMLSettings();
			case 'frameset':
				$this->View->Model->clearSessionVars();
				$this->View->settings->load();
				//FIXME: this causes the tree to load twice!
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), ($sid = we_base_request::int('sid', -1)) !== -1 ? '&sid=' . $sid : '');
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	//TODO: move editor-body relatetd stuff to weCustomerView! => note dependencies on jsOut_fieldTypesByName
	function getHTMLFieldsSelect(string $branch): we_html_select{
		$select = new we_html_select(['name' => 'branch']);

		$fields_names = $this->View->Model->getFieldsNames($branch);
		$this->jsOut_fieldTypesByName = [];
		foreach($fields_names as $val){
			$tmp = $this->View->getFieldProperties(($branch ? $branch . '_' : '') . $val);
			$this->jsOut_fieldTypesByName[$val] = ($tmp['type'] ?? '');
		}
		if($fields_names){
			foreach($fields_names as $k => $field){
				$field = (string) $field;
				$select->addOption((string) $k, (we_customer_customer::isProperty($field) ?
						$this->View->settings->getPropertyTitle($field) :
						$field)
				);
			}
		}

		return $select;
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$tabs = new we_gui_tabs();

		$branches_names = $this->View->Model->getBranchesNames();

		/* Uncomment for 9.2
		 * 
		$tabs->addTab('', 'fa-list', false, '\'' . g_l('modules_customer', '[all]') . '\'', ['id' => 'all', 'title' => g_l('modules_customer', '[all]')]);
		$allTabs[g_l('modules_customer', '[all]')] = 'all';

		$text = $this->View->Model->Username;
		 *
		 */

		// Remove for 9.2
		$tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, '\'' . g_l('modules_customer', '[common]') . '\'', ['id' => "common", 'title' => g_l('modules_customer', '[common]')]);
		$allTabs = [g_l('modules_customer', '[common]') => 'common'];
		$branchCount = 0;
		foreach($branches_names as $branch){
			$tabs->addTab('', 'fa-object-group', false, '\'' . $branch . '\'', ['id' => 'branch_' . $branchCount, 'title' => $branch]);
			$allTabs[$branch] = 'branch_' . $branchCount;
			$branchCount++;
		}
		$tabs->addTab('', 'fa-object-ungroup', false, '\'' . g_l('modules_customer', '[other]') . '\'', ['id' => 'other', 'title' => g_l('modules_customer', '[other]')]);
		$allTabs[g_l('modules_customer', '[other]')] = 'other';
//((top.content.activ_tab=="' . g_l('modules_customer','[other]') . '") )
		// Remove until here 9.2

		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$tabs->addTab('', 'fa-shopping-cart', false, '\'' . g_l('modules_customer', '[orderTab]') . '\'', ['id' => 'orderTab', 'title' => g_l('modules_customer', '[orderTab]')]);
			$allTabs[g_l('modules_customer', '[orderTab]')] = 'orderTab';
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$tabs->addTab('', 'fa-file-o', false, '\'' . g_l('modules_customer', '[objectTab]') . '\'', ['id' => 'objectTab', 'title' => g_l('modules_customer', '[objectTab]')]);
			$allTabs[g_l('modules_customer', '[objectTab]')] = 'objectTab';
		}
		$tabs->addTab('', 'fa-file', false, '\'' . g_l('modules_customer', '[documentTab]') . '\'', ['id' => 'documentTab', 'title' => g_l('modules_customer', '[documentTab]')]);
		$allTabs[g_l('modules_customer', '[documentTab]')] = 'documentTab';

		// Remove for 9.2
		// All tab should be last #11913
		$tabs->addTab('', 'fa-list', false, '\'' . g_l('modules_customer', '[all]') . '\'', ['id' => 'all', 'title' => g_l('modules_customer', '[all]')]);
		$allTabs[g_l('modules_customer', '[all]')] = 'all';

		$text = $this->View->Model->Username;
		// Remove until here 9.2

		//TODO: we have the following body in several modules!
		$body = we_html_element::body(['onresize' => "weTabs.setFrameSize()", 'onload' => "loaded();", 'id' => 'eHeaderBody',], we_html_element::div(['id' => 'main'], we_html_element::div([
						'id' => 'headrow'], we_html_element::b(str_replace(' ', '&nbsp;', g_l('modules_customer', '[customer]')) . ':&nbsp;') .
						we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . str_replace(' ', '&nbsp;', $text ?? '') . '</b> (ID:' . $this->View->Model->ID . ')'
						)
					) .
					$tabs->getHTML()
				)
		);

		return $this->getHTMLDocument($body, we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_top.js', '', ['id' => 'loadVarCustomerHeader', 'data-customerHeader' => setDynamicVar($allTabs)])
		);
	}

	protected function getHTMLEditorBody(): string{
		$hiddens = ['cmd' => 'we_customer_edit', 'pnt' => 'edbody', 'activ_sort' => 0];

		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen($this->jsCmd);
		}

		$branch = we_base_request::string('branch', g_l('modules_customer', '[common]'));

		return $this->getHTMLDocument(we_html_element::body(
					['class' => 'weEditorBody', 'onload' => "loaded=1;", 'onunload' => "doUnload();"], we_html_element::form(
						['name' => we_base_constants::WE_FORM, 'autocomplete' => 'off'], $this->View->getCommonHiddens($hiddens) .
						$this->View->getHTMLProperties($branch))), $this->View->getJSProperty());
	}

	protected function getHTMLTreeHeader(): string{
		$select = $this->View->getHTMLSortSelect();
		$select->setAttributes(['onchange' => "applySort();", 'style' => 'width:150px']);
		$select->selectOption($this->View->settings->getSettings('default_sort_view'));

		return we_html_element::form(['name' => 'we_form_treeheader'], we_html_element::hiddens([
					'pnt' => 'treeheader',
					'pid' => '0',
					'cmd' => 'no_cmd']) .
				$select->getHtml() .
				we_html_button::create_button(we_html_button::RELOAD, "javascript:applySort();") .
				we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('show_sort_admin')")
		);
	}

	protected function getHTMLTreeFooter(): string{
		return $this->getHTMLSearchTreeFooter();
	}

	private function getHTMLCustomerAdmin(): string{
		$branch = we_base_request::string("branch", g_l('modules_customer', '[other]'));
		$branch_select = we_base_request::string("branch", g_l('modules_customer', '[other]'));

		$select = $this->View->getHTMLBranchSelect(false);
		$select->setAttributes(['name' => "branch_select", 'class' => 'weSelect', 'onchange' => "selectBranch()", 'style' => "width:150px;"]);
		$select->selectOption($branch_select);

		$fields = $this->getHTMLFieldsSelect($branch);
		$fields->setAttributes(['name' => 'fields_select', 'size' => 15, 'onchange' => "setFieldsButtons(this.selectedIndex,this.length);", 'style' => 'width:350px;height:250px;']);
		//$hiddens = rray('name' => "field", 'value' => ''));

		$buttons_table = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('open_add_field')") .
			we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('open_edit_field')", '', '', true, false, '', false, '', '', 'editFieldButton') .
			we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete_field')", '', '', true, false, '', false, '', '', 'deleteFieldButton') .
			we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('move_field_up')", '', '', true, false, '', false, '', '', 'moveFieldUpButton') .
			we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('move_field_down')", '', '', true, false, '', false, '', '', 'moveFieldDownButton');

		$table = (new we_html_table(['class' => 'default', 'width' => '500'], 4, 5))
			->setCol(0, 0, g_l('modules_customer', '[branch]'), ['class' => "defaultfont lowContrast", 'style' => 'padding-right:10px;'])
			->setCol(0, 2, g_l('modules_customer', '[branch_select]'), ['class' => "defaultfont lowContrast"])
			->setCol(1, 0, we_html_tools::textInput('branch', $branch, 0, ['style' => "width:350px;", 'required' => 'required', 'placeholder' => g_l('modules_customer', '[field_branch_placeholder]')]), [
				'style' => 'padding-right:10px;'])
			->setCol(1, 2, $select->getHtml(), ['style' => 'padding-right:10px;'])
			->setCol(1, 3, we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('add_branch')", '', '', false, false, '', false, '', '', 'addBranchButton'), [
				'style' => 'padding-bottom:10px;'])
			->setCol(1, 4, we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('open_edit_branch')", '', '', false, false, '', false, '', '', 'editBranchButton'), [
				'style' => 'padding-bottom:10px;'])
			->setCol(2, 0, g_l('modules_customer', '[fields]'), ['class' => "defaultfont lowContrast", 'style' => 'vertical-align:top;'])
			->setCol(3, 0, $fields->getHtml(), ['style' => 'vertical-align:top;padding-right:10px;'])
			->setCol(3, 2, $buttons_table, ['style' => 'vertical-align:top;']);

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_admin.js') .
					we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "return false;"], we_html_element::hiddens([
							'cmd' => 'switchBranch',
							'pnt' => 'customer_admin']) .
						we_html_tools::dialogLayout($table->getHtml(), g_l('modules_customer', '[field_admin]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose()"))
					)
				)
		);
	}

	private function getHTMLFieldEditor(string $type, int $mode): string{
		$field = we_base_request::string('field', '');
		$branch = we_base_request::string('branch', g_l('modules_customer', '[other]'));

		$hiddens = we_html_element::hiddens([
				'pnt' => 'field_editor',
				'cmd' => 'no_cmd',
				'branch' => $branch,
				'art' => (string) $mode,
				($type === 'field' ? 'field' : '') => $field]);

		$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
		$curType = '';

		switch($type){
			case 'branch':
				$hiddens .= we_html_element::hidden('pnt', 'branch_editor');
				$edit = (new we_html_table(['width' => '300'], 1, 2))
					->setCol(0, 0, g_l('modules_customer', '[field_name]'), ['style' => 'vertical-align:middle;', 'class' => "defaultfont lowContrast"])
					->setCol(0, 1, we_html_tools::textInput("name", $branch), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);

				$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_branch')");
				break;
			default:
				$hiddens .= we_html_element::hidden('pnt', 'field_editor');
				$field_props = $this->View->getFieldProperties($field);

				$types = new we_html_select(['name' => 'field_type', 'class' => 'weSelect', 'style' => "width:200px;", 'onchange' => "fieldTypeChange(this.value);"]);
				$types->addOptions(array_combine(array_keys(we_customer_settings::FIELD_TYPES), array_keys(we_customer_settings::FIELD_TYPES)));
				if(isset($field_props['type'])){
					$types->selectOption($field_props['type']);
					$curType = $field_props['type'];
				} else {
					$curType = 'input';
				}

				/* $enc = (new we_html_select(['name' => "field_encrypt", 'class' => "weSelect", 'style' => "width:200px;"]))
				  ->addOptions([0 => g_l('global', '[no]'), 1 => g_l('global', '[yes]')])
				  ->selectOption(!empty($field_props['encrypt']) ? 1 : 0); */

				$edit = (new we_html_table(['width' => '300', 'id' => 'EditorRpc'], 5, 2))
					->setCol(0, 0, g_l('modules_customer', '[branch]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(0, 1, $branch, ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(1, 0, g_l('modules_customer', '[field_name]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(1, 1, we_html_tools::textInput('name', ($field_props['name'] ?? '')), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(2, 0, g_l('modules_customer', '[field_type]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(2, 1, $types->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(3, 0, g_l('modules_customer', '[field_default]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(3, 1, we_html_tools::textInput("field_default", ($field_props['default'] ?? '')), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					/* ->setCol(4, 0, g_l('modules_customer', '[encryptField]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					  ->setCol(4, 1, $enc->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']) */
					->setCol(5, 0, g_l('global', '[required_field]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(5, 1, we_html_forms::checkboxWithHidden((bool) ($field_props['field_required'] ?? false), 'field_required', '', false, '', '', false), ['style' => 'vertical-align:middle;',
					'class' => 'defaultfont']);

				$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_field')");
		}

		return
			$this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();fieldTypeChange('" . $curType . "');"], we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_admin.js') .
					we_html_element::form(['name' => we_base_constants::WE_FORM], $hiddens .
						we_html_tools::dialogLayout($edit->getHtml(), '', we_html_button::position_yes_no_cancel($save, '', $cancel)
						)
					)
				)
		);
	}

	public function getHTMLEditorRpc(string $field_type, int $mode, string $oldname): string{
		$branch = we_base_request::string('branch', g_l('modules_customer', '[other]'));
		$field_props = $this->View->getFieldProperties($oldname);

		$types = new we_html_select(['name' => 'field_type', 'class' => 'weSelect', 'style' => "width:200px;", 'onchange' => "fieldTypeChange(this.value);"]);
		$types->addOptions(array_combine(array_keys(we_customer_settings::FIELD_TYPES), array_keys(we_customer_settings::FIELD_TYPES)));
		$types->selectOption($field_type);

		// initialise table
		$curcol = 0;
		$edit = (new we_html_table(['width' => '300', 'id' => 'EditorRpc'], 5, 2))
			->setCol($curcol, 0, g_l('modules_customer', '[branch]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
			->setCol($curcol++, 1, $branch, ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
			->setCol($curcol, 0, g_l('modules_customer', '[field_name]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
			->setCol($curcol++, 1, we_html_tools::textInput('name', ($field_props['name'] ?? '')), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
			->setCol($curcol, 0, g_l('modules_customer', '[field_type]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
			->setCol($curcol++, 1, $types->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
			->setCol($defcol = $curcol, 0, g_l('modules_customer', '[field_default]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
			->setCol($curcol++, 1, we_html_tools::textInput("field_default", $field_props['default'] ?? ''), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);

		switch($field_type){
			case we_customer_settings::TYPE_MULTISELCT:
				$default = str_replace(',', "\r\n", $field_props['default']);
				$edit->setCol($defcol, 1, we_html_element::textArea(['name' => 'field_default', 'style' => 'vertical-align:middle; width: 150px;', 'placeholder' => g_l('modules_customer', '[select_textarea_placeholder]'),
						], $default));

			case we_customer_settings::TYPE_SELECT:
				$edit
					->setCol($curcol, 0, g_l('global', '[values]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol($curcol++, 1, we_html_element::textArea(['name' => 'field_values', 'style' => 'vertical-align:middle; width: 150px;', 'placeholder' => g_l('modules_customer', '[select_textarea_placeholder]'),
							'required' => 'required'], implode("\n", $field_props['values'] ?: []) ?: ''));
				break;
			case we_customer_settings::TYPE_INPUT:
			case we_customer_settings::TYPE_TEXTAREA:
				// prepare enc
				/* $enc = (new we_html_select(['name' => "field_encrypt", 'class' => "weSelect", 'style' => "width:200px;"]))
				  ->addOptions([0 => g_l('global', '[no]'), 1 => g_l('global', '[yes]')])
				  ->selectOption(!empty($field_props['encrypt']) ? 1 : 0); */

				/* $edit
				  ->setCol($curcol, 0, g_l('modules_customer', '[encryptField]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
				  ->setCol($curcol++, 1, $enc->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']); */
				break;
			case we_customer_settings::TYPE_COUNTRY:
				// similar to we_customer_view
				$langcode = array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de';

				$countrycode = array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';
				$countryselect = new we_html_select(['name' => 'field_default', 'style' => 'width:200px;', 'class' => 'wetextinput']);

				$topCountries = array_flip(WE_COUNTRIES_TOP);

				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$shownCountries = array_flip(WE_COUNTRIES_SHOWN);
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				if($oldLocale){
					setlocale(LC_ALL, $oldLocale);
				}
				if(WE_COUNTRIES_DEFAULT){
					$countryselect->addOption('--', WE_COUNTRIES_DEFAULT);
				}
				$countryselect->addOptions($topCountries);
				if(!empty($topCountries) && !empty($shownCountries)){
					$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
				}

				$countryselect->addOptions($shownCountries);
				$countryselect->selectOption($field_props['default'] ?? '');

				$edit->setCol($defcol, 1, $countryselect->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);
				break;

			case we_customer_settings::TYPE_LANGUAGE:
				if(!empty(we_base_WE::$Langs['FrontendLanguages'])){
					$frontendL = we_base_WE::$Langs['FrontendLanguages'];
					foreach($frontendL as &$lcvalue){
						$lccode = explode('_', $lcvalue);
						$lcvalue = $lccode[0];
					}
					unset($lcvalue);
					$languageselect = new we_html_select(['name' => 'field_default', 'style' => 'width:240px;', 'class' => 'wetextinput', 'onchange' => "top.content.setHot();"]);
					foreach(we_base_country::g_l_a('languages', '') as $languagekey => $languagevalue){
						if(in_array($languagekey, $frontendL, true)){
							$languageselect->addOption((string) $languagekey, (string) $languagevalue);
						}
					}
					$languageselect->selectOption($field_props['default'] ?? '');
					$languagehtml = $languageselect->getHtml();
					unset($languageselect);
				} else {
					$languagehtml = 'no FrontendLanguages defined';
				}
				$edit
					->setCol($defcol, 0, g_l('modules_customer', '[field_default]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol($defcol, 1, $languagehtml, ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);

				break;
			case we_customer_settings::TYPE_IMG:
				$cmd1 = "document.we_form.elements['default_value'].value";
				$imgId = (int) ($field_props['default'] ?? 0);
				$img = new we_document_image($imgId);
				$defaulthtml = '
				<table class="weEditTable">
					<tr>
						<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $img->getPreviewImg() . we_html_element::hidden('field_default', $imgId) . '</td>
					</tr>
					<tr>
						<td class="weEditmodeStyle" colspan="2" style="text-align:center">' .
					we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image', {id:'" . $imgId . "',table:WE().consts.tables.FILE_TABLE,targetID:'field_default',jsCmd:'refreshForm',open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})") . we_html_button::create_button(we_html_button::TRASH, "javascript:" . $cmd1 . "='';refreshForm();") .
					'</td>
					</tr>
				</table>';

				$edit->setCol($defcol, 1, $defaulthtml, ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);
				break;
			case we_customer_settings::TYPE_DATETIME:
			case we_customer_settings::TYPE_DATE:
				$edit->setCol($defcol, 1, we_html_tools::textInput("field_default", ($field_props['default'] ?? ''), 0, [], 'date'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);
				break;
			case we_customer_settings::TYPE_NUMBER:
				$edit->setCol($defcol, 1, we_html_tools::textInput("field_default", ($field_props['default'] ?? ''), 0, [], 'number'), ['style' => 'vertical-align:middle;',
					'class' => 'defaultfont']);
				break;
		}

		// add required if not img
		if($field_type !== we_customer_settings::TYPE_IMG){
			$edit
				->setCol($curcol, 0, g_l('global', '[required_field]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
				->setCol($curcol++, 1, we_html_forms::checkboxWithHidden($field_props['required'] ? true : false, 'field_required', '', false, '', '', false), ['style' => 'vertical-align:middle;',
					'class' => 'defaultfont']);
		}

		return $edit->getHtml();
	}

	function getHTMLBox(string $content, string $headline = '', int $width = 100, int $height = 50, int $w = 25, int $vh = 0, int $ident = 0, int $space = 5, string $headline_align = "left", string $content_align = "left"): string{
		$table = (new we_html_table(['width' => $width . 'px', 'height' => $height . 'px', 'class' => 'default', 'style' => 'margin-left:' . $ident . 'px;margin-top:' . $vh . 'px;margin-bottom:' . ($w && $headline ? $vh : 0) . 'px;'], 1, 2))
			->setCol(0, 0, str_replace(' ', '&nbsp;', $headline), ['style' => 'vertical-align:middle;text-align:' . $headline_align . ';padding-right:' . $space . 'px;',
				'class' => 'defaultfont lowContrast'])
			->setCol(0, 1, $content, ['style' => 'vertical-align:middle;text-align:' . $content_align]);
		return $table->getHtml();
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$sortField = we_base_request::string('sort', we_base_request::NOT_VALID_S);

		if($sortField === we_base_request::NOT_VALID_S){
			$sortField = $this->View->settings->getSettings('default_sort_view');
		}
		we_tree_customer::setSort(empty($this->View->settings->SortView[$sortField]) ? '' : $sortField);

		$offset = we_base_request::int('offset', 0);

		if(we_base_request::string('error')){
			$this->jsCmd->addMsg(g_l('modules_customer', '[error_download_failed]'), we_base_util::WE_MESSAGE_ERROR);
		}

		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_tree_customer::getItems($pid, $offset, $this->Tree->default_segment), 'sorted' => false]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd'])
					)
				)
		);
	}

	function getHTMLSearch(): string{//TODO: this is popup search editor: make separate frameset for popups!
		$colspan = 4;

		$mode = we_base_request::int('mode', 0);

		$hiddens = we_html_element::hiddens([
				'pnt' => 'search',
				'cmd' => 'search',
				'search' => '1',
				'mode' => (string) $mode
		]);

		$search_but = we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('search')");

		$search = (new we_html_table(['class' => 'default', 'width' => '550', 'height' => '50'], 4, 3))
			->setRowAttributes(0, ['style' => 'vertical-align:top'])
			->setCol(0, 0, g_l('modules_customer', '[search_for]'), ['class' => 'defaultfont', 'colspan' => '3', 'style' => 'padding-bottom: 3px;']);


		if($mode){
			$result = $this->getHTMLAddSearch($search);
			$foundItems = $this->advSearchFoundItems;
		} else {
			$search->setCol(1, 0, we_html_tools::textInput('keyword', we_base_request::string('keyword', ''), 0, [
					], 'text', '550px'), ['style' => 'padding-bottom:5px;']
			);

			$sw = we_html_button::create_button(we_html_button::DIRRIGHT, "javascript:we_cmd('switchToAdvance')");

			$search->setCol(3, 0, we_html_element::div(['class' => 'defaultfont'], g_l('modules_customer', '[advanced_search]')) .
				$sw .
				$search_but, ['style' => 'text-align:right', 'colspan' => $colspan]
			);
			$hiddens .= we_html_element::hidden('count', 1);

			$max_res = we_customer_settings::MaxSearchResults;
			$result = (($k = we_base_request::string('keyword')) && we_base_request::bool('search') ?
				$this->View->getSearchResults($k, $max_res) :
				[]);

			$foundItems = count($result);
		}

		// finally, build the result array
			$content = [];
			foreach($result as $id => $text){
				$content[] = [
					we_html_button::create_button(we_html_button::VIEW, "javascript:WE().util.dialogTop(window).opener.top.content.we_cmd('we_customer_edit'," . $id . ")"),
					$text,
				];
			}

		$table = (new we_html_table(['width' => '550', 'height' => '50'], 3, 1))
			->setCol(0, 0, $search->getHtml())
			->setCol(1, 0, g_l('modules_customer', '[num_data_sets]') . ($foundItems ? ' (' . $foundItems . ')' : ''))
			->setCol(2, 0, $this->getResultTable($content));
//need to call to have jsOut_fieldTypesByName filled
		$this->getHTMLFieldsSelect('');

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => ($mode ? '' : 'document.we_form.keyword.focus();')], $this->View->getJSSearch() .
					we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_functions.js', /* (we_base_request::bool('mode') ? "setTimeout(lookForDateFields, 1);" : '') */ '', [
						'id' => 'loadVarCustomerFuntions', 'data-customerFunctions' => setDynamicVar($this->jsOut_fieldTypesByName)]) .
					we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "we_cmd('search');return false;"], $hiddens .
						we_html_tools::dialogLayout(
							$table->getHtml(), g_l('modules_customer', '[search]'), we_html_button::position_yes_no_cancel('', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"))
						)
					)
				)
		);
	}

	private function getSearchListRow(array $content): string{
		return '<tr class="middlefont"><td style="min-width:70px;">' . $content[0] . '</td>
<td style="min-width:70px;">' . $content[1] . '</td></tr>';
	}

	private function getResultTable(array $content):string{
		$out = '';
		foreach($content as $row){
			$out .= $this->getSearchListRow($row);
		}
		return '<table class="contentTable" cellpadding="5" data-bla="hoi"><tbody style="background-color:#fff;width:100%" id="contentTbody">' .
						$out .
						'</tbody></table>'
		;
	}

	private function getHTMLSettings(): string{
		if(we_base_request::string('cmd') === 'save_settings'){
			$this->View->processCommands($this->jsCmd);
			$closeflag = true;
		} else {
			$closeflag = false;
		}

		$default_sort_view_select = $this->View->getHTMLSortSelect();
		$default_sort_view_select->setAttributes(['name' => "default_sort_view", 'style' => 'width:200px;']);
		$default_sort_view_select->selectOption($this->View->settings->getSettings('default_sort_view'));

		$cur = 0;
		$table = (new we_html_table(['class' => 'default', 'style' => 'margin-right:10px;'], 5, 2))
			->setCol($cur, 0, g_l('modules_customer', '[default_sort_view]') . ":&nbsp;", ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, $default_sort_view_select->getHtml())
			->setCol(++$cur, 0, g_l('modules_customer', '[treetext_format]') . ":&nbsp;", ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, we_html_tools::textInput("treetext_format", $this->View->settings->getSettings('treetext_format')));

		$default_order = (new we_html_select(['name' => 'default_order', 'style' => 'width:250px;', 'class' => 'weSelect']))
			->addOption('', g_l('modules_customer', '[none]'));

		foreach(we_customer_settings::ORDER_TABLE as $ord){
			$ordval = g_l('modules_customer', ($ord === 'ASC') ? '[ASC]' : '[DESC]');
			$default_order->addOption($ord, $ordval);
		}
		$default_order->selectOption($this->View->settings->getSettings('default_order'));

		$table
			->setCol(++$cur, 0, g_l('modules_customer', '[default_order]') . ':&nbsp;', ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, $default_order->getHtml());

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_settings')");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_tools::dialogLayout(
						we_html_element::hiddens(['pnt' => "settings", 'cmd' => '']) .
						$table->getHtml(), g_l('modules_customer', '[settings]'), we_html_button::position_yes_no_cancel($save, $close)
					)
				)
		);

		if($closeflag){
			$this->jsCmd->addCmd('close_dialog');
		}

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/settings.js'));
	}

	/**
	 *
	 * @param we_html_table $search
	 * @return string[]
	 */
	public function getHTMLAddSearch(we_html_table $search): array{
		$count = we_base_request::int('count');
		$search_arr = [];

		$search_but = we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('search')");
		$colspan = 4;

		$fieldDefinitions = $this->db->metadata(CUSTOMER_TABLE, we_database_base::META_FULL);
		for($i = 0; $i < $count; $i++){
			$search_arr[$i] = [
				'logic' => we_base_request::string('logic_' . $i),
				'branch' => ($branch = we_base_request::string('branch_' . $i)),
				'field' => ($field = we_base_request::string('field_' . $i)),
				'operator' => we_base_request::int('operator_' . $i),
				'value' => we_base_request::string('value_' . $i),
				'type' => ($field = substr($field, strlen($branch) + 1) ?: '') && !empty($fieldDefinitions[$field]) ? $fieldDefinitions[$field]['dbType'] : '',
			];
		}

		$advsearch = new we_html_table([], 1, 4);
		$branch = $this->View->getHTMLBranchSelect();
		$branch->setOptionVT(1, g_l('modules_customer', '[other]'), g_l('modules_customer', '[other]'));

		$c = 0;

		foreach($search_arr as $i => $cur){
			$branch->selectOption($cur['branch']);
			$field = (!$cur['branch'] ?
				$this->getHTMLFieldsSelect(g_l('modules_customer', '[common]')) :
				$this->getHTMLFieldsSelect($cur['branch']));

			$field->selectOption($cur['field']);

			$branch->setAttributes(['name' => 'branch_' . $i, 'class' => 'weSelect', 'onchange' => "we_cmd('selectBranch')", 'style' => "width:145px"]);
			$field->setAttributes(['name' => 'field_' . $i, 'style' => "width:145px", 'class' => 'weSelect', 'onchange' => "we_cmd('selectField')"]);

			if($i != 0){
				$advsearch->setCol($c, 0, we_html_tools::select('logic_' . $i, we_customer_view::SEARCH_LOGIC, 1, $cur['logic'], false, [], true, 70), ['colspan' => $colspan]);
				++$c;
			}

			switch(strtolower($cur['type'])){
				case 'timestamp':
					$type = 'datetime';
					break;
				case 'int':
				case 'tinyint':
					$type = 'int';
					break;
				default:
					$type = 'text';
			}

			$advsearch
				->setCol($c, 0, $branch->getHtml())
				->setCol($c, 1, $field->getHtml())
				->setCol($c, 2, we_html_tools::select('operator_' . $i, we_customer_view::SEARCH_OPERATORS, 1, $cur['operator'], false, [
						], true, 60))
				->setCol($c, 3, ($type === 'datetime' ? we_html_tools::getNewDateInput('value_' . $i, new DateTime($cur['value']), we_html_tools::FORMAT_DATE) :
						we_html_tools::textInput('value_' . $i, $cur['value'])) . we_html_element::hidden('type_' . $i, $type));
			++$c;
		}

		$advsearch->setCol($c, 0, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_search')") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_search')")
			, ['colspan' => $colspan, 'style' => 'padding-top:5px;']
		);

		$search
			->setCol(1, 0, we_html_element::hidden('count', $count) . $advsearch->getHtml())
			->setCol(3, 0, "<table class='default'><tr><td>" .
				we_html_element::div(['class' => "defaultfont lowContrast"], g_l('modules_customer', '[simple_search]')) .
				we_html_button::create_button('fa:btn_direction_left,fa-lg fa-caret-left', "javascript:we_cmd('switchToSimple')") .
				$search_but
				. '</td><td>&nbsp;</td></tr></table>', ['colspan' => $colspan, 'style' => 'text-align:right;padding-top:5px;']
		);
		$max_res = we_customer_settings::MaxSearchResults;
		$result = ($search_arr && we_base_request::bool('search') ? self::getAdvSearchResults($search_arr, $max_res) : []);

		$this->advSearchFoundItems = count($result);

		return $result;
	}

	/**
	 *
	 * @param array<int,array{logic:string,branch:string,field:string,operator:int,value:string}> $keywords
	 * @param int $res_num
	 * @return array
	 */
	private static function getAdvSearchResults(array $keywords, int $res_num): array{
		$where = '';
		$comm = g_l('modules_customer', '[common]');
		foreach($keywords as $search){
			$field = trim(str_replace($comm . '_', '', $search['field']));
			if(!$field){
				continue;
			}
			$where .= ' ' . $search['logic'] . ' ' .
				$field . ' ' . we_customer_view::SEARCH_OPERATORS[$search['operator']] . " '" .
				(is_numeric($search['value']) ? (float) $search['value'] : DB_WE::$inst->escape((string) $search['value'])) .
				"'";
		}

		return DB_WE::$inst->getAllFirstq('SELECT ID,CONCAT(Username, " (",Forename," ",Surname,")") AS user FROM ' . CUSTOMER_TABLE . ' WHERE ' . ($where ?: '0') . ' ORDER BY Username LIMIT 0,' . $res_num, false);
	}

}
