<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 135172fe7bdf $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2022-04-24 20:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_customer_view extends we_modules_view{

	public const SEARCH_OPERATORS = ['=', '<>', '<', '<=', '>', '>=', 'LIKE'];
	public const SEARCH_LOGIC = ['AND' => 'AND', 'OR' => 'OR'];

	/** @var we_customer_settings */
	var $settings;

	const ERR_SAVE_BRANCH = -10;
	const ERR_SAVE_FIELD_INVALID = -7;
	const ERR_SAVE_PROPERTY = -5;
	const ERR_SAVE_FIELD_EXISTS = -4;
	const ERR_SAVE_FIELD_NOT_EMPTY = -3;
	const SAVE_OK = 0;

	/** @var we_customer_customer */
	var $Model;

	function __construct(){
		parent::__construct(we_modules_info::CUSTOMER);
		$this->Model = new we_customer_customer();
		$this->settings = new we_customer_settings();
		$this->settings->load();
	}

	function getCommonHiddens(array $cmds = []): string{
		return we_html_element::hiddens([
					'cmd' => ($cmds['cmd'] ?? ''),
					'pnt' => ($cmds['pnt'] ?? 'edbody'),
					'cmdid' => ($cmds['cmdid'] ?? ''),
					'activ_sort' => ($cmds['activ_sort'] ?? ''),
					'branch' => we_base_request::string('branch', g_l('modules_customer', '[common]'))
		]);
	}

	function getJSTop(): string{
		return
				parent::getJSTop() .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_property.js', '', ['id' => 'loadVarCustomer_property', 'data-customer' => setDynamicVar(array_merge(
									$jsVars, [
						'username' => $this->Model->Username
		]))]);
	}

	function getJSSearch(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_search.js');
	}

	private function saveCustomer(we_base_jsCmd $jscmd): void{
		// Userinput may be empty
		$this->Model->Username = trim($this->Model->Username ?? '');
		if(!$this->Model->Username){
			$jscmd->addMsg(g_l('modules_customer', '[username_empty]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if($this->Model->filenameNotValid()){
			$jscmd->addMsg(g_l('modules_customer', '[we_filename_notValid]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$newone = ($this->Model->ID ? false : true);

		$exists = $this->Model->isDuplicateUsername();
		if($exists){
			$jscmd->addMsg(sprintf(g_l('modules_customer', '[username_exists]'), $this->Model->Username), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if($_SESSION['weS']['module_sessions']['customer']->Password != $this->Model->Password || $this->Model->LoginDenied || $this->Model->AutoLoginDenied){
//delete autologins, if password is changed
			$this->db->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE WebUserID=' . intval($this->Model->ID));
		}

		$saveOk = $this->Model->save();

		if($saveOk){
			$tt = strtr(addslashes(fS('SELECT ' . $this->settings->treeTextFormatSQL . ' AS treeFormat FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . intval($this->Model->ID))), [
				'>' => '&gt;', '<' => '&lt;']);
			if($newone){
				$jscmd->addCmd('loadTree', ['clear' => false, 'items' => [[
					'id' => $this->Model->ID,
					'typ' => 'item',
					'parentid' => 0,
					'text' => $tt,
					'disable' => 0,
					'tooltip' => (($this->Model->Forename || $this->Model->Surname ) ? $this->Model->Forename . "&nbsp;" . $this->Model->Surname : "")
						]], 'sorted' => true]);
			} else {
				$jscmd->addCmd('updateTreeEntry', [
					'id' => $this->Model->ID,
					'text' => $tt]);
				$jscmd->addCmd('updateTitle', $this->Model->Username);
			}
		}

		if($saveOk){
			$jscmd->addMsg(sprintf(g_l('modules_customer', '[customer_saved_ok]'), addslashes($this->Model->Username)), we_base_util::WE_MESSAGE_NOTICE);
		} else {
			// get required fields without input
			$requiredEmpty = $this->Model->getRequiredEmpty();
			$jscmd->addMsg(sprintf(g_l('modules_customer', '[customer_saved_nok]'), addslashes($this->Model->Username)) . ($requiredEmpty ? ' ' . g_l('modules_customer', '[customer_required_fields_empty]') . ' ' . implode(' ', $requiredEmpty) : '') , we_base_util::WE_MESSAGE_ERROR);
		}
	}

	/**
	 * @param we_base_jsCmd $jscmd
	 * @return void
	 */
	function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('cmd')){
			case 'we_customer_new':
				$this->Model = new we_customer_customer();
				$this->settings->initCustomerWithDefaults($this->Model);
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Username);
				break;
			case 'we_customer_edit':
				$this->Model = new we_customer_customer(we_base_request::int("cmdid"));
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Username);
				break;
			case 'we_customer_save':
				$this->saveCustomer($jscmd);
				return;
			case 'we_customer_delete':
				$oldid = $this->Model->ID;
				$this->Model->delete();
				$this->Model = new we_customer_customer();

				$jscmd->addMsg(g_l('modules_customer', '[customer_deleted]'), we_base_util::WE_MESSAGE_NOTICE);
				$jscmd->addCmd('deleteTreeEntry', $oldid);
				$jscmd->addCmd('loadHome');
				break;
			case 'switchPage':
				break;
			case 'save_field':
				$branch = we_base_request::string('branch');
				$field = we_base_request::string('field');
				$field_name = we_base_request::string('name');
				$field_type = we_base_request::string('field_type');
				$field_default = we_base_request::string('field_default');
				$field_encrypt = we_base_request::bool('field_encrypt');
				//new Fields as of 9.0.7: required, values
				$field_values = we_base_request::string('field_values');
				$field_required = we_base_request::bool('field_required');


				$saveret = $this->saveField($field, $branch, $field_name, $field_type, $field_default, $field_encrypt, $field_values, $field_required);
				switch($saveret){
					case self::ERR_SAVE_BRANCH:
						$jscmd->addMsg(g_l('modules_customer', '[branch_no_edit]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					case self::ERR_SAVE_FIELD_INVALID:
						$jscmd->addMsg(g_l('modules_customer', '[we_fieldname_notValid]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					case self::ERR_SAVE_PROPERTY:
						$jscmd->addMsg(sprintf(g_l('modules_customer', '[cannot_save_property]'), $field_name), we_base_util::WE_MESSAGE_ERROR);
						break;
					case self::ERR_SAVE_FIELD_EXISTS:
						$jscmd->addMsg(g_l('modules_customer', '[fieldname_exists]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					case self::ERR_SAVE_FIELD_NOT_EMPTY:
						$jscmd->addMsg(g_l('modules_customer', '[field_not_empty]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					default:
						$this->Model->reloadPersistenSlots();
						$jscmd->addCmd('submitForm');
						$jscmd->addCmd('refreshForm');
						$jscmd->addCmd('close_dialog');
				}

				break;
			case 'delete_field':
				$field = we_base_request::string('fields_select');
				$ber = '';
				$fname = we_customer_customer::transFieldName($field, $ber);

				$this->deleteField(($ber == '' && preg_match('%' . g_l('modules_customer', '[other]') . '%i', $field) ? $fname : $field));

				$this->Model->reloadPersistenSlots();
				$jscmd->addMsg(sprintf(g_l('modules_customer', '[field_deleted]'), $fname, $ber), we_base_util::WE_MESSAGE_NOTICE);
				$jscmd->addCmd('refreshForm');
				break;
			case 'move_field_up':
				$field = we_base_request::string('fields_select');
				$this->moveField($field, true);
				$jscmd->addCmd('refreshForm');
				break;
			case 'move_field_down':
				$field = we_base_request::string('fields_select');
				$this->moveField($field, false);
				$jscmd->addCmd('refreshForm');
				break;
			case 'save_branch':
				$branch_new = we_base_request::string('name', '');
				$branch_old = we_base_request::string('branch', '');

				if(in_array($branch_new, [g_l('modules_customer', '[common]'), g_l('modules_customer', '[other]'), g_l('modules_customer', '[all]')], true)){
					$jscmd->addMsg(g_l('modules_customer', '[branch_no_edit]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				if($branch_new != $branch_old){
					$arr = $this->Model->getBranchesNames();

					if(in_array($branch_new, $arr, true)){
						$jscmd->addMsg(g_l('modules_customer', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
				}

				if($this->saveBranch($branch_old, $branch_new) == self::ERR_SAVE_PROPERTY){
					$jscmd->addMsg(sprintf(g_l('modules_customer', '[cannot_save_property]'), $branch_old), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$this->Model->reloadPersistenSlots();
				$jscmd->addCmd('setNewBranchName', g_l('modules_customer', '[other]'), g_l('modules_customer', '[common]'));
				break;
			case 'add_sort':
				$cout = 0;
				$found = false;
				$cname = '';
				while(!$found){//FIXME: might be an endless loop
					$cname = g_l('modules_customer', '[sort_name]') . $cout;
					if(!in_array($cname, array_keys($this->settings->SortView))){
						$found = true;
					}
					$cout++;
				}
				$this->settings->SortView[$cname] = [];

				break;
			case 'del_sort':
				if(($i = we_base_request::string('sortindex'))){
					unset($this->settings->SortView[$i]);
				}
				break;
			case 'add_sort_field':
				if(($i = we_base_request::string('sortindex'))){
					$this->settings->SortView[$i][] = ['branch' => '', 'field' => '', 'order' => ''];
				}
				break;
			case 'del_sort_field':
				if(($i = we_base_request::string('sortindex', '')) !== '' &&
						($j = we_base_request::int('fieldindex', -1)) !== -1){

					unset($this->settings->SortView[$i][$j]);
				}
				break;
			case 'save_sort':
				$this->settings->save();
				$sorting = array_merge([g_l('modules_customer', '[no_sort]')], array_keys($this->settings->SortView));
				$jscmd->addMsg(g_l('modules_customer', '[sort_saved]'), we_base_util::WE_MESSAGE_NOTICE);
				$jscmd->addCmd('setSorts', $sorting);
				break;
			case 'applySort':
				break;
			case 'save_settings':
				foreach($this->settings->getAllSettings() as $k){
					$set = we_base_request::string($k, we_base_request::NOT_VALID_S);
					if($set !== we_base_request::NOT_VALID_S){
						$this->settings->setSettings($k, $set);
					}
				}
				foreach(array_keys($this->settings->properties) as $k){
					$set = we_base_request::string($k, we_base_request::NOT_VALID_S);
					if($set !== we_base_request::NOT_VALID_S){
						$this->settings->properties[$k] = $set;
					}
				}

				if($this->settings->save()){
					$jscmd->addMsg(g_l('modules_customer', '[settings_saved]'), we_base_util::WE_MESSAGE_NOTICE);
					$jscmd->addCmd('close_dialog');
				} else {
					$jscmd->addMsg(g_l('modules_customer', '[settings_not_saved]'), we_base_util::WE_MESSAGE_NOTICE);
				}
				break;
			default:
		}

//FIXME: this data is not deleted on close - fix this!
		$_SESSION['weS']['module_sessions']['customer'] = $this->Model;
	}

	function processVariables(): void{
		if(isset($_SESSION['weS']['module_sessions']['customer'])){
			$this->Model = $_SESSION['weS']['module_sessions']['customer'];
		}
		if(($sid = we_base_request::int('sid'))){
			$this->Model = new we_customer_customer($sid);
			$_SESSION['weS']['module_sessions']['customer'] = $this->Model;
		}
		$dates = we_base_util::convertDateInRequest('customer');
		$empty = we_base_request::intList('isEmpty');
		foreach(array_keys($this->Model->persistent_slots) as $varname){
			$varname = (string) $varname;
			switch($varname){
				case 'LoginDenied':
					if(we_base_request::bool('customer', false, 'LoginDenied')){
						$this->Model->LoginDenied = true;
					} elseif(we_base_request::string('customer', '', 'Username')){
						$this->Model->LoginDenied = false;
					}
					break;
				case 'Password':
					$pw = we_base_request::string('customer', we_customer_customer::NOPWD_CHANGE, 'Password');
					if($pw && $pw != we_customer_customer::NOPWD_CHANGE){//keep old pwd
						$this->Model->Password = we_customer_customer::cryptPassword($pw);
					}
					break;
				default:
					if(isset($empty[$varname])){
						$this->Model->{$varname} = NULL;
						break;
					}
					if(isset($dates[$varname])){
						$this->Model->{$varname} = $dates[$varname];
						break;
					}
					if(($v = we_base_request::string('customer', we_base_request::NOT_VALID_S, $varname)) !== we_base_request::NOT_VALID_S){
						/*$isEncField = $this->settings->retriveFieldAdd($varname, 'encrypt');
						if($isEncField){
							if($v != we_customer_customer::ENCRYPTED_DATA){
								//fixme: should we store bin data??
								$this->Model->{$varname} = we_customer_customer::cryptData($v, SECURITY_ENCRYPTION_KEY, false);
							}
						} else {*/
							$this->Model->{$varname} = $v;
						//}
					}
			}
		}
		if(we_base_request::string('pnt') !== 'sort_admin'){
			return;
		}
		$counter = we_base_request::int('counter', -1);

		if($counter !== -1){
			$this->settings->SortView = [];

			for($i = 0; $i < $counter; $i++){
				$sort_name = we_base_request::string('sort_' . $i) ?:
						g_l('modules_customer', '[sort_name]') . '_' . $i;

				$fcounter = we_base_request::int('fcounter_' . $i, 1);

				if($fcounter > -1){
					$this->settings->SortView[$sort_name] = [];
				}
				for($j = 0; $j < $fcounter; $j++){
					$new = [];
					if(($b = we_base_request::string('branch_' . $i . '_' . $j))){
						$new['branch'] = $b;
					}
					if(($field = we_base_request::string('field_' . $i . '_' . $j))){
						$new['field'] = ($new['branch'] == g_l('modules_customer', '[common]') ?
								str_replace(g_l('modules_customer', '[common]') . '_', '', $field) :
								$field);
					}
					if(($func = we_base_request::string('function_' . $i . '_' . $j))){
						$new['function'] = $func;
					}
					if(($ord = we_base_request::string('order_' . $i . '_' . $j))){
						$new['order'] = $ord;
					}
					$this->settings->SortView[$sort_name][$j] = $new;
				}
			}
		}
	}

	/**
	 *
	 * @param string $field
	 * @return array{null:bool,default:?mixed,values:array,required:bool,name:string,type:string}|array
	 */
	function getFieldProperties(string $field): array{
		// Fixme : I feel like there is a more elegant way to handle 'other' branch...
		if(strpos($field, g_l('modules_customer', '[other]')) === 0){
			$field = str_replace(g_l('modules_customer', '[other]') . '_', '', $field);
		}
		$m = $this->db->metadata(CUSTOMER_TABLE, we_database_base::META_FULL);
		$meta = $m[$field] ?? []; //FIXME: fields without prefix are named Allgeimein_ - this has to be removed
		$specialtype = $this->settings->retriveFieldAdd($field, 'type', we_customer_settings::TYPE_INPUT);
		$branch = '';

		return $meta ? [
			//'encrypt' => (bool) $this->settings->retriveFieldAdd($field, 'encrypt'),
			'null' => $meta['nullable'],
			'default' => trim($meta['default'] ?? '', '\''),
			'values' => $meta['values'],
			'required' => !$meta['nullable'] && !isset($meta['default']),
			'name' => we_customer_customer::transFieldName($meta['name'], $branch),
			'type' => $this->settings->getFieldType($meta['dbType'], $specialtype),
				] : [
			//'encrypt' => false,
			'null' => true,
			'default' => '',
			'values' => '',
			'required' => false,
			'name' => '',
			'type' => '',
				];
	}

	/**
	 * @param string $field - contains full field name with branch of an already existing field
	 * @param string $branch - branch name
	 * @param string $field_name - field name without branch name
	 * @param string $field_type - type of the field
	 * @param string $field_default - predefined values
	 * @param bool $encrypt - should userinput be encrypted?
	 * @param string $field_values - possible values for select and multiselect
	 * @param bool $field_required - is the field required?
	 * @return int
	 */
	function saveField(string $field, string $branch, string $field_name, string $field_type, string $field_default, bool $encrypt, string $field_values = '', bool $field_required = false): int{
		if($branch == g_l('modules_customer', '[common]')){
			return self::ERR_SAVE_BRANCH;
		}

		if($branch == g_l('modules_customer', '[other]')){
			$field = str_replace(g_l('modules_customer', '[other]') . '_', '', $field);
		}
		if(!$field_name){
			return self::ERR_SAVE_FIELD_NOT_EMPTY;
		}

		// Check for default fields in select and multiselect
		if(!$field_values && in_array($field_type, ['select', 'multiselect'], true)){
			return self::ERR_SAVE_FIELD_NOT_EMPTY;
		}
		// replace all newlines
		$new_field_values = str_replace(["\r\n", "\n", "\r"], ',', $field_values);

		// Default Vals for Multiselect must be prepared
		if($field_type === 'multiselect'){
			$field_default = str_replace(["\r\n", "\n", "\r"], ',', $field_default);
		}

		$h = $this->db->isColExist(CUSTOMER_TABLE, $field);

		$new_field_name = (($branch && $branch !== g_l('modules_customer', '[other]')) ? $branch . '_' : '') . $field_name;

		if(preg_match('|[^a-z0-9\_]|i', $new_field_name)){
			return self::ERR_SAVE_FIELD_INVALID;
		}

		if($field != $new_field_name && $this->db->isColExist(CUSTOMER_TABLE, $new_field_name)){
			return self::ERR_SAVE_FIELD_EXISTS;
		}

		if(we_customer_customer::isProperty($field) ||
				we_customer_customer::isProtected($field) ||
				we_customer_customer::isProperty($new_field_name) ||
				we_customer_customer::isProtected($new_field_name) ||
				($branch == g_l('modules_customer', '[other]') && self::isReserved($new_field_name))){
			return self::ERR_SAVE_PROPERTY;
		}

		if($h){
			$this->settings->removeFieldAdd($field);
		}
		$this->settings->storeFieldAdd($new_field_name, 'default', $field_default);
		$this->settings->storeFieldAdd($new_field_name, 'type', $field_type);
		//$this->settings->storeFieldAdd($new_field_name, 'encrypt', $encrypt ? '1' : '0');
		// add new vars since 9.0.7
		$this->settings->storeFieldAdd($new_field_name, 'required', $field_required ? '1' : '0');
		$this->settings->storeFieldAdd($new_field_name, 'values', $new_field_values ?: '');


		if($h){
			if($field != $new_field_name){
				$this->db->renameCol(CUSTOMER_TABLE, $field, $new_field_name);
			}
			$this->db->changeColType(CUSTOMER_TABLE, $new_field_name, ($encrypt ? 'BLOB NOT NULL' : $this->settings->getDbType($field_type, $new_field_name, $field_default)));
		} else {
			//search position to insert
			$meta = array_reverse($this->db->metaFieldNames(CUSTOMER_TABLE));
			$last = '';
			$isOther = ($branch == g_l('modules_customer', '[other]'));
			$branch .= '_';
			foreach($meta as $cur){
				$cur = (string) $cur;
				if(str_contains($cur, $branch) || ($isOther && !str_contains($cur, '_'))){
					$last = $cur;
					break;
				}
			}
			$this->db->addCol(CUSTOMER_TABLE, $new_field_name, ($encrypt ? 'BLOB NOT NULL' : $this->settings->getDbType($field_type, $new_field_name, $field_default)), $last);
		}

		$this->settings->save();
		return self::SAVE_OK;
	}

	private static function isReserved(string $field): bool{
		return we_database_base::isReservedField($field) ||
				in_array(trim($field), array_merge(we_customer_customer::PROPERTY_FIELDS, we_customer_customer::PROTECTED_FIELDS), true);
	}

	function deleteField(string $field): void{
		$this->db->delCol(CUSTOMER_TABLE, $field);
		$this->settings->removeFieldAdd($field);
		$this->settings->save();
	}

	function saveBranch(string $old_branch, string $new_branch): int{
		$ret = true;
		foreach($this->db->metaFieldNames(CUSTOMER_TABLE) as $k){
			if(str_contains($k, $old_branch)){
				$banche = '';
				$fieldname = we_customer_customer::transFieldName($k, $banche);
				if($banche == $old_branch && $fieldname != ''){
					$ret = $this->db->renameCol(CUSTOMER_TABLE, $k, $new_branch . '_' . $fieldname);
					if(!$ret){
						break;
					}
				}
			}
		}

		//FIXME:if we have failed, all entries are renamed, not only the successfull ones
		$this->settings->renameFieldAdds($old_branch, $new_branch);
		$this->settings->save();
		return $ret ? self::SAVE_OK : self::ERR_SAVE_PROPERTY;
	}

	/**
	 *
	 * @param string $keyword
	 * @param int $res_num
	 * @return string[]
	 */
	function getSearchResults(string $keyword, int $res_num = 0): array{
		if(!$res_num){
			$res_num = we_customer_settings::MaxSearchResults;
		}

		$arr = explode(' ', strToLower($keyword));
		$array = [
			'AND' => [$arr[0]],
			'OR' => [],
			'AND NOT' => []
		];


		for($i = 1; $i < count($arr); $i++){
			switch($arr[$i]){
				case 'not':
					$i++;
					$array['AND NOT'][] = $arr[$i];
					break;
				case 'and':
					$i++;
					$array['AND'][] = $arr[$i];
					break;
				case'or':
					$i++;
					$array['OR'][] = $arr[$i];
					break;
				default:
			}
		}
		$condition = '';

		foreach($array as $ak => $av){
			foreach($av as $value){
				$conditionarr = [];
				foreach(array_keys($this->Model->persistent_slots) as $field){
					if(!we_customer_customer::isProtected($field) && $field !== 'Password'){
						$conditionarr[] = '`' . $field . '` LIKE "%' . $value . '%"';
					}
				}
				$condition .= ($condition ?
						' ' . $ak . ' (' . implode(' OR ', $conditionarr) . ')' :
						' (' . implode(' OR ', $conditionarr) . ')'
						);
			}
		}

		return array_map('htmlspecialchars', $this->db->getAllFirstq('SELECT ID,CONCAT(Username, " (",Forename," ",Surname,")") AS user FROM ' . $this->db->escape($this->Model->table) . ($condition ? ' WHERE ' . $condition : '') . ' ORDER BY Username' . ' LIMIT 0,' . $res_num, false)); //@phan-suppress-current-line PhanPartialTypeMismatchArgumentInternal
	}

	function getHTMLBranchSelect(bool $with_common = true, bool $with_other = true): we_html_select{
		$branches_names = $this->Model->getBranchesNames();

		$select = new we_html_select(['name' => 'branch']);

		if($with_common){
			$select->addOption(g_l('modules_customer', '[common]'), g_l('modules_customer', '[common]'));
		}

		if($with_other){
			$select->addOption(g_l('modules_customer', '[other]'), g_l('modules_customer', '[other]'));
		}

		foreach($branches_names as $branch){
			$select->addOption((string) $branch, (string) $branch);
		}

		return $select;
	}

	function getHTMLSortSelect(bool $include_no_sort = true): we_html_select{
		$sort = new we_html_select(['name' => 'sort', 'class' => 'weSelect']);

		if($include_no_sort){
			$sort->addOption('', g_l('modules_customer', '[no_sort]'));
		}

		$sort_names = array_map('htmlspecialchars', array_keys($this->settings->SortView));
		$sort->addOptions(array_combine($sort_names, $sort_names));

		return $sort;
	}

	private function getHTMLFieldControl(string $field, $value = '', bool $isRequired = false, array $props = []): string{ //Code used, when data is in session, not intial/DB
		//$hasEncContent = $isEncrypted && $value;
//fixme: all fields have to handle null, which needs some extra checkbox and disables the field value
		$required = $isRequired || $props['required'] ? 'required' : '';
		$curValue = $this->Model->{$field} ?? NULL;
		$isValueNull = is_null($curValue) && !($props['type'] === we_customer_settings::TYPE_SELECT && in_array('', $props['values'], true)) && !($props['type'] === we_customer_settings::TYPE_MULTISELCT) && !$props['required'];
		$disabled = $isValueNull ? 'disabled' : '';
			switch($props['type']){
			case we_customer_settings::TYPE_INPUT:
				$def = $props['default'];
				return we_html_tools::textInput('customer[' . $field . ']', (string) (/*($hasEncContent ? we_customer_customer::ENCRYPTED_DATA :*/ $value/*)*/ ?? ''), 0, ['oninput' => "top.content.setHot();",
							'style' => 'width:240px;',$disabled => $disabled, $required => $required, 'placeholder' => $def]);
			case we_customer_settings::TYPE_TEXTAREA:
				$def = $props['default'];
				return we_html_element::textArea(['name' => 'customer[' . $field . ']',$disabled => $disabled , 'style' => "width:240px;", 'placeholder' => $def, 'class' => "wetextarea", $required => $required], /*($hasEncContent ? we_customer_customer::ENCRYPTED_DATA : */$value/*)*/ ?? '');
			case we_customer_settings::TYPE_NUMBER:
				return we_html_tools::textInput('customer[' . $field . ']', (string) ((int) ($value ?? 0)), 0, ['oninput' => "top.content.setHot();", 'style' => 'width:240px;',$disabled => $disabled ,
							$required => $required], 'number');
			case we_customer_settings::TYPE_MULTISELCT:
				$values = is_array($value) ? $value : explode(',', $value ?? '');
				$defs = array_combine($props['values'], $props['values']);

				//empty string may be present in this case the field is not required anymore
				if($required && in_array('', $defs, true)){
					$required = '';
				}
				return we_html_tools::select('customer[' . $field . ']', $defs, 1, $values, true, ['class' => 'newSelect', 'style' => 'width:220px;', $required => $required, $disabled => $disabled,
							'onchange' => "top.content.setHot();"]);

			case we_customer_settings::TYPE_COUNTRY:
				$langcode = array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de';

				$countrycode = array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';
				$countryselect = new we_html_select(['name' => 'customer[' . $field . ']', 'style' => 'width:240px;', 'class' => 'wetextinput', 'id' => ($field === 'Gruppe' ? 'yuiAcInputPathGroupX' : ''),
					 $disabled => $disabled, 'onchange' => "top.content.setHot();", $required => $required]);

				$topCountries = array_flip(WE_COUNTRIES_TOP);

				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$shownCountries = array_flip(WE_COUNTRIES_SHOWN);
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				if($oldLocale){
					setlocale(LC_ALL, $oldLocale);
				}
				if(WE_COUNTRIES_DEFAULT){
					$countryselect->addOption('--', WE_COUNTRIES_DEFAULT);
				}
				$countryselect->addOptions($topCountries);
				if(!empty($topCountries) && !empty($shownCountries)){
					$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
				}

				$countryselect->addOptions($shownCountries);
				$countryselect->selectOption($value);
				return $countryselect->getHtml();

			case we_customer_settings::TYPE_LANGUAGE:
				if(!empty(we_base_WE::$Langs['FrontendLanguages'])){
					$frontendL = we_base_WE::$Langs['FrontendLanguages'];
					foreach($frontendL as &$lcvalue){
						$lccode = explode('_', $lcvalue);
						$lcvalue = $lccode[0];
					}
					unset($lcvalue);
					$languageselect = new we_html_select(['name' => 'customer[' . $field . ']', 'style' => 'width:240px;', 'class' => 'wetextinput', 'id' => ($field === 'Gruppe' ? 'yuiAcInputPathGroupX' : ''),
						 $disabled => $disabled, 'onchange' => "top.content.setHot();", $required => $required]);
					foreach(we_base_country::g_l_a('languages', '') as $languagekey => $languagevalue){
						if(in_array($languagekey, $frontendL, true)){
							$languageselect->addOption((string) $languagekey, (string) $languagevalue);
						}
					}
					$languageselect->selectOption($value);
					return $languageselect->getHtml();
				}
				return 'no FrontendLanguages defined';

			case we_customer_settings::TYPE_SELECT:
				if(!$this->Model->ID && $value === null){
					$value = $props['default'] ?? '';
				}
				$defs = array_combine($props['values'], $props['values']);
				//empty string may be present in this case the field is not required anymore
				if($required && in_array('', $defs)){
					$required = '';
				}
				return we_html_tools::select('customer[' . $field . ']', $defs, 1, [$value], false, ['style' => 'width:220px;', $required => $required,
							 $disabled => $disabled, 'onchange' => "top.content.setHot();"]);
			case we_customer_settings::TYPE_DATE:
				$format = we_html_tools::FORMAT_DATE;
			case we_customer_settings::TYPE_DATETIME:
				//$def = $props['default'];
				$format = ($format ?? we_html_tools::FORMAT_DATE_TIME); //@phan-suppress-current-line PhanCoalescingNeverNull
				try{
					$value = $value ? new DateTime($value) : NULL;
				}catch(Exception $e){
					//t_e('Parse of time string failed', $e);
					$value = new DateTime();
				}

				return we_html_tools::getNewDateInput('customer[we_date_' . $field . '][%s]', $value, $format, false, $isRequired, NULL, NULL, '', [$disabled => $disabled]);
			case we_customer_settings::TYPE_PASSWORD:
				$pass = new we_customer_generatePassword(8, ['caps' => 3, 'small' => 3, 'nums' => 2, 'specs' => 0]);
				return '<input id="newSecretPass" type="hidden" name="_none" value="' . $pass->PassGen() . '" />' . we_html_tools::textInput('customer[' . $field . ']', $value ?? '', 32, [
							 $disabled => $disabled, 'oninput' => "top.content.setHot();",
							'style' => "width:190px;", 'autocomplete' => 'new-password', $required => $required], 'password') . we_html_button::create_button(we_html_button::RELOAD, '', '', "var el=document.getElementsByName('password')[0];el.type='text';el.value=document.getElementById('newSecretPass').value;");
			case we_customer_settings::TYPE_IMG:
				$cmd1 = "document.we_form.elements['customer[" . $field . "]'].value";
				$imgId = (int) ($value ?? 0);
				$img = new we_document_image($imgId);
				return '
<table class="weEditTable">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $img->getPreviewImg() . we_html_element::hidden('customer[' . $field . ']', $imgId, '', [ $disabled => $disabled,]) . '</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' .
						we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image', {id:'" . $imgId . "',table:WE().consts.tables.FILE_TABLE,targetID:'customer[" . $field . "]',jsCmd:'refreshForm',open_doc:" . (we_base_permission::hasPerm([
									we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})") . we_html_button::create_button(we_html_button::TRASH, "javascript:" . $cmd1 . "='';refreshForm();") .
						'</td>
	</tr>
</table>';
			default:
				return we_html_tools::textInput('customer[' . $field . ']', $value ?? '', 0, [ $disabled => $disabled, 'onchange' => "top.content.setHot();", 'style' => 'width:240px;']);
		}
	}

	/**
	 * @param array<string,mixed> $common
	 * @param bool $isAll
	 * @return \we_html_multiIconBoxContentLine
	 */
	private function getCommonTable(array $common, bool $isAll): we_html_multiIconBoxContentLine{
		$table = new we_html_table(['width' => '500', 'height' => '50', 'class' => 'customer'], 1, 2);
		$c = 0;
		$table->setRowAttributes(0, ['style' => 'vertical-align:top']);
		foreach($common as $pk => $pv){
			$pk = (string) $pk;
			switch($pk){
				case 'MemberSince_Date':
				case 'LastLogin_Date':
				case 'LastAccess_Date':
					'@phan-var DateTime $pv';
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), ($pv ? we_html_element::div(['class' => 'defaultfont lowContrast'], $pv->format(g_l('weEditorInfo', '[date_format]'))) : '-')));
					break;
				case 'ID':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), ($pv ? we_html_element::div(['class' => 'defaultfont lowContrast'], (string) $pv) : '-')));
					++$c;
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, '');
					break;
				case 'LoginDenied':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_element::div(['class' => 'defaultfont lowContrast'], we_html_forms::checkboxWithHidden((bool) $pv, 'customer[LoginDenied]', g_l('modules_customer', '[login_denied]'), false, '', 'top.content.setHot();'))));
					break;
				case 'AutoLoginDenied':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_element::div(['class' => 'defaultfont lowContrast'], we_html_forms::checkboxWithHidden((bool) $pv, 'customer[AutoLoginDenied]', g_l('modules_customer', '[login_denied]'), false, '', 'top.content.setHot();'))));
					break;
				case 'AutoLogin':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_element::div(['class' => 'defaultfont lowContrast'], we_html_forms::checkboxWithHidden((bool) $pv, 'customer[AutoLogin]', g_l('modules_customer', '[autologin_request]'), false, '', 'top.content.setHot();'))));
					break;
				case 'Password':
					$pass = new we_customer_generatePassword(8, ['caps' => 3, 'small' => 3, 'nums' => 2, 'specs' => 0]);
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_tools::textInput('customer[Password]', ($this->Model->ID ? we_customer_customer::NOPWD_CHANGE : ''), 0, [
										'onchange' => "top.content.setHot();", 'autocomplete' => 'new-password'], 'password', '190px') . '<input id="newSecretPass" type="hidden" name="_none" value="' . $pass->PassGen() . '" />' . we_html_button::create_button(we_html_button::RELOAD, '', '', "var el=document.getElementsByName('customer[Password]')[0];el.type='text';el.value=document.getElementById('newSecretPass').value;")));

					break;
				case 'Username':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_tools::textInput('customer[Username]', $pv ?: '', 0, [
										'onchange' => "top.content.setHot();",
										'id' => 'yuiAcInputPathName',
										'required' => 'required',
										'onblur' => "parent.edheader.weTabs.setTitlePath(this.value);"], 'text', '240px')));
					break;
				case 'failedLogins':
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable(sprintf(g_l('modules_customer', '[failedLogins]'), SECURITY_LIMIT_CUSTOMER_NAME_HOURS), we_html_element::div([
										'class' => 'defaultfont lowContrast',
										'id' => 'FailedCustomerLogins'], intval($common['failedLogins']) . ' / ' . SECURITY_LIMIT_CUSTOMER_NAME)));
					break;
				case 'resetFailed':
					$but = we_html_button::create_button(we_html_button::RESET, "javascript:resetLogins(" . $this->Model->ID . ");");
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable('', we_html_element::div(['class' => 'defaultfont lowContrast'], $but)));
					break;
				default:
//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable($this->settings->getPropertyTitle($pk), we_html_tools::textInput('customer[' . $pk . ']', $pv ?? '', 0, [
										'onchange' => "top.content.setHot();"], 'text', '240px')));
			}

			if(++$c % 2 == 0){
				$table->addRows();
			}
		}
		return (new we_html_multiIconBoxContentLine(g_l('modules_customer', ($isAll ? '[common]' : '[data]')), $table->getHtml()))
						->space(we_html_multiIconBox::SPACE_MED);
	}

	/**
	 *
	 * @param array<string,mixed> $other
	 * @param bool $isAll
	 * @return \we_html_multiIconBoxContentLine
	 */
	private function getOtherTable(array $other, bool $isAll): we_html_multiIconBoxContentLine{
		$table = new we_html_table(['width' => '500', 'height' => '50', 'class' => 'customer'], 1, 2);
		$c = 0;
		$table->setRowAttributes(0, ['style' => 'vertical-align:top']);


		foreach($other as $k => $v){
			$k = (string) $k;
			//$isEncField = (bool) ($this->settings->retriveFieldAdd($k, 'encrypt'));

			// $v may be DateTime Object
			if($v instanceof DateTime){
				$v = $v->format('Y-m-d H:i:s');
			}
			$props = $this->getFieldProperties($k);
			$isRequired = $props['required'];
			$control = $this->getHTMLFieldControl($k, $v, $isRequired, $props);
			if($control){
				$null = $props['null'];
				$curValue = $this->Model->{$k} ?? NULL;
				$isValueNull = is_null($curValue) && !($props['type'] === we_customer_settings::TYPE_SELECT && in_array('', $props['values'], true)) && !($props['type'] === we_customer_settings::TYPE_MULTISELCT) && !$props['required'];
				$default = $props['default'];
				$add = $null ? ' <label><i id="ban_' . $k . '" style="' . ($isValueNull ? 'visibility:visible' : 'visibility:hidden') . '" class="emptyCheck fa fa-ban"></i><i class="emptyCheck fa fa-clipboard"></i><input style="visibility:hidden;position:absolute" type="checkbox" onchange="we_cmd(\'toggleEmpty\',\'' . $k . '\',\'' . $default . '\')" value="1" name="isEmpty[' . $k . ']" ' . ($isValueNull ? 'checked="checked"' : '') . '></label>' : '';

				//@phan-suppress-next-line PhanRedundantConditionInLoop
				$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable('<span>' . $k . ($isRequired ? ' *' : '') . $add . '</span>' /*. ($isEncField ? $this->getEncryptionHandling($k, $v != '') : '')*/, $control));
				if(++$c % 2 === 0){
					$table->addRows();
					//@phan-suppress-next-line PhanRedundantConditionInLoop
					$table->setRowAttributes((int) ($c / 2), ['style' => 'vertical-align:top']);
				}
			}
		}
		return (new we_html_multiIconBoxContentLine(g_l('modules_customer', ($isAll ? '[other]' : '[data]')), $table->getHtml()))
						->space(we_html_multiIconBox::SPACE_MED);
	}

	/*private function getEncryptionHandling(string $field, bool $hasContent): string{//@phan-suppress-current-line PhanUnusedPrivateMethodParameter
		return '<span style="margin-left:1em;" class="fa fa-lock"></span>';
	}*/

	private function getCustomerObjects(): string{
		if(empty($this->Model->ID)){
			return g_l('modules_customer', '[NoObjects]');
		}

		$objs = DB_WE::$inst->getAllq('SELECT
	`of`.ID,
	`of`.ContentType,
	`of`.Path,
	`of`.Text,
	o.Path AS objectName,
	DATE_FORMAT(`of`.Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,
	`of`.isPublished,
	`of`.IsModified
FROM ' .
				OBJECT_FILES_TABLE . ' `of` JOIN ' .
				OBJECT_TABLE . ' o ON o.ID=`of`.TableID
WHERE
	`of`.WebUserID=' . intval($this->Model->ID) . '
ORDER BY
	`of`.Path');
		if(!$objs){
			return g_l('modules_customer', '[NoObjects]');
		}
		$objectStr = '<table class="defaultfont dialogBorderBox" style="width:600px;">' .
				'<thead><tr class="boxHeader">
	<th></th>
	<th>' . g_l('modules_customer', '[ID]') . '</th>
	<th>' . g_l('modules_object', '[class]') . '</th>
	<th>' . g_l('modules_customer', '[filename]') . '</th>
	<th>' . g_l('modules_customer', '[Aenderungsdatum]') . '</th>
</thead><tbody>';
		foreach($objs as $cur){
			$objectStr .= '<tr>
	<td>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE," . $cur['ID'] . ",'" . $cur['ContentType'] . "');") . '</td>
	<td>' . $cur['ID'] . '</td>
	<td>' . $cur['objectName'] . '</td>
	<td title="' . $cur['Path'] . '" class="cutText">' . $cur['Text'] . '</td>
	<td class="defaultfont ' . ($cur['isPublished'] ? ($cur['IsModified'] ? 'changed' : '') : 'notpublished') . '">' . $cur['Mod_Date'] . '</td>
</tr>';
		}
		return $objectStr . '</tbody></table>';
	}

	private function getCustomerDocuments(): string{
		if(empty($this->Model->ID)){
			return g_l('modules_customer', '[NoDocuments]');
		}
		$all = DB_WE::$inst->getAllq('SELECT
	f.ID,
	f.Path,
	f.ContentType,
	f.Text,
	f.isPublished,
	f.Mod_Date>f.Publish_Date AS modified,
	DATE_FORMAT(f.Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,
	c1.Dat AS title,
	c2.Dat AS description
FROM ' .
				FILE_TABLE . ' f LEFT JOIN ' .
				CONTENT_TABLE . ' c1 ON (c1.DID=f.ID AND c1.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c1.nHash=unhex(md5("' . we_contents_root::TITLE_FIELD . '"))) LEFT JOIN ' .
				CONTENT_TABLE . ' c2 ON (c2.DID=f.ID AND c2.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c2.nHash=unhex(md5("' . md5(we_contents_root::DESCRIPTION_FIELD) . '")))
WHERE
	f.WebUserID=' . intval($this->Model->ID) . '
ORDER BY
	f.Path');

		if(!$all){
			return g_l('modules_customer', '[NoDocuments]');
		}
		$documentStr = '<table class="defaultfont dialogBorderBox" style="width:600px;">
<thead><tr class="boxHeader">
	<th></th>
	<th>' . g_l('modules_customer', '[ID]') . '</th>
	<th>' . g_l('modules_customer', '[filename]') . '</th>
	<th>' . g_l('modules_customer', '[Aenderungsdatum]') . '</th>
	<th>' . g_l('modules_customer', '[Titel]') . '</th>
</tr></thead><tbody>';
		foreach($all as $item){
			$documentStr .= '<tr>
	<td>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE," . $item['ID'] . ",'" . $item['ContentType'] . "');") . '</td>
	<td>' . $item['ID'] . '</td>
	<td title="' . $item['Path'] . '"><div class="cutText">' . $item['Text'] . '</div></td>
	<td class="defaultfont ' .
					($item['isPublished'] ? ($item['modified'] ? 'changeddefaultfont' : '') : '')
					. '">' . $item['Mod_Date'] . '</td>
	<td title="' . $item['description'] . '">' . ($item['title'] ?: '-') . '</td>
</tr>';
		}
		return $documentStr . '</tbody></table>';
	}

	function getHTMLProperties(string $preselect = ''): string{
		$parts = new we_html_multiIconBoxContent();

		['branches' => $branches, 'fixed' => $common_tmp, 'other' => $other] = $this->Model->getBranches();
		$common = array_merge([
			'ID' => $this->Model->ID,
				], $common_tmp);

		$common['failedLogins'] = fI('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND Username="' . DB_WE::$inst->escape($this->Model->Username) . '" AND isValid="true" AND LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour)');
		if($common['failedLogins'] >= intval(SECURITY_LIMIT_CUSTOMER_NAME)){
			$common['resetFailed'] = '';
		}
		$isAll = false;
		switch($preselect){
			case g_l('modules_customer', '[common]'):
				$parts->add($this->getCommonTable($common, false));
				break;
			case g_l('modules_customer', '[other]'):
				$parts->add($this->getOtherTable($other, false));
				break;
			case g_l('modules_customer', '[orderTab]'):
				$parts->add(new we_html_multiIconBoxContentLine('', we_shop_shop::getCustomersOrderList((int) $this->Model->ID, false)));
				break;
			case g_l('modules_customer', '[objectTab]'):
				$parts->add(new we_html_multiIconBoxContentLine('', $this->getCustomerObjects()));
				break;
			case g_l('modules_customer', '[documentTab]'):
				$parts->add(new we_html_multiIconBoxContentLine('', $this->getCustomerDocuments()));
				break;
			case g_l('modules_customer', '[all]'):
				$isAll = true;
				$parts
						->add($this->getCommonTable($common, true))// no fold, always show for convenience
						->add($this->getOtherTable($other, true)
							->fold(true, true));
//no break;
			default:
				foreach($branches as $bk => $branch){
					$bk = (string) $bk;
					if($preselect && $preselect != g_l('modules_customer', '[all]')){
						if($bk != $preselect){
							continue;
						}
					}

					$table = new we_html_table(['width' => '500', 'height' => '50', 'class' => 'customer'], 1, 2);
					$c = 0;
					$table->setRowAttributes(0, ['style' => 'vertical-align:top;']);
					foreach($branch as $k => $v){
						//$isEncField = (bool) ($this->settings->retriveFieldAdd($bk . '_' . $k, 'encrypt'));
						//$isRequired = (bool) ($this->settings->retriveFieldAdd($bk . '_' . $k, 'required'));

						// $v may be DateTime Object
						if($v instanceof DateTime){
							$v = $v->format('Y-m-d H:i:s');
						}

						$props = $this->getFieldProperties($bk . '_' . $k);
						$isRequired = $props['required'];
						$control = $this->getHTMLFieldControl($bk . '_' . $k, $v, $isRequired, $props);

						if(!$control){
							continue;
						}
						$null = $props['null'];
						$curValue = $this->Model->{$bk . '_' . $k} ?? NULL;
						$isValueNull = is_null($curValue) && !($props['type'] === we_customer_settings::TYPE_SELECT && in_array('', $props['values'], true)) && !($props['type'] === we_customer_settings::TYPE_MULTISELCT) && !$props['required'];
						$default = $props['default'];
						$add = $null ? ' <label><i id="ban_' . $bk . '_' . $k . '" style="' . ($isValueNull ? 'visibility:visible' : 'visibility:hidden') . '" class="emptyCheck fa fa-ban"></i><i class="emptyCheck fa fa-clipboard"></i><input style="visibility:hidden;position:absolute" type="checkbox" onchange="we_cmd(\'toggleEmpty\',\'' . $bk . '_' . $k . '\',\'' . $default . '\')" value="1" name="isEmpty[' . $bk . '_' . $k . ']" ' . ($isValueNull ? 'checked="checked"' : '') . '></label>' : '';

//@phan-suppress-next-line PhanRedundantConditionInLoop
						$table->setCol((int) ($c / 2), $c % 2, we_html_tools::formElementTable('<span>' . $k . ($isRequired ? ' *' : '') . $add . '</span>' /*. ($isEncField ? $this->getEncryptionHandling($bk . '_' . $k, $v != '') : '')*/, $control));

						if(++$c % 2 === 1){
							$table->addRows();
							//@phan-suppress-next-line PhanRedundantConditionInLoop
							$table->setRowAttributes((int) ($c / 2), ['style' => 'vertical-align:top;']);
						}
					}
					$parts->add((new we_html_multiIconBoxContentLine(($isAll ? $bk : g_l('modules_customer', '[data]')), $table->getHtml()))
							->space(we_html_multiIconBox::SPACE_MED)
							// make foldable, if all Tabs are showing
							->fold($isAll, true));
				}
		}

		return ($isAll ? (new we_html_button('open', '', '$(".btn_direction_weMultibox_table button").click()'))->get() : '') . (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	private function moveField(string $field, bool $mvUp): void{
		$chng = ($mvUp ? -2 : 1);
		$fields = $this->db->metaFieldNames(CUSTOMER_TABLE);
		'@phan-var array<int,string> $fields';
		$pos = array_search($field, $fields);
		if($pos){
			$this->db->moveCol(CUSTOMER_TABLE, $field, $fields[$pos + $chng]);
			$this->Model->reloadPersistenSlots();
		}
	}

	public function getHTMLSortEditor(we_customer_frames $pob): string{
		$branches_names = $this->Model->getBranchesNames();
		$branch = $this->getHTMLBranchSelect();
		$branch->setOptionVT(1, g_l('modules_customer', '[other]'), g_l('modules_customer', '[other]'));

		$order = new we_html_select(['name' => 'order', 'style' => 'width:90px;']);
		foreach(we_customer_settings::ORDER_TABLE as $ord){
			$order->addOption($ord, $ord);
		}

		$function = new we_html_select(['name' => 'function', 'style' => 'width:130px;']);

		$counter = 0;
		$fhidden = '';
		$parts = new we_html_multiIconBoxContent();
		foreach($this->settings->SortView as $k => $sorts){
			$fcounter = $row_num = 0;

			$sort_table = (new we_html_table(['width' => '400', 'height' => '50'], 1, 5))
					->setCol(0, 0, we_html_element::b(g_l('modules_customer', '[sort_branch]')))
					->setCol(0, 1, we_html_element::b(g_l('modules_customer', '[sort_field]')))
					//->setCol(0, 2, we_html_element::b(g_l('modules_customer', '[sort_function]')));
					->setCol(0, 3, we_html_element::b(g_l('modules_customer', '[sort_order]')));

			foreach($sorts as $sort){
				if(!$sort['branch']){
					$sort['branch'] = ( $branches_names[0] ?? g_l('modules_customer', '[common]'));
				}
				assert(is_string($sort['branch']));
				$branch->setAttributes(['name' => 'branch_' . $counter . '_' . $fcounter, 'class' => 'weSelect', 'onchange' => "we_cmd('selectBranch')", 'style' => "width:180px;"]);
				$branch->selectOption($sort['branch']);

				$field = $pob->getHTMLFieldsSelect($sort['branch']);
				$field->setAttributes(['name' => 'field_' . $counter . '_' . $fcounter, 'style' => "width:180px;", 'class' => 'weSelect', 'onchange' => "we_cmd('selectBranch')"]);

				$fields_names = array_keys($this->Model->getFieldsNames($sort['branch']));
				if($sort['branch'] === g_l('modules_customer', '[common]') || $sort['branch'] === g_l('modules_customer', '[other]')){
					foreach($fields_names as &$fnv){
						$fnv = str_replace($sort['branch'] . '_', '', (string) $fnv);
					}
					unset($fnv);
				}

				if(!isset($sort['field'])){
					$sort['field'] = '';
				}

				if(!in_array($sort['field'], $fields_names, true)){
					$sort['field'] = array_shift($fields_names);
				}

				if($sort['branch'] == g_l('modules_customer', '[common]') && isset($sort['field'])){
					$field->selectOption(g_l('modules_customer', '[common]') . '_' . $sort['field']);
				} else if(isset($sort['field'])){
					$field->selectOption($sort['field']);
				}

				$function->setAttributes(['name' => 'function_' . $counter . '_' . $fcounter, 'class' => 'weSelect',]);

				$function->delAllOptions();
				$function->addOption('', '');
				foreach(array_keys(we_customer_settings::FUNCTION_TABLE) as $ftk){
					if(isset($sort['field'])/* && $this->settings->isFunctionForField($ftk, $sort["field"]) */){
						$function->addOption((string) $ftk, g_l('modules_customer', '[filter][' . $ftk . ']'));
					}
				}

				if(isset($sort['function'])){
					$function->selectOption($sort['function']);
				}

				$order->setAttributes(['name' => 'order_' . $counter . '_' . $fcounter, 'class' => 'weSelect',]);
				if(isset($sort['order'])){
					$order->selectOption($sort['order']);
				}

				$row_num = $fcounter + 1;
				$sort_table
						->setCol($row_num, 0, $branch->getHtml())
						->setCol($row_num, 1, $field->getHtml())
						->setCol($row_num, 2, $function->getHtml())
						->setCol($row_num, 3, $order->getHtml())
						->setCol($row_num, 4, we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_sort_field','" . $k . "',$fcounter)"));

				$fcounter++;
			}

			$row_num++;
			$sort_table->setCol($row_num, 4, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_sort_field',document.we_form.sort_" . $counter . ".value)"), [
				'style' => 'padding-top:5px;']);


			$fhidden .= we_html_element::hidden('fcounter_' . $counter, $fcounter);

			$parts->add(new we_html_multiIconBoxContentLine($k, $pob->getHTMLBox(we_html_element::input(['name' => 'sort_' . $counter, 'value' => $k, 'size' => '40']), g_l('modules_customer', '[name]'), 100, 50, 25, 0, 0, 50) .
							$sort_table->getHtml() .
							we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_sort','" . $k . "')")));

			$counter++;
		}

		$parts->add(new we_html_multiIconBoxContentLine('', we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_sort')") . we_html_element::div([
							'class' => 'defaultfont lowContrast'], g_l('modules_customer', '[add_sort_group]'))));

		$sort_code = (new we_html_multiIconBox($parts))->marginLeft()->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_sort')"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")))->get() .
				we_html_element::comment("hiddens start") .
				we_html_element::hiddens(['pnt' => "sort_admin",
					'cmd' => '',
					'counter' => "$counter",
					'sortindex' => '',
					'fieldindex' => '']) .
				$fhidden .
				we_html_element::comment('hiddens ends');

		$out = we_html_element::body(['class' => 'weDialogBody', 'onload' => "doScrollTo()"], we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_sortAdmin.js', '', [
							'id' => 'loadVarCustomer_sortAdmin', 'data-settings' => setDynamicVar([
								'default_sort_view' => $this->settings->getSettings('default_sort_view')
					])]) .
						we_html_element::form(['name' => we_base_constants::WE_FORM], $sort_code)
		);

		return $pob->getHTMLDocument($out);
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$hiddens = ['cmd' => 'home'];
		$content = we_html_button::create_button('fat:new_customer,fa-lg fa-user-plus', "javascript:we_cmd('we_customer_new');", '', "", !we_base_permission::hasPerm([
							we_base_perm::CUSTOMER_NEW]));

		return parent::getActualHomeScreen(we_modules_info::CUSTOMER, $jscmd, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', 0)));
	}

}
