<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a202423df34b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-03-22 21:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
// FIXME: reduce all onchange JS to single function calls

class we_doctype_docTypes extends we_base_model implements we_backup_restoreIF{ //switched from we_contents_base
	public const BACKUP_XML_TAG = 'we:doctype';

	/* The Text that will be shown in the tree-menue */
	/** @var string */
	var $Text = 'New DocType';

	/** @var string */
	var $Extension = DEFAULT_STATIC_EXT;

	/** @var int */
	var $ParentID = 0;

	/** @var int */
	var $ParentFileID = 0;

	/** @var string */
	var $Path = '';

	/** @var int */
	var $TemplateID = 0;

	/** @var bool */
	var $IsDynamic = false;

	/** @var bool */
	var $IsSearchable = false;

	/** @var int[] */
	var $Templates = [];

	/** @var string */
	var $Language = '';

	/** @var string */
	var $SubDir = we_contents_base::SUB_DIR_NO;

	/** @var int[] */
	var $Category = [];

	/** @var bool */
	protected $IsHot = false;

	/** @var string[] */
	public $LangLinks = [];

	/** @var string */
	public $ContentType = we_base_ContentTypes::DOCTYPE;

	public function __construct(int $id = 0){
		parent::__construct(DOC_TYPES_TABLE);

		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'Category' => we_base_request::INTLISTA,
			'Text' => we_base_request::STRING,
			'Extension' => we_base_request::STRING,
			'ParentID' => we_base_request::INT,
			'ParentFileID' => we_base_request::INT,
			'Path' => we_base_request::STRING,
			'TemplateID' => we_base_request::INT,
			'IsDynamic' => we_base_request::BOOL,
			'IsSearchable' => we_base_request::BOOL,
			'SubDir' => we_base_request::STRING,
			'Templates' => we_base_request::INTLISTA,
			'Language' => we_base_request::STRING,
		];
		if($id){
			$this->load($id);
		}
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		return $this->save();
	}

	public function save(bool $force = false, bool $rebuild = false, array $skipWrite = []): bool{
		$idArr = array_filter($this->Templates);
		$newIdArr = id2pathA($idArr, TEMPLATES_TABLE);
		$this->Templates = array_keys($newIdArr);
		$this->LangLinks = LANGLINK_SUPPORT ? we_base_request::stringList('LangDocType') : '';

		return parent::save($force);
	}

	public function we_save_exim(): bool{
		return parent::save(false);
	}

	public function setElementsFromHTTP(): void{
		foreach($this->persistent_slots as $name => $type){
			if(($tmp = we_base_request::_($type, $name, false)) !== false){
				$this->{$name} = $tmp;
			}
		}

		if(!$this->Language){
			$this->initLanguageFromParent();
		}
	}

	private function initLanguageFromParent(): void{
		$ParentFileID = $this->ParentFileID;
		$i = 0;
		while(!$this->Language){
			if($ParentFileID == 0 || $i > 20){
				$this->Language = (we_base_WE::$Langs['DefaultFrontendLanguage'] ?: 'de_DE');
			} elseif(($h = getHash('SELECT Language,ParentFileID FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($ParentFileID)))){
				['Language' => $this->Language, 'ParentFileID' => $ParentFileID] = $h;
			}
			$i++;
		}
	}

	protected function getLangLinks(): void{
		//skip reread of langlinks
	}

	public function addCat(array $ids): void{
		$this->Category = array_unique(array_filter(array_merge($this->Category, $ids)), SORT_NUMERIC);
	}

	public function delCat(int $id): void{
		if(($pos = array_search($id, $this->Category, false)) === false){
			return;
		}

		unset($this->Category[$pos]);
		$this->Category = array_filter($this->Category);
	}

	/**
	 * Returns "where query" for Doctypes depending on which workspace the user have
	 * @return         array
	 */
	public static function getDoctypeQuery(bool $forModuleTree = false, bool $forMenu = false): array{
		$paths = [];
		$ws = we_users_util::get_ws(FILE_TABLE);
		'@phan-var int[] $ws';
		if(!$ws || $forModuleTree){
			return [
				'join' => '',
				'where' => ($forModuleTree || $forMenu ? '1 ' : 'dt.IsFolder=0 ') . 'ORDER BY dt.Text'
			];
		}
		if(WE_DOCTYPE_WORKSPACE_BEHAVIOR){
			return [
				'join' => 'LEFT JOIN ' . FILE_TABLE . ' f ON (CONCAT(f.Path,"/") LIKE CONCAT(dtf.Path,"/%") AND f.ID IN(' . implode(',', $ws) . ') AND f.IsFolder=1 )',
				'where' => ($forMenu ? '' : 'dt.IsFolder=0 AND ') . '(ISNULL(dtf.ID) OR !ISNULL(f.ID)) ORDER BY dt.Path'
			];
		}

		foreach(id2pathA($ws) as $cur){
			$paths[] = 'dtf.Path LIKE "' . DB_WE::$inst->escape((string) $cur) . '/%"';
		}
		return ($paths ?
			['join' => '',
			'where' => ($forMenu ? '' : 'dt.IsFolder=0 AND ') . '(dt.ParentFileID IN(' . implode(',', $ws) . ') OR ' . implode(' OR ', $paths) . ' OR ISNULL(dtf.ID)) ORDER BY dt.IsFolder DESC,dt.Path'
			] : ['join' => '',
			'where' => ($forMenu ? '1' : 'dt.IsFolder=0') . ' ORDER BY dt.IsFolder DESC,dt.Path'
		]);
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'doctype_empty' => (g_l('alert', '[doctype_empty]')),
				'doctype_exists' => (g_l('alert', '[doctype_exists]')),
				'doctype_hochkomma' => (g_l('alert', '[doctype_hochkomma]')),
				'newDocTypeName' => g_l('weClass', '[newDocTypeName]'),
				'save_changed_doctype' => g_l('alert', '[save_changed_doctype]'),
				'doctype_delete_prompt' => g_l('weClass', '[doctype_delete_prompt]'),
		]);
	}

}
