<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8487b712d257 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-11-03 12:25 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class for documents */

abstract class we_document_base extends we_contents_root implements we_exim_xmlIF{

	public const BACKUP_XML_TAG = 'we:document';

	/** @var ?DateTime */
	var $Publish_Date = null;

	/** @var string */
	var $Language = '';

	/** @var bool If the file should only be saved in the db */
	var $IsDynamic = false;

	/** @var we_schedule_entry[] */
	var $schedArr = [];

	/** @var int[] Categories of the document */
	var $Category = [];

	/** @var int[] */
	protected $oldCategory = [];

	/** @var bool */
	var $IsSearchable = false;

	/** @var bool */
	var $InGlossar = false;

	/** @var string */
	private $DocStream = '';

	/** @var bool */
	public $parseFile = true;
// persistent in we_contents_object*, since saved in class
// temporary in document, holds Paths to stylesheets from we:css-tags that are user by tinyMCE

	/** @var int[] */
	public $CSS = [];

	/** @var array<string,bool> this array is used, to store document specific data for a page */
	protected $editorSaves = [];

	/** @var we_versions_model|null */
	public $versionsModel = null; // FIXME: set protected and make getter

	/** @var string */
	public $Variant = '';

	/** @var we_customer_documentFilter	 */
	var $documentCustomerFilter = null;

	public function __construct(int $id = 0){
		parent::__construct();
		$this->initByID($id);
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'IsDynamic' => we_base_request::BOOL,
			'Publish_Date' => we_base_request::DATE_TIME,
			'Category' => we_base_request::INTLISTA,
			'IsSearchable' => we_base_request::BOOL,
			'InGlossar' => we_base_request::BOOL,
			'Language' => we_base_request::STRING,
			'schedArr' => we_base_request::SERIALIZED,
			'parseFile' => we_base_request::BOOL,
			'editorSaves' => we_base_request::SERIALIZED
		]);

		if(isWE()){
			if(we_base_request::int('wecf_mode', -1) !== -1){
				$this->documentCustomerFilter = $this->documentCustomerFilter ?: new we_customer_documentFilter($this->Table, $this->ID);
				$this->documentCustomerFilter->getCustomerFilterFromRequest($this->ID);
			}
			if(!($this->Language) && $this->Table !== TEMPLATES_TABLE){
				$this->initLanguageFromParent();
			}
			if($this->EditPageNr == we_base_constants::WE_EDITPAGE_VERSIONS){
				if(!is_object($this->versionsModel)){
					$this->versionsModel = new we_versions_model(we_base_WE::$transaction);
				}
				$this->versionsModel->initByHttp();
			}
		}
	}

	function copyDoc(int $id): bool{
		if(!$id){
			return false;
		}
		$doc = new static();
		$doc->InitByID($id, $this->Table, self::LOAD_TEMP_DB);
		$parentIDMerk = $doc->ParentID;
		if($this->ID === 0){
			foreach(array_keys($this->persistent_slots) as $name){
				if($name != 'elements' && in_array($name, array_keys(get_object_vars($doc)))){
					$this->{$name} = $doc->{$name};
				}
			}
			$this->Creation_Date = new DateTime();
			$this->CreatorID = we_users_user::getID();

			$this->ID = 0;
			$this->OldPath = '';
			$this->Publish_Date = null;
			$this->Filename .= '_copy';
			$this->setParentID($parentIDMerk);
			$this->OldPath = $this->Path;
		}
		$this->elements = $doc->elements;
		foreach(array_keys($this->elements) as $n){
			$this->elements[$n]['cid'] = 0;
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		$this->InWebEdition = true;
		$this->Category = empty($doc->Category) ? [] : $doc->Category;
		$this->documentCustomerFilter = $doc->documentCustomerFilter;
		return true;
	}

	/** gets the filesize of the document */
	function getFilesize(): int{
		return strlen($this->getElementS('data'));
	}

// returns the whole document Alias - don't remove
	function getDocument(): string{
		return $this->getDocumentData();
	}

	public function initLanguageFromParent(): void{
		$ParentID = $this->ParentID;
		$i = 0;
		while(!$this->Language){
			if($ParentID == 0 || $i > 20){
				$this->Language = self::getDefaultLanguage() ?: 'de_DE';
				return;
			}
			['ParentID' => $ParentID, 'Language' => $this->Language] = getHash('SELECT Language,ParentID FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . $ParentID);

			$i++;
		}
	}

	protected function formInGlossar(): string{
		return (we_modules_info::isActive(we_modules_info::GLOSSARY) ?
				we_html_forms::checkboxWithHidden((bool) $this->InGlossar, 'we_' . $this->Name . '_InGlossar', g_l('weClass', '[InGlossar]'), false, '', '_EditorFrame.setEditorIsHot(true);') :
				'');
	}

	protected function formIsSearchable(): string{
		return we_html_forms::checkboxWithHidden((bool) $this->IsSearchable, 'we_' . $this->Name . '_IsSearchable', g_l('weClass', '[IsSearchable]'), false, '', '_EditorFrame.setEditorIsHot(true);');
	}

	public function formMetaInfos(we_base_jsCmd $jsCmd): string{
		return
				$this->formMetaField(self::TITLE_FIELD) .
				$this->formMetaField(self::DESCRIPTION_FIELD) .
				$this->formMetaField(self::KEYWORD_FIELD);
	}

	protected function formCategory(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('delete_all_cats')", '', '', $this->Category ? false : true);
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_cat'})");
		$cats = new we_chooser_multiDir($this->Category, 'delete_cat', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE, '', '', 'setScrollTo();');
		return $cats->get($jsCmd);
	}

	function formReferences(we_base_jsCmd $jsCmd): array{
		$groups = $values = [];
		$num = $c = 0;

		// look for use in collections
		$collections = $this->DB_WE->getAllq('SELECT
	v.ID as id,v.Path as path,"' . VFILE_TABLE . '" as "table",v.ContentType as ct
FROM
	' . VFILE_TABLE . ' v LEFT JOIN ' . FILELINK_TABLE . ' fl ON (v.ID=fl.ID)
WHERE
	fl.remObj=' . $this->ID . ' AND fl.DocumentTable="' . stripTblPrefix(VFILE_TABLE) . '" AND fl.type="collection" AND fl.remTable="' . stripTblPrefix($this->Table) . '"
ORDER BY
	v.Path');

		if($collections){
			$groups[g_l('global', '[vfile]')] = $collections;
		}

		if(empty($groups)){
			return ['form' => g_l('weClass', '[notReferenced]'), 'num' => 0];
		}
		$ref = [];
		foreach($groups as $name => $group){
			$num += count($group);
			$values[$name . ' (' . count($group) . ')'] = we_html_tools::OPTGROUP;
			foreach($group as $entry){
				$values[++$c] = ($entry['path'] ?? '');
				$ref['id_' . $c] = $entry;
			}
		}

		$button = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.we_openReference(window, document.we_form.References.value);");
		$jsCmd->addCmd('setReferences', $ref);
		$form = we_html_tools::formElementTable('', we_html_tools::select('References', $values, 1, '', false, [], true, 388), $button);

		return ['form' => $form, 'num' => $num];
	}

	protected function formNavigation(we_base_jsCmd $jsCmd): string{
		$isSee = we_editor_functions::isSeeMode();
		$navItems = $this->getNavigationItems();
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_navigation_edit_navi',0)", '', '', !(we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT]) && $this->ID && $this->Publish_Date), false);

		if(we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT]) && $isSee){
			$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:WE().util.showConfirm(window, '', '" . strtr(g_l('navigation', '[dellall_question]'), [
								"\n" => '\n']) . "',['delete_all_navi']);", '', '', !(we_base_permission::hasPerm([
								we_base_perm::NAVIGATION_EDIT]) && $navItems));
		} else {
			$delallbut = '';
		}
		$navis = new we_chooser_multiFile($navItems, 'delete_navi_ask', $delallbut . $addbut, 'we_navigation_edit_navi', we_base_ContentTypes::FOLDER_COMMON, we_base_ContentTypes::NAVIGATION, "setScrollTo(" . we_base_constants::WE_EDITPAGE_PROPERTIES . ");");
		$NoDelNavis = path_to_idA($navItems, NAVIGATION_TABLE);
		foreach($NoDelNavis as $id){
			'@phan-var int $id';
			$naviItem = new we_navigation_navigation($id);
			if(!$naviItem->hasAnyChilds()){
				unset($NoDelNavis[$id]);
			}
		}
		$navis->setDisabledDelItems($NoDelNavis, g_l('navigation', '[NoDeleteFromDocument]'));

		if(!$isSee || !we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])){
			$navis->setEditable(false)
					->setDeletable(false);
		}

		return $navis->get($jsCmd);
	}

	function addCat(array $ids): void{
		$this->Category = array_unique(array_merge(array_filter($this->Category), $ids), SORT_NUMERIC);
	}

	function delCat(int $id): void{
		$cats = array_filter($this->Category);
		if(($pos = array_search($id, $cats, false)) === false){
			return;
		}

		unset($cats[$pos]);
		$this->Category = $cats;
	}

	function addNavi(int $id, string $text, int $parentid, int $ordn): void{
		$text = urldecode($text); //Bug #3769
		if($this->ID){

			$ordn--;

			$ord = $ordn;

			$new_path = rtrim(id2path($parentid, NAVIGATION_TABLE), '/') . '/' . $text;
			$id = $id ?: path_to_id($new_path, NAVIGATION_TABLE);

			$naviItem = new we_navigation_navigation($id);

			$naviItem->Ordn = fI('SELECT MAX(Ordn) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . $parentid);
			$naviItem->ParentID = $parentid;
			$naviItem->LinkID = $this->ID;
			$naviItem->Text = $text;
			$naviItem->Path = $new_path;
			$naviItem->SelectionType = we_navigation_navigation::STYPE_DOCLINK;
			$naviItem->Selection = we_navigation_navigation::SELECTION_STATIC;
			if(!NAVIGATION_ENTRIES_FROM_DOCUMENT){
				$naviItem->IsFolder = true;
			}

			$naviItem->save();
			$naviItem->reorderAbs($ord);
		}
	}

	function delNavi(string $path): void{
		$path = urldecode($path); //Bug #3816
		$navis = $this->getNavigationItems();
		if(($pos = array_search($path, $navis)) === false){
			return;
		}
		$id = path_to_id($path, NAVIGATION_TABLE);
		$naviItem = new we_navigation_navigation($id);
		if(!$naviItem->hasAnyChilds()){
			$naviItem->delete();
			unset($navis[$pos]);
		}
	}

	function delAllNavi(): void{
		$navis = $this->getNavigationItems();
		foreach($navis as $path){
			$id = path_to_id((string) $path, NAVIGATION_TABLE);
			$naviItem = new we_navigation_navigation($id);
			if(!$naviItem->hasAnyChilds()){
				$naviItem->delete();
				if(($pos = array_search((string) $path, $navis)) === false){
					continue;
				}
				unset($navis[$pos]);
			}
		}
	}

	/*
	 * internal functions
	 */

	protected function getParentIDFromParentPath(): int{
		return fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . $this->DB_WE->escape($this->ParentPath) . '" AND IsFolder=1', -1);
	}

	function changeLink(string $name): void{//FIXME: can we store info in bdid? add info on type, if it is object or file?
		if(!isset($_SESSION['weS']['WE_LINK'])){
			return;
		}
		$this->setElementS($name, we_serialize($_SESSION['weS']['WE_LINK'], SERIALIZE_JSON), 'link');
		unset($_SESSION['weS']['WE_LINK']);
	}

	static function getNamesFromContent(string $content): array{
		$arr = $result = [];
		preg_match_all('/< ?we:[^>]+name="([^"]+)"[^>]*>/i', $content, $result, PREG_SET_ORDER);
		foreach($result as $val){
			$arr[] = $val[1];
		}
		return $arr;
	}

	function remove_image(string $name): void{
		$this->delElement($name);
		$this->delElement($name . we_document_image::ALT_FIELD);
		$this->delElement($name . we_document_image::TITLE_FIELD);
	}

	protected function isVersioned(): bool{
		return we_versions_version::CheckPreferencesCtypes($this->ContentType);
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$skipHook){
			$hook = new we_hook_handler('preSave', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}
		if($this->PublWhenSave){
			$this->Publish_Date = new DateTime();
		}

		if(!parent::we_save($resave)){
			return false;
		}

		$ret = $this->writeDocument();
		if(!$ret || ($this->errMsg != '')){
			return false;
		}

		$this->OldPath = $this->Path;

		if(!$resave){ // NO rebuild!
			$this->saveDocumentCustomerFilter();
		}

		if($this->isVersioned()){
			we_versions_version::save($this);
		}

		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('save', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		return $ret;
	}

	public function we_rewrite(): bool{
		return $this->writeDocument();
	}

	abstract protected function writeDocument(): bool;

	protected function writeMetaValues(): void{
		foreach($this->DB_WE->getAllq('SELECT ID,tag,type,mode,csv FROM ' . METADATA_TABLE) as $meta){
			'@phan-var array{ID:int,tag:string,type:string,mode:string,csv:int} $meta';
			if($meta['mode'] === 'auto' && $meta['type'] === 'textfield' && ($value = $this->getElementS($meta['tag']))){
				$values = $meta['csv'] ? array_map('trim', explode(',', $value)) : [$value];
				foreach($values as $v){
					$this->DB_WE->query('INSERT INTO ' . METAVALUES_TABLE . ' SET ' . we_database_base::arraySetter([
								'tagID' => $meta['ID'],
								'value' => $v
					]));
				}
			}
		}
	}

	function registerMediaLinks(bool $temp = false, bool $linksReady = false): bool{
		if($linksReady){
			return parent::registerMediaLinks($temp);
		}
		switch($this->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::OBJECT_FILE:
				//FIXME: maybe move this part do we_document_webEdition
				$c = 0;
				foreach($this->elements as $k => $v){
					$element = $v['type'] . '[name=' . ($k ?: 'NN' . ++$c) . ']';
					switch($v['type'] ?? ''){
						case 'audio':
						case 'binary':
						case 'href':
						case 'img':
						case 'video':
							if(!empty($v['bdid'])){
								$this->MediaLinks[$element] = $v['bdid'];
							} elseif(!empty($v['dat']) && is_numeric($v['dat'])){
								$this->MediaLinks[$element] = $v['dat'];
							}
							break;
						case 'link':
							/*
							 * documents: when no link is set but there is a default id in template $v['dat'] is serialized twice:
							 * we do not register such links, they belong to the template and are they are not stored in tblContent!
							 *
							 * objectfiles: here the default defined in class is stored in tblObject_X:
							 * it is no "dynamic" default and belongs to the object: so we register it as medialink of the object (and class)
							 */
							if(isset($v['dat']) && ($link = is_array($v['dat']) ? $v['dat'] : we_unserialize((string) $v['dat'], [], true)) && is_array($link)){
								if(isset($link['type']) && isset($link['id']) && isset($link['img_id'])){
									if($link['type'] === 'int' && $link['id']){
										$this->MediaLinks[$element] = $link['id'];
									}
									if($link['img_id']){
										$this->MediaLinks[$element] = $link['img_id'];
									}
								}
							}
							break;
						default:
							if(!empty($v['bdid'])){
								$this->MediaLinks[$element] = $v['bdid'];
							}
					}
				}
				break;
		}

		return parent::registerMediaLinks($temp);
	}

	protected function writeSiteDir(string $content): bool{
		if($this->isMoved()){
			we_base_file::deleteLocalFile($this->getSitePath(true));
		}
		try{
			return we_base_file::checkAndMakeFolder(dirname($this->getSitePath()), true) && we_base_file::save($this->getSitePath(), $content);
		}catch(we_exception_file $e){
			return false;
		}
	}

	protected function writeMainDir(string $content): bool{
		if($this->isMoved()){
			we_base_file::deleteLocalFile($this->getRealPath(true));
		}
		try{
			return we_base_file::checkAndMakeFolder(dirname($this->getRealPath()), true) && we_base_file::save($this->getRealPath(), $content);
		}catch(we_exception_file $e){
			return false;
		}
	}

	/**
	 * Rewrites the navigation cache files
	 *
	 */
	public function rewriteNavigation(): void{
		//FIXME: make this more efficient!
		// rewrite filter
		$this->saveDocumentCustomerFilter();

		$folders = $this->getNavigationFoldersForDoc();
		foreach($folders as $f){
			we_navigation_cache::delNavigationTree((int) $f);
		}
	}

	/**
	 * Gets the navigation folders for the current document
	 *
	 * @return int[]
	 */
	protected function getNavigationFoldersForDoc(): array{
		if($this->Table !== FILE_TABLE){
			return [];
		}

		$queries = ['( Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND LinkID=' . intval($this->ID) . ')',];
		if($this instanceof we_document_textContent){
			//FIXME: query should use ID, not parentID
			$queries[] = '(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND (DocTypeID="' . $this->DB_WE->escape($this->DocType) . '" OR FolderID=' . intval($this->ParentID) . '))';
		}
		if(($category = array_map('escape_sql_query', array_unique(array_filter(array_merge($this->Category, $this->oldCategory)))))){
			//FIXME: query should use ID, not parentID
			$queries[] = '(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_DOCTYPE . '" AND (CONCAT(",",Categories,",") REGEXP ",(' . implode('|', $category) . '),"))';
		}

		$ret = $this->DB_WE->getAllq('SELECT DISTINCT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' OR ', $queries), true);
		'@phan-var int[] $ret';
		return $ret;
	}

	protected function getDocumentToSave(): string{
		$this->DocStream = $this->DocStream ?: $this->getDocumentData();
		return $this->DocStream;
	}

	function getDocumentData(string $includepath = ''): string{
		return $this->getElementS('data');
	}

	/**
	 *
	 * @param int $val
	 * @param array<string,string> $attribs
	 * @param bool $pathOnly
	 * @return string
	 */
	public function getImgFieldByVal(int $val, array $attribs, bool $pathOnly): string{
		if(!$val && isset($attribs['id'])){
			$val = (int) $attribs['id'];
		}
		if(!$val){
			return '';
		}
		$img = new we_document_image($val);

		if(isset($attribs['name'])){
			$img->Name = $attribs['name'];
		}

		$altField = $img->Name . we_document_image::ALT_FIELD;
		$titleField = $img->Name . we_document_image::TITLE_FIELD;
		$cur = we_tag_listview::$current;
		if($cur){
			switch(true){
				case ($cur instanceof we_listview_variants):
					$altField = (we_base_variants::PREFIX . $cur->Position . '_' . $altField);
					$titleField = (we_base_variants::PREFIX . $cur->Position . '_' . $titleField);
					break;
				case ($cur instanceof we_listview_document):
					$alt = $cur->f($altField) ?: '';
					$title = $cur->f($titleField) ?: '';
				case ($cur instanceof we_listview_objectBase):
					$attribs['alt'] = !empty($alt) ? $alt : ($img->getElementS('alt', $attribs['alt'] ?? '', true));
					$attribs['title'] = !empty($title) ? $title : ($img->getElementS('title', $attribs['title'] ?? '', true));
					break;
			}
		}
		if(isset($attribs['alt'])){
			$attribs['alt'] = htmlspecialchars($attribs['alt']);
		}
		if(isset($attribs['title'])){
			$attribs['title'] = htmlspecialchars($attribs['title']);
		}
		if(!(we_base_request::string('we_cmd', '', 0) === 'reload_editpage' && ($img->Name === we_base_request::string('we_cmd', we_base_request::NOT_VALID_S, 1)) && we_base_request::string('we_cmd', '', 2) === 'change_image') && $this->issetElement($altField)){
			if(!we_tag_listview::$current){
				$attribs['alt'] = htmlspecialchars($this->getElementS($altField));
				$attribs['title'] = htmlspecialchars($this->getElementS($titleField));
			}
		}

//	when width or height are given, then let the browser adjust the image
		if($attribs){
			if(isset($attribs['width'])){
				$img->delElement('width');
			}
			if(isset($attribs['height'])){
				$img->delElement('height');
			}
			//FIXME: why do these
			unset($attribs['size'], $attribs['quality']);
			$img->initByAttribs(we_weTag_tag::removeAttribs($attribs, ['hyperlink', 'target']));
		}
		if(we_tag_listview::$current){
			$img->setElementS('name', $img->getElementS('name') . '_' . we_tag_listview::$current->count, 'attrib');
			$img->Name = $img->Name . '_' . we_tag_listview::$current->count;
		}

		switch($pathOnly ? 'path' : ($attribs['only'] ?? '')){
			case 'src': //TODO: make separate case for multi domain project to devide between path and src
				return $img->getHtml(false, true, false, 'src');
			case 'path':
				return (empty($attribs['thumbnail']) && empty($attribs['thumbnailid']) ? $img->Path : $img->getHtml(false, true, true) );
			case 'id':
				return (string) $img->ID;
			case 'parentpath':
				return $img->ParentPath;
			case 'filename':
				return $img->Filename;
			case 'extension':
				return $img->Extension;
			case 'filesize':
				return (string) $img->getFilesize();
		}

		return $img->getHtml(false, true);
	}

	/**
	 *
	 * @param int $val
	 * @param array<string,string> $attribs
	 * @return array{string,string,string,string,string,int}
	 */
	public static function getBinaryFieldByVal(int $val, array $attribs): array{
		$id = $val ?: (empty($attribs['id']) ? 0 : intval($attribs['id']));
		$bin = new we_document_other($id);
		return [$bin->getText(), $bin->Path, $bin->ParentPath, $bin->Filename, $bin->Extension, $bin->getFilesize()];
	}

	/**
	 *
	 * @param int $val
	 * @param array<string,string> $attribs
	 * @param bool $pathOnly
	 * @return string
	 */
	private static function getVideoFieldByVal(int $val, array $attribs, bool $pathOnly): string{
		$id = $val ?: (empty($attribs['id']) ? 0 : intval($attribs['id']));
		$video = new we_document_video($id);
		if(!empty($attribs)){
			$video->initByAttribs($attribs);
		}
		return $pathOnly ? $video->Path : $video->getHtml(false, we_base_WE::$editmode);
	}

	/**
	 *
	 * @param int|string $val
	 * @param array<string,string> $attribs
	 * @return string
	 */
	private static function getDateFieldByVal(/* mixed */ $val, array $attribs): string{
		$time = $val ?: time();
		$format = !empty($attribs['format']) ? $attribs['format'] : g_l('date', '[format][default]');
		switch(strtoupper($format)){
			case we_html_tools::FORMAT_DATE:
				$format = g_l('weEditorInfo', '[date_only_format]');
				break;
			case we_html_tools::FORMAT_TIME:
				[, $format] = explode(' ', g_l('date', '[format][php]'));
				break;
			case we_html_tools::FORMAT_DATE_TIME:
				$format = g_l('date', '[format][php]');
				break;
		}
		$langcode = (we_base_WE::$mainDoc['doc'] && strlen(we_base_WE::$mainDoc['doc']->Language) == 5 ? we_base_WE::$mainDoc['doc']->Language : we_base_WE::$Langs['DefaultFrontendLanguage']);
		try{
			$date = (is_numeric($time) ? new DateTime((date('Y-m-d H:i:s', (int) $time))) : new DateTime($time));
		}catch(Exception $e){
			$date = new DateTime();
		}
		//we need to set it explicitly
		$date->setTimezone(new DateTimeZone(date_default_timezone_get()));

		return we_base_country::dateformat($langcode, $date, $format);
	}

	/**
	 *
	 * @param string $val
	 * @param array<string,string> $attribs
	 * @param int $parentID
	 * @param int $classID
	 * @return string
	 */
	private function getSelectFieldByVal(string $val, array $attribs, int $parentID, int $classID): string{
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			if(strlen($val) == 0){
				return '';
			}
			if($classID && ($defVals = fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $classID))){
				$arr = we_unserialize($defVals);
				if(($keys = $arr['select_' . $attribs['name']]['selectKeys'] ?? []) && ($values = $arr['select_' . $attribs['name']]['selectValues'] ?? [])){
					return array_combine($keys, $values)[$val] ?? '';
				}
			}
		}
		return self::getTextFieldByVal($val, $attribs, $parentID);
	}

	/**
	 *
	 * @param string $val
	 * @param array<string,string> $attribs
	 * @param int $parentID
	 * @return string
	 */
	public function getTextFieldByVal(string $val, array $attribs, int $parentID): string{
		$val = self::parseInternalLinks($val, $parentID);
		$retval = preg_replace('/<\?xml[^>]+>/i', '', $val);
		$wysiwyg = we_weTag_tag::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL);
		$htmlspecialchars = we_weTag_tag::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL);
		$striphtml = we_weTag_tag::getAttribute('striphtml', $attribs, false, we_base_request::BOOL);
		$htmlentitydecode = we_weTag_tag::getAttribute('htmlEntityDecode', $attribs, false, we_base_request::BOOL);

		if(($htmlspecialchars || $htmlentitydecode || $striphtml) && (!$wysiwyg)){
			$tmp = $striphtml ? strip_tags($retval) : $retval;
			$tmp = $htmlentitydecode ? html_entity_decode($tmp, ENT_COMPAT, 'UTF-8') : $tmp;
			$retval = $htmlspecialchars ? htmlspecialchars($tmp, ENT_QUOTES) : $tmp;
		}
		$retval = preg_replace('-<(br|hr)([^/>]*)/? *>-i', '<${1}${2}/>', we_base_util::rmPhp($retval));

		if(preg_match('/^[\d.,]+$/', trim($retval))){
			$precision = we_weTag_tag::getAttribute('precision', $attribs, 2, we_base_request::INT);
			if(($num = we_weTag_tag::getAttribute('num_format', $attribs, '', we_base_request::STRING))){
				$retval = we_base_util::formatNumber(we_base_util::std_numberformat($retval), $num, $precision);
			}
		}
		return strtr($retval, ['##|n##' => "\n", '##|r##' => "\r"]);
	}

//FIXME: check if we can rid of this function, since it causes problems every change of tags since it also uses the given attribs array!

	/**
	 *
	 * @param string $val
	 * @param string $type
	 * @param array<string,string> $attribs
	 * @param bool $pathOnly
	 * @param int $parentID
	 * @param string $path
	 * @param int $classID
	 * @param string $fn
	 * @return string|array
	 */
	public function getFieldByVal(string $val, string $type, array $attribs = [], bool $pathOnly = false, int $parentID = 0, string $path = '', int $classID = 0, string $fn = 'this'){
		switch($type){
			case 'img':
				return $this->getImgFieldByVal(intval($val), $attribs, $pathOnly);
			case 'binary':
				return self::getBinaryFieldByVal(intval($val), $attribs);
			case 'video':
				return self::getVideoFieldByVal(intval($val), $attribs, $pathOnly);
			case 'link':
				return $this->getFieldLink(we_unserialize($val), $attribs, $pathOnly, $parentID, $path);
			case 'date':
				return self::getDateFieldByVal($val, $attribs);
			case 'select':
				return $this->getSelectFieldByVal($val, $attribs, $parentID, $classID);
			case 'href':
				return $this->getHref($attribs['name'], $fn);
			default:
				return self::getTextFieldByVal($val, $attribs, $parentID);
		}
	}

	//FIXME: check if we can rid of this function, since it causes problems every change of tags since it also uses the given attribs array!

	/**
	 *
	 * @param array<string,mixed> $attribs
	 * @param string $type
	 * @param bool $pathOnly
	 * @return string|array
	 */
	function getField(array $attribs, string $type = 'txt', bool $pathOnly = false){
		if(isset($attribs['_name_orig'])){
			unset($attribs['_name_orig']);
		}
//FIXME: should we use getElementType by default?! and use type only to get specific data??
		switch($type){
			case 'img':
				if(isset($attribs['showcontrol']) && !$attribs['showcontrol'] && isset($attribs['id']) && $attribs['id']){//bug 6433: siehe korrespondierende Änderung in we_tag_img
					unset($attribs['showcontrol']);
					$val = (int) ($attribs['id']);
				} else {
					$val = empty($attribs['name']) ? 0 : $this->getElementI($attribs['name']);
				}
				return $val ? $this->getImgFieldByVal($val, $attribs, $pathOnly) : '';
			case 'video':
				if(isset($attribs['showcontrol']) && !$attribs['showcontrol'] && isset($attribs['id']) && $attribs['id']){//bug 6433: siehe korrespondierende Änderung in we_tag_img
					unset($attribs['showcontrol']);
					$val = (int) ($attribs['id']);
				} else {
					$val = empty($attribs['name']) ? 0 : $this->getElementI($attribs['name']);
				}
				return $val ? self::getVideoFieldByVal($val, $attribs, $pathOnly) : '';
			case 'link':
				// FIXME: be sure we have ALL link elements saved the same way: as elementA or elementS
				if($this instanceof we_document_triggerDocument){
					return $this->getFieldLink($this->getElementA($attribs['name']), $attribs, $pathOnly);
				}
				return $this->getFieldLink(we_unserialize($this->getElementS($attribs['name'])), $attribs, $pathOnly);
			case 'href':
				// FIXME: be sure we have ALL href elements saved the same way: as elementA or elementS
				if(!isset($attribs['name'])){
					return '';
				}
				if($this instanceof we_document_triggerDocument){
					return self::getHrefByArray($this->getElementA($attribs['name']));
				}
				$val = $this->getElementS($attribs['name']);
				if($this instanceof we_contents_objectFile || we_base_util::isSerialized($val)){// we can not use '$this instanceof we_contents_objectFile' to identify objectFile, we have to use 'isset($this->TableID)' instead
					return self::getHrefByArray(we_unserialize($val));
				}
				return $this->getHref($attribs['name'], 'this');
			case 'select':
			case 'txt':
				$val = (string) ($this->getElementS($attribs['name']) ?: ($this->getElementI($attribs['name']) ?: ''));
				break;
			case 'date':
				return self::getDateFieldByVal((string) $this->getElementI($attribs['name']), $attribs);
			default:
				// FIXME: this seems not to be the place to do this... - it is e.g. used by we:var
				switch($this->getElementType($attribs['name'])){
					case 'int':
					//case 'checkbox': this is string
					case 'collection':
					case 'binary':
					case 'object':
						$val = (string) $this->getElementI($attribs['name']);
						break;
					case 'txt':
					case 'select':
						$val = (string) ($this->getElementS($attribs['name']) ?: ($this->getElementI($attribs['name']) ?: ''));
						break;
					default:
						$val = $this->getElementS($attribs['name']);
				}
		}

		$tableID = ($this instanceof we_document_triggerDocument || $this instanceof we_contents_objectFile ? $this->TableID : 0);

		return $this->getFieldByVal($val, $type, $attribs, $pathOnly, !we_base_WE::$mainDoc['doc'] ? $this->ParentID : we_base_WE::$mainDoc['doc']->ParentID, !we_base_WE::$mainDoc['doc'] ? $this->Path : we_base_WE::$mainDoc['doc']->Path, (int) ((isset($attribs['classid']) && isset($attribs['type']) && $attribs['type'] === 'select') ? $attribs['classid'] : $tableID)); //not instance due to we_showObject
	}

	private function getValFromSrc(string $fn, string $name, string $key = self::ETYPE_TEXT): string{
		switch($fn){
			default:
			case 'this':
				return $key === self::ETYPE_TEXT ? $this->getElementS($name) : (string) $this->getElementI($name);
			case 'listview':
				return we_tag_listview::$current->f($name);
		}
	}

	private function getHref(string $name, string $fn = 'this'): string{
		if($this->getValFromSrc($fn, $name . we_base_link::MAGIC_INT_LINK, self::ETYPE_NUMBER)){
			$intID = (int) $this->getValFromSrc($fn, $name . we_base_link::MAGIC_INT_LINK_ID, self::ETYPE_NUMBER);
			return id2path($intID);
		}
		return $this->getValFromSrc($fn, $name);
	}

	/**
	 * @param array $hrefArr
	 * @param string $only
	 * @return string
	 */
	static function getHrefByArray(array $hrefArr, string $only = ''): string{
		return (!empty($hrefArr['int']) ?
				(empty($hrefArr['intID']) ? '' : ($only === 'id' ? (string) $hrefArr['intID'] : id2path((int) $hrefArr['intID']))) :
				(empty($hrefArr['extPath']) ? '' : $hrefArr['extPath'])
				);
	}

	/*
	 * functions for scheduler pro
	 */

	function add_schedule(): void{
		$this->schedArr[] = we_schedule_entry::emptyEntry();
	}

	function del_schedule(int $nr): void{
		unset($this->schedArr[$nr]);
	}

	function add_schedcat(int $id, int $nr): void{
		$this->schedArr[$nr]->addCat($id);
	}

	function delete_schedcat(int $id, int $nr): void{
		$this->schedArr[$nr]->delCat($id);
	}

	protected function formMetaField(string $field): string{
		$props = we_metadata_metaData::getMetaDataField($field);
		if(empty($props) || $props['mode'] === 'none' || $props['type'] !== 'textfield'){
			$name = in_array($field, we_metadata_metaData::STANDARD_FIELDS, true) ? g_l('weClass', '[' . $field . ']') : $field;
			$cont = $this->getElementS($field);
			return $this->formTextArea('txt', $field, $name . '<span class="wordcounter">' . g_l('wysiwyg', '[words]') . ': <span id="' . $field . 'Len">' . str_word_count($cont) . '/' . strlen($cont) . '</span> ' . g_l('wysiwyg', '[chars]') . '</span>', 0, 0, [
						'onchange' => "_EditorFrame.setEditorIsHot(true);", 'onkeydown' => "if(event.key==='Enter'){WE().handler.keyListener.cancelEvent(event);return false;}", 'oninput' => "WE().util.setTextareaCount(this.value,document.getElementById('" . $field . "Len'));",
						'class' => 'resizeable']);
		}

		$leading = $props['csv'] ? '-- ' . g_l('button', '[add][value]') . ' -- ' : '-- ' . g_l('button', '[select][value]') . ' --';
		$values = we_metadata_metaData::getDefinedMetaValues(true, $leading, $field, (bool) $props['closed'], ($props['closed'] && $props['csv']));
		$inputName = 'we_' . $this->Name . '_txt[' . $field . ']';

		$onchange = "metaFieldSelectProposal(this, '" . $inputName . "', " . ($props['csv'] ? "true" : "false") . ");";
		$mouseover = ['onmouseover' => "this.parentNode.getElementsByClassName('meta_icons')[0].style.display='inline-block';",
			'onmouseout' => "this.parentNode.getElementsByClassName('meta_icons')[0].style.display='none';",
				//'onclick' => "this.parentNode.getElementsByClassName('meta_icons')[0].style.display='none';",
		];

		$input = we_html_tools::textInput($inputName, ($this->getElementS($field) ?: (isset($GLOBALS['meta'][$field]) ? $GLOBALS['meta'][$field]['default'] : '')), 0, [
						], 'txt', '308px', false, false, (bool) $props['closed']);
		$sel = we_html_tools::select('we_tmp_' . $this->Name . '_select[' . $field . ']', $values, 1, '', false, ['onchange' => $onchange], true, 200);

		// FIXME: if we want the icons make icon-css and better js

		$iconCsv = we_html_element::div(['title' => g_l('metadata', '[txtIconCsv]'), 'class' => 'metafield'], '&nbsp;c,s,v&nbsp;');
		$iconClosed = we_html_element::div(['title' => g_l('metadata', '[txtIconClosed]'), 'class' => 'metafield'], '&nbsp;<i class="fa fa-key"></i>&nbsp;');
		$iconAuto = we_html_element::div(['title' => g_l('metadata', '[txtIconAuto]'), 'class' => 'metafield'], '&nbsp;&nbsp;<i class="fa fa-sign-in"></i>&nbsp;&nbsp;');
		$icons = we_html_element::div(['style' => 'display:none;', 'class' => 'meta_icons'], ($props['closed'] ? $iconClosed . '&nbsp;' : '') . ($props['csv'] ? $iconCsv . '&nbsp;' : '') . (($props['mode'] === 'auto') && !$props['closed'] ? $iconAuto : ''));

		try{
			$name = g_l('weClass', '[' . $field . ']', true);
		}catch(we_exception_base $e){
			$name = $field;
		}
		return we_html_element::div($mouseover, we_html_tools::formElementTable($name, $input, $sel . $icons));
	}

	/**
	 * @return array<string,array{type:string,content:string,attributes:array}> with the field names and attributes
	 */
	function getVariantFields(): array{
// overwrite
		return [];
	}

	/**
	 * @desc	the function modifies document EditPageNrs set
	 */
	function checkTabs(): void{
		if(!$this->canHaveVariants(true)){

			$ind = array_search(we_base_constants::WE_EDITPAGE_VARIANTS, $this->EditPageNrs, true);
			if($ind !== false){
				unset($this->EditPageNrs[$ind]);
			}
		}
	}

	/**
	 * get styles for textarea or object
	 * @return array<int,string>
	 */
	public function getDocumentCss(): array{
		// IMPORTANT: we must re-establish the original order after using id2pathA!
		$ret = [];
		$tmp = id2pathA($this->CSS, FILE_TABLE);
		foreach($this->CSS as $id){
			unset($ret[$id]); // IMPORTANT!
			$ret[$id] = $tmp[$id] ?? '';
		}

		return array_filter($ret);
	}

	/**
	 * get styles for textarea or object
	 */
	public function addDocumentCss(int $stylesheet): void{
		$this->CSS[] = $stylesheet;
	}

	public static function parseInternalLinks(string $text, int $pid, string $path = ''): string{
		$regs = [];

		// mark all img tags to make them unique for further str_replace
		$text = preg_replace_callback('|(<img[^>]*)>|is', function ($reg){
			static $c = 0;
			return trim($reg[1]) . ' data-mark="' . ++$c . '">';
		}, $text);

		$text = preg_replace('|\sdata-mark="[^"]"|', '', $text);

		if(preg_match_all('/(href|src)="(' . we_base_link::TYPE_INT_PREFIX . '|\?id=)(\\d+)(&amp;|&|#|\?)?("|[^"]+")/i', $text, $regs, PREG_SET_ORDER)){
			$allIds = [];
			foreach($regs as $reg){
				$allIds[] = intval($reg[3]);
			}

			$allDocs = DB_WE::$inst->getAllFirstq('SELECT ID,Path,UNIX_TIMESTAMP(Publish_Date) AS Published,IsDynamic FROM ' . FILE_TABLE . ' WHERE ID IN(' . implode(',', $allIds) . ')' . (!empty(we_base_WE::$we_doc->InWebEdition) ? '' : ' AND IsPublished=1'), true, MYSQLI_ASSOC);
			'@phan-var array<int,array<string,mixed>> $allDocs';
			foreach($regs as $reg){
				$foo = $allDocs[$reg[3]] ?? [];
				if($foo && $foo['Path']){
					$path_parts = pathinfo($foo['Path']);
					if(WYSIWYGLINKS_DIRECTORYINDEX_HIDE && seoIndexHide($path_parts['basename'])){
						$foo['Path'] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
					}
					$text = str_replace($reg[1] . '="' . $reg[2] . $reg[3] . $reg[4] . $reg[5], $reg[1] . '="' .
							$foo['Path'] .
							(!$foo['IsDynamic'] && !DISABLE_MEDIA_MODTIME ? '?m=' . $foo['Published'] . ($reg[4] === '#' ? '#' : '&') :
							($reg[4] ?: '')
							) . $reg[5], $text);
				} else {
					$text = preg_replace([
						'-<(a|img) [^>]*' . $reg[1] . '="' . $reg[2] . $reg[3] . '("|&|&amp;|\?|#)[^>]*>(.*)</a>-Ui',
						'-<(a|img) [^>]*' . $reg[1] . '="' . $reg[2] . $reg[3] . '(\?|&|&amp;|"|#)[^>]*>-Ui',
							], ['${3}',
						''
							], $text);
				}
			}
		}

		if(preg_match_all('/src="' . we_base_link::TYPE_THUMB_PREFIX . '(\\d+),(\\d+)"/i', $text, $regs, PREG_SET_ORDER)){
			$text = preg_replace('/(="' . we_base_link::TYPE_THUMB_PREFIX . '[^>]* )width="[^"]*"([^>]* )height="[^"]*"([^>]*>)/U', '$1$2$3', $text);
			foreach($regs as $reg){
				[, $imgID, $thumbID] = $reg;

				$thumbObj = new we_contents_thumbnail();
				if($thumbObj->initByImageIDAndThumbID((int) $imgID, (int) $thumbID)){
					$widthHeight = $thumbObj->isImageSvg() ? ((($width = $thumbObj->getOutputWidth()) ? ' width="' . $width . '"' : '') .
							(($height = $thumbObj->getOutputHeight()) ? ' height="' . $height . '"' : '')) : '';
					$text = str_replace('src="' . we_base_link::TYPE_THUMB_PREFIX . $imgID . ',' . $thumbID . '"', 'src="' . $thumbObj->getOutputPath(false, true) . '"' . $widthHeight, $text);
				} else {
					if(($path = id2path((int) $imgID))){
						$text = str_replace('src="' . we_base_link::TYPE_THUMB_PREFIX . $imgID . ',' . $thumbID . '"', 'src="' . $path . '"', $text);
					} else {
						// FIXME: is this the correct placeholder??
						$text = preg_replace('|<img[^>]+src="' . we_base_link::TYPE_THUMB_PREFIX . $imgID . ',' . $thumbID . '"[^>]+>|Ui', '', $text);
					}
				}
			}
		}
		try{
			if(we_modules_info::isActive(we_modules_info::OBJECT)){
				$we_doc = we_base_WE::getWeDoc();
				if(preg_match_all('/href="' . we_base_link::TYPE_OBJ_PREFIX . '(\\d+)(\\??)("|[^"]+")/i', $text, $regs, PREG_SET_ORDER)){
					foreach($regs as $reg){
						try{
							$href = we_contents_objectFile::getObjectHref((int) $reg[1], $pid, $path, 0, WYSIWYGLINKS_DIRECTORYINDEX_HIDE, WYSIWYGLINKS_OBJECTSEOURLS, $we_doc->InWebEdition);
						}catch(we_exception_base $e){//we_link_not_published
							$href = '';
						}

						if($href){
							$text = ($reg[2] === '?' ?
									str_replace('href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '?', 'href="' . $href . '&amp;', $text) :
									str_replace('href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . $reg[2] . $reg[3], 'href="' . $href . $reg[2] . $reg[3], $text));
						} else {
							$text = preg_replace([
								'-<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '("|&|&amp;|\?|#)[^>]*>(.*)</a>-Ui',
								'-<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '("|&|&amp;|\?|#)[^>]*>-Ui',
									], ['${2}', ''], $text);
						}
					}
				}
			}
		}catch(we_exception_base $e){
			//we_doc unavailable
		}

		return preg_replace('/\<a>(.*)\<\/a>/siU', '${1}', $text);
	}

	/**
	 *
	 * @return string[] array of paths
	 */
	private function getNavigationItems(): array{
		if($this->Table === FILE_TABLE && $this->ID && $this->InWebEdition){
			$ret = $this->DB_WE->getAllq('SELECT Path FROM ' . NAVIGATION_TABLE . ' WHERE Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND LinkID=' . intval($this->ID), true);
			'@phan-var string[] $ret';
			return $ret;
		}
		return [];
	}

	/**
	 * Get data which was saved persistent in this session by the editor
	 * @param string $name name of the variable to save
	 * @return bool false if not found, data if found
	 */
	public function getEditorPersistent(string $name): bool{
		return $this->editorSaves[$name] ?? false;
	}

	/**
	 * Save data in editor persistent in session
	 * @param string $name name of the variable to save
	 * @param bool $value the data to save. Don't save objects! Just data & arrays.
	 */
	public function setEditorPersistent(string $name, bool $value): void{
		$this->editorSaves[$name] = $value;
	}

	protected static function makeBlockName(string $block, string $field): string{
		return str_replace('####BLOCKNR####', '[0-9]+', preg_quote(
						str_replace('[0-9]+', '####BLOCKNR####', $field) .
						'blk_' .
						str_replace('[0-9]+', '####BLOCKNR####', $block) . '__') . '[0-9]+');
	}

	protected function initVariantDataFromDb(): void{
		if(!$this->canHaveVariants()){
			return;
		}
		if(($tmp = $this->getElementS(we_base_variants::ELEMENT_NAME))){
			// unserialize the variant data when loading the model
			$this->setElementA(we_base_variants::ELEMENT_NAME_ARRAY, we_unserialize($tmp), 'variant');
			// now register variant fields in document
			we_base_variants::setVariantDataForModel($this);
		}
	}

	function __sleep(): array{
		return array_merge(parent::__sleep(), ['documentCustomerFilter']);
	}

	public function getDocumentCustomerFilter(): ?we_customer_documentFilter{
		return $this->documentCustomerFilter;
	}

	protected function saveDocumentCustomerFilter(): void{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER) || empty($this->documentCustomerFilter)){
			return;
		}
		$this->documentCustomerFilter->setModelId($this->ID);
		$this->documentCustomerFilter->save();
	}

	public function applyWeDocumentCustomerFilterFromFolder(): void{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return;
		}
		$tmpFolder = we_folder_base::instance($this->Table, $this->ParentID);
		$this->documentCustomerFilter = $tmpFolder->documentCustomerFilter;
	}

	public function getDocumentState(): int{
		if($this->Publish_Date === null){
			return self::STATE_UNPUBLISHED;
		}
		return ($this->Mod_Date > $this->Publish_Date ? self::STATE_MODIFIED : self::STATE_PUBLISHED);
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$ret = parent::we_load($from);

		//if($ret){
		//some docs don't have data (xml)....?!
		if(isWE() && we_modules_info::isActive(we_modules_info::SCHEDULER)){//no need to load schedule data, if not in editmode
			$this->schedArr = we_schedule_scheduler::getScheduleTable($this->ID, $this->Table);
		}
		$this->initVariantDataFromDb();
		//}
		return $ret;
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
