<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9cb719edca8d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-10-06 19:05 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_webEdition extends we_document_textContent{

	public const BACKUP_XML_TAG = 'we:document';

	/** @var int ID of the templates that is used from the document */
	var $TemplateID = 0;

	/** @var int ID of the template that is used from the parked document (Bug Fix #6615) */
	protected $temp_template_id = 0;

	/** @var int[] Categories of the parked document (Bug Fix #6615) */
	protected $temp_category = [];

	/** @var ?bool */
	private $hasVariants = null;

	/** @var ?array */
	protected $hidePages = null;

	/** @var string assigned and read in im-/export */
	public $DocTypeName = '';

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::WEDOCUMENT;

	/** @var bool true if this document is new */
	protected $isNew = true;

	public function __construct(int $id = 0){
		if(isWE() && !$id && !empty($_SESSION['prefs']['DefaultTemplateID'])){
			$this->TemplateID = (int) $_SESSION['prefs']['DefaultTemplateID'];
		}
		parent::__construct($id);
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'TemplateID' => we_base_request::INT,
			//'TemplatePath' => we_base_request::STRING,
			'hidePages' => we_base_request::RAW,
			'controlElement' => we_base_request::RAW,
			'temp_template_id' => we_base_request::INT,
			'temp_category' => we_base_request::INTLISTA,
			'isNew' => we_base_request::BOOL,
		]);

		if($this->ID){
			$this->isNew = false;
		}

		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_VARIANTS;

			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && (we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER, we_base_perm::CUSTOMER_CAN_CHANGE_DOCS]))){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
		}
	}

	public function initByID(int $id, string $table = '', int $from = self::LOAD_MAIN_DB): void{
		if($id){
			$this->isNew = false;
		}
		parent::initByID($id, $table, $from);
	}

	public function isNew(): bool{
		return $this->isNew;
	}

	/**
	 *
	 * @param string $formname
	 * @param int $tid
	 * @param int $doctype
	 * @param int[] $categories
	 * @param int $docID
	 * @param bool $wewrite
	 * @return \we_document_webEdition
	 */
	public static function initDocument(string $formname = we_base_constants::DEFAULT_FORM, int $tid = 0, int $doctype = 0, array $categories = [], int $docID = 0, bool $wewrite = false): we_document_webEdition{
		if((!we_tag_sessionStart::$started) || (!isset($_SESSION['weS']['we_data']['we_document_session' . $formname])) || $wewrite){
			we_tag_write::$docElements[$formname] = $doc = new we_document_webEdition();
			$doc->we_new();
			if($docID){
				$doc->initByID($docID);
			} else {
				$doc->changeDoctype($doctype);
				if($tid){
					$doc->setTemplateID($tid);
				}
			}
			if($categories && (!$docID || $wewrite)){
				$doc->Category = $categories;
			}
			if(we_tag_sessionStart::$started){
				$doc->saveInSession('we_document_session' . $formname);
			}
		} else {
			if($docID){
				we_tag_write::$docElements[$formname] = $doc = new we_document_webEdition();
				$doc->initByID($docID);
			} else {
				$doc = we_contents_root::getSessionDocument('we_document_session' . $formname);
				assert($doc instanceof we_document_webEdition);
				we_tag_write::$docElements[$formname] = $doc;
			}
			$doc->Category = $categories ?: $doc->Category;
		}

		if(($ret = we_base_request::string('we_returnpage'))){
			$doc->setElementS('we_returnpage', $ret);
		}
		if(isset($_REQUEST['we_ui_' . $formname]) && is_array($_REQUEST['we_ui_' . $formname])){
			$dateFields = we_base_util::convertDateInRequest('we_ui_' . $formname); //FIXME: this can't be we_base_request at the moment
			foreach($dateFields as $name => $date){
				$doc->setElementI($name, (int) $date->format('U'), 'date');
			}
			foreach($_REQUEST['we_ui_' . $formname] as $n => $v){
				'@phan-var string $n';
				$v = we_base_util::rmPhp($v);
				$doc->setElementS($n, $v);
			}
		}

		if(($cats = we_base_request::_(we_base_request::WEFILELISTA, 'we_ui_' . $formname . '_categories', we_base_request::NOT_VALID_AS)) !== we_base_request::NOT_VALID_AS){
			$doc->Category = path_to_idA($cats, CATEGORY_TABLE);
		}
		if(($cats = we_base_request::intList('we_ui_' . $formname . '_Category', we_base_request::NOT_VALID_AI)) !== we_base_request::NOT_VALID_AI){
			$_REQUEST['we_ui_' . $formname . '_Category'] = implode(',', $cats);
		}
		foreach($doc->persistent_slots as $slotname => $type){
			switch($slotname){
				case 'categories':
					break;
				default:
					$doc->$slotname = we_base_request::_($type, 'we_ui_' . $formname . '_' . $slotname, $doc->$slotname);
			}
		}

		we_document_image::checkAndPrepare($formname, $doc);
		we_document_video::checkAndPrepare($formname, $doc);
		we_document_audio::checkAndPrepare($formname, $doc);
		we_document_other::checkAndPrepare($formname, $doc);

		if(we_tag_sessionStart::$started){
			$doc->saveInSession('we_document_session' . $formname);
		}
		return $doc;
	}

	public function makeSameNew(array $keep = []): void{
		parent::makeSameNew(array_merge($keep, ['TemplateID', 'IsDynamic']));
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				if(we_base_WE::$mainDoc['doc']){
					we_base_WE::$mainDoc['doc']->InWebEdition = true; //Bug 3417
				}
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;
				we_base_WE::setWeDoc($this);
				$file = preg_replace('/.tmpl$/i', '.php', $this->getTemplatePath());
				return file_exists($file) ? (new we_editor_showTempDocumentData($this, $jsCmd))->setFile($file) : new we_editor_noTmpl($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = false;
				we_base_WE::setWeDoc($this);
				$file = preg_replace('/.tmpl$/i', '.php', $this->getTemplatePath());
				return file_exists($file) ? (new we_editor_showTempDocumentData($this, $jsCmd))->setFile($file) : new we_editor_noTmpl($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VALIDATION:
				return new we_editor_validateDocument($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VARIANTS:
				we_base_WE::setWeDoc($this);
				return new we_editor_variants($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	private function formIsDynamic(bool $disabled = false): string{
		$v = $this->IsDynamic;
		if(!$disabled){
			$n = 'we_' . $this->Name . '_IsDynamic';
			return we_html_forms::checkboxWithHidden($v ? true : false, $n, g_l('weClass', '[IsDynamic]'), false, '', "_EditorFrame.setEditorIsHot(true);WE().layout.propEdit.switchExt(window," . intval($this->ID) . ",'" . $this->Name . "');");
		}
		return we_html_forms::checkboxWithHidden($v ? true : false, '', g_l('weClass', '[IsDynamic]'), false, '', '', true);
	}

	protected function formDocTypeTempl(): string{
		$disable = (we_base_permission::hasPerm([we_base_perm::WORK_DOCEXTENSION_EDIT]) ?
				(in_array($this->ContentType, [we_base_ContentTypes::HTML, we_base_ContentTypes::WEDOCUMENT]) && $this->Publish_Date) :
				true);

		return
				$this->formDocType(($this->Publish_Date !== null)) . we_html_element::br() .
				$this->formTemplatePopup(($this->Publish_Date !== null)) . we_html_element::br() .
				$this->formIsDynamic($disable) .
				$this->formIsSearchable() .
				$this->formInGlossar();
	}

	protected function formDocType(bool $disable = false): string{
		if($disable){
			$name = ($this->DocType ? fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType)) : g_l('weClass', '[nodoctype]'));
			return g_l('weClass', '[doctype]') . ': ' . $name;
		}
		$dtq = we_doctype_docTypes::getDoctypeQuery();

		return $this->formSelectFromDB('DocType', DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'], 'dt.ID', 'dt.Text', g_l('weClass', '[doctype]'), $dtq['where'], 1, [
					$this->DocType], false, ['onchange' => ($this->DocType ?
					"WE().util.showConfirm(window, '', '" . strtr(g_l('weClass', '[doctype_changed_question]'), ["\n" => '\n']) . "',['doctype_changed']);" :
					"we_cmd('doctype_changed');")], '', we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.we_cmd('we_doctype_edit')", '', '', (!we_base_permission::hasPerm([
									we_base_perm::WORK_DOCTYPE_EDIT]))), ((we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED]) || ($this->ID && empty($this->DocType)) ) ) ? [
					'', g_l('weClass', '[nodoctype]')] : []);
	}

	private function formTemplateWindow(): string{
		$weSuggest = we_gui_suggest::getInstance();
		$textname = 'we_' . $this->Name . '_TemplateName';
		$idname = 'we_' . $this->Name . '_TemplateID';
		$ueberschrift = g_l('weClass', '[template]');

		[$styleTemplateLabel, $styleTemplateLabelLink] = ($this->TemplateID ?
				['display:none', 'display:inline'] :
				['display:inline', 'display:none']);

		$myid = $this->TemplateID ?: '';
		$path = id2path((int) $myid, TEMPLATES_TABLE);

		$weSuggest->setAcId('Template')
				->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE])
				->setInput($textname, $path)
				->setLabel(we_html_element::span(['id' => 'TemplateLabel', 'style' => $styleTemplateLabel], $ueberschrift) . we_html_element::span(['id' => 'TemplateLabelLink',
							'style' => $styleTemplateLabelLink], $ueberschrift))
				->setMaxResults(10)
				->setResult($idname, (string) $myid)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(TEMPLATES_TABLE)
				->setWidth(0)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'reload_hot_editpage',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"))
				//->setIsDropFromTree(true);// deactivated
				->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:goTemplate(" . $myid . ");"));

		return $weSuggest->getHTML();
	}

	// creates the Template PopupMenue
	private function formTemplatePopup(bool $disable): string{
		$templateFromDoctype = ($this->DocType ?
				fS('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType) . ' LIMIT 1') :
				'');
		if($disable){
			$myid = (int) ($this->TemplateID ?: 0);
			$path = ($myid ? fS('SELECT IF(Display!="",Display,Path) FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . $myid) : '');

			if($this->DocType){
				return ($templateFromDoctype ?
						$this->xformTemplatePopup() :
						we_html_tools::formElementTable(g_l('weClass', '[template]'), $path)
						);
			}
			$pop = (we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && we_editor_functions::isNormalMode() ?
					$path . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:goTemplate(" . $myid . ");") : $path);

			return g_l('weClass', '[template]') . ': ' . $pop;
		}

		if($this->DocType){
			// if a Doctype is set and this Doctype has defined some templates, just show a select box
			return ($templateFromDoctype ?
					$this->xformTemplatePopup() :
					$this->formTemplateWindow() );
		}
		return $this->formTemplateWindow();
	}

	private function xformTemplatePopup(): string{
		$ws = we_users_util::get_ws(TEMPLATES_TABLE);
		'@phan-var int[] $ws';

		['TemplateID' => $TID, 'Templates' => $Templates] = getHash('SELECT TemplateID,Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType));
		$tlist = ($TID ?: '') . ($Templates ? ',' . $Templates : '');

		if($tlist){
			$temps = we_base_request::filterIntList($tlist);
		} else {
			$temps = [];
			foreach($ws as $wid){
				we_base_util::pushChilds($temps, (int) $wid, TEMPLATES_TABLE, false);
			}
		}
		if($this->TemplateID){
			$temps[] = $this->TemplateID;
			$TID = $this->TemplateID;
		}
		$templist = array_unique($temps);

		$openButton = (we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && we_editor_functions::isNormalMode() ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:goTemplate(document.we_form.elements.we_" . $this->Name . "_TemplateID.options[document.we_form.elements.we_" . $this->Name . "_TemplateID.selectedIndex].value)") : '');

		if($templist){
			$foo = [];
			foreach($templist as $tid){

				if(($tid == $this->TemplateID) || we_users_util::in_workspace($tid, $ws, TEMPLATES_TABLE)){
					$foo[] = $tid;
				}
			}

			return $this->formSelectFromDB2('TemplateID', TEMPLATES_TABLE, 'ID', 'IF(Display!="",Display,Path)', g_l('weClass', '[template]'), 'IsFolder=0 AND ID IN (' . ($foo ? implode(',', $foo) : -1) . ') ORDER BY Path', 1, [
						$TID], false, "we_cmd('template_changed');_EditorFrame.setEditorIsHot(true);", [
						'required' => 'required'], '', $openButton, ['', '']);
		}
		return $this->formSelectFromDB('TemplateID', TEMPLATES_TABLE, 'ID', 'IF(Display!="",Display,Path)', g_l('weClass', '[template]'), 'IsFolder=0 ORDER BY Path ', 1, [
					$this->TemplateID], false, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], '', $openButton);
	}

	/**
	 * @return string
	 * @desc Returns the metainfos for the selected file.
	 */
	public function formMetaInfos(we_base_jsCmd $jsCmd): string{
		//	Collect data from meta-tags
		//will evaluate the tags => we get meta data set
		$oldEdit = $this->EditPageNr; //FIXME: cache data
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;
		we_gui_suggest::setStaticInstance(false); //avoid modification on suggestor
		$this->editor($jsCmd); //execute only - do we need to show??
		$this->EditPageNr = $oldEdit;
		//restore contenttype if modified
		we_html_tools::forceHeaders();

		we_gui_suggest::setStaticInstance(true);

		//	if a meta-tag is set all information are in array $GLOBALS['meta']
		return parent::formMetaInfos($jsCmd);
	}

	public function getTemplatePath(): string{
		$path = ($this->TemplateID ? id2path((int) $this->TemplateID, TEMPLATES_TABLE) : '');
		return $path ? TEMPLATES_PATH . $path : '';
	}

	public function setTemplateID(int $templID): void{
		$this->TemplateID = $templID;
	}

	public function we_new(): void{
		parent::we_new();
	}

	private static function getFieldType(string $tagname, array $attribs, bool $useTextarea): string{
		switch($tagname){
			case 'list':
				return 'block';
			case 'textarea':
				if(!$useTextarea){
					return 'txt';
				}
			//no break;
			case self::INTERNAL_VAR://internal data which is never saved
			case 'formfield':
			case 'img':
			case 'block':
			case 'link':
			case 'object':
			case 'href':
			case 'customer':
				return $tagname;
			case 'input':
				return (empty($attribs['type']) || $attribs['type'] !== 'date') ? 'txt' : 'date';
			default:
				return 'txt';
		}
	}

	/**
	 * @return string
	 * @desc this function returns the code of the template this document bases on
	 */
	function getTemplateCode(bool $completeCode = true): string{
		return fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . intval($this->TemplateID) . ' AND c.nHash=unhex(md5("' . ($completeCode ? we_contents_template::FULLCODE : 'data') . '"))');
	}

	protected function getFieldTypes(string $templateCode, bool $useTextarea = false): array{
		$blocks = $fieldTypes = [];
		$tp = new we_weTag_parser($templateCode, $this->getPath());
		$tags = $tp->getTagsWithAttributes(false, true);
		'@phan-var array<int,array{name:string,attribs:array<string,string|string[]>,close:bool,weTag:string}> $tags';
		foreach($tags as $tag){
			$tagname = $tag['name'];
			if(!$tag['close']){// starttag found
				if(!empty($tag['attribs']['name'])){
					switch($tag['name']){
						case 'var':
						case 'field':
							continue 2;
						default:
							$name = (string) $tag['attribs']['name'];
							$ft = self::getFieldType($tag['name'], $tag['attribs'], $useTextarea);
							if($blocks){
								$foo = end($blocks);
								$blockname = (string) $foo['name'];
								$blocktype = (string) $foo['type'];
								switch($blocktype){
									case 'block':
										$name = self::makeBlockName($blockname, $name);
										break;
								}
							}
							$fieldTypes[$name] = $ft;
							switch($tag['name']){
								case 'block':
									$blocks[] = ['name' => $name, 'type' => $tag['name']];
									break;
							}
					}
				}
			} else { // endtag found
				switch($tagname){
					case 'block':
						if($blocks){
							array_pop($blocks);
						}
						break;
				}
			}
		}
		return $fieldTypes;
	}

	public function correctFields(): void{//FIXME: is this still relevant???
		// this is new for shop-variants
		$this->correctVariantFields();
		$regs = [];
		//FIXME: it is better to use $this->getUsedElements - only we:input type="date" is not handled... => this will call the TP which is not desired since this method is called on save in frontend
		//FIXME: getUsedElements is not available on save
		$types = $this->getFieldTypes($this->getTemplateCode());

		foreach($this->elements as $name => &$v){
			switch($v['type'] ?? ''){
				case 'block':
					$this->setElementType($name, 'block');
					break;
				case 'txt':
					//Bugfix for buggy tiny implementation where internal links looked like href="/img.gif?id=123" #7210
					if(!empty($v['dat'])){
						$dat = (string) $v['dat'];
						if(preg_match_all('@src="/[^">]+\\?id=(\\d+)([&|#][^">]+["]|["])@i', $dat, $regs, PREG_SET_ORDER)){
							foreach($regs as $reg){
								$dat = str_replace($reg[0], 'src="' . we_base_link::TYPE_INT_PREFIX . $reg[1] . '"', $dat);
							}
						}
						if(preg_match_all('@src="/[^">]+\?thumb=(\d+,\d+)([&|#][^">]+["]|["])@i', $dat, $regs, PREG_SET_ORDER)){
							foreach($regs as $reg){
								$dat = str_replace($reg[0], 'src="' . we_base_link::TYPE_THUMB_PREFIX . $reg[1] . '"', $dat);
							}
						}
						$dat = we_wysiwyg_editor::parseThumbnailsWidthHeight($dat);

						$v['dat'] = preg_replace('_\s+data-we\w+="[^"]+"_', '', $dat); //currently unused, attributes set from we are filtered out
					}
				case 'attrib':
				case 'variant':
				case 'formfield':
				case 'date':
				case 'image':
				case 'link':
				case 'img':
				case 'href':
				case 'object':
				case 'customer':
					if(isset($types[$name])){
						$this->setElementType($name, $types[$name]);
					}
					break;
				case self::INTERNAL_VAR://internal data which is never saved
					break;
				default:
					switch($types[$name] ?? ''){
						case 'link':
							// FIXME: make sure fixed types are written to tblFile too!
							$this->setElementType($name, $types[$name]);
							break;
						default:
							$this->setElementType($name, 'txt');
					}
			}
		}
	}

	/*
	 * this function is used to replace to prepare wysiwyg img sources for db
	 * it also writes img sources and hrefs to $this->MediaLinks
	 *
	 * when $isRebuildMediaLinks it only writes $this->MediaLinks (img sources come from db and must not be changed)
	 */

	function parseTextareaFields(string $rebuildMode = 'default'): void{
		switch($rebuildMode){
			case 'default':
				foreach($this->nextElement('txt') as $name){
					$oldVal = $this->getElementS($name);
					if(!$oldVal){
						continue;
					}
					$value = we_wysiwyg_editor::reparseInternalLinks($oldVal);
					$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::getAllMediaLinks($value, $name)); //true: replace internal file paths
					if($oldVal != $value){
						$this->setElementS($name, $value);
					}
				}
				return;

			//FIXME: implement textarea as element-type for textareas!
			case 'main':
			case 'temp'://rebuilding from tblTemporaryDoc
				foreach($this->nextElement('txt') as $name){
					'@phan-var string $name';
					$ct = $this->getElementS($name);

					if(preg_match('/(src|href)="' . we_base_link::TYPE_INT_PREFIX . '/', $ct)){
						// do we need this for normal rebuild? it's obsolete for rebuilding medialinks!
						//self::parseInternalLinks($elem['dat'], 0, '', true);
						// FIXME: use we_wysiwyg_editor::registerMediaLinks when available
						$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::getAllMediaLinks($ct, $name));
					}
				}
				return;
		}
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		// First off correct corupted fields
		$this->correctFields();

//FIXME: maybe use $this->getUsedElements() to unset unused elements?! add setting to do this? check rebuild!
		// Bug Fix #6615
		$this->temp_template_id = $this->TemplateID;
		$this->temp_category = $this->Category;

		// Last step is to save the webEdition document
		$out = parent::we_save($resave, $skipHook);
		if($out){
			$this->unregisterMediaLinks(false, true);
			$this->parseTextareaFields();
			$out = $this->registerMediaLinks(true);
		}

		if(LANGLINK_SUPPORT){
			$out = $out && $this->setLanguageLink();
		}

		$this->writeMetaValues();
		return $out;
	}

	public function we_publish(bool $DoNotMark = false, bool $saveinMainDB = true, bool $skipHook = false): bool{
		$this->temp_template_id = $this->TemplateID;
		$this->temp_category = $this->Category;
		$out = parent::we_publish($DoNotMark, $saveinMainDB, $skipHook);
		if($out){
			if($DoNotMark){
				// when called directly by rebuild we must prepare elements as is normally done in we_save
				$this->correctFields();
				$this->parseTextareaFields('main');
				// TODO: we should try to throw out obsolete elements from temporary! but this affects static docs only!
				// TODO: when doing rebuild media link test all elements against template!
				$this->unregisterMediaLinks(true, false);
				$out = $this->registerMediaLinks(); // last param: when rebuilding static docs do not delete temp entries!
			} else {
				$this->unregisterMediaLinks();
				$out = $this->registerMediaLinks(false, true);
			}
		}

		return $out;
	}

	public function we_unpublish(bool $skipHook = false): bool{
		$oldPublished = $this->Publish_Date;
		$ret = ($this->ID ? parent::we_unpublish($skipHook) : false);

		// if document was modified before unpublishing, the actual version is in tblTemporaryDoc: we unregister temp=0
		// otherwise we have nothing to do
		if($ret && $oldPublished && ($this->Mod_Date > $oldPublished)){
			$this->unregisterMediaLinks(true, false);
		}

		return $ret;
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$ret = parent::we_load($from);
		if(!$this->isEmptyElement(we_base_variants::ELEMENT_NAME_ARRAY)){
			$this->setElementS(we_base_variants::ELEMENT_NAME, $this->getElementS(we_base_variants::ELEMENT_NAME_ARRAY));
			$this->setElementA(we_base_variants::ELEMENT_NAME_ARRAY, we_unserialize($this->getElementS(we_base_variants::ELEMENT_NAME_ARRAY)));
		}
		return $ret;
	}

	function getDocumentData(string $includepath = ''): string{
		extract($GLOBALS, EXTR_SKIP); // globalen Namensraum herstellen.
		$editpageSave = $this->EditPageNr;
		we_base_WE::$saveStatic = true;
		if(we_base_WE::$mainDoc['doc']){
			$inWebEditonSave = we_base_WE::$mainDoc['doc']->InWebEdition;
			we_base_WE::$mainDoc['doc']->InWebEdition = false;
			$this->InWebEdition = false;
		} else {
			$inWebEditonSave = $this->InWebEdition;
			$this->InWebEdition = false;
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PREVIEW;
		$we_include = $includepath ?: $this->editor();
		if(is_object($we_include)){
			$contents = $we_include->show();
		} elseif(is_file($we_include)){
			ob_start();
			include($we_include);
			$contents = ob_get_clean() ?: '';
		} else {
			t_e('File ' . $we_include . ' not found!');
			$contents = '';
		}
		//restore contenttype if modified
		//we_html_tools::forceHeaders();
		$this->EditPageNr = $editpageSave;
		if(we_base_WE::$mainDoc['doc']){
			$this->InWebEdition = we_base_WE::$mainDoc['doc']->InWebEdition = $inWebEditonSave;
		} else {
			$this->InWebEdition = $inWebEditonSave;
		}
		we_base_WE::$saveStatic = false;
		if(we_base_WE::$we_EDITOR){ //  fix for php5, in editor we_doc was replaced by we_base_WE::$we_doc from we:include tags
			we_base_WE::setWeDoc($this);
		}

		return $contents;
	}

	protected function scheduleToBeforeNow(): bool{
		//FIXME: check schedarray!
		return false; //($this->To < time()));
	}

	protected function isVariantNamesValid(): bool{
		$variationFields = we_base_variants::getAllVariationFields($this);

		if(!empty($variationFields)){
			$i = 0;
			while($this->issetElement(we_base_variants::PREFIX . $i)){
				if(!trim($this->getElementS(we_base_variants::PREFIX . $i++))){
					return false;
				}
			}
		}

		return true;
	}

	// returns the filesize of the document
	function getFilesize(): int{
		return (file_exists(WE_DOCUMENT_ROOT . $this->Path) ?
				(filesize(WE_DOCUMENT_ROOT . $this->Path) ?: 0) : //das ist ungenau
				0);
	}

	public static function getDynamicDocumentToSave(int $ID): string{
		$c = 'constant';
		$date = date('Y-m-d, H:i');
		return <<<EOT
<?php /* Generated by WE {$c('WE_VERSION')}, REV {$c('WE_SVNREV')}, {$date} */
\$GLOBALS['WE_IS_DYN'] = 1;
\$_REQUEST['we_cmd'] = [1 => {$ID}];
require((is_dir('{$c('WE_INCLUDES_PATH')}') ? '{$c('WE_INCLUDES_PATH')}' : (\$_SERVER['DOCUMENT'.'_ROOT'] . '{$c('WE_INCLUDES_DIR')}')) . 'we_showDocument.inc.php');
EOT;
	}

	protected function getDocumentToSave(): string{
		static $cache = [];
		if($this->IsDynamic){
			we_base_variants::setVariantDataForModel($this);
			we_base_variants::correctModelFields($this);
			return self::getDynamicDocumentToSave($this->ID);
		}

		if(isset($cache[$this->ID])){
			return $cache[$this->ID];
		}

		$doc = $this->getDocumentData();
		$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst);
// --> Glossary Replacement
		$useGlossary = ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (!we_base_WE::$mainDoc['id'] || we_base_WE::$mainDoc['id'] == $this->ID)) && ($this->InGlossar) && we_glossary_replace::useAutomatic());

		// --> Glossary Replacement
		$doc = ($useGlossary ? we_glossary_replace::doReplace($doc, $this->Language) : $doc);
		$doc = ($urlReplace ? preg_replace($urlReplace, array_keys($urlReplace), $doc) : $doc);

		return $cache[$this->ID] = $doc;
	}

	/**
	 * @return void
	 * @desc This function sets special fields in the document to control i.e. the existing EDIT_PAGES or the available buttons
	 * 		for this document, use with tags we:hidePages and we:controlElement
	 */
	public function setDocumentControlElements(): void{
		////FIXME: use Tagparser & save this to DB
		//	get code of the matching template
		$templateCode = $this->getTemplateCode();

		//	First set hidePages from document ...
		$this->setHidePages($templateCode);

		//	now set information about buttons of document
		$this->setControlElements($templateCode);
	}

	public function executeDocumentControlElements(): void{
		// here we must check, if setDocumentControlElements() already worked
		if($this->controlElement !== null){
			$this->setDocumentControlElements();
		}
		//	disable hidePages
		$this->disableHidePages();
	}

	/**
	 * @return void
	 * @param string $templatecode
	 * @desc	if tag we:controlElement exists in template, this function sets the given control-elements in persistent_slot
	 * 		they are disabled in document later
	 *
	 */
	function setControlElements(string $templatecode): void{
		if(!$this->InWebEdition || !str_contains($templatecode, '<we:controlElement')){
			return;
		}
		$tags = we_weTag_parser::itemize_we_tag('we:controlElement', $templatecode);
		//	we need all given tags ...

		if(isset($this->persistent_slots['controlElement'])){
			$this->controlElement = [];
		} else {
			$this->persistent_slots['controlElement'] = we_base_request::RAW;
		}

		foreach($tags[2] as $cur){ //	go through all matches
			we_weTag_util::tag('controlElement', array_merge(['doc' => $this], we_weTag_parser::parseAttribs($cur)));
		}
	}

	/**
	 * @return void
	 * @param string $templatecode
	 * @desc	if tag we:hidePages exists in template, this function sets the given pages in persistent_slot
	 *
	 */
	private function setHidePages(string $templatecode): void{
		if(!$this->InWebEdition || !str_contains($templatecode, '<we:hidePages')){
			return;
		}
		//	delete exisiting hidePages ...
		if(isset($this->persistent_slots['hidePages'])){
			$this->hidePages = [];
		} else {
			$this->persistent_slots['hidePages'] = we_base_request::RAW;
		}

		$tags = we_weTag_parser::itemize_we_tag('we:hidePages', $templatecode);

		foreach($tags[2] as $cur){ //	go through all matches
			$this->hidePages = array_merge($this->hidePages, we_weTag_tag::getAttribute('pages', we_weTag_parser::parseAttribs($cur), '', we_base_request::STRING_LIST));
		}
		$this->disableHidePages();
	}

	/**
	 * @return void
	 * @desc disables the editpages saved in persistent_slot hidePages inside webEdition
	 */
	private function disableHidePages(): void{
		if(!$this->InWebEdition || empty($this->hidePages)){
			return;
		}
		if(in_array('all', $this->hidePages, true)){
			$this->EditPageNrs = [];
			return;
		}

		foreach($this->hidePages as $hide){
			if(isset(we_tag_hidePages::MNEMONIC_EDITPAGES[$hide]) && ($key = array_search(we_tag_hidePages::MNEMONIC_EDITPAGES[$hide], $this->EditPageNrs)) !== false){
				unset($this->EditPageNrs[$key]);
			}
		}
	}

	/**
	 * checks if this document is allowed to have variants
	 * and if it has some fields defined for variants.
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants(bool $checkFields = false): bool{
		if(($this->TemplateID == 0)){
			return false;
		}

		if($this->hasVariants !== null){
			return (bool) $this->hasVariants;
		}

		if($this->InWebEdition){
			//FIXME: check if we can add more for the like query e.g. type?
			return ($this->hasVariants = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE DID=' . intval($this->TemplateID) . ' AND DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND nHash=unhex(md5("' . we_base_variants::ALLVARIANT_FIELD . '"))'));
		}

		if(!$this->isEmptyElement(we_base_variants::ELEMENT_NAME_ARRAY)){
			// $tmp = $this->getElementA(we_base_variants::ELEMENT_NAME_ARRAY, [], true);
			// $this->setElementS(we_base_variants::ELEMENT_NAME, we_serialize($tmp), 'variant');
			return ($this->hasVariants = true);
		}
		$vars = we_unserialize($this->getElementS(we_base_variants::ELEMENT_NAME));
		return ($this->hasVariants = !empty($vars));
	}

	private function correctVariantFields(): void{//FIXME: is this still relevant???
		we_base_variants::correctModelFields($this);
	}

	function getVariantFields(): array{
		if($this->TemplateID == 0){
			return [];
		}
		$template = new we_contents_template($this->TemplateID);
		return $template->getVariantFields();
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[document]'), $this->formDocTypeTempl(), we_html_multiIconBox::PROP_DOC)))
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[metainfo]'), $this->formMetaInfos($jsCmd), we_html_multiIconBox::PROP_META)))
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[language]'), $this->formLangLinks(), we_html_multiIconBox::PROP_LANG))->fold())
								->add(we_modules_info::isActive(we_modules_info::CATEGORY) ? (new we_html_multiIconBoxContentLine(g_l('global', '[categorys]'), $this->formCategory($jsCmd), we_html_multiIconBox::PROP_CATEGORIES))->fold() : null)
								->add((new we_html_multiIconBoxContentLine(g_l('global', '[navigation]'), $this->formNavigation($jsCmd), we_html_multiIconBox::PROP_NAVI))->fold())
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[copyWeDoc]'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
								->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
								, 'PropertyPage'))
						->marginLeft()
						->get();
	}

	public function changeDoctype(int $dt = 0, bool $force = false): void{
		if((!$this->ID) || $force){
			if($dt){
				$this->DocType = $dt;
			}
			if($this->DocType && ($rec = getHash('SELECT dt.*,dtf.ID AS ParentID,dtf.Path AS ParentFilePath FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID WHERE dt.ID=' . intval($this->DocType)))){
				$this->Extension = $rec['Extension'];
				if($rec['ParentFileID'] == 0 || $rec['ParentID']){
					$this->ParentPath = $rec['ParentFilePath'] ?: '/';
					$this->ParentID = (int) $rec['ParentFileID'];
				}
				if($this->ContentType === we_base_ContentTypes::WEDOCUMENT){
					// only switch template, when current template is not in Templates
					$templates = explode(',', $rec['Templates']);
					if(!in_array($this->TemplateID, $templates)){
						$this->setTemplateID((int) $rec['TemplateID']);
					}
					$this->IsDynamic = (bool) $rec['IsDynamic'];
				}
				$this->IsSearchable = (bool) $rec['IsSearchable'];
				$this->Category = we_base_request::filterIntList($rec['Category']);
				$this->Language = $rec['Language'];
				$this->ParentPath = rtrim($this->ParentPath, '/');
				switch($rec['SubDir']){
					case self::SUB_DIR_YEAR:
						$this->ParentPath .= '/' . date('Y');
						break;
					case self::SUB_DIR_YEAR_MONTH:
						$this->ParentPath .= '/' . date('Y') . '/' . date('m');
						break;
					case self::SUB_DIR_YEAR_MONTH_DAY:
						$this->ParentPath .= '/' . date('Y') . '/' . date('m') . '/' . date('d');
						break;
					default:
				}

				if(!$this->checkPathDiffAndCreate()){
					$folder = new we_folder_document();
					$folder->initByPath($this->ParentPath);
					$this->ParentID = $folder->ID;
				}

				// get Customerfilter of parent
				if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
					$this->documentCustomerFilter = we_customer_documentFilter::getFilterByIdAndTable($this->ParentID, $this->Table);
				}
			}
		}
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => $this->ID, 'type' => we_shop_shop::DOCUMENT, 'req' => []];
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_document_textContent::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(): void{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return;
		}
		$this->documentCustomerFilter = we_customer_documentFilter::getFilterByIdAndTable($this->ID, FILE_TABLE);
	}

	public function toXML(array $attribs = [], array $extraSlots = [], string $content = ''): string{
		//$this->TemplatePath = we_base_file::clearPath('/' . str_replace(WE_DOCUMENT_ROOT, '', $this->getTemplatePath()));
		$this->DocTypeName = fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . intval($this->DocType));
		return parent::toXML($attribs, ['DocTypeName' => we_base_request::STRING]);
	}

}
