<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f0214b726cb5 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-04-04 16:47 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_contentProvider{
	const CODING_NONE = '';
	const CODING_ENCODE = 'encode';
	const CODING_SERIALIZE = 'serial';
	const CODING_ATTRIBUTE = 'coding';
	const DONT_IMEXPORT_PROPS = ['elements', 'wasUpdate', 'nHash'];

	static function getInstance(string $we_ContentType, int $ID = 0, string $table = ''): we_base_mainmodel{
		switch($we_ContentType){
			case we_base_ContentTypes::DOCTYPE:
				return new we_doctype_docTypes($ID);
			case we_base_ContentTypes::CATEGORY:
			case we_base_ContentTypes::CATEGORIES;
				$we_doc = new we_category_category();
				$we_doc->load($ID);
				return $we_doc;
			case we_base_ContentTypes::NAVIGATION:
				return new we_navigation_navigation($ID);
			case we_base_ContentTypes::NAVIGATIONRULE:
				$we_doc = new we_navigation_rule();
				$we_doc->we_load($ID);
				return $we_doc;
			case we_base_ContentTypes::THUMBNAIL:
				$we_doc = new we_exim_thumbnailExport();
				$we_doc->we_load($ID);
				return $we_doc;
			case 'we_backup_dbTable':
			case 'we_backup_table':
				$we_doc = new we_backup_dbTable($table, true);
				return $we_doc;
			case 'we_backup_tableItem':
				$we_doc = new we_backup_tableItem($table);
				if($ID){
					$we_doc->load($ID);
				}
				return $we_doc;

			case we_base_ContentTypes::BACKUP_BINARY:
				return new we_backup_binary($ID);
			case we_base_ContentTypes::VERSION:
				return new we_backup_version($ID);
			// fix for classes
			case we_base_ContentTypes::OBJECT:
				return new we_backup_object($ID);
			// fix ends ------------------------------------------------
		}

		switch($we_ContentType){
			case we_base_ContentTypes::FOLDER_COMMON:
				$we_Table = $table ?: FILE_TABLE;
				break;
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::TEMPLATE:
				$we_Table = TEMPLATES_TABLE;
				break;
			case we_base_ContentTypes::OBJECT:
				$we_Table = OBJECT_TABLE;
				break;
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::OBJECT_FILE:
				$we_Table = OBJECT_FILES_TABLE;
				break;
			case we_base_ContentTypes::FOLDER_VFILE:
				$we_Table = VFILE_TABLE;
				break;
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			default:
				$we_Table = FILE_TABLE;
		}
		return we_contents_base::initDoc($we_ContentType, $ID, $we_Table);
	}

	public static function getTagName(we_exim_xmlIF $object): string{
		if(defined(($const = get_class($object) . '::BACKUP_XML_TAG'))){
			return constant($const);
		}
		return we_document_base::BACKUP_XML_TAG;
	}

	protected static function needCoding(string $content): bool{
		return we_base_util::isSerialized($content) || preg_match('!([\x0-\x08\x0e-\x19\x11\x12<>&\x7F-\xFF])!', $content); //exclude x9:\t,x10:\n,x13:\r,x20:space
	}

	protected static function needCdata(string $content): bool{
		return preg_match('-[<>&]-', $content) ? true : false;
	}

	protected static function needSerialize($content): bool{
		return is_array($content);
	}

	/**
	 *
	 * @param mixed $content
	 * @return array{string,array}
	 */
	public static function formatXMLData($content): array{
		if($content instanceof DateTime){
			return [$content->format(we_database_base::DATE_TIME_FORMAT), []];
		}

		if(self::needSerialize($content)){
			return [
				we_exim_contentProvider::encode(we_serialize((array) $content, SERIALIZE_JSON)),
				[we_exim_contentProvider::CODING_ATTRIBUTE => we_exim_contentProvider::CODING_SERIALIZE]
			];
		} elseif(is_string($content) && (we_exim_contentProvider::needCoding($content) || we_exim_contentProvider::needCdata($content))){//fix for faulty parser
			return [
				we_exim_contentProvider::encode($content),
				[we_exim_contentProvider::CODING_ATTRIBUTE => we_exim_contentProvider::CODING_ENCODE]
			];
		}
		return [$content, []];
	}

	static function isBinary(int $id): bool{
		return fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . $id . ' AND ContentType IN("' . we_base_ContentTypes::IMAGE . '","' . we_base_ContentTypes::AUDIO . '","' . we_base_ContentTypes::VIDEO . '","' . we_base_ContentTypes::APPLICATION . '","' . we_base_ContentTypes::PDF . '") LIMIT 1');
	}

	public static function encode(string $data): string{
		return base64_encode($data);
	}

	static function decode(string $data): string{
		return base64_decode($data) ?: '';
	}

	public static function getDecodedData(string $type, string $data){
		switch($type){
			case self::CODING_ENCODE:
				return self::decode($data);
			case self::CODING_SERIALIZE:
				$dec = self::decode($data);
				return we_unserialize($dec, '', true) ?: $dec;
			case self::CODING_NONE:
			default:
				return $data;
		}
	}

}
