<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6be7c86edc20 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-01-27 21:53 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_refTable implements Countable{
	/** @var we_exim_refData[] */
	private $Storage = [];

	/** @var int */
	var $current = 0;

	/** @var array<string,array{user:string,id:int}> */
	var $Users = []; // username => id

	/**
	 *
	 * @param null|we_exim_element|we_backup_object|we_contents_objectFile|we_folder_objectFile|we_navigation_navigation|we_navigation_rule|we_exim_thumbnailExport $object
	 * @param array $extra
	 * @return void
	 */
	function add(?we_base_mainmodel $object, array $extra = []): void{
		$rd = new we_exim_refData($object, $extra);
		if(!$object){
			$rd->Table = we_exim_ExIm::getTableForCT($rd->ContentType, $rd->Table ?? '');
		}
		if($this->hasPerms($rd)){
			$this->Storage[] = $rd;
		}
	}

	/**
	 *
	 * @param int[] $ids
	 * @return void
	 */
	function addToUsers(array $ids): void{
		foreach($ids as $id){
			$key = basename(id2path($id, USER_TABLE));
			if($key){
				$this->Users[$key] = ['user' => $key, 'id' => $id];
			}
		}
	}

	private function hasPermsByTable(string $table, int $id): bool{
		switch($table){
			case DOC_TYPES_TABLE:
			case CATEGORY_TABLE:
			case NAVIGATION_RULE_TABLE:
				$allowed = true;
				break;
			default:
				$allowed = fB('SELECT 1 FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id . ' ' . we_exim_ExIm::queryForAllowed($table));
		}
		switch($table){
			case FILE_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]);
			case TEMPLATES_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]);
			case OBJECT_TABLE :
				return $allowed && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]);
			case OBJECT_FILES_TABLE :
				return $allowed && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]);
			case VFILE_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE]);
			case DOC_TYPES_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::WORK_DOCTYPE_EDIT]);
			case CATEGORY_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT]);
			case NAVIGATION_TABLE:
				return $allowed && we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT]);
			case FILELINK_TABLE:
				return true;
		}
		return false;
	}

	private function hasPerms(we_exim_refData $rd): bool{
		if($rd->Table){
			return $this->hasPermsByTable($rd->Table, $rd->ID);
		}
		switch($rd->ContentType){
			case we_base_ContentTypes::BACKUP_BINARY:
			case 'weBinary':
			case we_base_ContentTypes::NAVIGATIONRULE:
			case we_base_ContentTypes::THUMBNAIL:
				return true;
			default:
				return false;
		}
	}

	function moveItemsToEnd(string $ct): void{
		$regular = $moved = [];
		foreach($this->Storage as $elem){
			if($elem->ContentType === $ct){
				$moved[] = $elem;
			} else {
				$regular[] = $elem;
			}
		}
		$this->Storage = array_merge($regular, $moved);
	}

	function update(we_base_mainmodel $object): void{
		$param = [
			'ID' => $object->ID,
			'ContentType' => $object->ContentType
		];
		foreach($this->Storage as $k => $ref){
			if($ref->match($param)){
				$this->Storage[$k]->init($object);
			}
		}
	}

	function exists(array $params): bool{
		foreach($this->Storage as $ref){
			if($ref->match($params)){
				return true;
			}
		}
		return false;
	}

	function setProp(array $id, string $name, /* mixed */ $value): bool{
		foreach($this->Storage as $ref){
			if($ref->match($id)){
				$this->{$name} = $value;
				return true;
			}
		}
		return false;
	}

	function reset(): void{
		$this->current = 0;
	}

	function getNext(): ?we_exim_refData{
		if(isset($this->Storage[$this->current])){
			$id = $this->current;
			$this->current++;
			return $this->Storage[$id];
		}
		$this->reset();
		return null;
	}

	function getLast(): ?we_exim_refData{
		return ($this->Storage ?
			$this->Storage[count($this->Storage) - 1] :
			null);
	}

	function getRef(array $param): ?we_exim_refData{
		foreach($this->Storage as $ref){
			if($ref->match($param)){
				return $ref;
			}
		}
		return null;
	}

	public function __sleep(): array{
		return ['Storage'];
	}

	public function getCount(): int{
		return count($this->Storage);
	}

	function getNewOwnerID(int $id): int{
		foreach($this->Users as $user){
			if($user['id'] == $id &&
				($newid = fI('SELECT ID FROM ' . USER_TABLE . ' WHERE Username=\'' . DB_WE::$inst->escape($user['user']) . '\''))){
				return $newid;
			}
		}
		return 0;
	}

	public function count(): int{
		return count($this->Storage);
	}

}
