<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f9d7b286feee $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 01:23 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
// FIXME: move to some new class we_exim_preparer => no exporting in modul anymore

class we_export_preparer extends we_exim_ExIm{
	/** @var array<string,mixed> */
	var $options = [];

	/** @var we_exim_searchPatterns  */
	protected $PatternSearch;

	/** @var we_database_base */
	protected $db;

	private const searchCT = [
		we_base_ContentTypes::WEDOCUMENT,
		we_base_ContentTypes::TEMPLATE,
		we_base_ContentTypes::DOCTYPE,
		we_base_ContentTypes::CATEGORIES,
		we_base_ContentTypes::CATEGORY,
		we_base_ContentTypes::OBJECT,
		we_base_ContentTypes::OBJECT_FILE,
		we_base_ContentTypes::IMAGE
	];

	function __construct(){
		parent::__construct();
		$this->PatternSearch = new we_exim_searchPatterns();
		$this->db = new DB_WE();
	}

	private function getDocumentIncludes(string $text, int $level): void{
		$match = [];

		foreach($this->PatternSearch->doc_patterns['id'] as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $i => $include){
					if(!str_contains(strtolower($match[0][$i]), 'type="template"')){
						$this->addToDepArray($level, (int) $include);
					}
				}
			}
		}

		foreach($this->PatternSearch->doc_patterns['path'] as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $path){
					$include = path_to_id($path, FILE_TABLE);
					$this->addToDepArray($level, $include);
				}
			}
		}
	}

	private function getObjectIncludes(string $text, int $level): void{
		$match = [];

		foreach($this->PatternSearch->obj_patterns['id'] as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $include){
					$this->addToDepArray($level, (int) $include, we_base_ContentTypes::OBJECT_FILE, OBJECT_FILES_TABLE);
				}
			}
		}

		foreach($this->PatternSearch->class_patterns as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $include){
					$this->addToDepArray($level, (int) $include, we_base_ContentTypes::OBJECT, OBJECT_TABLE);
				}
			}
		}
	}

	private function getExternalLinked(string $text, int $level): void{
		$match = [];
		foreach($this->PatternSearch->ext_patterns as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $external){
					$path = $this->isPathLocal($external);
					if($path && $path != '/'){
						$id = path_to_id($path, FILE_TABLE);
						$this->addToDepArray($level, $id, ($id ? '' : we_base_ContentTypes::BACKUP_BINARY));
					}
				}
			}
		}
	}

	private function getNavigation(string $text, int $level): void{
		$navrules = $match = [];
		foreach($this->PatternSearch->navigation_patterns as $pattern){
			if(!preg_match_all($pattern, $text, $match)){
				continue;
			}
			foreach($match[2] as $value){
				if(!is_numeric($value)){
					continue;
				}
				$path = ($value ? id2path((int) $value, NAVIGATION_TABLE) : '');

				$this->addToDepArray($level, (int) $value, we_base_ContentTypes::NAVIGATION, NAVIGATION_TABLE);
				$navrules[] = $value;

				$ids = $this->db->getAllq('SELECT ID FROM ' . NAVIGATION_TABLE . ' WHERE Path LIKE "' . $this->db->escape($path) . '/%"', true);
				foreach($ids as $id){
					$this->addToDepArray($level, (int) $id, we_base_ContentTypes::NAVIGATION, NAVIGATION_TABLE);
					$navrules[] = $id;
				}
			}
		}
		if($navrules){
			$this->getNavigationRule($navrules, $level);
		}
	}

	private function getNavigationRule(array $naviid, int $level): void{
		$ids = $this->db->getAllq('SELECT ID FROM ' . NAVIGATION_RULE_TABLE . ' WHERE NavigationID IN (' . implode(',', $naviid) . ')');
		foreach($ids as $id){
			$this->addToDepArray($level, (int) $id, we_base_ContentTypes::NAVIGATIONRULE, NAVIGATION_RULE_TABLE);
		}
	}

	private function getThumbnail(string $text, int $level): void{
		$match = $values = [];
		foreach($this->PatternSearch->thumbnail_patterns as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $value){
					$values[] = $this->db->escape($value);
				}
			}
		}
		if($values){
			$ids = $this->db->getAllq('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE Text IN ("' . (implode('","', $values)) . '")', true);
			foreach($ids as $id){
				$this->addToDepArray($level, (int) $id, we_base_ContentTypes::THUMBNAIL, THUMBNAILS_TABLE);
			}
		}
		$values = [];
		foreach($this->PatternSearch->thumbnailid_patterns as $pattern){
			if(preg_match_all($pattern, $text, $match)){
				foreach($match[2] as $value){
					$values[] = $this->db->escape($value);
				}
			}
		}
		if($values){
			$ids = $this->db->getAllq('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE ID IN (' . (implode(',', $values)) . ')');
			foreach($ids as $id){
				$this->addToDepArray($level, (int) $id, we_base_ContentTypes::THUMBNAIL, THUMBNAILS_TABLE);
			}
		}
	}

	private function getIncludesFromWysiwyg(string $text, int $level): void{
		$match = [];
		/*
		  if(is_array($text)){ // shop exception - handle array in the content
		  foreach($text as $item1){
		  if(!is_array($item1)){
		  continue;
		  }
		  foreach($item1 as $item2){
		  if(!is_array($item2)){
		  continue;
		  }
		  foreach($item2 as $item3){
		  if(is_array($item3) && in_array(we_contents_base::ETYPE_NUMBER, array_keys($item3)) && !empty($item3[we_contents_base::ETYPE_NUMBER])){
		  $this->addToDepArray($level, $item3[we_contents_base::ETYPE_NUMBER]);
		  }
		  }
		  }
		  }
		  return;
		  } */
		foreach($this->PatternSearch->wysiwyg_patterns as $patterns){
			foreach($patterns as $pattern){
				if(preg_match_all($pattern, $text, $match)){
					foreach($match[2] as $k => $include){
						$isobj = (str_contains($match[1][$k], we_base_link::TYPE_OBJ_PREFIX));
						$this->addToDepArray($level, (int) $include, ($isobj ? we_base_ContentTypes::OBJECT_FILE : ''), ($isobj ? OBJECT_FILES_TABLE : ''));
					}
				}
			}
		}
	}

	private function isPathLocal(string $path): string{
		if(str_contains(strtolower($path), strtolower($_SERVER['SERVER_NAME']))){
			$path = str_replace([we_http_util::getServerUrl(), we_http_util::getServerUrl(true)], '', $path);
			//try again with password
		}
		return (is_readable(WE_DOCUMENT_ROOT . $path) ? $path : '');
	}

	private function addToDepArray(int $level, int $id, string $ct = '', string $table = ''): void{
		if(!$ct){
			$table = $table ?: FILE_TABLE;
			$ct = fS('SELECT ContentType FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		}

		if($ct){
			$new = [
				'ID' => $id,
				'ContentType' => $ct,
				'level' => $level
			];
			if(!$this->RefTable->exists($new)){
				$this->RefTable->add(null, $new);
			}
		}
	}

	private function getDepFromArray(array $array): array{
		$ret = ['docs' => [], 'objs' => []];

		if(!empty($array['id'])){
			$ret['docs'][] = $array['id'];
		}

		if(!empty($array['img_id'])){
			$ret['docs'][] = $array['img_id'];
		}

		if(!empty($array['obj_id'])){
			$ret['objs'][] = $array['obj_id'];
		} else {
			foreach($array as $value){
				if($value && is_array($value)){
					$ret = array_merge_recursive($ret, $this->getDepFromArray($value));
				}
			}
		}
		return $ret;
	}

	private function getDependent(we_base_mainmodel $object, int $level): void{
		$tag = constant(get_class($object) . '::BACKUP_XML_TAG');
		$hasObj = we_modules_info::isActive(we_modules_info::OBJECT);
		assert($object instanceof we_contents_base);
		if(isset($object->Table) && ($this->options['handle_document_includes'] ||
			$this->options['handle_document_linked']) && !empty($object->elements)){
			foreach($object->elements as $ek => $ev){

				if($this->options['handle_document_linked']){

					if(str_contains($ek, 'longdescid')){
						if(($val = $object->getElementI($ek))){
							$this->addToDepArray($level, $val);
						}
					} else if($hasObj && str_contains($ek, 'ObjID')){
						if(($val = $object->getElementI($ek))){
							$this->addToDepArray($level, $val, we_base_ContentTypes::OBJECT_FILE);
						}
					}

					if(isset($ev['dat'])){
						$dat = we_unserialize((string) $ev[we_contents_base::ETYPE_TEXT], [], true);
						if(!is_array($dat) && $this->options['handle_document_linked']){
							$this->getExternalLinked((string) $ev[we_contents_base::ETYPE_TEXT], $level);
						}
					}
				}

				if($this->options['handle_document_includes']){
					if(str_contains($ek, 'intID')){
						if(isset($ev[we_contents_base::ETYPE_NUMBER])){
							$this->addToDepArray($level, (int) $ev[we_contents_base::ETYPE_NUMBER]);
						}
					} else if(isset($ev[we_contents_base::ETYPE_TEXT])){
						$dat = we_unserialize($ev[we_contents_base::ETYPE_TEXT], [], true);
						if(is_array($dat)){
							$elarray = $this->getDepFromArray($dat);
							foreach($elarray as $elk => $elv){
								foreach($elv as $id){
									if($id){
										$this->addToDepArray($level, $id, ($elk === 'docs' ? '' : we_base_ContentTypes::OBJECT_FILE));
									}
								}
							}
						} else {
							$this->getIncludesFromWysiwyg($ev[we_contents_base::ETYPE_TEXT], $level);
						}
					}
					if(!empty($ev[we_contents_base::ETYPE_NUMBER])){
						$this->addToDepArray($level, $ev[we_contents_base::ETYPE_NUMBER]);
					}
				}
			}
		}

		if($tag === we_contents_template::BACKUP_XML_TAG){//FIXME: what about recursive template includes, ...
			assert($object instanceof we_contents_template);
			$data = $object->getDocumentCode();
			/* FIXME: use tp parser and use e.g. new syntax for $newTmplPattern - and move code to specific class!
			  $tp = new we_weTag_parser($data);
			  $allTags = $tp->getTagsGroupedByTag();
			 */
			$match = [];
			if($this->options['handle_def_templates']){
				if($object->MasterTemplateID){
					$this->addToDepArray($level, $object->MasterTemplateID, we_base_ContentTypes::TEMPLATE, TEMPLATES_TABLE);
				}
				foreach($this->PatternSearch->tmpl_patterns as $include_pattern){
					if(preg_match_all($include_pattern, $data, $match)){
						foreach($match[2] as $value){
							$this->addToDepArray($level, (int) $value, we_base_ContentTypes::TEMPLATE, TEMPLATES_TABLE);
						}
					}
				}
			}
			if($this->options['handle_document_includes']){
				$this->getDocumentIncludes($data, $level);
			}
			if($this->options['handle_object_includes'] && $hasObj){
				$this->getObjectIncludes($data, $level);
			}
			if($this->options['handle_document_linked']){
				$this->getExternalLinked($data, $level);
			}
			if($this->options['handle_navigation']){
				$this->getNavigation($data, $level);
			}
			if($this->options['handle_thumbnails']){
				$this->getThumbnail($data, $level);
			}
		}

		if($object instanceof we_document_webEdition && $object->TemplateID && $this->options['handle_def_templates']){
			$this->addToDepArray($level, $object->TemplateID, we_base_ContentTypes::TEMPLATE, TEMPLATES_TABLE);
		}

		if($object instanceof we_contents_objectFile && $this->options['handle_def_classes']){
			$this->addToDepArray($level, $object->TableID, we_base_ContentTypes::OBJECT, OBJECT_TABLE);
		}

		if($object instanceof we_document_webEdition && $object->DocType && $tag != we_doctype_docTypes::BACKUP_XML_TAG && $this->options['handle_doctypes']){
			$this->addToDepArray($level, $object->DocType, we_base_ContentTypes::DOCTYPE, DOC_TYPES_TABLE);
		}

		if(!empty($object->Category) && $tag != we_category_category::BACKUP_XML_TAG && $this->options['handle_categorys']){
			foreach($object->Category as $cat){
				//FIXME: is this the correct contenttype?!
				$this->addToDepArray($level, (int) $cat, we_base_ContentTypes::CATEGORY, CATEGORY_TABLE);
			}
		}

		if($this->options['handle_object_embeds'] && we_modules_info::isActive(we_modules_info::OBJECT) && $tag === we_contents_objectFile::BACKUP_XML_TAG){
			foreach($object->elements as $key => $value){
				if(preg_match('|' . we_listview_base::OBJECT_FIELD_PREFIX . '[0-9]+|', $key) && isset($value[we_contents_base::ETYPE_TEXT])){
					$this->addToDepArray($level, $value[we_contents_base::ETYPE_TEXT], we_base_ContentTypes::OBJECT_FILE, OBJECT_FILES_TABLE);
				}
				if(preg_match('|' . we_listview_base::OBJECT_FIELD_PREFIX . '[0-9]+_path|', $key) && isset($value[we_contents_base::ETYPE_TEXT])){
					$this->addToDepArray($level, path_to_id($value[we_contents_base::ETYPE_TEXT], OBJECT_FILES_TABLE), we_base_ContentTypes::OBJECT_FILE, OBJECT_FILES_TABLE);
				}
				switch(($value['type'] ?? '')){
					case 'img':
					case 'binary':
						$this->addToDepArray($level, $value[we_contents_base::ETYPE_NUMBER] ?? $value[we_contents_base::ETYPE_TEXT]);
				}
			}
		}
	}

	private function makeExportList(): void{
		$step = 0;
		while(($id = $this->RefTable->getNext())){
			$search = in_array($id->ContentType, self::searchCT, true);
			if($search || $this->options['handle_owners']){
				$doc = we_exim_contentProvider::getInstance($id->ContentType, $id->ID);

				if($search && $this->options['export_depth'] > $id->level){
					if(!isset($this->analyzed[$id->ContentType])){
						$this->analyzed[$id->ContentType] = [];
					}
					if(!in_array($id->ID, $this->analyzed[$id->ContentType], true)){
						$l = (int) $id->level + 1;
						$this->getDependent($doc, $l);
						$this->analyzed[$id->ContentType][] = $id->ID;
					}
				}
				// collect owners info
				if($this->options['handle_owners']){
					$uids = [];
					if($doc instanceof we_contents_root && $doc->CreatorID && !in_array($doc->CreatorID, $this->RefTable->Users)){
						$uids = [(int) $doc->CreatorID];
					}
					if($doc->Owners){
						$uids = array_merge($uids, $doc->Owners);
					}
					if(!empty($uids)){
						$this->RefTable->addToUsers($uids);
					}
				}
			}
			$step++;
			if(10 < $step){ //FIXME: removed BACKUP_STEPS, should be handled equal to backup
				break;
			}
		}
	}

	public function prepareExport(array $ids = []): void{
		//we_updater::fixInconsistentTables(); // FIXME: this call kills all elements!

		if($this->options['handle_def_templates'] ||
			$this->options['handle_doctypes'] ||
			$this->options['handle_categorys'] ||
			$this->options['handle_def_classes'] ||
			$this->options['handle_document_includes'] ||
			$this->options['handle_document_linked'] ||
			$this->options['handle_object_includes'] ||
			$this->options['handle_object_embeds'] ||
			$this->options['handle_class_defs'] ||
			$this->options['handle_owners'] ||
			$this->options['handle_navigation'] ||
			$this->options['handle_thumbnails']
		){
			$this->makeExportList();
		}

		// move objects to the end of the reftable because objects should be imported after classes
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->RefTable->moveItemsToEnd('objectFile');
		}
	}

	public function loadPreserves(): void{
		parent::loadPreserves();
		if(isset($_SESSION['weS']['ExImOptions'])){
			$this->options = $_SESSION['weS']['ExImOptions'];
		}
	}

	public function savePreserves(): void{
		parent::savePreserves();
		$_SESSION['weS']['ExImOptions'] = $this->options;
	}

	public static function unsetPreserves(): void{
		parent::unsetPreserves();
		if(isset($_SESSION['weS']['ExImOptions'])){
			unset($_SESSION['weS']['ExImOptions']);
		}
	}

}
