<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: cf15d77ce62c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-03-04 18:50 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_fileupload{

	const USERINPUT_TYPE_IMG = 'WE_UI_IMG_DATA_ID_';
	const USERINPUT_TYPE_BINARY = 'WE_UI_BINARY_DATA_ID_';
	const USERINPUT_TYPE_VIDEO = 'WE_UI_VIDEO_DATA_ID_';
	const USERINPUT_TYPE_AUDIO = 'WE_UI_AUDIO_DATA_ID_';
	const USERINPUT_TYPE_DEL = 'WE_UI_DEL_CHECKBOX_';

	/** @var int */
	private static $maxUploadSizeMB = -1;

	/** @var string */
	protected $rpcUrl = WEBEDITION_DIR . 'rpc.php?cmd=uploadProcess';

	/** @var string */
	protected $name = 'weFileSelect';

	/** @var int */
	protected $maxUploadSizeBytes = 0;
	//protected $maxChunkCount = 0;

	/** @var string */
	protected $genericFileNameTemp = '';

	/** @var bool */
	protected $doCommitFile = true;

	/** @var bool */
	protected $isDragAndDrop = true;

	/** @var string[] */
	protected $noPermsCT = [];

	/** @var array<string,array> */
	protected $typeCondition = [
		'accepted' => [
			'cts' => '',
			'exts4cts' => '',
			'exts' => '',
			'all' => '',
		],
		'forbidden' => [
			'cts' => '',
			'exts4cts' => '',
			'exts' => '',
			'all' => '',
			'allForbiddenByUploader' => ''
		],
	];

	/** @var array<string,array> */
	protected $typeConditionArrs = [
		'accepted' => [
			'cts' => [],
			'exts4cts' => [],
			'exts' => [],
			'all' => [],
		],
		'forbidden' => [
			'cts' => [],
			'exts4cts' => [],
			'exts' => [],
			'all' => [],
		]
	];

	/** @var string[] */
	protected $editables = ['.jfif', '.jpg', '.png', '.gif', '.webp']; // TODO: use this in js too

	/** @var string[] */
	protected $metaImportables = ['.jfif', '.jpg', '.png', '.webp', /* , '.pdf' */];

	/** @var array<string,array> */
	/* obsolete?
	  protected $permToUpload = [
	  'images' => false,
	  'editables' => false,
	  'metaImportables' => false
	  ];
	 *
	 */

	// partly obsolete

	/** @var array<string,bool|int|string|array> */
	protected $imageEditProps = [
		'parentID' => 0,
		'sameName' => we_exim_ExIm::COLLISION_RENAME,
		'importMetadata' => false,
		'isSearchable' => false,
		'parseFile' => true,
		'thumbnails' => [],
		'categories' => [],
		'importDirs' => true,
	];

	/** @var array<string,bool> */
	protected $perm = [];

	protected const CHUNK_SIZE = 128;
	const REPLACE_BY_UNIQUEID = '##REPLACE_BY_UNIQUEID##';
	const REPLACE_BY_FILENAME = '##REPLACE_BY_FILENAME##';

	protected function __construct(string $name){
		$this->name = $name;
		$this->isDragAndDrop = !(we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 11) || we_base_browserDetect::isOpera();
		$this->maxUploadSizeBytes = (self::getMaxUploadSizeMB()) * we_base_util::MBYTE;
		//$this->maxChunkCount = (self::getMaxUploadSizeMB()) * 1024 / self::CHUNK_SIZE;
	}

	public function getCss(): string{
		return '';
	}

	public function getJs(): string{
		return '';
	}

	public function setIsDragAndDrop(bool $isDragAndDrop = true): void{
		$this->isDragAndDrop = $isDragAndDrop;
	}

	public static function isDragAndDrop(): bool{
		return true; //!((we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 11) || we_base_browserDetect::isOpera());
	}

	public static function getMaxUploadSizeMB(): int{
		if(self::$maxUploadSizeMB === -1){
			self::$maxUploadSizeMB = (int) (defined('FILE_UPLOAD_MAX_UPLOAD_SIZE') && !we_base_permission::hasPerm([we_base_perm::UPLOAD_DISABLE_SIZE_LIMIT]) ? FILE_UPLOAD_MAX_UPLOAD_SIZE : 0);
		}

		return self::$maxUploadSizeMB;
	}

	public static function getMaxUploadSizeB(): int{
		return self::getMaxUploadSizeMB() * 1024 * 1024;
	}

	/**
	 *
	 * @return string[]
	 */
	protected function getForbiddenCtsFromPerms(): array{
		return array_filter([
			(we_base_permission::hasPerm([we_base_perm::WORK_VIDEO_NEW]) ? '' : we_base_ContentTypes::VIDEO),
			(we_base_permission::hasPerm([we_base_perm::WORK_VIDEO_NEW]) ? '' : we_base_ContentTypes::AUDIO),
			(we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW]) ? '' : we_base_ContentTypes::IMAGE),
			(we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW]) ? '' : we_base_ContentTypes::PDF),
			(we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW]) ? '' : we_base_ContentTypes::APPLICATION),
			(we_base_permission::hasPerm([we_base_perm::WORK_JS_NEW]) ? '' : we_base_ContentTypes::JS),
			(we_base_permission::hasPerm([we_base_perm::WORK_HTACCESS_NEW]) ? '' : we_base_ContentTypes::HTACCESS),
			(we_base_permission::hasPerm([we_base_perm::WORK_TEXT_NEW]) ? '' : we_base_ContentTypes::XML),
			(we_base_permission::hasPerm([we_base_perm::WORK_TEXT_NEW]) ? '' : we_base_ContentTypes::TEXT),
			(we_base_permission::hasPerm([we_base_perm::WORK_CSS_NEW]) ? '' : we_base_ContentTypes::CSS),
			(we_base_permission::hasPerm([we_base_perm::WORK_HTML_NEW]) ? '' : we_base_ContentTypes::HTML),
		]);
	}

	/**
	 *
	 * @param string $field
	 * @param string[] $contentTypes
	 * @param string[] $extensions
	 * @return void
	 */
	protected function setTypeCondition(string $field = 'accepted', array $contentTypes = [], array $extensions = []): void{
		$allForbiddenByUploader = '';

		if($field != 'accepted'){
			$cts = $extsByCts = //nur we cts
					$newCts = [];
			foreach($contentTypes as $ct){
				$ct = strtolower($ct);
				if(in_array($ct, we_base_ContentTypes::getContentTypes(FILE_TABLE, true), true)){
					$extsByCts = array_unique(array_merge($extsByCts, we_base_ContentTypes::getExtension($ct)));
					$newCts = we_base_ContentTypes::getRealContentTypes($ct);
					$cts = array_unique(empty($newCts) ? array_merge($cts, [$ct]) : array_merge($cts, $newCts, [$ct]));
				}
			}
			$allForbiddenByUploader = implode(',', array_filter(array_unique(array_merge($cts, $extensions, $extsByCts))));
		}

		$contentTypes = $field !== 'accepted' ? array_merge($contentTypes, $this->noPermsCT) : $contentTypes;
		$cts = $extsByCts = $newCts = [];
		foreach($contentTypes as $ct){
			$ct = strtolower($ct);
			if(in_array($ct, we_base_ContentTypes::getContentTypes(FILE_TABLE, true), true)){
				$extsByCts = array_unique(array_merge($extsByCts, we_base_ContentTypes::getExtension($ct)));
				$newCts = we_base_ContentTypes::getRealContentTypes($ct);
				$cts = array_unique(empty($newCts) ? array_merge($cts, [$ct]) : array_merge($cts, $newCts, [$ct]));
			}
		}

		$this->typeConditionArrs[$field] = [
			'cts' => $cts,
			'exts' => $extensions,
			'exts4cts' => $extsByCts,
			'all' => array_filter(array_unique(array_merge($cts, $extensions, $extsByCts)))
		];

		$typeCondition = [
			'cts' => $cts ? ',' . implode(',', $cts) . ',' : '',
			'exts4cts' => $extsByCts ? ',' . implode(',', $extsByCts) . ',' : '',
			'exts' => $extensions ? ',' . strtolower(implode(',', array_unique($extensions))) . ',' : ''
		];
		$typeCondition['all'] = str_replace(',,', ',', $typeCondition['cts'] . $typeCondition['exts4cts'] . $typeCondition['exts']);

		if($field !== 'accepted'){
			$typeCondition['allForbiddenByUploader'] = $allForbiddenByUploader;
		}

		$this->typeCondition[$field] = $typeCondition;
	}

	protected function setUploadPerms(): void{
		$this->setUploadPerm('editables');
		$this->setUploadPerm('images');
		$this->setUploadPerm('metaImportables');
		$this->setUploadPerm('parsables');
	}

	protected function setUploadPerm(string $what): void{
		$permittedExts = [];
		switch($what){
			case 'editables':
				$permittedExts = $this->editables;
				break;
			case 'images':
				$permittedExts = we_base_ContentTypes::getExtension(we_base_ContentTypes::IMAGE);
				break;
			case 'metaImportables':
				$permittedExts = $this->metaImportables;
				break;
			case 'parsables':
				$permittedExts = we_base_ContentTypes::getExtension(we_base_ContentTypes::CSS);
				break;
		}

		if(empty($this->typeConditionArrs['accepted']['all'])){
			$intersect = array_intersect($this->typeConditionArrs['forbidden']['all'], $permittedExts);
			$this->perm[$what] = empty($this->typeConditionArrs['forbidden']['all']) || !empty(array_diff($permittedExts, $intersect));
			return;
		}

		$diff = array_diff($this->typeConditionArrs['accepted']['all'], $this->typeConditionArrs['forbidden']['all']);
		$this->perm[$what] = !empty(array_intersect($diff, $permittedExts));
		return;
	}

	protected function isPermToUpload(string $what): bool{
		return $this->perm[$what];
	}

	public function loadImageEditPropsFromSession(): void{
		$this->imageEditProps = $_SESSION['weS']['we_fileupload']['imageEditProps'] ?? $this->imageEditProps;
		if(!$this->imageEditProps['parentID']){
			$this->imageEditProps['parentID'] = we_users_util::get_def_ws(FILE_TABLE);
		}
		//FIXME: should we use latest selector value??
	}

	public function saveImageEditPropsInSession(): void{
		$_SESSION['weS']['we_fileupload']['imageEditProps'] = $this->imageEditProps;
		$_SESSION['weS']['we_fs_lastDir'][FILE_TABLE] = $this->imageEditProps['parentID'];
	}

}
