<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f5c46a8ebfcc $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-02-16 23:50 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_resp_base extends we_fileupload{
	/** @var string */
	protected $name = 'we_File';

	/** @var bool */
	protected $importFile = false;

	/** @var array<string,string> */
	protected $response = [
		'status' => '',
		'fileNameTemp' => '',
		'mimePhp' => 'none',
		'message' => '',
		'completed' => '',
		'weDoc' => ''
	];

	/** @var array{partNum:int,partCount:int,formnum:int,formcount:int,doCommitFile:bool} */
	protected $controlVars = [
		'partNum' => 1,
		'partCount' => 1,
		'partSize' => 0,
		'formnum' => 0,
		'formcount' => 0,
		'doCommitFile' => true,
	];

	/** @var array{genericFileNameTemp:string,fileTemp:string,fileDef:string,parentID:int,parentDir:string,weFileName:string,weFileSize:int,weFileCt:string,sameName:string,innerPath:string} */
	protected $fileVars = [
		'genericFileNameTemp' => '',
		'fileTemp' => '',
		'fileDef' => '',
		'parentID' => 0,
		'parentDir' => '',
		'weFileName' => '',
		'weFilePath' => '',
		'weFileSize' => 1,
		'weFileCt' => '',
		'sameName' => we_exim_ExIm::COLLISION_RENAME,
		'innerPath' => '',
	];

	/** @var array{transaction:string,importMetadata:bool,categories:int[],isSearchable:bool,parseFile:bool,importDirs:bool,title:string,alt:string,thumbs:int[],focusX:int,focusY:int} */
	protected $docVars = [
		'transaction' => '',
		'importMetadata' => false,
		'categories' => [],
		'isSearchable' => true,
		'importDirs' => false,
		'parseFile' => true,
		'title' => '',
		'alt' => '',
		'thumbs' => [],
		'focusX' => 0,
		'focusY' => 0
	];

	/** @var array */
	protected $FILES = [];

	/* const GET_PATH_ONLY = 1;
	  const GET_NAME_ONLY = 2;
	  const FORCE_DOC_ROOT = true;
	  const MISSING_DOC_ROOT = true;
	 */

	public function __construct(string $name = ''){
		$this->name = $name ?: $this->name;
		$this->initByHttp();
	}

	protected function initByHttp(): void{
		$this->FILES = $_FILES;
		$this->fileVars = [
			'genericFileNameTemp' => we_base_request::string('genericFilename', $this->fileVars['genericFileNameTemp']),
			'fileTemp' => we_base_request::string('weFileNameTemp', $this->fileVars['fileTemp']),
			'fileDef' => $this->fileVars['fileDef'],
			'parentID' => we_base_request::int('fu_file_parentID', $this->fileVars['parentID']),
			'parentDir' => we_base_request::_(we_base_request::EXTFILE, 'fu_file_parentDir', $this->fileVars['parentDir']), // FIXME: set back to URL when base_request can handle not fully qualifieds
			'weFileName' => we_base_request::_(we_base_request::FILENAME, 'weFileName', $this->fileVars['weFileName']),
			'weFilePath' => we_base_request::_(we_base_request::FILE, 'weFilePath', $this->fileVars['weFilePath']),
			'weFileSize' => we_base_request::int('weFileSize', $this->fileVars['weFileSize']),
			'weFileCt' => we_base_request::string('weFileCt', $this->fileVars['weFileCt']),
			'sameName' => we_base_request::string('fu_file_sameName', $this->fileVars['sameName']),
			'innerPath' => '',
		];
		$this->controlVars = [
			'doCommitFile' => we_base_request::bool('doCommitFile', true),
			'partNum' => we_base_request::int('wePartNum', $this->controlVars['partNum']),
			'partCount' => we_base_request::int('wePartCount', $this->controlVars['partCount']),
			'partSize' => we_base_request::int('wePartSize', $this->controlVars['partSize']),
			'formnum' => we_base_request::int('weFormNum', $this->controlVars['formnum']),
			'formcount' => we_base_request::int('weFormCount', $this->controlVars['formcount']),
		];
	}

	public function processRequest(): array{
		if(!(isset($this->FILES[$this->name]) && strlen((string) $this->FILES[$this->name]['tmp_name']))){
			return array_merge($this->response, ['status' => 'failure', 'message' => g_l('importFiles', '[php_error]'), 'filename' => $this->fileVars['weFileName']]);
		}

		$this->fileVars['weFileCt'] = we_base_ContentTypes::getFromFilename((string) $this->FILES[$this->name]['name']); //compare mime and ct by extension
		if($this->importFile && !we_base_permission::hasPerm(($perm = we_base_ContentTypes::getPermission($this->fileVars['weFileCt'])))){
			return array_merge($this->response, ['status' => 'failure', 'message' => 'no perms: ' . g_l('perms_workpermissions', '[' . $perm[0] . ']'), 'filename' => $this->fileVars['weFileName']]);
		}

		if($this->controlVars['partNum'] > 1 && !$this->fileVars['fileTemp']){
			return array_merge($this->response, ['status' => 'failure', 'message' => 'inconsistent_params', 'filename' => $this->fileVars['weFileName']]);
		}

		if($this->controlVars['partSize'] != $this->FILES[$this->name]['size']){
			t_e('retransmit, size not matching!', $this->controlVars['partSize'], $this->FILES[$this->name]['size']);
			return array_merge($this->response, ['status' => 'retransmit', 'fileNameTemp' => $this->fileVars['fileTemp']]);
		}

		$chunkName = $this->controlVars['partNum'] === 1 ? $this->makeFileTemp() : TEMP_DIR . we_base_file::getUniqueId();
		$chunkFile = WE_DOCUMENT_ROOT . $chunkName;
		if(!@move_uploaded_file((string) $this->FILES[$this->name]['tmp_name'], $chunkFile)){
			return array_merge($this->response, ['status' => 'failure', 'message' => 'move_file_error', 'filename' => $this->fileVars['weFileName']]);
		}
		we_base_file::insertIntoCleanUp($chunkFile, 3600);

		// if not first chunk append it to tempFile, else tmpFile = chunkFile
		if($this->controlVars['partNum'] === 1){
			$this->fileVars['fileTemp'] = $chunkName;
		} else {
			file_put_contents(WE_DOCUMENT_ROOT . $this->fileVars['fileTemp'], file_get_contents($chunkFile), FILE_APPEND);
			we_base_file::deleteLocalFile($chunkFile);
		}

		if(($maxSize = self::getMaxUploadSizeB()) && filesize(WE_DOCUMENT_ROOT . $this->fileVars['fileTemp']) > $maxSize){
			return array_merge($this->response, ['status' => 'failure', 'message' => 'errorFileSize', 'filename' => $this->fileVars['weFileName']]);
		}

		// everything works fine but upload not finished yet: get next chunk
		if($this->controlVars['partCount'] !== $this->controlVars['partNum'] && $this->controlVars['partCount'] !== 1){
			return array_merge($this->response, ['status' => 'continue', 'fileNameTemp' => $this->fileVars['fileTemp']]);
		}

		// last chunk appended: start post process and get some response to send
		return $this->postProcess();
	}

	protected function postprocess(): array{
		if(!$this->controlVars['doCommitFile']){ // most simple variant: we just leave tempFile in tmp directory and inform GUI that it can go on and grab it
			return array_merge($this->response, ['status' => 'success', 'fileNameTemp' => $this->fileVars['fileTemp'], 'completed' => 1]);
		}

		if(!$this->checkSetFile()){
			return $this->response;
		}

		if(!copy(WE_DOCUMENT_ROOT . $this->fileVars['fileTemp'], WE_DOCUMENT_ROOT . strtr($this->fileVars['fileDef'], ['\\' => '/', '//' => '/']))){
			return $this->response = array_merge($this->response, ['status' => 'failure', 'message' => 'error_copy_file', 'completed' => 1, 'filename' => $this->fileVars['weFileName']]);
		}

		return array_merge($this->response, ['status' => 'success', 'fileNameTemp' => $this->fileVars['fileTemp'], 'completed' => 1, 'weDoc' => ['id' => 0, 'path' => $this->fileVars['fileDef'],
				'text' => $this->fileVars['weFileName']], 'commited' => 1]);
	}

	public function commitUploadedFile(bool $absolute = false): string{
		$file = WE_DOCUMENT_ROOT . $this->fileVars['fileTemp'];
		if(!file_exists($file) || !is_readable($file)){
			throw new we_exception_file('no valid file uploaded');
		}
		/* if(!$this->checkFileType($file)){
		  throw new we_exception_file('filetype not ok');
		  } */

		return $absolute ? $file : $this->fileVars['fileTemp'];
	}

	protected function checkSetFile(): bool{
		if($this->fileVars['parentDir'] && in_array(intval($this->fileVars['parentID']), [-1, 0])){ // upload without importing to WE
			$path = $this->fileVars['parentDir'];
		} else {
			// check WS
			$path = $this->fileVars['parentID'] ? id2path($this->fileVars['parentID']) : '';
		}
		$path = rtrim($path, '/') . '/';

		if(file_exists(WE_DOCUMENT_ROOT . $path . $this->fileVars['weFileName'])){
			switch($this->fileVars['sameName']){
				case we_exim_ExIm::COLLISION_OVERWRITE:
					if(path_to_id($path . $this->fileVars['weFileName'])){
						$this->response = array_merge($this->response, ['status' => 'failure', 'message' => g_l('fileselector', '[can_not_overwrite_we_file]'), 'completed' => 1, 'filename' => $this->fileVars['weFileName']]);
						return false;
					}
					break;
				case we_exim_ExIm::COLLISION_RENAME:
					$z = 0;
					$regs = [];
					if(preg_match('|^(.+)(\.[^\.]+)$|', $this->fileVars['weFileName'], $regs)){
						$ext = $regs[2];
						$name = $regs[1];
					} else {
						$ext = '';
						$name = $this->fileVars['weFileName'];
					}
					$tmp = $name . '_' . $z . $ext;
					while(file_exists(WE_DOCUMENT_ROOT . $path . $tmp)){
						$tmp = $name . "_" . ++$z . $ext;
					}
					$this->fileVars['weFileName'] = $tmp;
					break;
				default:
					$this->response = array_merge($this->response, ['status' => 'failure', 'message' => g_l('importFiles', '[same_name]'), 'completed' => 1, 'filename' => $this->fileVars['weFileName']]);
					return false;
			}
		}
		$this->fileVars['fileDef'] = $path . "/" . $this->fileVars['weFileName'];

		return true;
	}

	protected function checkSetParentId(): void{

	}

	protected function makeFileTemp(): string{
		return $this->fileVars['genericFileNameTemp'] ? strtr($this->fileVars['genericFileNameTemp'], [self::REPLACE_BY_UNIQUEID => we_base_file::getUniqueId(), self::REPLACE_BY_FILENAME => $this->fileVars['weFileName']]) : TEMP_DIR . we_base_file::getUniqueId();
	}

	/* private function checkFileType(string $file = ''): bool{
	  return true; // FIXME: make same test as in JS
	  } */
}
