<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a202423df34b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-03-22 21:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_ui_editor extends we_fileupload_ui_base{

	protected $isExternalBtnUpload = true;
	protected $parentID = [
		'setField' => false,
		'presetId' => IMAGESTARTID_DEFAULT,
		'presetDir' => '',
		'setFixed' => false,
	];

	public function __construct(string $name, array $acceptedCT = [], array $acceptedExt = [], array $forbiddenCT = [], array $forbiddenExt = [], bool $doImport = true){
		$this->doImport = $doImport;
		if($this->doImport){
			$this->noPermsCT = $this->getForbiddenCtsFromPerms();
		}

		parent::__construct('we_File', $acceptedCT, $acceptedExt, $forbiddenCT, $forbiddenExt);

		$this->type = 'editor';
		$this->doCommitFile = true;

		$this->controls = [
			'showControls' => self::CONTROLS_SHOW_TRUE,
			'showControlsInitialState' => self::CONTROLS_SHOW_TRUE,
			'showBtnsUploadCancel' => self::CONTROLS_SHOW_FALSE,
			'showEditOptions' => $this->isPermToUpload('editables') ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_FALSE,
			'showImportMeta' => self::CONTROLS_SHOW_FALSE
		];

		$this->doImport = $doImport;
		$this->responseClass = $this->doImport ? 'we_fileupload_resp_import' : $this->responseClass = 'we_fileupload_resp_base';

		$this->loadImageEditPropsFromSession();
	}

	public function setFieldParentID(array $parentID = []): void{
		$this->parentID = array_merge($this->parentID, $parentID);
	}

	// move to fileupload.scss and dete
	public function getCss(): string{
		return parent::getCss() . we_html_element::cssElement('
			.paddingTop{
				padding-top: 12px;
			}
		');
	}

	public function getHtml(bool $returnRows = false): string{
		$formUploader = we_html_element::div(['id' => 'imageUpload', 'class' => 'fileuploadEditor', 'style' => 'display:block;width:384px;'], $this->getHiddens() .
						we_html_element::div(['style' => 'width:542px'], we_html_element::div(['id' => 'div_fileupload_right', 'style' => "position:relative;"], $this->getFormUploader($this->getDropZoneContent())
								)
						)
		);

		$moreHiddens = [];

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', $formUploader))->rightHead())
				->add($this->getFormParentID())
				->add($this->getFormSameName());

		if($this->doImport){
			$parts->add($this->getFormOptions(true, $this->isPermToUpload('parsables'), $this->isPermToUpload('metaImportables')));

			if(!$this->isPermToUpload('parsables')){
				$moreHiddens['fu_doc_parseFile'] = '';
			}
			if(!$this->isPermToUpload('metaImportables')){
				$moreHiddens['fu_doc_importMetadata'] = '';
			}

			if(we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT]) && we_modules_info::isActive(we_modules_info::CATEGORY)){
				$parts->add($this->getFormCategories());
			} else {
				$moreHiddens['fu_doc_categories'] = '';
			}

			if($this->isPermToUpload('images')){
				$parts
						->add($this->getFormImageAttributes())
						->add($this->getFormThumbnails());
			} else {
				$moreHiddens['fu_doc_alt'] = '';
				$moreHiddens['fu_doc_title'] = '';
				$moreHiddens['fu_doc_thumbs'] = '';
			}
		} else {
			$moreHiddens['fu_doc_isSearchable'] = '';
			$moreHiddens['fu_doc_categories'] = '';
			$moreHiddens['fu_doc_alt'] = '';
			$moreHiddens['fu_doc_title'] = '';
			$moreHiddens['fu_doc_thumbs'] = '';
			$moreHiddens['fu_doc_importMetadata'] = '';
		}

		$box = (new we_html_multiIconBox($parts))->marginLeft()->get();

		return we_html_element::hiddens($moreHiddens) . $box;
	}

	public function getHtmlFooter(): string{
		return we_html_element::div(['class' => 'editfooter fileuploadEditor_footer', 'style' => ''], we_html_element::div(['class' => 'containerBtn'], we_html_element::div([
									'class' => 'elemBtnFooter'], we_html_button::create_button(we_html_button::UPLOAD, "javascript:weFileUpload_instance.startUpload();", '', '', true, false, '_btn_footer', true, '', 'noMarginLeft')) .
								we_html_element::div(['class' => 'elemBtnFooter hidden'], we_html_button::create_button(we_html_button::CANCEL, "javascript:weFileUpload_instance.cancelUpload();", '', '', true, false, '_btn_footer')) .
								we_html_element::div(['class' => 'elemBtnFooter'], we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose()", '', '', false, false, '_btn_footer'))
						)
		);
	}

	public function getFormImageAttributes(): we_html_multiIconBoxContentLine{
		$html = we_html_element::div([], we_html_element::label([], 'Alternativ Text') . '<br/>' . we_html_tools::textInput('fu_doc_alt', '', 0, [], 'text', '378px')) .
				we_html_element::div([], we_html_element::label([], 'Titel') . '<div>' . we_html_tools::textInput('fu_doc_title', '', 0, [], 'text', '378px') . '</div>');
		$headline = 'Attribute';

		return ((new we_html_multiIconBoxContentLine($headline, $html, we_html_multiIconBox::PROP_ATTRIB))->fold()->space(we_html_multiIconBox::SPACE_MED));
	}

	function getFormCategories(we_base_jsCmd $jsCmd = null): we_html_multiIconBoxContentLine{
		$table = (new we_html_table([
					'id' => 'CategoriesBlock',
					'style' => 'display: block;',
					'class' => 'default withSpace'
						], 2, 1))
				->setCol(0, 0, we_html_element::div([
							'id' => 'categoriesDiv',
							'class' => 'blockWrapper',
							'style' => 'width:378px; height: 80px; border: #AAAAAA solid 1px;'
				]))
				->setCol(1, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats();") . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'addCat'})"), [
			'colspan' => '2', 'style' => 'text-align:right']
		);

		$html = $table->getHtml() . we_html_element::hiddens([
					'fu_doc_categories' => '0'
		]);
		$headline = g_l('global', '[categorys]');

		if($jsCmd && !empty($this->imageEditProps['categories'])){
			$jsCmd->addCmd('addCat', ['allPaths' => array_values(id2pathA($this->imageEditProps['categories'], CATEGORY_TABLE))]);
		}

		return ((new we_html_multiIconBoxContentLine($headline, $html, we_html_multiIconBox::PROP_CATEGORIES))->fold(empty($this->imageEditProps['categories']))->space(we_html_multiIconBox::SPACE_MED));
	}

	public function getFormOptions(bool $getSearchable = true, bool $getParse = true, $getMeta = true): we_html_multiIconBoxContentLine{
		$html = ($getSearchable ? we_html_element::div([], we_html_forms::checkboxWithHidden((bool) $this->imageEditProps['isSearchable'],
								'fu_doc_isSearchable',
								g_l('importFiles', '[imgsSearchable_label]'),
								false, 'defaultfont', '')) : ''
				) .
				($getParse ? we_html_element::div([], we_html_forms::checkboxWithHidden((bool) $this->imageEditProps['parseFile'],
								'fu_doc_parseFile',
								g_l('weClass', '[parseFile]'),
								false, 'defaultfont', '')) : ''
				) .
				($getMeta ? we_html_element::div(['id' => 'div_importMeta', 'class' => 'importMeta'],
						we_html_forms::checkboxWithHidden((bool) $this->imageEditProps['importMetadata'],
								'fu_doc_importMetadata',
								g_l('importFiles', '[import_metadata]'),
								false, 'defaultfont', '')
				) : ''
				);

		$headline = g_l('export', '[options]');

		return (new we_html_multiIconBoxContentLine($headline, $html))->rightHead();
	}

	public function getFormSameName(): we_html_multiIconBoxContentLine{
		$value = $this->imageEditProps['sameName'] ?? we_exim_ExIm::COLLISION_RENAME;
		$html = we_html_element::div(['style' => 'margin:10px 0 0 0;'], we_html_element::div(['style' => 'margin-top:10px'], we_html_forms::radiobutton(we_exim_ExIm::COLLISION_OVERWRITE, $value === we_exim_ExIm::COLLISION_OVERWRITE, 'sameName', g_l('importFiles', '[sameName_overwrite]'), false, 'defaultfont', 'document.we_form.fu_file_sameName.value=this.value;') .
								we_html_forms::radiobutton(we_exim_ExIm::COLLISION_RENAME, $value === we_exim_ExIm::COLLISION_RENAME, 'sameName', g_l('importFiles', '[sameName_rename]'), false, "defaultfont", 'document.we_form.fu_file_sameName.value=this.value;') .
								we_html_forms::radiobutton(we_exim_ExIm::COLLISION_SKIP, $value === we_exim_ExIm::COLLISION_SKIP, 'sameName', g_l('importFiles', '[sameName_nothing]'), false, "defaultfont", 'document.we_form.fu_file_sameName.value=this.value;')
						) .
						we_html_element::hidden('fu_file_sameName', $value)
		);

		$headline = g_l('importFiles', '[sameName_headline]');

		return (new we_html_multiIconBoxContentLine($headline, $html))->rightHead()->noline();
	}

	public function getFormImportDirs(): we_html_multiIconBoxContentLine{
		$html = we_html_element::div([], we_html_forms::checkboxWithHidden((bool) $this->imageEditProps['importDirs'],
								'fu_doc_importDirs',
								g_l('importFiles', '[importDirs_label]'),
								false, 'defaultfont', ''
		));
		return (new we_html_multiIconBoxContentLine(g_l('importFiles', '[importDirs_headline]'), $html))->noline()->rightHead();
	}

	public function getFormParentID(string $formName = we_base_constants::WE_FORM): we_html_multiIconBoxContentLine{
		if($this->parentID['presetDir']){
			// in sselect when uploading to ext dir we have (string) parentDir in $this->parentID: 'preset' with no id to path!
			$id = !($id = path_to_id((string) $this->parentID['presetDir'])) ? -1 : $id;
			$parentDir = $id === -1 ? $this->parentID['presetDir'] : '';
			$parentID = !in_array($id, [0, -1]) ? $id : 0;
		} else {
			$parentDir = '';
			$parentID = $this->parentID['presetId'] ?: ($this->imageEditProps['parentID'] ?: (IMAGESTARTID_DEFAULT ?: 0));
		}

		$parentID = (int) ($parentID && fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($parentID) . ' AND IsFolder=1') ? $parentID : 0);

		if(($ws = we_users_util::get_ws(FILE_TABLE))){
			'@phan-var int[] $ws';
			if(!(we_users_util::in_workspace($parentID, $ws, FILE_TABLE))){
				if($parentID && $this->parentID['setFixed']){ //only warn, if a parentID was ever set.
					t_e('wrong uploader configuration: fixed parentID not in user\'s ws', $ws, $this->parentID['presetId']);
				}
				$parentID = we_users_util::get_def_ws(FILE_TABLE);
			}
		}

		if(!$this->parentID['setFixed']){
			$weSuggest = we_gui_suggest::getInstance();
			$weSuggest->setAcId('fu_file_parentID')
					->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
					->setInput('fu_file_parentDir', $parentID ? id2path($parentID, FILE_TABLE) : '/')
					->setMaxResults(10)
					->setResult('fu_file_parentID', (string) $parentID)
					->setSelector(we_gui_suggest::DirSelector)
					->setWidth(326)
					->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:$parentID,table:WE().consts.tables.FILE_TABLE,targetID:'fu_file_parentID',targetName:'fu_file_parentDir',filter:WE().consts.contentTypes.FOLDER_DOCUMENT});"));
			$html = $weSuggest->getHTML() . we_html_element::hidden('fu_file_parentFixed', '0');
		} else {
			$parentPath = $parentDir ?: id2path($parentID);
			$html = we_html_element::input(['value' => $parentPath, 'disabled' => 'disabled']) .
					we_html_button::create_button(we_html_button::SELECT, '', '', '', true) .
					we_html_element::hiddens(['fu_file_parentID' => (string) $parentID,
						'fu_file_parentDir' => $parentDir,
						'fu_file_parentFixed' => '1',
			]);
		}

		$headline = g_l('importFiles', '[destination_dir]');

		return (new we_html_multiIconBoxContentLine($headline, $html, we_html_multiIconBox::PROP_PATH))->noline();
	}

	public function getFormThumbnails(we_base_jsCmd $jsCmd = null): we_html_multiIconBoxContentLine{
		$thumbnails = new we_html_select([// FIXME: use we_cmd in onchange!
			'multiple' => 'multiple',
			'name' => 'thumbnails_tmp',
			'id' => 'thumbnails_tmp',
			'class' => 'defaultfont',
			'size' => 6,
			'style' => 'width: 378px;',
			'onchange' => "this.form.fu_doc_thumbs.value='';for(var i=0;i<this.options.length;i++){if(this.options[i].selected){this.form.fu_doc_thumbs.value +=(this.options[i].value + ',');}};this.form.fu_doc_thumbs.value=this.form.fu_doc_thumbs.value.replace(/^(.+),$/,'$1');"
		]);
		$entries = DB_WE::$inst->getAllq('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');
		foreach($entries as $entry){
			$attribs = [
				'title' => $entry['description'],
				(in_array($entry['ID'], $this->imageEditProps['thumbnails'], true) ? 'selected' : '') => 'selected'
			];
			$thumbnails->addOption((string) $entry['ID'], $entry['Text'], $attribs);
		}

		if($jsCmd && !empty($this->imageEditProps['thumbnails'])){
			$jsCmd->addCmd('setFocus', 'thumbnails_tmp');
		}

		$html = $thumbnails->getHtml() . we_html_element::hidden('fu_doc_thumbs', implode(',', $this->imageEditProps['thumbnails']));
		$headline = g_l('thumbnails', '[create_thumbnails]');

		return ((new we_html_multiIconBoxContentLine($headline, $html, we_html_multiIconBox::PROP_IMAGE))->fold(empty($this->imageEditProps['thumbnails']))->space(we_html_multiIconBox::SPACE_MED));
	}

	public function getJs(): string{
		return self::getJSFileuploader() . self::getJSUtils() . $this->getJSDialog() . $this->getJSInit();
	}

	public function getJSDialog(): string{
		$cats = we_base_request::filterIntList($this->imageEditProps['categories']);
		return we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_fileupload.js', '', ['id' => 'loadVarFileupload_ui_editor', 'data-editor' => setDynamicVar([
						'trashButton' => we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER),
						'categoriesDivSize' => 368,
						'variantCats' => (we_modules_info::isActive(we_modules_info::CATEGORY) && $cats ? id2pathA($cats, CATEGORY_TABLE) : [])
		])]);
	}

	public static function showFrameset(): void{
		$acceptedCT = explode(',', stripslashes(we_base_request::string('we_cmd', '', 'accept')));
		$doImport = we_base_request::bool('we_cmd', true, 'import');
		$importToID = we_base_request::int('we_cmd', 0, 'parentid');
		$importToDir = we_base_request::string('we_cmd', '', 'parentdir');
		$setFixedImportTo = we_base_request::bool('we_cmd', false, 'fixed');
		$isPreset = we_base_request::bool('we_cmd', false, 'preset');
		$cmdUploadFinished = we_base_request::string('we_cmd', '', 'writeback');

		$fileUpload = new self('we_File', $acceptedCT, [], [], [], $doImport);
		$fileUpload->setIsPreset($isPreset);
		$fileUpload->setFieldParentID(['setField' => true, 'presetId' => $importToID, 'presetDir' => $importToDir, 'setFixed' => $setFixedImportTo]);
		$fileUpload->setCmd('cmdUploadFinished', $cmdUploadFinished);

		echo we_html_tools::getHtmlTop('fileupload', $fileUpload->getJs() .
				$fileUpload->getCss(), we_html_element::body(['class' => 'weDialogBody'], we_html_element::form([
								], we_html_element::div([
									'id' => 'we_fileupload_editor', 'class' => 'weDialogBody', 'style' => 'position:absolute;top:0;bottom:40px;left:0;right:0;overflow: auto;'], $fileUpload->getHtml()) .
								we_html_element::div(['id' => 'we_fileupload_footer', 'class' => '', 'style' => 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden;'], $fileUpload->getHtmlFooter())
						)
				)
		);
	}

}
