<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 47eed3b93e07 $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2022-01-12 12:32 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_import_functions{

	/**
	 * @return boolean
	 * @param integer $parentID
	 * @param integer $templateID
	 * @param array<string,mixed> $fields
	 * @param string[] $dateFields
	 * @param integer $doctypeID
	 * @param int[] $categories
	 * @param string $filename
	 * @param boolean $isDynamic
	 * @param string $extension
	 * @param boolean $publish
	 * @param boolean $IsSearchable
	 * @desc imports a document into webedition: used by we_importWizardWE and we_importWizardCSV
	 */
	public static function importDocument(int $parentID, int $templateID, array $fields, array $dateFields, int $doctypeID, array $categories, string $filename, bool $isDynamic, string $extension, bool $publish, bool $IsSearchable, string $conflict = we_exim_ExIm::COLLISION_RENAME): bool{

		// erzeugen eines neuen webEdition-Dokument-Objekts
		$we_doc = new we_document_webEdition();

		$we_doc->we_new();

		$we_doc->Extension = $extension;
		if($filename){
			$filename = self::correctFilename($filename, 'document');
			$we_doc->Filename = $filename;
		}

		$we_doc->setParentID($parentID);
		// IF NAME OF OBJECT EXISTS, WE HAVE TO CREATE A NEW NAME
		if(($file_id = fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($we_doc->Path) . '"'))){
			if($conflict === we_exim_ExIm::COLLISION_RENAME){
				$z = 0;
				$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
				while(fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($we_doc->ParentID))){
					$z++;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
				}
				$we_doc->Filename = $we_doc->Filename . '_' . $z;
				$we_doc->Path = $we_doc->getPath();
			} else if($conflict === we_exim_ExIm::COLLISION_REPLACE){
				$we_doc->initById($file_id);
			} else {
				return true;
			}
		}

		$we_doc->DocType = $doctypeID;
		$we_doc->setTemplateID($templateID);
		$we_doc->Category = $categories;

		$we_doc->ContentType = we_base_ContentTypes::WEDOCUMENT;

		$we_doc->IsDynamic = $isDynamic;
		$we_doc->IsSearchable = $IsSearchable;
		foreach($fields as $fieldName => $fieldValue){
			if($fieldValue instanceof DateTime){
				$fieldValue = $fieldValue->format('U');
			}
			//quick hack for numbers and dates
			if(is_numeric($fieldValue)){
				$we_doc->setElementI($fieldName, (int) $fieldValue, in_array($fieldName, $dateFields, true) ? 'date' : 'txt');
			} else {
				$we_doc->setElementS($fieldName, (string) $fieldValue);
			}
		}

		// SAVE DOCUMENT
		if(!$we_doc->we_save()){
			return false;
		}
		// PUBLISH OR EXIT
		return ($publish ?
				$we_doc->we_publish() :
				true
				);
	}

	/**
	 * @return boolean
	 * @param integer $classID
	 * @param array<string,mixed> $fields
	 * @param int[] $categories
	 * @param string $filename
	 * @param boolean $publish
	 * @desc imports an object into webEdition
	 */
	public static function importObject(int $classID, array $fields, array $categories, string $filename, bool $publish, bool $issearchable, int $parentID = 0, string $conflict = we_exim_ExIm::COLLISION_RENAME): bool{

		// INIT OBJECT
		$object = new we_contents_objectFile();
		$object->we_new();
		$object->TableID = $classID;
		$object->setRootDirID(true);
		$object->resetParentID();
		$object->restoreDefaults();
		if($categories){
			$object->Category = $categories;
		}
		$object->IsSearchable = $issearchable;
		if($parentID){
			$object->setParentID($parentID);
		}

		// IF WE HAVE TO GIVE THE OBJECT A NAME
		if($filename || $filename === '0'){
			$object->Filename = we_import_functions::correctFilename($filename, 'objectFile');
			$object->Path = $object->getParentPath() . (($object->getParentPath() != '/') ? '/' : '') . $object->getText();
			// IF NAME OF OBJECT EXISTS, WE HAVE TO CREATE A NEW NAME
			if(($file_id = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . intval($object->ParentID) . ' AND Filename="' . DB_WE::$inst->escape($object->Filename) . '"'))){
				switch($conflict){
					case we_exim_ExIm::COLLISION_REPLACE:
						$object->initByID($file_id);
						break;
					case we_exim_ExIm::COLLISION_RENAME:
						$z = 0;
						$footext = $object->getText($z);
						while(fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Filename="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($object->ParentID))){
							$z++;
							$footext = $object->getText($z);
						}
						$object->Filename = $footext;
						$object->Path = $object->getParentPath() . (($object->getParentPath() != '/') ? '/' : '') . $object->getText();
						break;
					default:
						return true;
				}
			}
		}

		// FILL FIELDS OF OBJECT
		foreach($fields as $fieldName => $fieldValue){
			if($fieldValue instanceof DateTime){
				$fieldValue = (int) $fieldValue->format('U');
				$object->setElementI($fieldName, $fieldValue);
			} else {
				$object->setElementS($fieldName, $fieldValue);
			}
		}
		// SAVE OBJECT
		if(!$object->we_save()){
			return false;
		}
		// PUBLISH OR EXIT
		return ($publish ?
				$object->we_publish() :
				true);
	}

	/**
	 * @return string
	 * @param string $fileName
	 * @param string $type
	 * @desc corrects the filename if it contains invalid chars
	 */
	static function correctFilename(string $fileName, $type = 'document'): string{
		return $fileName ? substr(trim(we_base_util::convertDisallowedChars($fileName, $type), '/'), 0, 200) : 'newfile';
	}

	static function correctExtension(string $ext): string{
		$exts = explode('.', $ext);
		$extension = $ext ? array_pop($exts) : '';
		return $extension ? '.' . substr(we_base_util::convertDisallowedChars(strtolower($extension), 'extension'), 0, 16) : '';
	}

	/**
	 * @return int
	 * @param string $datestring
	 * @param string $format
	 * @desc converts a $datestring which represent a date to an unix timestamp with the given $format. If $format is empty, $datestring has to be a valid English date format
	 */
	static function parseDate(string $datestring, string $format = ''): DateTime{
		/*if(!$format){
			$format = 'Y-m-d H:i:s';
		}*/
		// for now just Y-m-d H:i:s
		return new DateTime($datestring);
	}

}
