<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 967c728473a5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-07-01 17:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_wizardXML extends we_import_wizard{

	protected function getWizBody(): string{
		$formAttribs = [
			'name' => we_base_constants::WE_FORM,
			'enctype' => 'multipart/form-data'
		];

		$jsCmd = new we_base_jsCmd();
		$hiddens = [
			'pnt' => 'wizbody',
			'type' => we_exim_ExIm::TYPE_XML,
			'v[type]' => we_exim_ExIm::TYPE_XML,
			'step' => (string) ($this->step),
			'mode' => (string) ($this->mode),
			'button_state' => '0'
		];
		switch($this->step){
			case 2:
				$this->fileUploaderResp = new we_fileupload_resp_base();
				break;
			default:
				$this->fileUploader = new we_fileupload_ui_base('uploaded_xml_file', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::XML), we_base_ContentTypes::getExtension(we_base_ContentTypes::XML));
				$this->fileUploader->setCmd('cmdFileselectOnclick', 'fileupload_doOnFileSelect');
				$this->fileUploader->setGenericFileName(TEMP_DIR . 'we_xml_' . we_fileupload::REPLACE_BY_UNIQUEID . '.xml');
		}
		switch($this->step){
			default:
			case 1:
				$formAttribs['onsubmit'] = 'return false;';
				$content = $this->getStep1($jsCmd);
				break;
			case 2:
				$content = $this->getStep2();
				break;
			case 3:
				$content = $this->getStep3();
				break;
		}
		$doOnLoad = !we_base_request::bool('noload');
		return we_html_tools::getHtmlTop('', ($this->fileUploader ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
				we_html_element::jsScript(JS_DIR . 'import_wizardWizbody.js') .
				$jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody',
					'onload' => $doOnLoad ? "parent.wiz_next('wizbusy', WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=import&pnt=wizbusy&mode=" . $this->mode . "&type=" . we_exim_ExIm::TYPE_CSV . "'); self.focus();" : ""
					], we_html_element::form($formAttribs, we_html_element::hiddens($hiddens) .
						$content
					)
				)
		);
	}

	/**
	 * Generic XML Import Step 1
	 *
	 * @return string
	 */
	private function getStep1(we_base_jsCmd $jsCmd): string{
		$DB_WE = we_database_base::$inst;
		$v = we_base_request::stringList('v');

		if(isset($v['docType']) && $v['docType'] != -1 && we_base_request::bool('doctypeChanged')){
			$values = getHash('SELECT ParentID,Extension,IsDynamic,Category FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($v['docType']));
			$v['store_to_id'] = $values['ParentID'];
			$v['store_to_path'] = id2path($values['ParentID']);
			$v['we_Extension'] = $values['Extension'];
			$v['is_dynamic'] = $values['IsDynamic'];
			$v['docCategories'] = $values['Category'];
		}

		$hdns = we_html_element::hiddens([
				'v[importDataType]' => '',
				'v[import_from]' => ($v['import_from'] ?? ''),
				'v[docCategories]' => ($v['docCategories'] ?? ''),
				'v[objCategories]' => ($v['objCategories'] ?? ''),
				//'v[store_to_id]', 'value' => (isset($v['store_to_id']) ? $v['store_to_id'] : 0))).
				'v[collision]' => ($v['collision'] ?? we_exim_ExIm::COLLISION_RENAME),
				'doctypeChanged' => '0',
				'v[we_TemplateID]' => '0',
				//'v[we_TemplateName]', 'value' => '/')).
				'v[is_dynamic]' => ($v['is_dynamic'] ?? '0'),
				'v[btnState_next]' => 'enabled',
				'v[btnState_back]' => 'enabled'
		]);


		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			$hdns .= we_html_element::hidden('v[import_type]', self::TYPE_DOCUMENT);
		}

		$v['import_type'] = $v['import_type'] ?? self::TYPE_DOCUMENT;

		$importFromButton = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT])) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'v[fserver]', 'all_Types', document.we_form.elements['v[fserver]'].value, 'setFormField,v[rdofloc],1,radio,0');") : "";
		$inputLServer = we_html_tools::textInput('v[fserver]', ($v['fserver'] ?? '/'), 255, ['readonly' => 'readonly'], 'text', '300px');
		$importFromServer = we_html_tools::formElementTable('', $inputLServer, $importFromButton);

		//FIXME: still need condition?
		$inputLLocal = $this->fileUploader->getHTML();
		$importFromLocal = we_html_tools::formElementTable('', $inputLLocal);
		$rdoLServer = we_html_forms::radiobuttonV('lServer', $v['rdofloc'], 'v[rdofloc]', g_l('import', '[fileselect_server]'));
		$rdoLLocal = we_html_forms::radiobuttonV('lLocal', $v['rdofloc'], 'v[rdofloc]', g_l('import', '[fileselect_local]'));
		$tblRow = 0;
		$importLocs = (new we_html_table(['class' => 'default'], 6, 1))
			->setCol($tblRow++, 0, $rdoLServer)
			->setCol($tblRow++, 0, $importFromServer)
			->setCol($tblRow++, 0, $rdoLLocal, ['style' => 'padding-top:4px;'])
			//FIXME: still need condition?
			//$importLocs->setCol($tblRow++, 0, $this->fileUploader->getHtmlAlertBoxes());
			->setCol($tblRow++, 0, $importFromLocal);

		$optid = 0;
		$DTselect = (new we_html_select(['name' => 'v[docType]',
				'class' => 'weSelect',
				'onclick' => (we_modules_info::isActive(we_modules_info::OBJECT)) ? "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);" : '',
				'onchange' => "WE().util.dialogTop(window).setFormField('doctypeChanged', 1, 'hidden'); WE().util.dialogTop(window).weChangeDocType(this);",
				'style' => 'width: 300px'
				]))
			->insertOption($optid, '-1', g_l('import', '[none]'));

		$v['docType'] = $v['docType'] ?? -1;
		foreach($DB_WE->getAllq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.IsFolder=0 ORDER BY dt.Text') as $rec){
			$optid++;
			$DTselect->insertOption($optid, (string) $rec['ID'], $rec['Text']);
			if($v['docType'] == $rec['ID']){
				$DTselect->selectOption($rec['ID']);
			}
		}
		$doctypeElement = we_html_tools::formElementTable(g_l('import', '[doctype]'), $DTselect->getHTML());

		/*		 * * templateElement *************************************************** */
		/* $ueberschrift = (we_base_permission::hasPerm([we_base_perm::CAN_SEE_TEMPLATES])?
		  '<a href="javascript:goTemplate(document.we_form.elements[\'' . $idname . '\'].value)">' . g_l('import', '[template]') . '</a>':
		  g_l('import', '[template]')); */

		$myid = $v['we_TemplateID'] ?? 0;
		//$path = f('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($myid), 'Path', $DB_WE);

		/*		 * ******************************************************************** */
		$weSuggest = we_gui_suggest::getInstance();

		$TPLselect = new we_html_select(['name' => 'docTypeTemplateId',
			'class' => 'weSelect',
			'onclick' => (we_modules_info::isActive(we_modules_info::OBJECT)) ? "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);" : '',
			//'onchange'  => "we_submit_form(self.document.we_form, 'wizbody', '".$this->path."');",
			'style' => 'width: 300px'
		]);

		if($v['docType'] != -1 && !$TPLselect->isEmpty()){
			$displayDocType = 'display:block';
			$displayNoDocType = 'display:none';
			$templates = $DB_WE->getAllFirstq('SELECT t.ID,t.Path
				FROM ' . DOC_TYPES_TABLE . ' dt JOIN
				' . TEMPLATES_TABLE . ' t ON FIND_IN_SET(t.ID,dt.Templates)
				WHERE dt.ID=' . intval($v['docType']), false);
			'@phan-var array<int,string> $templates';
			$TPLselect->addOptions($templates);
			if(!empty($v['we_TemplateID'])){
				$TPLselect->selectOption($v['we_TemplateID']);
			}
		} else {
			$displayDocType = 'display:none';
			$displayNoDocType = 'display:block';
		}

		$templateElement = we_html_element::div(['id' => 'docTypeLayer', 'style' => $displayDocType], we_html_tools::formElementTable(g_l('import', '[template]'), $TPLselect->getHTML()));

		$weSuggest->setAcId('TmplPath')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE])
			->setInput('v[we_TemplateName]', (string) ($v['we_TemplateName'] ?? ''), ['onfocus' => "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);"])
			->setMaxResults(10)
			->setResult('noDocTypeTemplateId', (string) $myid)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(TEMPLATES_TABLE)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:top.wizbody.document.we_form.elements.noDocTypeTemplateId.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'noDocTypeTemplateId',targetName:'v[we_TemplateName]',jsCmd:'reload_editpage',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"))
			->setLabel(g_l('import', '[template]'));

		$templateElement .= we_html_element::div(['id' => 'noDocTypeLayer', 'style' => $displayNoDocType], $weSuggest->getHTML());

		$docCategories = (we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->formCategory2($jsCmd, 'doc', isset($v['docCategories']) ? we_base_request::filterIntList($v['docCategories']) : [
			]) : '');

		$weSuggest->setAcId('DirPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('v[store_to_path]', ($v['store_to_path'] ?? '/'), ['onfocus' => "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);"])
			->setMaxResults(10)
			->setRequired(true)
			->setResult('v[store_to_id]', (string) ($v['store_to_id'] ?? 0))
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:WE().util.dialogTop(window).wizbody.document.we_form.elements['v[store_to_id]'].value,table:'" . FILE_TABLE . "',targetID:'v[store_to_id]',targetName:'v[store_to_path]',rootDirID:'0'})"))
			->setLabel(g_l('import', '[import_dir]'));

		$storeTo = $weSuggest->getHTML();

		$radioDocs = we_html_forms::radiobuttonV(self::TYPE_DOCUMENT, $v['import_type'], 'v[import_type]', g_l('import', '[documents]'));
		$radioObjs = we_html_forms::radiobuttonV(self::TYPE_OBJECT, $v['import_type'], 'v[import_type]', g_l('import', '[objects]'), true, '', "self.document.we_form.elements['v[store_to_path]'].value='/'; WE().layout.weSuggest.checkRequired(window,self.document.we_form.elements['v[store_to_path]'].id); if(self.document.we_form.elements['v[we_TemplateName]']!==undefined) { self.document.we_form.elements['v[we_TemplateName]'].value=''; WE().layout.weSuggest.checkRequired(window,self.document.we_form.elements['v[we_TemplateName]'].id); }", (!we_modules_info::isActive(we_modules_info::OBJECT)));

		$v['classID'] = $v['classID'] ?? -1;
		$CLselect = new we_html_select(['name' => 'v[classID]',
			'class' => 'weSelect',
			'onclick' => "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 1);",
			'style' => 'width: 150px'
		]);
		$ac = we_users_util::getAllowedClasses();
		if($ac){
			$entries = $DB_WE->getAllFirstq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' WHERE ID IN(' . implode(',', $ac) . ') ORDER BY Text', false);
			'@phan-var array<int,string> $entries';
			$CLselect->addOptions($entries)
				->selectOption((string) $v['classID']);
		} else {
			$CLselect->addOption('-1', g_l('import', '[none]'));
		}

		$objClass = (new we_html_table(['class' => 'default'], 2, 2))
			->setCol(0, 0, g_l('import', '[class]'), ['style' => 'vertical-align:top;width:130px;', 'class' => 'defaultfont lowContrast'])
			->setCol(0, 1, $CLselect->getHTML(), ['style' => 'width:150px;']);

		$objCategories = $this->formCategory2($jsCmd, 'obj', isset($v['objCategories']) ? we_base_request::filterIntList($v['objCategories']) : []);
		$objCats = (new we_html_table(['class' => 'default'], 2, 2))
			->setCol(0, 0, g_l('import', '[categories]'), ['style' => 'vertical-align:top;width:130px;', 'class' => 'defaultfont lowContrast'])
			->setCol(0, 1, $objCategories, ['style' => 'width:150px;']);

		$objects = (new we_html_table(['class' => 'default'], 3, 2))
			->setCol(0, 0, $radioObjs, ['colspan' => '3', 'class' => 'withBigSpace'])
			->setColAttributes(1, 0, ['style' => 'width:50px;'])
			->setCol(1, 1, $objClass->getHTML())
			->setCol(2, 1, $objCats->getHTML());

		$specifyDoc = (new we_html_table(['class' => 'default'], 1, 2))
			->setCol(0, 1, we_html_forms::checkbox('3', (bool) ($v['is_dynamic'] ?? false), 'chbxIsDynamic', g_l('import', '[isDynamic]'), true, 'defaultfont', "this.form.elements['v[is_dynamic]'].value=this.checked? 1 : 0; switchExt();"), [
				'style' => 'vertical-align:bottom'])
			->setCol(0, 0, we_html_tools::formElementTable(g_l('import', '[extension]'), we_html_tools::getExtensionPopup('v[we_Extension]', ($v['we_Extension'] ?? '.html'), we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT), 100)), [
			'style' => 'padding-right:20px;']);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[import]'), $importLocs->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED))
			->add((new we_html_multiIconBoxContentLine((we_modules_info::isActive(we_modules_info::OBJECT)) ? $radioDocs : g_l('import', '[documents]'), $doctypeElement . ' ' . $templateElement . ' ' . $storeTo . ' ' . $specifyDoc->getHTML() . ' ' .
				($docCategories ? we_html_tools::formElementTable(g_l('import', '[categories]'), $docCategories) : '')))
			->space(we_html_multiIconBox::SPACE_MED)
			->noline());

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$parts->add((new we_html_multiIconBoxContentLine($radioObjs, we_html_tools::formElementTable(g_l('import', '[class]'), $CLselect->getHTML()) . ' ' .
						$objects->getHtml() .
						we_html_tools::formElementTable(g_l('import', '[categories]'), $objCategories)))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline());
		}

		$znr = -1;

		return $hdns .
				(new we_html_multiIconBox($parts, 'xml'))
				->marginLeft()
				->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
				->headline(g_l('import', '[gxml_import]'))
				->get();
	}

	/**
	 * Generic XML Import Step 2
	 *
	 */
	private function getStep2(): string{
		$parts = new we_html_multiIconBoxContent();
		$hdns = "\n";
		$v = we_base_request::stringList('v');
		$upload_error = '';

		if($v['rdofloc'] === 'lLocal'){
			try{
				$v['import_from'] = $this->fileUploaderResp->commitUploadedFile();
			} catch (we_exception_file $e){
				$upload_error = $e->getMessage();
			}
		}

		$vars = ['rdofloc', 'fserver', 'flocal', 'importDataType', 'docCategories', 'objCategories', 'store_to_id', 'is_dynamic', 'import_from', 'docType',
			'we_TemplateName', 'we_TemplateID', 'store_to_path', 'we_Extension', 'import_type', 'classID', 'sct_node', 'rcd', 'from_elem', 'to_elem', 'collision'];
		foreach($vars as $var){
			$hdns .= we_html_element::hidden('v[' . $var . ']', $v[$var] ?? '');
		}
		$hdns .= we_html_element::hiddens(['v[mode]' => '0', 'v[cid]', 'value' => '-2']);
		$hasChildNode = $isSingleNode = false;

		if((file_exists(WE_DOCUMENT_ROOT . $v['import_from']) && is_readable(WE_DOCUMENT_ROOT . $v['import_from']))){
			$xp = new we_xml_parser(WE_DOCUMENT_ROOT . $v['import_from']);
			$xmlWellFormed = ($xp->parseError === '');

			$recs = [];
			if($xmlWellFormed){
				// Node-set with paths to the child nodes.
				$xp->evaluate('*/child::*');
				$children = $xp->nodes[$xp->root]['children'];

				foreach($children as $key => $value){
					$flag = true;
					for($k = 1; $k < ($value + 1); $k++){
						if(!$xp->hasChildNodes($xp->root . '/' . $key . '[' . $k . ']')){
							$flag = false;
						}
					}
					if($flag){
						$recs[(string) $key] = (string) $value;
					}
				}
				$isSingleNode = (count($recs) == 1);
				$hasChildNode = (!empty($recs));
			}
			if($xmlWellFormed && $hasChildNode){
				$rcdSelect = (new we_html_select([
						'name' => 'we_select',
						'class' => 'weSelect',
						(($isSingleNode) ? 'disabled' : '') => (($isSingleNode) ? 'disabled' : ''),
						'onchange' => "onChangeSelectXMLNode(this);"
						]))
					->addOptions($recs);
				$firstOptVal = reset($recs);
				if(isset($v['rcd']) && ($key = array_search($v['rcd'], $recs)) !== false){
					$rcdSelect->selectOption($key);
				}

				$tblSelect = (new we_html_table([], 1, 7))
					->setCol(0, 1, $rcdSelect->getHtml())
					->setColAttributes(0, 2, ['width' => '20'])
					->setCol(0, 3, g_l('import', '[num_data_sets]'))
					->setCol(0, 4, we_html_tools::textInput('v[from_iElem]', '1', 5, [], 'text', '50px', false, ($isSingleNode && ($firstOptVal == 1))))
					->setCol(0, 5, g_l('import', '[to]'))
					->setCol(0, 6, we_html_tools::textInput('v[to_iElem]', (string) $firstOptVal, 5, [], 'text', '50px', false, ($isSingleNode && ($firstOptVal == 1))));

				$tblFrame = (new we_html_table([], 3, 2))
					->setCol(0, 0, ($isSingleNode) ? we_html_tools::alertAttentionBox(g_l('import', '[well_formed]') . ' ' . g_l('import', '[select_elements]'), we_html_tools::TYPE_INFO, 530) :
						we_html_tools::alertAttentionBox(g_l('import', '[xml_valid_1]') . ' ' . count($recs) . ' ' . g_l('import', '[xml_valid_m2]'), we_html_tools::TYPE_INFO, 530), [
						'colspan' => '2', 'class' => 'defaultfont'])
					->setColAttributes(1, 0, ['colspan' => '2'])
					->setCol(2, 1, $tblSelect->getHtml());

				$parts->add((new we_html_multiIconBoxContentLine('', $tblFrame->getHtml()))->noline());
			} else {
				$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', (!$xmlWellFormed ? '[not_well_formed]' : '[missing_child_node]')), we_html_tools::TYPE_ALERT, 530)))->noline());
			}
		} else {
			$xmlWellFormed = false;

			if($upload_error){ // uploaded file nok: get error from uploader
				$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox($upload_error, we_html_tools::TYPE_ALERT, 530)))->noline());
			} else { // file from server nok
				if(!file_exists(WE_DOCUMENT_ROOT . $v['import_from'])){
					$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_exists]') . WE_DOCUMENT_ROOT . $v['import_from'], we_html_tools::TYPE_ALERT, 530)))->noline());
				} elseif(!is_readable(WE_DOCUMENT_ROOT . $v['import_from'])){
					$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_readable]'), we_html_tools::TYPE_ALERT, 530)))->noline());
				}
			}
		}

		$znr = -1;

		return $hdns .
			we_html_element::hiddens(['v[btnState_next]' => (($xmlWellFormed && $hasChildNode) ? 'enabled' : 'disabled'),
				'v[btnState_back]' => 'enabled'
			]) .
				(new we_html_multiIconBox($parts, 'xml'))
				->marginLeft()
				->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
				->headline(g_l('import', '[select_data_set]'))
				->get();
	}

	private function getStep3(): string{
		$v = we_base_request::stringList('v');
		if(isset($v['att_pfx'])){
			$v['att_pfx'] = base64_encode($v['att_pfx']);
		}
		$records = we_base_request::_(we_base_request::RAW, 'records', []);
		$we_flds = we_base_request::_(we_base_request::RAW, 'we_flds', []);
		$attrs = we_base_request::_(we_base_request::RAW, 'attrs', []);
		foreach($attrs as $name => $value){
			$attrs[$name] = base64_encode($value);
		}

		$hdns = $this->getHdns('v', $v) .
			($records ? $this->getHdns('records', $records) : '') .
			($we_flds ? $this->getHdns('we_flds', $we_flds) : '') .
			($attrs ? $this->getHdns('attributes', $attrs) : '') .
			//$hdns .= ' => 'v[cid]', 'value' => -2));
			we_html_element::hiddens([
				'v[pfx_fn]' => ((!isset($v['pfx_fn'])) ? '0' : $v['pfx_fn']),
				(isset($v['rdo_timestamp']) ? 'v[sTimeStamp]' : '') => ($v['rdo_timestamp'] ?? ''),
				'v[btnState_next]' => ((we_base_request::int('mode') != 1) ? 'enabled' : 'disabled'),
				'v[btnState_back]' => 'enabled'
		]);

		$records = $dateFields = [];
//FIXME: we must keep track of the type for writing this to tblContent!
		switch($v['import_type']){
			case self::TYPE_DOCUMENT:
				$templateCode = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . intval($v['we_TemplateID']) . ' AND c.nHash=unhex(md5("' . we_contents_template::FULLCODE . '")');
				$tp = new we_weTag_parser($templateCode);

				$records = [we_contents_root::TITLE_FIELD, we_contents_root::DESCRIPTION_FIELD, we_contents_root::KEYWORD_FIELD];
				foreach($tp->getTagsWithAttributes() as $tag){
					if(empty($tag['attribs']['name'])){
						continue;
					}
					switch($tag['name']){
						case 'var':
						case 'field':
							continue 2;
						// tags with text content, links and hrefs
						case 'input':
							if(!empty($tag['attribs']['type']) && $tag['attribs']['type'] === 'date'){
								$dateFields[] = $tag['attribs']['name'];
							}
						case 'textarea':
						case 'href':
						case 'link':
							$records[] = $tag['attribs']['name'];
							break;
					}
				}
				$records = array_unique($records);
				break;
			case self::TYPE_OBJECT:
				$classFields = self::getClassFields((int) $v['classID']);
				foreach($classFields as $classField){
					if(self::isTextField($classField['type']) || self::isNumericField($classField['type']) || self::isDateField($classField['type'])){
						$records[] = $classField['name'];
					}
					if(self::isDateField($classField['type'])){
						$dateFields[] = $classField['name'];
					}
				}
				break;
		}
		$xp = new we_xml_parser(WE_DOCUMENT_ROOT . $v['import_from']);
		$nodeSet = $xp->evaluate($xp->root . '/' . $v['rcd'] . '[1]/child::*');
		$val_nodes = [];

		foreach($nodeSet as $node){
			'@phan-var string $node';
			$nodeName = $xp->nodeName($node);
			$val_nodes = array_merge($val_nodes, [$nodeName => $nodeName]);

			/* if($xp->hasAttributes($node)){
			  $val_attrs = $val_attrs + ['@n:' => g_l('import', '[none]')];
			  $attributes = $xp->getAttributes($node);

			  foreach($attributes as $name => $value){
			  $tmp_attrs = [$name => $name];
			  $val_attrs = $val_attrs + $tmp_attrs;
			  }
			  } */
		}

		//$val_attrs = $val_attrs ?: ['@n:' => g_l('import', '[none]')];

		$th = [g_l('import', '[we_flds]'), g_l('import', '[rcd_flds]'), g_l('import', '[attributes]')];
		$rows = [];

		$i = 0;
		foreach($records as $record){
			$hdns .= we_html_element::hidden('records[' . $i . ']', $record);
			$sct_we_fields = (new we_html_select([
					'name' => 'we_flds[' . $record . ']',
					'class' => 'weSelect',
					]))
				->addOption('', g_l('import', '[any]'));

			foreach($val_nodes as $value => $text){
				$sct_we_fields->addOption(htmlspecialchars($value), $text);
				if(isset($we_flds[$record])){
					if($value == $we_flds[$record]){
						$sct_we_fields->selectOption($value);
					}
				} elseif($value == $record){
					$sct_we_fields->selectOption($value);
				}
			}
			switch($record){
				case we_contents_root::TITLE_FIELD:
					$new_record = g_l('import', '[we_title]');
					break;
				case we_contents_root::DESCRIPTION_FIELD:
					$new_record = g_l('import', '[we_description]');
					break;
				case we_contents_root::KEYWORD_FIELD:
					$new_record = g_l('import', '[we_keywords]');
					break;
				default:
					$new_record = '';
			}
			$rows[] = [
				($new_record != '') ? $new_record : $record,
				$sct_we_fields->getHTML(),
				we_html_tools::textInput('attrs[' . $record . ']', (empty($attrs[$record]) ? '' : (base64_decode($attrs[$record]) ?: '')), 255, [], 'text', '100px')
			];
			$i++;
		}

		// Associated prefix selector.
		$asocPfx = (new we_html_table(['class' => 'default'], 1, 1))
			->setCol(0, 0, g_l('import', '[pfx]') . we_html_element::br(2) .
			we_html_tools::textInput('v[asoc_prefix]', ($v['asoc_prefix'] ?? g_l('import', ($v['import_type'] === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'))), 255, [
				'onclick' => "WE().util.dialogTop(window).setFormField('v[rdo_filename]', true, 'radio', 0);"], "text", '150px'));

		// Assigned record or attribute field selectors.
		$rcdPfxSelect = (new we_html_select(['name' => 'v[rcd_pfx]',
				'class' => 'weSelect',
				'onclick' => "WE().util.dialogTop(window).setFormField('v[pfx_fn]', 1, 'hidden'); WE().util.dialogTop(window).setFormField('v[rdo_filename]', true, 'radio', 1);",
				'style' => 'width: 150px'
				]))
			->addOptions($val_nodes);

		if(isset($v['rcd_pfx'])){
			$rcdPfxSelect->selectOption($v['rcd_pfx']);
		}


		$attPfxSelect = we_html_tools::textInput('v[att_pfx]', (empty($v['att_pfx']) ? '' : (base64_decode($v['att_pfx']) ?: '')), 255, ['onclick' => "WE().util.dialogTop(window).setFormField('v[rdo_filename]', true, 'radio', 1);"], "text", '100px');

		$asgndFld = new we_html_table(['class' => 'default'], 1, 3, [
			new we_html_tr([
				new we_html_td(g_l('import', '[rcd_fld]') . we_html_element::br(2) . $rcdPfxSelect->getHTML()),
				new we_html_td('', ['width' => '20px']),
				new we_html_td(g_l('import', '[attributes]') . we_html_element::br(2) . $attPfxSelect)
		])]);

		// Filename selector.
		$fn = new we_html_table(['class' => 'default'], 3, 1, [new we_html_tr([new we_html_td(we_html_forms::radiobutton('0', (!isset($v['rdo_filename']) ? true : ($v['rdo_filename'] == 0)), 'v[rdo_filename]', g_l('import', '[auto]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=0;"))]),
			new we_html_tr([new we_html_td($asocPfx->getHTML(), ['style' => 'padding-left:25px;'])]),
			new we_html_tr([new we_html_td(we_html_forms::radiobutton('1', (!isset($v['rdo_filename']) ? false : ($v['rdo_filename'] == 1)), "v[rdo_filename]", g_l('import', '[asgnd]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=1;"), [
					'style' => 'padding-top:5px;'])]),
			new we_html_tr([new we_html_td($asgndFld->getHTML(), ['style' => 'padding-left:25px;'])])
		]);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', '<br/>' . we_html_tools::dialogBorder(510, $rows, $th, 'defaultfont')));
		if(!empty($dateFields)){
			// Timestamp
			$tStamp = (new we_html_table(['class' => 'default withSpace'], 4, 1))
				->setCol(0, 0, we_html_forms::radiobuttonV('Unix', $v['rdo_timestamp'], 'v[rdo_timestamp]', g_l('import', '[uts]'), true, 'defaultfont', '', false, g_l('import', '[unix_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(1, 0, we_html_forms::radiobuttonV('GMT', $v['rdo_timestamp'], 'v[rdo_timestamp]', g_l('import', '[gts]'), true, 'defaultfont', '', false, g_l('import', '[gmt_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(2, 0, we_html_forms::radiobuttonV('Format', $v['rdo_timestamp'], 'v[rdo_timestamp]', g_l('import', '[fts]'), true, 'defaultfont', '', false, g_l('import', '[format_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(3, 0, we_html_tools::textInput('v[timestamp]', ($v['timestamp'] ?? ''), 0, ['onclick' => "WE().util.dialogTop(window).setFormField('v[rdo_timestamp]', true, 'radio', 2);"], 'text', '150px'), [
				'style' => 'padding-left:25px;']);

			$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[format_date]'), $tStamp->getHTML()))
					->space(we_html_multiIconBox::SPACE_MED)
			);
			if(!isset($v['dateFields'])){
				$hdns .= we_html_element::hidden('v[dateFields]', implode(',', $dateFields));
			}
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[name]'), $fn->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline());

		$conflict = $v['collision'] ?? we_exim_ExIm::COLLISION_RENAME;
		$fn_colsn = (new we_html_table(['class' => 'default withSpace'], 3, 1))
			->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $conflict, 'nameconflict', g_l('import', '[rename]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_RENAME . "';"))
			->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_REPLACE, $conflict, 'nameconflict', g_l('import', '[replace]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_REPLACE . "';"))
			->setCol(2, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $conflict, 'nameconflict', g_l('import', '[skip]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_SKIP . "';"));

		$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[name_collision]'), $fn_colsn->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED2)
		);

		$znr = -1;

		return $hdns .
				(new we_html_multiIconBox($parts, 'xml'))
				->marginLeft()
				->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
				->headline(g_l('import', '[assign_record_fields]'))
				->get();
	}

	protected function getWizCmdProgressStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attributes): string{
		$fields = [];
		$xp = new we_xml_parser($v['uniquePath'] . '/temp_' . $v['cid'] . '.xml');
		foreach($records as $record){
			$nodeSet = $xp->evaluate($xp->root . '/' . $we_flds[$record]);
			$xPath = '';
			$loop = 0;
			$firstNode = '';
			foreach($nodeSet as $node){
				'@phan-var string $node';
				if($loop == 0){
					$firstNode = $node;
					$loop++;
				}
				$list = $xp->getAttributes($node);
				$flag = true;
				$decAttrs = we_weTag_parser::parseAttribs(base64_decode($attributes[$record]) ?: '');
				foreach($decAttrs as $key => $value){
					if(!isset($list[$key]) || $list[$key] != $value){
						$flag = false;
					}
				}
				if($flag){
					$xPath = $node;
					break;
				}
			}
			if($xPath == ''){
				$xPath = $firstNode;
			}
			$fields = array_merge($fields, [$record => $xp->getData($xPath)]);
		}
		if($v['pfx_fn'] == 1){
			$v['rcd_pfx'] = $xp->getData($xp->root . '/' . $v['rcd_pfx'] . '[1]');
			if($v['rcd_pfx'] == ''){
				$v['rcd_pfx'] = g_l('import', ($v['import_type'] === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'));
			}
		}
		return parent::getWizCmdProgressStepBase($jsCmd, $v, $this->getHdns('v', $v) . $this->getHdns('records', $records) . $this->getHdns('we_flds', $we_flds) . $this->getHdns('attributes', $attributes), $fields);
	}

	protected function getWizCmdPrepareStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attrs, array $attributes): string{
		$parse = new we_xml_splitFile(WE_DOCUMENT_ROOT . $v['import_from']);
		$parse->splitFile('*/' . $v['rcd'], $v['from_elem'] ?? false, $v['to_elem'] ?? false, 1);
		return parent::getWizCmdPrepareStepBase($jsCmd, $this->getHdns('v', $v) .
				$this->getHdns('records', $records) .
				$this->getHdns('we_flds', $we_flds) .
				$this->getHdns('attributes', $attributes) .
				$this->getHdns('attrs', $attrs) .
				we_html_element::hiddens([
					'v[numFiles]' => (string) $parse->fileId,
					'v[uniquePath]' => $parse->path
		]));
	}

}
