<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8487b712d257 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-11-03 12:25 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_newsletter_util{
	/** @var int set to unset */
	public static $REMOVENEWSLETTER_STATUS = 0;

	/** @var int set to unset */
	public static $WRITENEWSLETTER_STATUS = 0;

	/** @var string */
	public static $MAIL = '';

	/** @var string */
	public static $TITLE = '';

	/** @var string */
	public static $SALUTATION = '';

	/** @var string */
	public static $FIRSTNAME = '';

	/** @var string */
	public static $LASTNAME = '';

	/** @var string */
	public static $CONFIRMLINK = '';

	public const PLACEHOLDER_PREFIX = '####PLACEHOLDER:DB::CUSTOMER_TABLE:';

	/** @var string */
	public static $CUSTOMERID = '';

	/**
	 *
	 * @param bool $customer
	 * @param string $emailField
	 * @param string[] $abos
	 * @param string[] $paths
	 * @param string $unsubscribe_mail
	 * @return bool
	 */
	static function unsubscribeNewsletter(bool $customer, string $emailField, array $abos, array $paths, string $unsubscribe_mail): bool{
		$db = DB_WE::$inst;
		self::$REMOVENEWSLETTER_STATUS = 0;
		$unsubscribe_mail = strtolower(preg_replace("|[\r\n,]|", '', $unsubscribe_mail));
		self::$MAIL = $unsubscribe_mail;
		if(!we_check_email($unsubscribe_mail)){
			self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_INVALID; // E-Mail ungueltig
			return false;
		}

		$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE subscribe_mail="' . $db->escape($unsubscribe_mail) . '"');

		$emailExists = ($customer ?
			self::removeFromDB($emailField, $unsubscribe_mail, $abos) :
			self::removeFromFile($paths, $unsubscribe_mail));

		if($emailExists){
			self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
			return true;
		}
		self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
		return false;
	}

	/**
	 * @param string $emailField
	 * @param string $unsubscribe_mail
	 * @param string[] $abos
	 * @return bool
	 */
	private static function removeFromDB(string $emailField, string $unsubscribe_mail, array $abos): bool{
		$db = DB_WE::$inst;
		$meta = $db->metadata(CUSTOMER_TABLE);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int,default:string}> $meta';
		$tmp = [];
		foreach($abos as $abo){
			$tmp[] = '"' . $db->escape($abo) . '"';
		}
		$where = ' WHERE ' . $db->escape($emailField) . '="' . $db->escape($unsubscribe_mail) . '"';
		$hash = getHash('SELECT ' . implode(',', $tmp) . ' FROM ' . CUSTOMER_TABLE . $where);
		unset($tmp);
		$update = [];
		if(!$hash){
			return false;
		}
		$emailExists = false;
		foreach($abos as $abo){
			$fieldDefaults = explode(',', (isset($meta[$abo]) ? $meta[$abo]['default'] : '')); //FIXME:default vals does not work
			$aboNeg = count($fieldDefaults) > 1 ? $fieldDefaults[0] : '';

			$dbAbo = $hash[$abo];
			if(!empty($dbAbo) || $dbAbo != $aboNeg){
				$update[$abo] = $aboNeg;
				$emailExists = true;
			}
		}
		if($emailExists){
			$fields = [
				'Mod_Date' => sql_function('NOW()'),
				'ModifiedBy' => 'frontend',
			];
			$hook = new we_hook_handler('customer_preSave', ['customer' => &$fields, 'from' => 'tag', 'type' => 'modify', 'tagname' => 'addDelNewsletterEmail', 'isSubscribe' => false,
				'isUnsubscribe' => true]);
			$hook->executeHook();
			$db->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter(array_merge($update, $fields)) . ' ' . $where);
		}
		return $emailExists;
	}

	/**
	 *
	 * @param string[] $paths
	 * @param string $unsubscribe_mail
	 * @return bool
	 */
	private static function removeFromFile(array $paths, string $unsubscribe_mail): bool{
		$emailExists = false;
		foreach($paths as $path){
			'@phan-var string $path';
			$path = WE_DOCUMENT_ROOT . we_base_file::addPreSlash($path);

			if(!file_exists(dirname($path))){
				t_e('file ' . $path . ' doesn\'t exist');
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return false;
			}

			// #4158
			while(($lock = we_base_file::lock(basename(__FILE__))) == false){
				usleep(500000);
			}

			$file = file($path);
			if(!$file){
				$lock !== false && we_base_file::unlock($lock);
				continue;
			}

			$fileChanged = false;
			$regex = '|,' . preg_quote($unsubscribe_mail, '|') . ',|i';
			foreach($file as $i => $line){
				if(preg_match($regex, $line)){
					$emailExists = true;
					unset($file[$i]);
					$fileChanged = true;
				}
			}

			if($fileChanged && !we_base_file::save($path, implode("\n", array_map('trim', $file)) . "\n")){
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return false;
			}
			if($lock){
				we_base_file::unlock($lock);
			}
		}
		return $emailExists;
	}

	/**
	 *
	 * @param string $confirmid
	 * @param string $mail
	 * @param string $subscribe_mail
	 * @return array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]}
	 */
	static function getNewsletterFields(string $confirmid, string $mail, string $subscribe_mail): array{
		if($confirmid){
			if(!preg_match('|^([a-f0-9]){32}$|', $confirmid)){
				t_e('confirmid was not valid hexstring of length 32', $confirmid);
				throw new we_exception_newsletter('err', we_newsletter_base::STATUS_CONFIRM_FAILED);
			}
			$h = getHash('SELECT subscribe_mail,subscribe_salutation,subscribe_title,subscribe_firstname,subscribe_lastname,lists FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE confirmID=x\'' . DB_WE::$inst->escape($confirmid) . '\' AND subscribe_mail="' . DB_WE::$inst->escape(strtolower($mail)) . '"');
			'@phan-var array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]} $h';
			if(!$h){
				throw new we_exception_newsletter('err', we_newsletter_base::STATUS_CONFIRM_FAILED);
			}
			$h['lists'] = explode(',', (string) $h['lists']);
			return $h;
		}

		$subscribe_mail = trim(preg_replace("|[\r\n,]|", '', $subscribe_mail));
		if(!$subscribe_mail){
			throw new we_exception_newsletter('err', we_newsletter_base::STATUS_EMAIL_INVALID);
		}

		if(!we_check_email($subscribe_mail)){
			throw new we_exception_newsletter('err', we_newsletter_base::STATUS_EMAIL_INVALID); // E-Mail ungueltig
		}

		return [
			'subscribe_mail' => $subscribe_mail,
			'subscribe_salutation' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::STRING, 'we_subscribe_salutation__', ''))),
			'subscribe_title' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::STRING, 'we_subscribe_title__', ''))),
			'subscribe_firstname' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::STRING, 'we_subscribe_firstname__', ''))),
			'subscribe_lastname' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::STRING, 'we_subscribe_lastname__', ''))),
			'lists' => [],
		];
	}

	/**
	 *
	 * @param int $adminmailid
	 * @param string $adminemail
	 * @param string $adminsubject
	 * @param array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]} $f
	 * @param bool $includeimages
	 * @return void
	 * @throws we_exception_mail
	 */
	static function mailNewSuccessfullNewsletterActiviation(int $adminmailid, string $adminemail, string $adminsubject, string $adminFrom, array $f, bool $includeimages): void{
		if(!($adminmailid && ($mailadr = we_mail_address::parse($adminemail)))){//inform admin of the new account
			throw new we_exception_mail('attribute adminmailid or adminemail missing');
		}
		$phpmail = new we_mail_mail([$mailadr], $adminsubject, we_mail_address::parse($adminFrom), we_mail_address::parse($adminFrom));

		$adminmailtextHTML = strtr(
			(($adminmailid > 0) && we_base_file::isWeFile($adminmailid, FILE_TABLE) ? we_getDocumentByID($adminmailid, '') : '')
			, [
			'###MAIL###' => $f['subscribe_mail'],
			'###SALUTATION###' => $f['subscribe_salutation'],
			we_newsletter_base::TITLE_REPLACE_TEXT => $f['subscribe_title'],
			we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $f['subscribe_firstname'],
			we_newsletter_base::LASTNAME_REPLACE_TEXT => $f['subscribe_lastname'],
		]);
		$phpmail->addHTMLPart($adminmailtextHTML);
		$phpmail->setIsEmbedImages($includeimages);
		$phpmail->buildMessage();
		$phpmail->send();
	}

	/**
	 *
	 * @param we_database_base $db
	 * @param string $type
	 * @param string $customer_email_field
	 * @param array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]} $f
	 * @param string[] $abos
	 * @param string[] $paths
	 * @param int $mailid
	 * @param bool $includeimages
	 * @param int $expires
	 * @param int $confirmDocumentID
	 * @param string $subject
	 * @param string $from
	 * @param string[] $toCC
	 * @param string[] $toBCC
	 * @return void
	 */
	public static function addDoubleOptIn(we_database_base $db, string $type, string $customer_email_field, array $f, array $abos, array $paths, int $mailid, bool $includeimages, int $expires, int $confirmDocumentID, string $subject, string $from, array $toCC, array $toBCC): void{
		if(!$mailid){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			return;
		}

		$confirmID = md5(uniqid($f['subscribe_mail'], true));
		$lists = [];
		$emailExistsInAllAbos = false;
		switch($type){
			case 'customer':
				// #5589 start
				if(empty($customer_email_field)){
					t_e('Error in newsletter settings: The field for the customer mail-address is not set');
					return;
				}
				if(($hash = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customer_email_field . '="' . $db->escape($f['subscribe_mail']) . '"'))){
					foreach($abos as $cAbo){
						if(empty($hash[$cAbo])){
							$lists[] = $cAbo;
						}
					}
					if(empty($lists)){// subscriber exists in all lists
						$emailExistsInAllAbos = true;
					}
					// #5589 end
				} else {
					$lists = $abos; //#9002
				}

				break;
			case 'csv':
				foreach($paths as $p){
					$realPath = realpath(str_starts_with($p, '/') ? (WE_DOCUMENT_ROOT . $p) : (WE_DOCUMENT_ROOT . '/' . $p)) ?: '';
					if(!$realPath || !file_exists($realPath)){
						t_e('newsletter file not found', $p, $realPath);
						self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						return;
					}
					try{
						$file = we_base_file::load($realPath);
					} catch (we_exception_file $e){
						t_e('newsletter file unable to read', $p, $realPath);
						self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						return;
					}
					if(!preg_match("%[\r\n]" . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file) && !preg_match('%^' . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file)){
						$lists[] = $p;
					}
				}
				if(empty($lists)){// #5135 subscriber exists in all lists
					$emailExistsInAllAbos = true;
				}
				break;
			case 'emailonly':
				$lists = $abos;
				break;
		}
		if($emailExistsInAllAbos){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
			return;
		}

		$db->query('REPLACE INTO ' . NEWSLETTER_CONFIRM_TABLE . ' SET ' .
			we_database_base::arraySetter([
				'confirmID' => sql_function('x\'' . $confirmID . '\''),
				'subscribe_mail' => strtolower($f['subscribe_mail']),
				'subscribe_salutation' => $f['subscribe_salutation'],
				'subscribe_title' => $f['subscribe_title'],
				'subscribe_firstname' => $f['subscribe_firstname'],
				'subscribe_lastname' => $f['subscribe_lastname'],
				'lists' => implode(',', $lists),
				'expire' => sql_function('NOW() + INTERVAL ' . $expires . ' MINUTE ') // in secs
		]));

		$use_https_refer = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="use_https_refer"');
		$protocol = ($use_https_refer ? 'https://' : 'http://');

		$port = $use_https_refer ? 443 : 80;
		$basehref = $protocol . $_SERVER['SERVER_NAME'] . ':' . $port;

		$cnt = 0;
		$confirmLink = ($confirmDocumentID ? id2path($confirmDocumentID, FILE_TABLE) : we_http_util::getScriptName()) . '?' . http_build_query(['confirmID' => $confirmID,
				'mail' => $f['subscribe_mail']]);
		if(($urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true, true))){
			$confirmLink = str_replace('//', $protocol, preg_replace($urlReplace, array_keys($urlReplace), $confirmLink, -1, $cnt));
		}

		self::$MAIL = $f['subscribe_mail'];
		self::$TITLE = we_newsletter_base::TITLE_REPLACE_TEXT;
		self::$SALUTATION = $f['subscribe_salutation'];
		self::$FIRSTNAME = $f['subscribe_firstname'];
		self::$LASTNAME = $f['subscribe_lastname'];
		self::$CONFIRMLINK = ($cnt == 0 ? $protocol . $_SERVER['SERVER_NAME'] . ($port !== 80 ? ':' . $port : '') : '') . $confirmLink;

		$mailtextHTML = we_base_file::isWeFile($mailid, FILE_TABLE) ? we_getDocumentByID($mailid) : '';
		$mailtextHTML = str_replace(we_newsletter_base::TITLE_REPLACE_TEXT, $f['subscribe_title'], ($f['subscribe_title'] ? preg_replace('%([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '%', '${1} ' . $f['subscribe_title'], $mailtextHTML) : $mailtextHTML));

		$placeholderfieldsmatches = [];
		preg_match_all('/' . we_newsletter_util::PLACEHOLDER_PREFIX . '(.[^#]{1,200})####/', $mailtextHTML, $placeholderfieldsmatches);
		$placeholderfields = $placeholderfieldsmatches[1];
		unset($placeholderfieldsmatches);

		$customerHash = ($type === 'customer' ?
			array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customer_email_field . '="' . $db->escape($f['subscribe_mail']) . '"'), we_customer_customer::getEncryptedFields()) :
			[]);
		if(!empty($placeholderfields)){
			foreach($placeholderfields as $phf){
				$placeholderReplaceValue = $customerHash[$phf] ?? '';
				//$mailtext = str_replace(self::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $mailtext);
				$mailtextHTML = str_replace(self::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $mailtextHTML);
			}
		}
		$to = we_mail_address::parse($f['subscribe_mail']);
		if(!$to){
			throw new we_exception_mail('no recipient given');
		}
		$phpmail = new we_mail_mail([$to], $subject, we_mail_address::parse($from), we_mail_address::parse($from));

		foreach($toCC as $cc){
			if(!str_contains($cc, '@')){
				if(!empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser'][$cc]) && str_contains($_SESSION['webuser'][$cc], '@')){ //wenn man registrierten Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$cc])){
						$phpmail->addCc(we_mail_address::parse($_SESSION['webuser'][$cc]));
					}
				} else if(($tmpcc = we_base_request::string($cc)) &&
					str_contains($tmpcc, '@') &&
					we_check_email(($tmpcc = we_base_request::_(we_base_request::EMAIL, $cc)))){
					$phpmail->addCc(we_mail_address::parse($tmpcc));
				}
			} elseif(we_check_email($cc)){
				$phpmail->addCc(we_mail_address::parse($cc));
			}
		}

		foreach($toBCC as $bcc){
			if(!str_contains($bcc, '@')){
				if(!empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser'][$bcc]) && str_contains($_SESSION['webuser'][$bcc], '@')){ //wenn man registrierte Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$bcc])){
						$phpmail->addBcc(we_mail_address::parse($_SESSION['webuser'][$bcc]));
					}
				} else if(($tmpbcc = we_base_request::string($bcc)) &&
					str_contains($tmpbcc, '@') &&
					we_check_email(($tmpbcc = we_base_request::_(we_base_request::EMAIL, $bcc)))){
					$phpmail->addBcc(we_mail_address::parse($tmpbcc));
				}
			} elseif(we_check_email($bcc)){
				$phpmail->addBcc(we_mail_address::parse($bcc));
			}
		}

		if($includeimages){
			$phpmail->setIsEmbedImages($includeimages);
		} else {
			$phpmail->setBaseDir($basehref);
		}

		$phpmail->addHTMLPart($mailtextHTML);
		//what about attachments/inlines??
		$phpmail->buildMessage();
		$phpmail->send();
		we_tag_addDelNewsletterEmail::$isDoubleOptIn = true;
	}

	/**
	 *
	 * @param string[] $paths
	 * @param array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]} $insertRow
	 * @return void
	 */
	static function addToFile(array $paths, array $insertRow): void{
		$emailwritten = false;
		$dr = WE_DOCUMENT_ROOT;
		foreach($paths as $p){
			$realPath = (string) WE_DOCUMENT_ROOT . we_base_file::addPreSlash($p);
			if(!$realPath || !file_exists(dirname($realPath)) || !str_contains($realPath, $dr)){
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return;
			}

			while(($lock = we_base_file::lock(basename(__FILE__))) == false){
				usleep(500000);
			}

			// make sure the file exists before loading it!
			try{
				we_base_file::save($realPath, '', 'ab+');
			} catch (we_exception_base $e){
				$lock !== false && we_base_file::unlock($lock);
				t_e('save of file ' . $p . ' failed');
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return;
			}

			if(($file = we_base_file::load($realPath)) && (
				preg_match("%[\r\n]" . $insertRow['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file) ||
				preg_match('%^' . $insertRow['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file))){
				// E-Mail schon vorhanden => Nix tun
			} else {
				$row = $insertRow['subscribe_mail'] . ',' . $insertRow['subscribe_salutation'] . ',' . $insertRow['subscribe_title'] . ',' . $insertRow['subscribe_firstname'] . ',' . $insertRow['subscribe_lastname'] . "\n";
				we_base_file::save($realPath, $row, 'ab+');
				$emailwritten = true;
				$lock !== false && we_base_file::unlock($lock);
			}
		}
		if(!$emailwritten){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
		}
	}

	static function addToDB(array $customerFieldPrefs, array $f, array $abos): void{
		try{
			$customerEmailField = we_newsletter_view::getCustomerEmailField();
		} catch (Exception $ex){
			t_e('warning', $ex->getMessage());
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			return;
		}

		$uid = ($uid = fI('SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE Username="' . DB_WE::$inst->escape($f['subscribe_mail']) . '"')) ? $uid :
			($customerEmailField ? fI('SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customerEmailField . '="' . DB_WE::$inst->escape($f['subscribe_mail']) . '"') : 0);

		$fields = (!$uid ? [
			'Username' => $f['subscribe_mail'],
			'Password' => we_customer_customer::cryptPassword((new we_customer_generatePassword(12, ['caps' => 3, 'small' => 4, 'nums' => 3, 'specs' => 2]))->PassGen()),
			'MemberSince_Date' => new DateTime(),
			'LoginDenied' => 0,
			'LastLogin_Date' => null,
			'LastAccess_Date' => null,
			(!empty($customerFieldPrefs['customer_salutation_field']) && $customerFieldPrefs['customer_salutation_field'] !== 'ID' ? $customerFieldPrefs['customer_salutation_field'] : '') => $f['subscribe_salutation'],
			(!empty($customerFieldPrefs['customer_title_field']) && $customerFieldPrefs['customer_title_field'] !== 'ID' ? $customerFieldPrefs['customer_title_field'] : '') => $f['subscribe_title'],
			(!empty($customerFieldPrefs['customer_firstname_field']) && $customerFieldPrefs['customer_firstname_field'] !== 'ID' ? $customerFieldPrefs['customer_firstname_field'] : '') => $f['subscribe_firstname'],
			(!empty($customerFieldPrefs['customer_lastname_field']) && $customerFieldPrefs['customer_lastname_field'] !== 'ID' ? $customerFieldPrefs['customer_lastname_field'] : '') => $f['subscribe_lastname'],
			($customerEmailField !== 'Username' ? $customerEmailField : '') => $f['subscribe_mail'],
			] : [
			'Mod_Date' => new DateTime(),
			'ModifiedBy' => 'frontend',
			($customerEmailField !== 'Username' ? $customerEmailField : '') => $f['subscribe_mail'],
			]
			);

		$hook = new we_hook_handler('customer_preSave', ['customer' => &$fields, 'from' => 'tag', 'type' => (!$uid ? 'new' : 'modify'), 'tagname' => 'addDelNewsletterEmail',
			'isSubscribe' => true, 'isUnsubscribe' => false]);
		$hook->executeHook();

		if(!(DB_WE::$inst->query($uid ?
					'UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($fields) . ' WHERE ID=' . $uid :
					'INSERT INTO ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($fields)))){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
		}

		$set = [];
		$checkAbos = [];
		$meta = DB_WE::$inst->metadata(CUSTOMER_TABLE);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int,default:string}> $meta';
		foreach($abos as $abo){
			if(!empty($meta[$abo]['default'])){
				$setVals = explode(',', $meta[$abo]['default']);
			} else if(!empty($meta['Newsletter_Ok']['default'])){
				$setVals = explode(',', $meta['Newsletter_Ok']['default']);
			} else {
				$setVals = ['', '1'];
			}

			[$setDefault, $setVal] = (count($setVals) > 1 ? $setVals : ['', $setVals[0]]);

			if(!DB_WE::$inst->isColExist(CUSTOMER_TABLE, $abo)){
				DB_WE::$inst->addCol(CUSTOMER_TABLE, $abo, 'VARCHAR(200) DEFAULT "' . DB_WE::$inst->escape($setDefault) . '"', 'Username');
			}
			$set[$abo] = $setVal;
			$checkAbos[] = DB_WE::$inst->escape($abo);
		}

		$existingAbos = array_filter(getHash('SELECT ' . implode(', ', $checkAbos) . ' FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customerEmailField . '="' . DB_WE::$inst->escape($f['subscribe_mail']) . '"'));
		if(count($existingAbos) === count($checkAbos)){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
		} else {
			DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set) . ' WHERE ' . $customerEmailField . '="' . DB_WE::$inst->escape($f['subscribe_mail']) . '"');
		}
	}

}
