<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d1e386d2ec90 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-23 18:42 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_objectfileFolderAction extends we_rpcCmd_folderAction{

	function execute(): we_rpc_response{
		$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 'transaction');
		$we_doc = we_contents_root::getSessionDocument($transaction);
		assert($we_doc instanceof we_folder_objectFile);

		if(!$we_doc->ID){
			return $this->resp;
		}

		$ids = [];
		$arr = we_base_request::string('we_cmd', '', 1);

		if($arr){
			$allDocs = explode(',', $arr);
			foreach($allDocs as $v){
				$parts = explode('_', $v, 2);
				$ids[] = $parts[1] === OBJECT_FILES_TABLE ? intval($parts[0]) : 0;
			}
			$ids = array_filter($ids);
		}

		if(!$ids){
			return $this->resp;
		}

		// FIXME: send processed items and some message
		$message = '';
		$jsCmd = new we_base_jsCmd();
		switch(we_base_request::string('we_cmd', '', 0)){
			case we_folder_base::ACTION_DELETE:
				$message = self::deleteObjects($ids, $jsCmd);
				break;
			case we_folder_base::ACTION_UNPUBLISH:
				$we_doc->publishObjects($ids, false);
				break;
			case we_folder_base::ACTION_PUBLISH:
				$we_doc->publishObjects($ids);
				break;
			case we_folder_base::ACTION_CATEGORY_ADD:
				$cats = we_base_request::intList('we_cmd', [], 2);
				if(!$cats){
					break;
				}
				self::changeCats([OBJECT_FILES_TABLE => $ids], $cats, true);
				break;
			case we_folder_base::ACTION_CATEGORY_DELETE:
				$cats = we_base_request::intList('we_cmd', [], 2);
				if(!$cats){
					break;
				}
				self::changeCats([OBJECT_FILES_TABLE => $ids], $cats, false);
				break;
			case we_folder_base::ACTION_UNSEARCHABLE:
				$we_doc->setObjectProperty($ids, 'IsSearchable', false);
				break;
			case we_folder_base::ACTION_SEARCHABLE:
				$we_doc->setObjectProperty($ids, 'IsSearchable', true);
				break;
			case we_folder_base::ACTION_COPY_WS:
				$we_doc->setObjectProperty($ids, 'Workspaces');
				break;
			case we_folder_base::ACTION_COPY_TID:
				$we_doc->setObjectProperty($ids, 'TriggerID');
				break;
		}
		$this->resp->setData('message', $message);
		if(!$jsCmd->isEmpty()){
			$this->resp->setData('jscmd', $jsCmd->getRPCCmds());
		}
		return $this->resp;
	}

	public static function deleteObjects(array $IDs, we_base_jsCmd $jsCmd): string{
		if(!$IDs){
			return '';
		}

		$deletedItems = [];

		foreach($IDs as $tid){
			$tid = (int) $tid;
			if(we_base_permission::checkIfRestrictUserIsAllowed($tid, OBJECT_FILES_TABLE)){
				we_base_delete::deleteEntry($tid, OBJECT_FILES_TABLE, false, false);
				$deletedItems[] = $tid;
			}
		}
		// write some answer message and send $deletedItems
		$jsCmd->addCmd('closeDeletedEntries', OBJECT_FILES_TABLE, $deletedItems);
		return '';
	}

}
