<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e5bf7f91c9c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 13:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_shop{
	const IGNORED_EDIT_FIELDS = [
		'ID', 'Username', 'Password', 'MemberSince_Date', 'LastLogin_Date', 'LastAccess_Date', 'ParentID', 'Path', 'IsFolder', 'Text', 'AutoLogin', 'AutoLoginDenied',
		'ModifiedBy', 'Mod_Date'];
	const IGNORED_EXTRA_SHOW_FIELDS = ['Forename', 'Surname'];
	// name of request array for shopping items
	const ARTICLE_CUSTOM_FIELD = 'we_sacf';
	const CART_CUSTOM_FIELD = 'we_sscf';
	//const WE_SHOP_CART_CUSTOMER_FIELD = 'we_shopCustomer';
	const TITLE_FIELD_NAME = 'shoptitle';
	//const WE_SHOP_DESCRIPTION_FIELD_NAME = 'shopdescription';
	const VAT_FIELD_NAME = 'shopvat'; // due to the names of old fields (shoptitle, shopdescription) - we must name shopvat
	const CATEGORY_FIELD_NAME = 'shopcategory';
	//const WE_SHOP_PRICE_IS_NET_NAME = 'we_shopPriceIsNet';
	//const WE_SHOP_PRICENAME = 'we_shopPricename';
	const SHIPPING_NAME = 'we_shopPriceShipping';
	const DOCUMENT = 'w';
	const OBJECT = 'o';

	//const WE_SHOP_CALC_VAT = 'we_shopCalcVat';

	static function getCustomersOrderList(int $customerId, bool $sameModul = true): string{
		$weShopStatusMails = we_shop_statusMails::getShopStatusMails();

		$hidden = array_keys(array_filter($weShopStatusMails->FieldsHiddenCOV, function ($v): bool{
				return $v == 1;
			}));
		$showBaseFields = array_diff(we_shop_statusMails::BaseDateFields, $hidden);
		$showDateFields = array_diff(we_shop_statusMails::StatusFields, $hidden);
		$showAdvanced = array_diff($showDateFields, $showBaseFields);

		$format = [];
		foreach($showBaseFields as $field){
			$format[] = 'DATE_FORMAT(' . $field . ',"' . g_l('date', '[format][mysql]') . '") AS format' . $field;
		}
		$advanced = ($showAdvanced ?
			DB_WE::$inst->getAllFirstq('SELECT CONCAT(odt.ID,odt.type),DATE_FORMAT(odt.date,"' . g_l('date', '[format][mysql]') . '") FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_DATES_TABLE . ' odt ON odt.ID=o.ID WHERE odt.type IN ("' . implode('","', $showAdvanced) . '") AND customerID=' . $customerId . ' ORDER BY odt.ID', false) :
			[]);

		$all = DB_WE::$inst->getAllFirstq('SELECT ID,shopname' . ($format ? ',' . implode(',', $format) : '') . ' FROM ' . SHOP_ORDER_TABLE . ' WHERE customerID=' . $customerId . ' ORDER BY ID DESC');

		$orderStr = '<table style="width:100%">';
		if($all){
			$orderStr .= '<tr><td></td><td><b>' . g_l('modules_shop', '[orderList][order]') . '</b></td>';

			foreach($showDateFields as $field){
				$orderStr .= '<td><b>' . $weShopStatusMails->FieldsText[$field] . '</b></td>';
			}

			$orderStr .= '</tr>';

			foreach($all as $id => $entries){

				$orderStr .= '<tr>';
				if(we_base_permission::hasPerm([we_base_perm::SHOP_EDIT_ORDER])){
					$orderStr .= ($sameModul ?
						('<td>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.content.editor.location=WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=editor&bid=" . $id . "';") . '</td>') :
						('<td>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.content.document.location=WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&bid=" . $id . "';") . '</td>')
						);
				} else {
					$orderStr .= '<td></td>';
				}
				$orderStr .= '<td>' . $id . ($entries['shopname'] ? ' (' . $entries['shopname'] . /* '. ' . g_l('modules_shop', '[orderList][order]') . */ ')' : '') . '</td>';
				foreach($showDateFields as $field){
					$val = (in_array($field, $showBaseFields) ?
						$entries['format' . $field] :
						$advanced[$id . $field]);

					$orderStr .= '<td>' . ( $val ?: '-' ) . '</td>';
				}

				$orderStr .= '</tr>';
			}
		} else {
			$orderStr .= '<tr><td>' . g_l('modules_shop', '[orderList][noOrders]') . '</td></tr>';
		}
		$orderStr .= '</table>';

		return $orderStr;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'del_shop' => g_l('modules_shop', '[del_shop]'),
				'delete_alert' => g_l('modules_shop', '[delete_alert]'),
				'delete_shipping' => g_l('modules_shop', '[delete][shipping]'),
				'field_empty_js_alert' => g_l('modules_shop', '[field_empty_js_alert]'),
				'no_order_there' => (g_l('modules_shop', '[no_order_there]')),
				'no_number' => g_l('modules_shop', '[keinezahl]'),
				'no_perms' => (g_l('modules_shop', '[no_perms]')),
				'nothing_to_delete' => (g_l('modules_shop', '[nothing_to_delete]')),
				'nothing_to_save' => (g_l('modules_shop', '[nothing_to_save]')),
				'preferences_save_alert' => g_l('modules_shop', '[preferences][save_alert]'),
				'vat_confirm_delete' => g_l('modules_shop', '[vat][js_confirm_delete]'),
				'tree' => [
					'treeYearClick' => g_l('modules_shop', '[treeYearClick]'),
					'treeYear' => g_l('modules_shop', '[treeYear]'),
				]
		]);
	}

	static function getAllOrderYears(): array{
		$years = DB_WE::$inst->getAllq('SELECT DISTINCT YEAR(DateOrder) a FROM ' . SHOP_ORDER_TABLE . ' ORDER BY a DESC', true);
		if(array_search(date('Y'), $years) === false){
			array_unshift($years, date('Y'));
		}
		return $years;
	}

}
