<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c4fc1a31b4b9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 18:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::SHOP);

abstract class we_tag_createShop extends we_weTag_tag{
	/** @var we_shop_Basket[] */
	static $baskets = [];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['shopname' => false], __CLASS__);
		if(!we_modules_info::isActive(we_modules_info::SHOP)){
			return self::modulMissingError('Shop', __CLASS__);
		}

		$deleteshop = self::getAttribute('deleteshop', $attribs, false, we_base_request::BOOL);
		$deleteshoponlogout = self::getAttribute('deleteshoponlogout', $attribs, false, we_base_request::BOOL);
		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);

		if(!isset($_SESSION)){
			we_base_WE::$session = we_base_sessionHandler::inst();
		}

		$sName = we_base_request::string('shopname');
		if(isset($_SESSION[$shopname . '_save']) && (we_base_request::bool('deleteshop') && (!$sName || $sName === $shopname) || $deleteshop)){ // delete shop
			unset($_SESSION[$shopname . '_save']);
		}
		if(we_tag_sessionStart::$logout && $deleteshoponlogout){
			unset($_SESSION[$shopname . '_save']);
		}

		self::$baskets[$shopname] = new we_shop_Basket(($_SESSION[$shopname . '_save'] ?? []));
		self::$baskets[$shopname]->initCartFields();
		$_SESSION[$shopname . '_save'] = self::$baskets[$shopname]->getCartProperties();
		return '';
	}

	public static function getBasket(string $name): we_shop_Basket{
		if(!isset(self::$baskets[$name])){
			throw new we_exception_base('no shop basket with name ' . $name . ' found');
		}
		return self::$baskets[$name];
	}

}
