<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 7e2066a95770 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-11 17:47 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_css extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['id' => false], __CLASS__);
		$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$data = getHash('SELECT Path,CONCAT("m=",UNIX_TIMESTAMP(Publish_Date)) AS Pub FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND ID=' . $id);
		if(!$data){
			return '';
		}
		$src = $data['Path'];
		$m = DISABLE_MEDIA_MODTIME ? '' : $data['Pub'];
		self::$doc = we_base_WE::getWeDoc();

		$nolink = false;
		switch(self::getAttribute('applyto', $attribs, defined('CSSAPPLYTO_DEFAULT') ? CSSAPPLYTO_DEFAULT : 'around', we_base_request::STRING)){
			case 'around' :
				break;
			case 'wysiwyg' :
				$nolink = true;
			case 'all' :
				switch(self::getAttribute('media', $attribs, '', we_base_request::STRING)){
					case '':
					case 'screen':
					case 'all':
						// we still need addDocumentCss() because JS getDocumentCss() won't apply it when applyTo=wysiwyg!
						self::$doc->addDocumentCss($id);
						break;
				}
				break;
		}
		//	remove not needed elements
		$attribs = self::removeAttribs($attribs, ['id', 'applyto']);
		$rel = self::getAttribute('rel', $attribs, 'stylesheet', we_base_request::STRING);
		if($rel === 'preload'){
			$attribs['as'] = 'style';
			header('Link: <' . $src . '>; rel=preload; as=style', false);
		}
		$src .= (we_base_WE::$editmode ?
			// these stylesheets are applied to wysiwyg using addDocumentCss depending on attribute "applyto":
			// so prevent JS getDocumentCss() from applying it again or when applyto=around!
			'?wysiwyg=0' . ($m ? '&' . $m : '') :
			($m ? '?' . $m : '')
			);

		$attribs['rel'] = $rel;
		$attribs['type'] = 'text/css';
		$attribs['href'] = $src;

		return $nolink ? '' : self::htmlTag('link', $attribs) . "\n";
	}

}
