<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 64a321d0ad06 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-21 10:33 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_date extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$format = self::getAttribute('format', $attribs, g_l('date', '[format][default]'), we_base_request::RAW);
		['code' => $langcode] = self::getOutLang($attribs);

		switch(strtolower(self::getAttribute('type', $attribs, 'php', we_base_request::STRING))){
			case 'js':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);

				$js = 'function getDateS(d){
	switch(d){
		case 1:
		case 21:
		case 31:
			return "st";
		case 2:
		case 22:
			return "nd";
		case 3:
		case 23:
			return "rd";
		default:
			return "th";
	}
}
function getDateWord(f,dateObj){
	var l_day_Short = ["' . implode('","', $days['abbreviated']) . '"];
	var l_monthLong = ["' . implode('","', $months['wide']) . '"];
	var l_dayLong = ["' . implode('","', $days['wide']) . '"];
	var l_monthShort = ["' . implode('","', $months['abbreviated']) . '"];
	switch(f){
		case "D":
			return l_day_Short[dateObj.getDay()];
		case "l":
			return l_dayLong[dateObj.getDay()];
		case "F":
			return l_monthLong[dateObj.getMonth()];
		case "M":
			return l_monthShort[dateObj.getMonth()];
	}
}';

				$ret = $js_arr = [];

				for($i = 0; $i < strlen($format); $i++){
					switch($format[$i]){
						case '\\'://skip next char
							$i++;
							$ret[] = '"' . $format[$i] . '"';
							continue 2;
						default:
							$ret[] = '"' . $format[$i] . '"';
							continue 2;
						case 'Y':
							$js_arr['Y'] = 'var Y = heute.getYear()+(heute.getYear()<1900?1900:0);';
							break;
						case 'y':
							$js_arr['y'] = 'var y = (""+heute.getYear()).slice(-2);';
							break;
						case 'a':
							$js_arr['a'] = "var a = (heute.getHours() > 11) ? 'pm' : 'am';";
							break;
						case 'A':
							$js_arr['A'] = "var A = (heute.getHours() > 11) ? 'PM' : 'AM';";
							break;
						case 's':
							$js_arr['s'] = 'var s = heute.getSeconds();';
							break;
						case 'm':
							$js_arr['m'] = "var m = ('0'+(heute.getMonth()+1)).slice(-2);";
							break;
						case 'n':
							$js_arr['n'] = "var n = heute.getMonth()+1;";
							break;
						case 'd':
							$js_arr['d'] = "var d = ('00'+heute.getDate()).slice(-2);";
							break;
						case 'j':
							$js_arr['j'] = "var j = heute.getDate();";
							break;
						case 'h':
							$js_arr['h'] = "var h = (heute.getHours()%12);";
							break;
						case 'H':
							$js_arr['H'] = "var H = ('0'+heute.getHours()).slice(-2);";
							break;
						case 'g':
							$js_arr['g'] = 'var g = (heute.getHours()%12);';
							break;
						case 'G':
							$js_arr['G'] = 'var G = heute.getHours();';
							break;
						case 'i':
							$js_arr['i'] = "var i = ('0'+heute.getMinutes()).slice(-2);";
							break;
						case 'S':
							$js_arr['S'] = 'var S = getDateS(heute.getDate());';
							break;
						case 'D':
							$js_arr['D'] = "var D = getDateWord('D',heute);";
							break;
						case 'F':
							$js_arr['F'] = "var F = getDateWord('F',heute);";
							break;
						case 'l':
							$js_arr['l'] = "var l = getDateWord('l',heute);";
							break;
						case 'M':
							$js_arr['M'] = "var M = getDateWord('M',heute);";
							break;
					}
					$ret[] = $format[$i];
				}
				return we_html_element::jsElement(
						'(function(){
	var heute = new Date();' .
						$js .
						implode('', $js_arr) . '
	document.write(' . stripslashes(implode('+', $ret)) . ');
})();');
			case 'php':
			default:

				return we_base_country::dateformat($langcode, new DateTime(), $format);
		}
	}

}
