<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 883c0197a34e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-08-03 19:59 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifVar extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['name' => false, 'match' => true], __CLASS__);

		$match = self::getAttribute('match', $attribs, '', we_base_request::STRING);
		$operator = self::getAttribute('operator', $attribs, 'equal', we_base_request::STRING);

		if(!str_contains($match, ',')){
			$size = 1;
			$matchArray = [];
		} else {
			$matchArray = array_filter(explode(',', $match));
			$size = count($matchArray);
			$regs = [];
			if($size == 1 && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $match, $regs)){
				$match = (isset($regs[2]) ?
						self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
						($GLOBALS[$regs[1]] ?? ''));
			}
		}

		$val = we_tag_var::tag(self::removeAttribs($attribs, ['match']));

		return ($size == 1 ?
				self::op($operator, $val, $match) :
				(in_array($val, $matchArray)));
	}

	protected static function op(string $operator, /* mixed */ $first, /* mixed */ $match): bool{
		switch($operator){
			default:
			case 'equal':
				return $first == $match;
			case 'less':
				return $first < $match;
			case 'less|equal':
				return $first <= $match;
			case 'greater':
				return $first > $match;
			case 'greater|equal':
				return $first >= $match;
			case 'contains':
				return (str_contains((string) $first, (string) $match));
			case 'isin':
				return (str_contains((string) $match, (string) $first));
		}
	}

}
