<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 26815396621d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-03-22 19:14 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

class we_tag_xmlnode extends we_weTag_tag{
	/** @var _we_tag_xmlnode_struct */
	private $struct = null;

	/** @var string */
	private $to = '';

	/** @var string */
	private $nameTo = '';

	/** @var array<int,array{xpath:string,parent:int,url?:string,feed?:string}> */
	private static $xpaths = [];

	/** @var int[] */
	private static $xstack = [];

	/** @var ?string */
	private static $xsuperparent = null;

	/** @var we_tag_xmlnode[] */
	private static $instances = [];

	private function __construct(_we_tag_xmlnode_struct $struct, string $to, string $nameTo){
		$this->struct = $struct;
		$this->to = $to;
		$this->nameTo = $nameTo;
		ob_start();
	}

	private function doRun(): bool{
		while($this->struct->next()){
			if($this->struct->hasChild()){
				self::$xsuperparent = $this->struct->getNode();
				return true;
			}
			echo $this->struct->getFeedData();
		}
		echo self::redirectTagOutput(ob_get_clean() ?: '', $this->nameTo, $this->to);
		//remove this instance
		array_pop(self::$instances);
		return false;
	}

	public static function run(): bool{
		return end(self::$instances)->doRun();
	}

	public static function parse(array $attribs, string $content): string{
		return '<?php if(' . we_weTag_parser::printTag('xmlnode', $attribs) . '){
	while(we_tag_xmlnode::run()){
		?>' . $content . '<?php
	}
}?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['xpath' => false], __CLASS__);

		$feed = self::getPostName(self::getAttribute('feed', $attribs, '', we_base_request::STRING));
		$url = self::getAttribute('url', $attribs, '', we_base_request::URL);
		$encoding = 'UTF-8';

		$pind_name = count(self::$xstack) - 1;
		if($pind_name < 0){
			$pind_name = 0;
			$parent_name = 0;
		} else {
			$parent_name = self::$xstack[$pind_name];
		}

		$ind_name = count(self::$xpaths) + 1;
		self::$xpaths[$ind_name] = [
			'xpath' => $attribs['xpath'],
			'parent' => $parent_name
		];
		$feed_name = self::getFeedName($url, $feed, $ind_name, $parent_name, $pind_name);
		if($feed_name === null){//no feed name found
			return false;
		}

		// find feed
		$nodes_name = self::getNodeName($feed_name, $ind_name, $parent_name, $pind_name);

		self::$xstack[] = $ind_name;
		//append instance
		self::$instances[] = new self(new _we_tag_xmlnode_struct($nodes_name, $feed_name, $encoding), self::getAttribute('to', $attribs, 'screen', we_base_request::STRING), self::getAttribute('nameto', $attribs, '', we_base_request::STRING));
		return true;
	}

	protected static function getFeedName(string $url, string $feed, int $ind_name, int $parent_name, int $pind_name): ?we_xml_browser{
		if($url){
			$feed_name = new we_xml_browser($url);
			self::$xpaths[$ind_name]['url'] = $url;
			return $feed_name;
		}
		if($feed){
			$feed_name = we_tag_xmlfeed::$xmlfeeds[$feed];
			self::$xpaths[$ind_name]['feed'] = $feed;

			return $feed_name;
		}

		if(empty($parent_name)){
			return null;
		}
		for($c_name = $pind_name; $c_name > -1; $c_name--){
			$otac_name = self::$xstack[$c_name];
			if(isset(self::$xpaths[$otac_name])){
				if(!empty(self::$xpaths[$otac_name]['url'])){
					$feed_name = new we_xml_browser(self::$xpaths[$otac_name]['url']);
					self::$xpaths[$ind_name]['url'] = self::$xpaths[$otac_name]['url'];
					return $feed_name;
				}
				if(!empty(self::$xpaths[$otac_name]['feed'])){
					$feed_name = we_tag_xmlfeed::$xmlfeeds[self::$xpaths[$otac_name]['feed']];
					self::$xpaths[$ind_name]['feed'] = self::$xpaths[$otac_name]['feed'];
					return $feed_name;
				}
			}
		}

		return null;
	}

	protected static function getNodeName(we_xml_browser $feed_name, int $ind_name, int $parent_name, int $pind_name): array{
		$nodes_name = (self::$xsuperparent !== null ? $feed_name->evaluate(self::$xsuperparent . '/' . self::$xpaths[$ind_name]['xpath']) : []) ?: $feed_name->evaluate(self::$xpaths[$ind_name]['xpath']);

		if(empty($nodes_name) && !empty($parent_name)){
			for($c_name = $pind_name; $c_name > -1; $c_name--){
				$otac_name = self::$xstack[$c_name];
				if(isset(self::$xpaths[$otac_name])){
					if(!empty(self::$xpaths[$otac_name]['xpath'])){
						self::$xpaths[$ind_name]['xpath'] = self::$xpaths[$otac_name]['xpath'] . '/' . self::$xpaths[$ind_name]['xpath'];
						$nodes_name = $feed_name->evaluate(self::$xpaths[$ind_name]['xpath']);
					}
				}
			}
		}
		return $nodes_name;
	}

}

class _we_tag_xmlnode_struct{
	/** @var string[] */
	private $nodes_name;

	/** @var we_xml_browser */
	private $feed_name;

	/** @var bool */
	private $init = false;

	/** @var string */
	private $xmlCharset = '';

	/** @var string */
	private $docCharset = '';

	/**
	 *
	 * @param string[] $nodes_name
	 * @param we_xml_browser $feed_name
	 * @param string $encoding
	 */
	function __construct(array $nodes_name, we_xml_browser $feed_name, string $encoding){
		$this->nodes_name = $nodes_name;
		$this->feed_name = $feed_name;
		$this->xmlCharset = $this->feed_name->mainXmlEncoding;
		$this->docCharset = $encoding;
	}

	function next(): bool{
		if($this->init){
			return next($this->nodes_name) !== false;
		}
		$this->init = true;
		return reset($this->nodes_name) !== false;
	}

	function hasChild(): bool{
		return $this->feed_name->hasChildNodes(current($this->nodes_name) ?: '');
	}

	function getFeedData(): string{
		$ret = $this->feed_name->getData(current($this->nodes_name) ?: '');
		return (
			$this->docCharset === $this->xmlCharset ?
			$ret :
			(mb_convert_encoding($ret, $this->docCharset, $this->xmlCharset) ?: '')
			);
	}

	function getNode(): string{
		return current($this->nodes_name) ?: '';
	}

}
