<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f9d7b286feee $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 01:23 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_users extends we_tree_base{

	protected function customJSFile(): string{
		$startloc = (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
			0 :
			fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . we_users_user::getID())
			);

		return
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'mod_tree.js', "initTree('" . we_modules_info::USERS . "'," . $startloc . ");") .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_tree.js');
	}

	public static function getItems(int $ParentId, int $Offset = 0, int $Segment = 500, bool $sort = false): array{
		$items = [];
		if(we_base_permission::hasPerm([we_base_perm::USER_NEW, we_base_perm::USER_GROUP_NEW, we_base_perm::USER_SAVE, we_base_perm::USER_GROUP_SAVE, we_base_perm::USER_DELETE,
				we_base_perm::USER_GROUP_DELETE])){

			$parent_path = (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
				'/' :
				str_replace('\\', '/', dirname(id2path(we_users_user::getID(), USER_TABLE))));

			foreach(DB_WE::$inst->getAllq('SELECT ID,ParentID,username,CONCAT(First," ",Second) AS Name,Type,Permissions,LoginDenied FROM ' . USER_TABLE . ' WHERE Path LIKE "' . DB_WE::$inst->escape($parent_path) . '%" AND ParentID=' . $ParentId . ' ORDER BY username ASC') as $rec){
				switch($rec['Type']){
					case we_users_user::TYPE_USER_GROUP:
						$items[] = [
							'id' => intval($rec['ID']),
							'parentid' => intval($rec['ParentID']),
							'text' => addslashes($rec['username']),
							'typ' => self::TYP_FOLDER,
							'open' => false,
							'contenttype' => we_base_ContentTypes::USERGROUP,
							'table' => USER_TABLE,
							'loaded' => false,
							'checked' => false,
						];
						break;
					default:
						$p = we_unserialize($rec['Permissions']);

						$items[] = [
							'id' => intval($rec['ID']),
							'parentid' => intval($rec['ParentID']),
							'text' => addslashes($rec['username'] . ' (' . $rec['Name'] . ')'),
							'typ' => self::TYP_ITEM,
							'open' => false,
							'contenttype' => ($rec['Type'] == we_users_user::TYPE_ALIAS ? we_base_ContentTypes::USERALIAS : we_base_ContentTypes::USER),
							'table' => USER_TABLE,
							'class' => (!empty($p[we_base_perm::ADMINISTRATOR]) ? 'bold ' : '') . ($rec['LoginDenied'] ? 'red' : '')
						];
				}
			}
		}
		return $items;
	}

}
