<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ef93478307da $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 17:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_validation_base{
	/** @var string */
	public static $error = '';

	static function getAllCategories(): array{
		return ['xhtml' => g_l('validation', '[category_xhtml]'),
			'links' => g_l('validation', '[category_links]'),
			'css' => g_l('validation', '[category_css]'),
			'accessibility' => g_l('validation', '[category_accessibility]')
		];
	}

	static function saveService(we_validation_service $validationService): ?we_validation_service{
		// before saving check if another validationservice has this name
		if(fB('SELECT 1 FROM ' . VALIDATION_SERVICES_TABLE . ' WHERE name="' . DB_WE::$inst->escape($validationService->name) . '"
AND ID!=' . intval($validationService->id) . ' LIMIT 1')){
			self::$error = g_l('validation', '[edit_service][servicename_already_exists]');
			return null;
		}

		$qSet = we_database_base::arraySetter([
				'category' => $validationService->category,
				'name' => $validationService->name,
				'host' => $validationService->host,
				'path' => $validationService->path,
				'method' => $validationService->method,
				'varname' => $validationService->varname,
				'checkvia' => $validationService->checkvia,
				'additionalVars' => $validationService->additionalVars,
				'ctype' => $validationService->ctype,
				'fileEndings' => $validationService->fileEndings,
				'active' => $validationService->active
		]);
		$query = ($validationService->id != 0 ?
			'UPDATE ' . VALIDATION_SERVICES_TABLE . ' SET ' . $qSet . ' WHERE ID=' . intval($validationService->id) :
			'INSERT INTO ' . VALIDATION_SERVICES_TABLE . ' SET ' . $qSet);

		if(DB_WE::$inst->query($query)){
			$validationService->id = $validationService->id ?: DB_WE::$inst->getInsertId();
			return $validationService;
		}
		return null;
	}

	static function deleteService(we_validation_service $validationService): bool{
		if($validationService->id != 0){
			if(DB_WE::$inst->query('DELETE FROM ' . VALIDATION_SERVICES_TABLE . ' WHERE ID=' . intval($validationService->id))){
				return true;
			}
			return false;
		}
		//  not saved entry - must not be deleted from db
		return true;
	}

	static function getValidationServices(string $extension, string $mode = 'edit'): array{
		switch($mode){
			case 'edit':
				$all = DB_WE::$inst->getAllq('SELECT * FROM ' . VALIDATION_SERVICES_TABLE);
				break;
			case 'use':
				$all = DB_WE::$inst->getAllq('SELECT * FROM ' . VALIDATION_SERVICES_TABLE . ' WHERE fileEndings LIKE "%' . DB_WE::$inst->escape($extension) . '%" AND active=1');
				break;
			default:
				return [];
		}
		'@phan-var array<int,array{ID:int,category:string,name:string,host:string,path:string,method:string,varname:string,checkvia:string,additionalVars:string,ctype:string,fileEndings:string,active:int}> $all';
		$ret = [];
		foreach($all as $cur){
			$ret[] = new we_validation_service((int) $cur['ID'], 'custom', $cur['category'], $cur['name'], $cur['host'], $cur['path'], $cur['method'], $cur['varname'], $cur['checkvia'], $cur['ctype'], $cur['additionalVars'], $cur['fileEndings'], (bool) $cur['active']);
		}
		return $ret;
	}

}
