<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d938a246a5b9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-29 12:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Voting
 *
 */
class we_voting_list extends we_listview_base{//FIXME this should be an instance of we_listview_base
	//properties

	var $Version;

	/**
	 * Default Constructor
	 * Can load or create new Newsletter depends of parameter
	 */
	public function __construct(string $name, int $groupid, int $version, int $rows, int $offset, bool $desc, string $order, bool $subgroup, int $start){
		parent::__construct($name, $rows, $offset, $order, $desc);
		$this->Version = $version;
		$this->start = $start;
		if($this->start == 0){
			$this->start += $offset;
		}

		$childs_query = '';
		if($groupid != 0){
			$childs = [$groupid];
			if($subgroup){
				we_base_util::readChilds([$groupid], $childs, VOTING_TABLE, true, '', 'IsFolder', ['1']);
			}
			$childs_query = 'ParentID IN (' . implode(',', $childs) . ')';
		}

		$limit = ($rows || $this->start ?
				' LIMIT ' . $this->start . ',' . ($rows == 0 ? 9999999 : $rows) : '');

		if($order != ''){
			$order_sql = ' ORDER BY ' . $order;
			if($desc){
				$order_sql .= ' DESC ';
			} else {
				$order_sql .= ' ASC ';
			}
		} else {
			$order_sql = '';
		}
//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = fI('SELECT COUNT(1) FROM ' . VOTING_TABLE . ' WHERE IsFolder=0 ' . (!empty($childs_query) ? ' AND ' . $childs_query : '') . $order_sql);

		$this->DB_WE->query('SELECT ID FROM ' . VOTING_TABLE . ' WHERE IsFolder=0 ' . (!empty($childs_query) ? ' AND ' . $childs_query : '') . $order_sql . $limit);
	}

	public function next_record(): bool{
		if($this->DB_WE->next_record()){
			we_tag_voting::$currentVoting = new we_voting_voting(intval($this->DB_WE->f('ID')));
			we_tag_voting::$currentVoting->setDefVersion($this->Version);
			return true;
		}
		return false;
	}

	public function getNextLink(array $attribs, string $content = ''): string{
		if(!$this->hasNextPage()){
			return $content;
		}

		$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);
		$foo = $this->start + $this->rows;
		$attribs['href'] = we_weTag_util::tag('url', ['id' => ($urlID ?: 'self'), 'hidedirindex' => false]) . '?' . htmlspecialchars(we_http_util::makeQueryString(['_we_vl_start_' . $this->name => $foo]));
		$attribs['rel'] = 'next';

		return we_weTag_tag::htmlTag('a', $attribs, $content, true);
	}

	public function hasNextPage(bool $parentStart = false): bool{
		return (($this->start + $this->rows) < $this->anz_all);
	}

	public function getBackLink(array $attribs, string $content = ''): string{
		if($this->hasPrevPage()){
			$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);
			$foo = $this->start - $this->rows;
			$attribs['href'] = we_weTag_util::tag('url', ['id' => ($urlID ?: 'self'), 'hidedirindex' => false]) . '?' . htmlspecialchars(we_http_util::makeQueryString(['_we_vl_start_' . $this->name => $foo]));
			$attribs['rel'] = 'prev';

			return we_weTag_tag::htmlTag('a', $attribs, $content, true);
		}
		return $content;
	}

	function hasPrevPage(bool $parentStart = false): bool{
		return (abs($this->start) != abs($this->offset));
	}

}
