<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: afda860cd6fe $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-14 14:42 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_weTag_attribute{
	/** @var int */
	protected $Id;

	/** @var string */
	protected $Name;

	/** @var boolean */
	protected $Required;

	/** @var string	 */
	protected $Module;

	/** @var string	 */
	protected $Value;

	/** @var string	 */
	protected $Description;

	/** @var bool	 */
	protected $Deprecated;

	/** @var string */
	protected $defaultValue;

	/** @var we_weTag_twbase */
	protected $tag;

	abstract function getCodeForTagWizard(we_html_tr $row): void;

	/**
	 * @param string $name
	 * @param boolean $required
	 * @param string $module
	 */
	public function __construct(we_weTag_twbase $tag, string $name, string $default = '', bool $required = false, string $module = '', string $description = '', bool $deprecated = false){
		static $count = 0;
		$this->Id = ++$count;
		$this->tag = $tag;
		$this->Name = $name;
		$this->Required = $required;
		$this->Module = $module;
		// set value occasionally
		$this->Value = we_base_request::string('attributes', '', $name);
		try{
			$this->Description = $description ?: html_entity_decode(g_l('tag', '[' . $tag->Name . '][attributes][' . $name . '][description]', true, true));
		} catch (we_exception_base $e){
			$this->Description = '';
		}
		$this->Deprecated = $deprecated;
		try{
			$this->defaultValue = $default ?: html_entity_decode(g_l('tag', '[' . $tag->Name . '][attributes][' . $name . '][default]', true, true));
		} catch (we_exception_base $e){
			$this->defaultValue = '';
		}
	}

	/**
	 * @return string
	 */
	protected function getLabelCodeForTagWizard(): string{
		return
			($this->tag->getTypeAttribute() || $this->Description ? we_html_tools::alertAttentionBox(htmlspecialchars($this->Description), we_html_tools::TYPE_HELP, 0, false) : '') .
			we_html_element::label([
				'id' => 'label_' . $this->getIdName(),
				'class' => 'defaultfont',
				'for' => $this->getIdName(),
				/* 'style' => ($this->Description ? 'border-bottom-style: dotted;border-bottom-width: 1px;border-spacing: 2px;cursor:help;' : '') . ($this->Deprecated ? 'text-decoration:line-through;' : ''),
				  'title' => htmlentities($this->Description ?: ''), */
				], $this->Name . ($this->Required ? '*' : ''));
	}

	/**
	 * @return string
	 */
	public function getName(): string{
		return $this->Name;
	}

	/**
	 * @return string
	 */
	public function getIdName(): string{
		return 'id' . $this->Id . '_' . $this->Name;
	}

	/**
	 * @return boolean
	 */
	public function IsRequired(): bool{
		return $this->Required;
	}

	public function IsDeprecated(): bool{
		return $this->Deprecated;
	}

	public function getDescription(): string{
		return $this->Description;
	}

	/**
	 * checks if this attribute should be used, checks if needed modules are installed
	 * @return boolean
	 */
	public function useAttribute(): bool{
		return ($this->Module === '' || we_modules_info::isActive($this->Module));
	}

	/**
	 * checks if this option should be used, checks if needed modules are installed
	 * @param we_weTag_option[] $options
	 * @return we_weTag_option[]
	 */
	public static function getUseOptions(array $options): array{
		$useOptions = [];
		foreach($options as $option){
			if($option->useOption()){
				$useOptions[] = $option;
			}
		}
		return $useOptions;
	}

}
