<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d1077a540408 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-10-20 12:39 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_parser{

	/** @var int */
	private $lastpos = 0;

	/** @var array<int,string> */
	private $tags = [];

	/** @var string */
	public static $curFile = '';

	/** @var string */
	private $errStr = '';

	//private $AppListviewItemsTags = [];

	public function __construct(string $content = '', string $curFile = ''){
		self::$curFile = $curFile;
		//init Tags
		if($content != ''){
			$this->setAllTags($content);
		}
	}

	/**
	 *
	 * @return string[]
	 */
	public function getAllTags(): array{
		return $this->tags;
	}

	/* fixme: check regex: (["'])(?:(?=(\\?))\2.)*?\1
	  ([""']) match a quote; ((?=(\\?))\2.) if backslash exists, gobble it, and whether or not that happens, match a character; *? match many times (non-greedily, as to not eat the closing quote); \1 match the same quote that was use for opening.
	 */

	private function setAllTags(string $code): void{
		$this->tags = [];
		$foo = [];
		preg_match_all('%</?we:([[:alnum:]_-]+)([ \t\n\r]*[[:alnum:]_-]+[ \t]*=[ \t]*"[^"]*")*[ \t\n\r]*/?>?%i', $code, $foo, PREG_SET_ORDER);
		foreach($foo as $f){
			$this->tags[] = $f[0];
		}
	}

	/**
	 * @return	array
	 * @param	string $tagname
	 * @param	string $code
	 * @desc		function separates a complete XML tag in several pieces
	 * 			returns array with this information
	 * 			tagname without <> .. for example "we:hidePages"
	 * 			[0][x] = complete Tag
	 * 			[1][x] = start tag
	 * 			[2][x] = parameter as string
	 */
	public static function itemize_we_tag(string $tagname, string $code): array{
		$matches = [];
		preg_match_all('/(<' . $tagname . '([^>]*)>)/U', $code, $matches);
		return $matches;
	}

	public function parseTags(string &$code, int $start = 0, /* mixed */ $ende = false): bool{
		if($start == 0 && !$this->checkOpenCloseTags($code)){
			throw new we_exception_parser($this->errStr);
		}
		$this->lastpos = 0;
		$ende = $ende ?: count($this->tags);
		for($ipos = $start; $ipos < $ende;){
			//t_e($ipos,$this->tags[$ipos],$ende);
			if($this->tags[$ipos]){
				$tmp = $this->parseTag($code, $ipos); //throws exception
				$this->tags[$ipos] = '';
				$ipos += $tmp;
			} else {
				$ipos++;
			}
		}
		$this->lastpos = 0;
		return true;
	}

	private function checkOpenCloseTags(string &$code): bool{
		$Counter = [];
		'@phan-var int[] $Counter';
		foreach($this->tags as $tag){
			$matches = [];
			if(preg_match_all('|<(/?)we:([[:alnum:]_-]+)([ \t\n\r]*[[:alnum:]_-]+[ \t]*=[ \t]*"[^"]*")*[ \t\n\r]*(/)?>?|smi', $tag, $matches)){
				if($matches[2][0] !== null){
					if(!isset($Counter[$matches[2][0]])){
						$Counter[$matches[2][0]] = 0;
					}

					if($matches[1][0] === '/'){
						$Counter[$matches[2][0]]--;
					} else {
						//selfclosing-Tag
						if($matches[4][0] === '/'){
							continue;
						}
						$Counter[$matches[2][0]]++;
					}
				}
			}
		}

		$ErrorMsg = '';
		$err = '';
		$isError = false;
		foreach($Counter as $tag => $counter){
			if($counter < 0){
				$err .= sprintf(g_l('parser', '[missing_open_tag]'), 'we:' . $tag) . "\\n";
				$ErrorMsg .= we_weTag_tag::parseError(sprintf(g_l('parser', '[missing_open_tag]') . ' (' . abs($counter) . ')', 'we:' . $tag));

				$isError = true;
			} elseif($counter > 0){
				$err .= sprintf(g_l('parser', '[missing_close_tag]'), 'we:' . $tag) . "\\n";
				$ErrorMsg .= we_weTag_tag::parseError(sprintf(g_l('parser', '[missing_close_tag]') . ' (' . abs($counter) . ')', 'we:' . $tag));
				$isError = true;
			}
		}
		if($isError){
			$code = $ErrorMsg;
		}
		$this->errStr = $err;
		return !$isError;
	}

	private function searchEndtag(string $tagname, string $code, int $tagPos, int $ipos): array{
		$tagcount = 0;
		$endtags = [];

		$endtagpos = $tagPos;
		$regs = [];
		for($i = $ipos + 1; $i < count($this->tags); $i++){
			if(preg_match('|(< ?/ ?we ?: ?' . $tagname . '[^a-z])|i', $this->tags[$i], $regs)){
				$endtags[] = $regs[1];
				if($tagcount){
					$tagcount--;
				} else {
					// found endtag
					for($n = 0; $n < count($endtags); $n++){
						$endtagpos = strpos($code, $endtags[$n], $endtagpos + 1);
					}
					$this->tags[$i] = '';
					return ['pos' => $endtagpos, 'no' => $i];
				}
			} else {
				if(preg_match('|(< ?we ?: ?' . $tagname . '[^a-z])|i', $this->tags[$i])){
					$tagcount++;
				}
			}
		}
		return ['pos' => false, 'no' => false];
	}

	public static function parseAttribs(string $attr): array{
		//remove comment-attribute (should never be seen), and obsolete cachelifetime
		$removeAttribs = ['cachelifetime', 'comment'];
		$attribs = [];
		$regs = [];
		preg_match_all('/([^=]+)=[ \t]*"([^"]*)"/', $attr, $regs, PREG_SET_ORDER);

		if(!empty($regs)){
			foreach($regs as $f){
				if(!in_array($f[1], $removeAttribs)){
					$val = $f[2];
					$attribs[trim($f[1])] = $val;
				}
			}
		}
		return $attribs;
	}

	/**
	 * @param bool $withBlocknames
	 * @param bool $withCloseTags
	 * @return array<string,array<int,array{name:string,attribs:array<string,string|string[]>,close:bool,weTag:string}>>
	 */
	public function getTagsGroupedByTag(bool $withBlocknames = false, bool $withCloseTags = false): array{
		$ret = [];
		foreach($this->getTagsWithAttributes($withBlocknames, $withCloseTags) as $pos => $tag){
			$ret[$tag['weTag']][$pos] = $tag;
		}
		return $ret;
	}

	/**
	 * @param bool $withBlocknames
	 * @param bool $withCloseTags
	 * @return array<int,array{name:string,attribs:array<string,string|string[]>,close:bool,weTag:string}>
	 */
	public function getTagsWithAttributes(bool $withBlocknames = false, bool $withCloseTags = false): array{
		$regs = $blocks = $ret = [];

		foreach($this->tags as $ipos => $tag){
			preg_match('%<(/?)we:([[:alnum:]_-]+)[ \t\n\r]*(.*)/?>?%msi', $tag, $regs);
			if(!$regs[1]){//open tag
				$attribs = (empty($regs[3]) ? [] : self::parseAttribs($regs[3]));
				$ret[$ipos] = [
					'name' => $regs[2],
					'attribs' => $attribs + ($withBlocknames ? ['weblock' => $blocks] : []),
					'close' => false,
					'weTag' => $tag
				];
				if($withBlocknames && $regs[2] === 'block' && !empty($attribs['name'])){
					array_unshift($blocks, 'blk_' . $attribs['name']);
				}
			} else {//close tag
				if($withBlocknames && $regs[2] === 'block'){
					array_shift($blocks);
				}
				if($withCloseTags){
					$ret[$ipos] = [
						'name' => $regs[2],
						'attribs' => ($withBlocknames ? ['weblock' => $blocks] : []),
						'close' => true,
						'weTag' => $tag
					];
				}
			}
		}
		return $ret;
	}

	private function parseTag(string &$code, int $ipos): int{
		$tag = $this->tags[$ipos];
		$regs = [];
		//$endTag = false;
		preg_match('%<(/?)we:([[:alnum:]_-]+)([ \t\n\r]*[[:alnum:]_-]+[ \t]*=[ \t]*"[^"]*")*[ \t\n\r]*(/?)(>?)%mi', $tag, $regs);
		$endTag = ($regs[1] === '/');
		if($endTag){
			//there should not be any endtags
			$code = str_replace($tag, '', $code);
			return 1;
		}

		$selfclose = ($regs[4] === '/');
		$gt = $regs[5];
		$tagname = $regs[2];

		//@Lukas: =>384
		if(!$gt){
			$data = '';
			for($i = $ipos - 2; $i < $ipos + 5; $i++){
				$data .= "\n" . ($i == $ipos ? '!-> ' : '    ') . $this->tags[$i];
			}
			throw new we_exception_parser(we_weTag_tag::parseError(sprintf(g_l('parser', '[incompleteTag]'), $tagname), 'Parsed tags around:' . $data));
		}
		//tags which need an endtag are not allowed to be selfclosing
		//$selfclose&=!in_array($tagname, self::$CloseTags);
		preg_match('%</?we:[[:alnum:]_-]+[ \t\n\r]*(.*)' . $regs[4] . $regs[5] . '%msi', $regs[0], $regs);
		$attr = trim($regs[1]);

		$attribs = self::parseAttribs($attr);
		//$attr = self::printArray($attribs);

		if(isset($attribs['name'])){
			$len = strlen($attribs['name']);
			if($len == 0 || $len > 255){
				throw new we_exception_parser(we_weTag_tag::parseError(sprintf(g_l('parser', ($len == 0 ? '[name_empty]' : '[name_to_long]')), $tagname), $tag));
			}
		}
		$fnName = 'we_tag_' . $tagname;
		try{
			we_weTag_util::includeTagFile($fnName);
		}catch(we_exception_file $e){
			throw new we_exception_parser($e->getMessage());
		}

		$tagPos = strpos($code, (string) $tag, $this->lastpos) ?: 0;
		$this->lastpos = $tagPos;
		$endeStartTag = $tagPos + strlen($tag);
		$endeEndTagPos = 0;
		if($selfclose){
			$content = '';
			$endTagNo = -1;
		} else {
			['pos' => $endTagPos, 'no' => $endTagNo] = $this->searchEndtag($tagname, $code, $tagPos, $ipos);
			if($endTagPos === false){
				throw new we_exception_parser(we_weTag_tag::parseError(sprintf(g_l('parser', '[start_endtag_missing]'), $tagname), $tag));
			}
			$endeEndTagPos = (int) strpos($code, '>', $endTagPos) + 1;
			$content = (string) substr($code, $endeStartTag, ((int) $endTagPos - $endeStartTag)) ?: '';
			//only 1 exception: comment tag should be able to contain partly invalid code (e.g. missing attributes etc)
			if(($tagname != 'comment') && (($ipos + 1) < $endTagNo)){
				$this->parseTags($content, ($ipos + 1), $endTagNo); //throws exception
			}
		}

		//t_e($tag, $tagPos, $endeStartTag, $endTagPos, $ipos, $content,$this->tags);
		$tagClass = 'we_tag_' . $tagname;
		if(!class_exists($tagClass, false)){
			throw new we_exception_parser('unable to parse tag ' . $tagClass . '. This tag is old and needs an update!');
		}
		/* call specific function for parsing this tag
		 * $attribs is the attribs string, $content is content of this tag
		 * return value is parsed again and inserted
		 */
		try{
			$content = $tagClass::parse($attribs, $content);
			$code = substr($code, 0, $tagPos) .
					$content .
					substr($code, ($endeEndTagPos ?: $endeStartTag));
		}catch(we_exception_parser $e){
			$code = $e->getMessage();
			throw $e;
		}
		return ($endTagNo !== -1 ? (int) ($endTagNo - $ipos) : 1);
	}

	/**
	 *
	 * @staticvar int $no
	 * @param string $name
	 * @param array<string,string|int|bool|float> $attribs
	 * @param string $content
	 * @param bool $cslash
	 * @param bool $directContent
	 * @return string
	 */
	public static function printTag(string $name, array $attribs = [], string $content = '', bool $cslash = false, bool $directContent = false): string{
		$var = '';
		if(!empty($attribs['to']) && !in_array($name, [//FIXME: minimize this list!!
					'a',
					'addPercent',
					'back',
					'calculate',
					'categorySelect',
					'checkForm',
					'description',
					'href',
					'keywords',
					'navigationEntries',
					'next',
					'select',
					'sessionLogout',
					'setVar',
					'sidebar',
					'textarea',
					'title',
					'xmlnode',
				])){
			$nameTo = (string) ($attribs['nameto'] ?? $attribs['name']);
			if($nameTo){//target variable is given via variable
				switch($attribs['to']){
					case 'local':
						$var = self::getToVar('', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
					case 'global':
						$var = self::getToVar('GLOBALS', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
					case 'request':
						$var = self::getToVar('_REQUEST', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
					case 'post':
						$var = self::getToVar('_POST', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
					case 'get':
						$var = self::getToVar('_GET', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
					case 'session':
						$var = self::getToVar('_SESSION', $nameTo);
						unset($attribs['nameto']);
						$attribs['to'] = 'parsed';
						break;
				}
			}
		}
		$attr = self::printArray($attribs, false);
		static $no = 0;
		return $var . 'we_weTag_util::tag(\'' . $name . '\'' .
				($attr ? ',' . $attr : ($content ? ',[]' : '')) .
				($content ?
				($directContent ? ',' . $content :
				(',' . ($cslash && str_contains($content, '\'') ? '<<<\'WE' . $no . 'EOS\'
' . $content . '
WE' . ($no++) . 'EOS
' : '\'' . $content . '\'') )
				) : '')
				. ')';
	}

	private static function getToVar(string $prefix, string $name): string{
		if(!str_contains($name, '[')){
			$quote = str_contains($name, '$') ? '"' : '\'';
			return '__WEVAR__$' . ($prefix ? $prefix . '[' . $quote . $name . $quote . ']' : $name) . '=';
		}
		$arr_matches = preg_split('/\]\[|\[|\]/', $name);
		$last = count($arr_matches) - 1;
		unset($arr_matches[$last--]); //preg split has an empty element at the end

		$prefix = $prefix ?: array_shift($arr_matches);
		$data = '';
		foreach($arr_matches as $cur){
			$quote = str_contains($cur, '$') ? '"' : '\'';
			$data .= '[' . ($cur ? $quote . $cur . $quote : '') . ']';
		}
		return '__WEVAR__$' . $prefix . $data . '=';
	}

	/**
	 * @param array<string,string|int|bool|float> $array
	 * @param bool $printEmpty
	 * @return string
	 */
	public static function printArray(array $array, bool $printEmpty = true): string{
		$ret = [];
		foreach($array as $key => $val){
			switch($key){
				case 'comment':
				case 'cachelifetime':
					continue 2;
				default:
					$regs = [];
					if(is_string($val) && preg_match('|^\\\\\$([^\[]+)(\[[^]]+\])?$|', $val, $regs)){
						$val = '$GLOBALS[\'' . $regs[1] . '\']' . (empty($regs[2]) ? '' : strtr($regs[2], ['[' => '[\'', ']' => '\']']));
						$quotes = '';
					} else {
						$v = (string) $val;
						$skipQuotes = (is_numeric($val) && $val[0] != '+') ||
								is_bool($val) ||
								(in_array($v, ['true', 'false']) && !str_starts_with($key, 'pass_')) || //bool
								(!str_contains($v, '${') && preg_match('/^\$[_[:alpha:]][_[:alnum:]]*$/', $v)) || //full php variable
								str_contains($v, '::$') || //static class variable
								preg_match('/[_[:alpha:]][_[:alnum:]]*::[_[:alpha:]][_[:alnum:]]*/', $v)//class constants
						;
						$quotes = $skipQuotes ? '' :
								((str_contains($v, '$') ) || (str_contains($v, '\'')) ? '"' : '\'');
						$val = $skipQuotes ? $val : ($quotes . $val . $quotes);
					}
					$ret[] = '\'' . $key . '\'=>' . $val;
			}
		}

		return (!empty($ret) || $printEmpty ? '[' . implode(',', $ret) . ']' : '');
	}

}

//FIXME:temporary fix for user tags
class we_tag_tagParser extends we_weTag_parser{

}
