/* global top, treeData, WE */

/**
 * webEdition CMS
 *
 * $Rev: 4925140e1525 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-02-11 13:20 +0100 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

var treeData;

var container = function () {
	this.wasdblclick = false;
	this.tout = null;
	this.len = 0;
	this.elements = [];
	this.state = 0;
	this.startloc = 0;
	this.table = "";
	this.selection = "";
	this.selection_table = "";
	this.mod = "";
	this.we_scrollY = {};
	return this;
};

container.prototype = {
	node_layouts: {
		item: 'item',
		group: 'group',
		threedots: 'changed',
		itemDisabled: 'disabled',
		groupDisabled: 'disabled',
		groupDisabledOpen: 'disabled',
		itemChecked: 'checked_item',
		groupChecked: 'checked_group',
		groupOpen: 'group',
		groupCheckedOpen: 'checked_group',
		itemNotpublished: 'notpublished',
		itemCheckedNotpublished: 'checked_notpublished',
		itemChanged: 'changed',
		itemCheckedChanged: 'checked_changed',
		itemSelected: 'selected_item',
		itemSelectedNotpublished: 'selected_notpublished_item',
		itemSelectedChanged: 'selected_changed_item',
		groupSelected: 'selected_group',
		groupSelectedOpen: 'selected_open_group'
	},
	tree_layouts: {
		0: 'tree',
		1: 'tree',
		2: 'tree',
		3: 'tree'
	},
	tree_states: {
		edit: 0,
		select: 1,
		selectitem: 2,
		selectgroup: 3
	},
	getLayout: function () {
		return this.tree_layouts[this.state];
	},
	setState: function (state) {
		this.state = state;
		if (this.state === this.tree_states.edit) {
			for (var i = 1; i <= this.len; i++) {
				if (this.elements[i].checked) {
					this.elements[i].checked = 0;
				}
			}
		}
	},
	selectNode: function (table, node) {
		var ind;
		if (node) {
			if (this.selection !== "" && this.selection_table === this.table) {
				ind = this.indexOfEntry(this.selection);
				if (ind !== -1) {
					var oldnode = this.get(this.selection);
					oldnode.selected = 0;
					oldnode.applylayout();
				}
			}
			if (table === this.table) {
				ind = this.indexOfEntry(node);
				if (ind !== -1) {
					var newnode = this.get(node);
					newnode.selected = 1;
					newnode.applylayout();
				}
			}
			this.selection_table = table;
			this.selection = node;
		}
	},
	unselectNode: function () {
		if (this.selection !== "" && this.table === this.selection_table) {
			var ind = this.indexOfEntry(this.selection);
			if (ind !== -1) {
				var node = this.get(this.selection);
				node.selected = 0;
				if (node.applylayout) {
					node.applylayout();
				}
			}
		}
		this.selection_table = "";
		this.selection = "";
	},
	clear: function () {
		for (var i = 0; i <= this.elements.length; i++) {
			delete this.elements[i];
		}
		this.len = 0;
	},
	indexOfEntry: function (id) {
		for (var ai = 1; ai <= this.len; ai++) {
			if (this.elements[ai].id == id) {
				return ai;
			}
		}
		return -1;
	},
	add: function (object) {
		this.elements[++this.len] = object;
	},
	addSort: function (object) {
		this.len++;
		var oLower = object.text.toLowerCase();
		for (var i = this.len; i > 0; i--) {
			if (i > 1 && (
					this.elements[i - 1].text.toLowerCase() > oLower ||
					this.elements[i - 1].parentid != object.parentid ||
					(this.elements[i - 1].typ !== object.typ))
					) {
				this.elements[i] = this.elements[i - 1];
				continue;
			}
			this.elements[i] = object;
			break;
		}
	},
	deleteEntry: function (id, draw) {
		var ind = this.indexOfEntry(id);
		if (ind > 0) {
			for (var ai = ind; ai <= this.len - 1; ai++) {
				this.elements[ai] = this.elements[ai + 1];
			}
			delete this.elements[this.len];
			this.len--;
			if (draw || draw === undefined) {
				drawTree();
			}
		}
	},
	deleteEntries: function (table, ids) {
		if (this.table !== table) {
			return;
		}
		for (var i = 0; i < ids.length; i++) {
			this.deleteEntry(ids[i], false);
		}
		drawTree();
	},
	setSegment: function (id) {
		var node = this.get(id);
		node.showSegment();
	},
	drawThreeDots: function (nf, ai) {
		return '<span class="treeKreuz kreuzungend"></span>' +
				'<span name="_' + nf.elements[ai].id + "\" onclick=\"treeData.setSegment('" + nf.elements[ai].id + '\');" class="threedots"><i class="fa fa-' + (nf.elements[ai].contenttype === 'arrowup' ? 'caret-up' : 'caret-down') + '"></i></span><br/>';
	},
	drawItem: function (nf, ai) {
		return '<span class="treeKreuz ' + (ai === nf.len ? "kreuzungend" : "kreuzung") + '"></span>' + this.clickHandler(nf.elements[ai]);
	},
	drawGroup: function (nf, ai, zweigEintrag) {
		return  "<span ondblclick=\"treeData.setScrollY();treeData.openClose('" + nf.elements[ai].id + "')\" onclick=\"treeData.setScrollY();treeData.openClose('" + nf.elements[ai].id + "')\" class='treeKreuz fa-stack " + (ai === nf.len ? "kreuzungend" : "kreuzung") + "'><i class='fa fa-square fa-stack-1x we-color'></i><i class='fa fa-caret-" + (nf.elements[ai].open ? "down" : "right") + " fa-stack-1x'></i></span>" +
				this.clickHandler(nf.elements[ai]) +
				(nf.elements[ai].open ?
						this.draw(nf.elements[ai].id, zweigEintrag + '<span class="' + (ai === nf.len ? "" : "strich ") + 'treeKreuz "></span>') :
						"");
	},
	drawSort: function (nf, ai, zweigEintrag) {
		//overwritten
	},
	makeFoldersOpenString: function () {
		var op = "";
		for (var i = 1; i <= this.len; i++) {
			if (this.elements[i].typ === "group" && this.elements[i].open) {
				op += this.elements[i].id + ",";
			}
		}
		op = op.substring(0, op.length - 1);
		return op;
	},
	updateEntry: function (attribs) {
		if (attribs.table && this.table !== attribs.table) {
			return;
		}
		var updated = false;
		var aname;
		for (var ai = 1; ai <= this.len; ai++) {
			if (this.elements[ai].id == attribs.id) {
				updated = true;
				for (aname in attribs) {
					this.elements[ai][aname] = attribs[aname];
				}
				break;
			}
		}
		if (updated) {
			drawTree();
		}
	},
	search: function (eintrag) {
		var nf = new container();
		for (var ai = 1; ai <= this.len; ai++) {
			if (this.elements[ai].parentid == eintrag) {
				nf.add(this.elements[ai]);
			}
		}
		return nf;
	},
	makeNewEntry: function (attribs) {
		if (attribs.table && this.table && this.table !== attribs.table) {
			return;
		}
		var pos = this.indexOfEntry(attribs.parentid);
		if (this.elements[pos] && this.elements[pos].loaded) {
			attribs.typ = ([
				WE().consts.contentTypes.FOLDER_COMMON,
				WE().consts.contentTypes.FOLDER_DOCUMENT,
				WE().consts.contentTypes.FOLDER_TEMPLATE,
				WE().consts.contentTypes.FOLDER_OBJECTFILE,
				WE().consts.contentTypes.FOLDER_VFILE
			].indexOf(attribs.contenttype) !== -1 ? "group" : "item");
			attribs.tooltip = attribs.id;
			attribs.disabled = 0;
			attribs.selected = 0;
			if (attribs.typ === "item") {
				attribs.published = 1;
			}

			this.addSort(new Node(attribs));
			drawTree();
		}
	},
	clickHandler: function (cur) {
		let row = '<span class="treeEntry' + (cur.iscollectionitem ? ' collectionitem' : '') + '" ',
				href = false,
				select = false;
		if (this.selection_table === this.table && cur.id == this.selection) {
			cur.selected = 1;
		}
		switch (cur.disabled ? -1 : this.state) {
			case - 1:
				href = false;
				break;
			case this.tree_states.selectitem:
				href = cur.typ !== "group";
				select = true;
				break;
			case this.tree_states.selectgroup:
				href = cur.typ !== "item";
				select = true;
				break;
			case this.tree_states.select:
				href = true;
				select = true;
				break;
			default:
				href = true;
				if (cur.path) {
					row += "draggable=\"" + (cur.iscollectionitem ? "false" : "true") + "\" ondragstart=\"this.ownerDocument.defaultView.treeStartDrag(event,'" + (
							[
								WE().consts.contentTypes.FOLDER_COMMON,
								WE().consts.contentTypes.FOLDER_DOCUMENT,
								WE().consts.contentTypes.FOLDER_TEMPLATE,
								WE().consts.contentTypes.FOLDER_OBJECTFILE,
								WE().consts.contentTypes.FOLDER_VFILE
							].indexOf(cur.contenttype) !== -1 ? 'dragFolder' : 'dragItem') + "','" + cur.table + "'," + parseInt(cur.id) + ",'" + cur.contenttype + "','" + cur.path.replace(/'/g, "\\'").replace(/"/g, '') + "'," + (cur.tableid ? parseInt(cur.tableid) : 0) + ")\" ";
				}
				row += "name=\"_" + cur.id + "\" ondblclick=\"return treeData.checkDblClick('" + cur.id + "');\" onclick=\"treeData.tout=setTimeout(treeData.dblClickTmout.bind(treeData),300,'" + cur.id + "');return true;\"";
		}
		row += (select && href ? "onclick=\"treeData.checkNode('img_" + cur.id + "')\"" : '') +
				//close open span tag
				">" +
				WE().util.getTreeIcon(cur.contenttype, cur.open, cur.text.replace(/^.*\./, ".")) +
				(cur.inschedule > 0 ? '<i class="inscheduler fa fa-clock-o"></i>' : '') +
				(select && href ? '<i class="fa fa-' + (cur.checked ? 'check-' : '') + 'square-o wecheckIcon" name="img_' + cur.id + '"></i>' : '') +
				(cur.isprotected > 0 ? '<i class="isprotected fa fa-lock"></i>' : '') +
				'<label id="lab_' + cur.id + '"' + (cur.tooltip !== "" ? ' title="' + (cur.tooltip ? cur.tooltip : cur.id) + '"' : "") + ' class="' + cur.getLayout() + (cur.class ? ' ' + cur.class : '') + '">' + WE().util.htmlspecialchars(cur.text) + "</label>" +
				"</span><br/>";
		return row;
	},
	draw: function (startEntry, zweigEintrag) {
		var nf = this.search(startEntry);
		var row = "";
		for (var ai = 1; ai <= nf.len; ai++) {
			row += zweigEintrag;
			var pind = this.indexOfEntry(nf.elements[ai].parentid);
			if (pind !== -1) {
				if (this.elements[pind].open) {
					switch (nf.elements[ai].typ) {
						case "item":
							row += this.drawItem(nf, ai);
							break;
						case "threedots":
							row += this.drawThreeDots(nf, ai);
							break;
						case "folder":
							row += this.drawFolder(nf, ai, zweigEintrag);
							break;
						case "group":
							row += this.drawGroup(nf, ai, zweigEintrag);
							break;
						case "sort":
							row += this.drawSort(nf, ai, zweigEintrag);
							break;
						case "shop":
							row += this.drawShop(nf, ai, zweigEintrag);
							break;
					}
				}
			}
		}
		return row;
	},
	openClose: function (id) {
		if (id === "") {
			return;
		}

		var node = this.get(id);
		var openstatus = (node.open ? 0 : 1);

		node.open = openstatus;
		if (openstatus && !node.loaded) {
			top.content.cmd.location = top.getFrameset() + "&pnt=cmd&pid=" + id;
		} else {
			drawTree();
		}
		if (openstatus) {
			node.loaded = true;
		}
	},
	get: function (eintrag) {
		for (var ai = 1; ai <= this.len; ai++) {
			if (this.elements[ai].id == eintrag) {
				return this.elements[ai];
			}
		}
		return new container();
	},
	checkNode: function (imgName) {
		var
				tmp,
				object_name = imgName.substring(4, imgName.length);

		for (var i = 1; i <= this.len; i++) {
			if (this.elements[i].id != object_name) {
				continue;
			}
			if (this.elements[i].checked) {
				this.elements[i].checked = 0;
				this.elements[i].applylayout();
				try {
					if (window.document.getElementsByName(imgName)) {
						tmp = window.document.getElementsByName(imgName)[0];
						tmp.classList.remove('fa-check-square-o');
						tmp.classList.add('fa-square-o');
					}
				} catch (e) {

				}
				break;
			}
			this.elements[i].checked = 1;
			this.elements[i].applylayout();
			try {
				if (window.document.getElementsByName(imgName)) {
					tmp = window.document.getElementsByName(imgName)[0];
					tmp.classList.remove('fa-square-o');
					tmp.classList.add('fa-check-square-o');
				}
			} catch (e) {

			}
			break;
		}
	},
	parentChecked: function (start) {
		for (var i = 1; i <= this.len; i++) {
			if (this.elements[i].id === start) {
				if (this.elements[i].checked) {
					return true;
				}

				if (this.elements[i].parentid && this.elements[i].parentid !== this.elements[i].id) {
					if (this.parentChecked(this.elements[i].parentid)) {
						return true;
					}
				}
			}
		}

		return false;
	},
	setScrollY: function () {
		var el;
		if (this.we_scrollY && (el = top.document.getElementById("treetable"))) {
			this.we_scrollY[this.table] = el.scrollTop;
		}
	},
	scrollToY: function () {
		var el;
		if (this.we_scrollY && (el = top.document.getElementById("treetable"))) {
			el.scrollTop = (this.we_scrollY[this.table] ? this.we_scrollY[this.table] : 0);
		}
	},
	checkDblClick: function (id) {
		this.wasdblclick = true;
		window.clearTimeout(this.tout);
		this.doClick(id);
		return true;
	},

	dblClickTmout: function (id) {
		if (!this.wasdblclick) {
			this.doClick(id);
		} else {
			this.wasdblclick = false;
		}
	},
	doClick: function () {
		//override!
	}
};

function treeStartDrag(evt, type, table, id, ct, path, tableid) { // TODO: throw out setData
	if (WE().layout.dragNDrop !== undefined && WE().layout.dragNDrop !== null) {
		WE().layout.dragNDrop.dataTransfer.text = type + ',' + table + ',' + id + ',' + ct + ',' + path + ',' + tableid;
		evt.dataTransfer.setData('text', type + ',' + table + ',' + id + ',' + ct + ',' + path + ',' + tableid);
	}
}

var Node = function (attribs) {
	for (var aname in attribs) {
		this[aname] = attribs[aname];
	}
	return this;
};

Node.prototype = {
	table: "",
	pos: 0,
	typ: "",
	tooltip: "",
	disabled: 0,
	selected: 0,
	published: 1,
	loaded: false,

	rootEntry: function (id, text, rootstat, offset) {
		return new Node({
			id: id,
			text: text,
			open: 1,
			loaded: true,
			typ: "root",
			offset: offset,
			rootstat: rootstat
		});
	},
	getLayout: function () {
		var layout_key = (this.typ === "group" ? "group" : "item") +
				(this.typ === "item" && this.published === 0 ? "Notpublished" : "");
		return treeData.node_layouts[layout_key];
	},
	showSegment: function () {
		var parentnode = this.get(this.parentid);
		parentnode.clear();
		window.we_cmd("loadFolder", this.table, parentnode.id, "", "", "", this.offset);
	},
	applylayout: function (layout) {
		if (window.document.getElementById("lab_" + this.id)) {
			window.document.getElementById("lab_" + this.id).className = (layout ? layout : this.getLayout());
		}
	},
	clear: function () {//FIXME this sould be improved
		var deleted = 0;
		for (var ai = 1; ai <= treeData.len; ai++) {
			if (treeData.elements[ai].parentid != this.id) {
				continue;
			}
			if (treeData.elements[ai].contenttype === "group") {
				deleted += treeData.elements[ai].clear();
			} else {
				treeData.elements.splice(ai, 1);
				ai--;
				//treeData.elements[treeData.len] = null;
				treeData.len--;
			}
			deleted++;
		}
		drawTree();
		return deleted;
	}
};

function drawTree() {
	var tree = (top.content && top.content.editor ? top.content : top);
	tree.document.getElementById("treetable").innerHTML = "<div class=\"" + treeData.getLayout() + "\">" +
			treeData.draw(treeData.startloc, "") +
			"</div>";
}

function initTree() {
	treeData = new container();
}

function startTree(pid, offset, openFolders) {
	window.pid = pid ? pid : 0;
	window.offset = offset ? offset : 0;
	top.content.cmd.location = top.getFrameset() + "&pnt=cmd&pid=" + window.pid + "&open=" + (openFolders ? openFolders.join() : '');
}
