/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 4925140e1525 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-02-11 13:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
WE().util = {
	dialogTop: WE().layout.dialogTop,
	multi_editMulti: null,
	multi_edit: null,
	weSetCookie: function (doc, name, value, expires, path, domain) {
		doc.cookie = name + "=" + encodeURIComponent(value) +
				((expires === undefined) ? "" : "; expires=" + expires.toGMTString()) +
				((path === undefined) ? "" : "; path=" + path) +
				((domain === undefined) ? "" : "; domain=" + domain);
	},
	weGetCookie: function (doc, name) {
		var cname = name + "=";
		var dc = doc.cookie;
		if (dc.length > 0) {
			var begin = dc.indexOf(cname);
			if (begin !== -1) {
				begin += cname.length;
				var end = dc.indexOf(";", begin);
				if (end === -1) {
					end = dc.length;
				}
				return decodeURIComponent(dc.substring(begin, end));
			}
		}
		return null;
	},
	hashCode: function (s) {
		/*jshint bitwise:false */
		return s.split("").reduce(function (a, b) {
			a = ((a << 5) - a) + b.charCodeAt(0);
			return a & a;
		}, 0);
	},
	we_sbmtFrm: function (target, url, source) {
		if (source === undefined) {
			source = WE().layout.weEditorFrameController.getVisibleEditorFrame();
		}

		if (source) {
			try {
				if (source.we_submitForm !== undefined && source.we_submitForm) {
					return source.we_submitForm(target.name, url);
				}
				if (source.contentWindow && source.contentWindow.we_submitForm) {
					return source.contentWindow.we_submitForm(target.name, url);
				}
			} catch (e) {
				WE().t_e("Submit problem", source.name, target.name, url);
			}
		}
		return false;
	},
	hasPerm: function (perm) {
		return (WE().session.permissions.ADMINISTRATOR || WE().session.permissions[perm]);
	},
	/**
	 * This function sets incons inside elements of a given class. The element must have the property data-contenttype and data-extension set to determine the correct icon
	 * @param string classname the elements classname
	 * @returns noting
	 */
	setIconOfDocClass: function (doc, classname) {
		var elements = doc.getElementsByClassName(classname);
		for (var i = 0; i < elements.length; i++) {
			elements[i].innerHTML = this.getTreeIcon(elements[i].getAttribute("data-contenttype"), false, elements[i].getAttribute("data-extension"));
		}
	},
	/**
	 * Get a file icon out of a given type, used in tree, selectors & tabs
	 * @param {string} contentType
	 * @param {boolean} open
	 * @returns icon to be drawn as html-code
	 */
	getTreeIcon: null,
	sprintf: function (argum) {
		if (!arguments || arguments.length === 0) {
			return;
		}

		var regex = /([^%]*)%([%ds])(.*)/,
				arr,
				iterator = 0,
				ret = "",
				check = argum,
				type;

		while ((arr = regex.exec(check))) {
			ret += arr[1];
			type = arr[2];
			check = arr[3];

			switch (type) {
				case "d":
					iterator++;
					ret += parseInt(arguments[iterator]) ? parseInt(arguments[iterator]) : 0;
					break;
				case "s":
					iterator++;
					ret += arguments[iterator];
					break;
				case "%":
					ret += "%";
			}
		}
		return ret + check;
	},
	htmlspecialchars(text) {
		let map = {
			'&': '&amp;',
			'<': '&lt;',
			'>': '&gt;',
			'"': '&quot;',
			"'": '&#039;'
		};

		return text.replace(/[&<>"']/g, function (m) {
			return map[m];
		});
	},
	IsDigitPercent: function (e) {
		var key = e.keyCode;
		switch (key) {
			case 37://%
			case 0:
			case 46://.
			case 101://e
			case 109://m
			case 13:
			case 8://???
			case 63272://???
				return true;
		}
		return (((key >= 48) && (key <= 57))/*0-9*/ || (key <= 63235 && key >= 63232));
	},
	getWe_cmdArgsArray: function (arr) {
		if (arr.length > 0 && typeof arr[0] === "object") {
			return arr[0];
		}
		return arr;
	},
	getWe_cmdArgsUrl: function (args, base) {
		var
				url = (base === undefined ? WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?" : base),
				pos = 0;

		if (Array.isArray(args)) {
			for (var i = 0; i < args.length; i++) {
				if (typeof (args[i]) !== 'object') {
					url += "we_cmd[" + pos + "]=" + encodeURIComponent(args[i]) + (i < (args.length - 1) ? "&" : "");
					pos++;
				} else if (args[i] !== null && typeof (args[i]) === 'object') {
					//url += this.getWe_cmdArgsUrlIterate(args[i]);
					url += Object.keys(args[i]).map(this.getWe_cmdArgsUrlMap.bind(this, args[i])).join("&");
				}
			}
		} else {
			//url += this.getWe_cmdArgsUrlIterate(args[i]);
			url += Object.keys(args).map(this.getWe_cmdArgsUrlMap.bind(this, args)).join("&");
		}

		return url;
	},
	/*
	 getWe_cmdArgsUrlIterate: function(obj) {
	 var params = [];
	 for(var key in obj){
	 if(obj.hasOwnProperty(key)) {
	 params.push("we_cmd[" + key + "]=" + encodeURIComponent(obj[key]));
	 }
	 }
	 return params.join("&");
	 },
	 */
	getWe_cmdArgsUrlMap: function (obj, key) {
		return "we_cmd[" + key + "]=" + encodeURIComponent(obj[key]);
	},
	showConfirm: null,
	showMessage: null,
	clip: function (doc, unique, width) {
		var text = doc.getElementById("td_" + unique);
		var btn = doc.getElementById("btn_" + unique).firstChild;

		if (text.classList.contains("cutText")) {
			text.classList.remove("cutText");
			text.style.maxWidth = "";
			btn.classList.remove("fa-caret-right");
			btn.classList.add("fa-caret-down");
		} else {
			text.classList.add("cutText");
			text.style.maxWidth = "100%";
			btn.classList.remove("fa-caret-down");
			btn.classList.add("fa-caret-right");
		}
	},
	setTextareaCount: function (value, trgt) {
		var words = value.match(/\w+/g);
		trgt.innerHTML = (words ? words.length : 0) + " / " + value.length;

	},
	hasPermDelete: function (eTable, isFolder) {
		if (eTable === "") {
			return false;
		}
		if (WE().session.permissions.ADMINISTRATOR) {
			return true;
		}
		switch (eTable) {
			case WE().consts.tables.FILE_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_DOC_FOLDER : WE().session.permissions.DELETE_DOCUMENT);
			case WE().consts.tables.TEMPLATES_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_TEMP_FOLDER : WE().session.permissions.DELETE_TEMPLATE);
			case WE().consts.tables.OBJECT_FILES_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_OBJECTFILE : WE().session.permissions.DELETE_OBJECTFILE);
			case WE().consts.tables.OBJECT_TABLE:
				return (isFolder ? false : WE().session.permissions.DELETE_OBJECT);
			case WE().consts.tables.VFILE_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_COLLECTION_FOLDER : WE().session.permissions.DELETE_COLLECTION);
			default:
				return false;
		}
	},
	validate: {
		email: function (email) {
			email = email.replace(/".*"/g, "y");
			email = email.replace(/\\./g, "z");
			var parts = email.split("@");
			if (parts.length !== 2) {
				return false;
			}
			if (!WE().util.validate.domain(parts[1])) {
				return false;
			}
			return (parts[0].match(/(.+)/));
		},
		domain: function (domain) {
			var parts = domain.split(".");
			//if(parts.length>3 || parts.length<1) return false;
			//if(parts.length===1 && !WE().util.validate.domainname(parts[0])) return false;
			for (var i = 0; i < (parts.length - 1); i++) {
				if (!WE().util.validate.domainname(parts[i])) {
					return false;
				}
			}
			return (parts[parts.length - 1].match(/^[a-z][a-z]+$/i));
		},
		domainname: function (domainname) {
			var pattern = /^[^_\-\s/=?\*"'#!§$%&()\[\]\{\};:,°<>\|][^\s/=?\*"'#!§$%&()\[\]\{\};:,°<>\|]+$/i;
			return (domainname.match(pattern));
		},
		date: function () {
			// TODO
		},
		currency: function () {
			// TODO
		}
	},
	/**
	 * load consts if not present in global scope
	 * @param {HTMLDocument} doc reference to the current document, where we can load the consts
	 * @param {string} check if WE().consts doesn't have "check" as property, we load the consts with a request
	 * @returns {nothing}
	 */
	loadConsts: function (doc, check, optUncache) {
		var cur = WE().consts;
		var found = true;
		var what = check.split(".");
		for (var i = 0; i < what.length; i++) {
			if (cur[what[i]] === undefined || !cur[what[i]]) {
				found = false;
				break;
			}
			cur = cur[what[i]];
		}
		//FIXME optUncache should be used to check for changes and update them
		if (found) {
			return;
		}
		$.ajax({
			async: false,
			type: "GET",
			cache: true,
			url: WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=loadJSConsts&we_cmd[1]=" + check + "&u=" + WE().consts.uncache + (optUncache == undefined ? '' : optUncache),
			success: function (data) {
				cur[what[i]] = data;
			},
			dataType: "json"
					//timeout: 2000
		}).fail(function (jqxhr, textStatus, error) {
			WE().t_e('JS rpc (loadConsts) failed', textStatus, error, jqxhr.responseText, this.url);
		});
	},
	/**
	 * load variable data from a tag
	 * @param {document} doc the document containing the tag
	 * @param {string} id element containing the data
	 * @param {string} dataname the attribute of the element containing the data
	 * @returns {Array|Object} the decoded data
	 */
	getDynamicVar: function (doc, id, dataname) {
		var el = doc.getElementById(id);
		return (el ?
				this.decodeDynamicVar(el, dataname) :
				null
				);
	},
	/**
	 * internal for decoding the data of a tag
	 */
	decodeDynamicVar: function (el, dataname) {
		var data = el.getAttribute(dataname);
		return data ? JSON.parse(window.atob(data)) : null;
	},
	/**
	 * @param {string} url
	 * @param {string} data string of data to post, e.g. JSON.stringify(array)
	 * @param {function} success function with data, function (weResponse)  => weResponse.DataArray.data holds the data by WE
	 * @param {string} type default json
	 * @returns {nothing}
	 */
	rpc: function (url, data, success, type) {
		return $.ajax({
			type: "POST",
			url: url,
			data: data,
			success: success,
			dataType: (type ? type : "json")
					//timeout: 2000
		}).fail(function (jqxhr, textStatus, error) {
			if (jqxhr.status > 0) {//it is only an error if status > 0 (e.g. site moved, ...)
				WE().t_e('JS rpc failed', textStatus, error, jqxhr.responseText, this.url, jqxhr);
			}
		});
	},
	processJSCmd: function (cmd, cmdData) {
		var caller = (this && this.window === this ? this : window);

		//Keep switch clean, only few statements should be here, everthing else goes to specific js file
		switch (cmd) {
			case 'msg':
				WE().util.showMessage(cmdData.msg, cmdData.prio ? cmdData.prio : WE().consts.message.WE_MESSAGE_NOTICE, caller);
				break;
			case 'close':
				caller.top.close();
				break;
			case 'close_dialog':
				WE().util.dialogTop(caller).doClose();
				break;
			case 'close_delayed':
				window.setTimeout(caller.top.close.bind(caller.top), cmdData);
				break;
			case 'close_dialog_delayed':
				window.setTimeout(WE().util.dialogTop(caller).doClose.bind(WE().util.dialogTop(caller).doClose), cmdData);
				break;
			case 'history.back':
				history.back();
				break;
			case 'we_cmd':
				if (caller.we_cmd) {
					caller.we_cmd.apply(caller, cmdData);
				} else if (caller.parent.we_cmd) {
					caller.parent.we_cmd.apply(caller, cmdData);
				} else if (caller.parent.parent.we_cmd) {
					caller.parent.parent.we_cmd.apply(caller, cmdData);
				} else if (caller.top.we_cmd) {
					caller.top.we_cmd.apply(caller, cmdData);
				} else {
					top.we_cmd.apply(caller, cmdData);
				}
				break;
			case 'location':
				switch (cmdData.doc) {
					case 'document':
						caller.document.location = cmdData.loc;
						break;
					case 'body':
						caller.top.body.document.location = cmdData.loc;
						break;
				}
				break;
			default:
				//if nothing matched, we set arg[0]=cmd & pass the whole argument to we_cmd
				if (caller.we_cmd) {//direct match
					caller.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.parent.we_cmd) {//match content frame
					caller.parent.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.parent.parent.we_cmd) {//in view component (on modules)
					caller.parent.parent.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.top.we_cmd) {
					caller.top.we_cmd.call(caller, cmd, cmdData);
				} else { // at last try the main we_cmd
					top.we_cmd.call(window, cmd, cmdData);
				}
		}
	},
	removeDiacritics: function (str) {
		var patterns = [
			{replacement: 'A', pattern: /[\u0041\u24B6\uFF21\u00C0\u00C1\u00C2\u1EA6\u1EA4\u1EAA\u1EA8\u00C3\u0100\u0102\u1EB0\u1EAE\u1EB4\u1EB2\u0226\u01E0\u01DE\u1EA2\u00C5\u01FA\u01CD\u0200\u0202\u1EA0\u1EAC\u1EB6\u1E00\u0104\u023A\u2C6F]/g},
			{replacement: 'AA', pattern: /[\uA732]/g},
			{replacement: 'AE', pattern: /[\u00C6\u01FC\u01E2\u00C4]/g},
			{replacement: 'AO', pattern: /[\uA734]/g},
			{replacement: 'AU', pattern: /[\uA736]/g},
			{replacement: 'AV', pattern: /[\uA738\uA73A]/g},
			{replacement: 'AY', pattern: /[\uA73C]/g},
			{replacement: 'B', pattern: /[\u0042\u24B7\uFF22\u1E02\u1E04\u1E06\u0243\u0182\u0181]/g},
			{replacement: 'C', pattern: /[\u0043\u24B8\uFF23\u0106\u0108\u010A\u010C\u00C7\u1E08\u0187\u023B\uA73E]/g},
			{replacement: 'D', pattern: /[\u0044\u24B9\uFF24\u1E0A\u010E\u1E0C\u1E10\u1E12\u1E0E\u0110\u018B\u018A\u0189\uA779]/g},
			{replacement: 'DZ', pattern: /[\u01F1\u01C4]/g},
			{replacement: 'Dz', pattern: /[\u01F2\u01C5]/g},
			{replacement: 'E', pattern: /[\u0045\u24BA\uFF25\u00C8\u00C9\u00CA\u1EC0\u1EBE\u1EC4\u1EC2\u1EBC\u0112\u1E14\u1E16\u0114\u0116\u00CB\u1EBA\u011A\u0204\u0206\u1EB8\u1EC6\u0228\u1E1C\u0118\u1E18\u1E1A\u0190\u018E]/g},
			{replacement: 'F', pattern: /[\u0046\u24BB\uFF26\u1E1E\u0191\uA77B]/g},
			{replacement: 'G', pattern: /[\u0047\u24BC\uFF27\u01F4\u011C\u1E20\u011E\u0120\u01E6\u0122\u01E4\u0193\uA7A0\uA77D\uA77E]/g},
			{replacement: 'H', pattern: /[\u0048\u24BD\uFF28\u0124\u1E22\u1E26\u021E\u1E24\u1E28\u1E2A\u0126\u2C67\u2C75\uA78D]/g},
			{replacement: 'I', pattern: /[\u0049\u24BE\uFF29\u00CC\u00CD\u00CE\u0128\u012A\u012C\u0130\u00CF\u1E2E\u1EC8\u01CF\u0208\u020A\u1ECA\u012E\u1E2C\u0197]/g},
			{replacement: 'J', pattern: /[\u004A\u24BF\uFF2A\u0134\u0248]/g},
			{replacement: 'K', pattern: /[\u004B\u24C0\uFF2B\u1E30\u01E8\u1E32\u0136\u1E34\u0198\u2C69\uA740\uA742\uA744\uA7A2]/g},
			{replacement: 'L', pattern: /[\u004C\u24C1\uFF2C\u013F\u0139\u013D\u1E36\u1E38\u013B\u1E3C\u1E3A\u0141\u023D\u2C62\u2C60\uA748\uA746\uA780]/g},
			{replacement: 'LJ', pattern: /[\u01C7]/g},
			{replacement: 'Lj', pattern: /[\u01C8]/g},
			{replacement: 'M', pattern: /[\u004D\u24C2\uFF2D\u1E3E\u1E40\u1E42\u2C6E\u019C]/g},
			{replacement: 'N', pattern: /[\u004E\u24C3\uFF2E\u01F8\u0143\u00D1\u1E44\u0147\u1E46\u0145\u1E4A\u1E48\u0220\u019D\uA790\uA7A4]/g},
			{replacement: 'NJ', pattern: /[\u01CA]/g},
			{replacement: 'Nj', pattern: /[\u01CB]/g},
			{replacement: 'O', pattern: /[\u004F\u24C4\uFF2F\u00D2\u00D3\u00D4\u1ED2\u1ED0\u1ED6\u1ED4\u00D5\u1E4C\u022C\u1E4E\u014C\u1E50\u1E52\u014E\u022E\u0230\u022A\u1ECE\u0150\u01D1\u020C\u020E\u01A0\u1EDC\u1EDA\u1EE0\u1EDE\u1EE2\u1ECC\u1ED8\u01EA\u01EC\u00D8\u01FE\u0186\u019F\uA74A\uA74C]/g},
			{replacement: 'OE', pattern: /[\u00D6]/g},
			{replacement: 'OI', pattern: /[\u01A2]/g},
			{replacement: 'OO', pattern: /[\uA74E]/g},
			{replacement: 'OU', pattern: /[\u0222]/g},
			{replacement: 'P', pattern: /[\u0050\u24C5\uFF30\u1E54\u1E56\u01A4\u2C63\uA750\uA752\uA754]/g},
			{replacement: 'Q', pattern: /[\u0051\u24C6\uFF31\uA756\uA758\u024A]/g},
			{replacement: 'R', pattern: /[\u0052\u24C7\uFF32\u0154\u1E58\u0158\u0210\u0212\u1E5A\u1E5C\u0156\u1E5E\u024C\u2C64\uA75A\uA7A6\uA782]/g},
			{replacement: 'S', pattern: /[\u0053\u24C8\uFF33\u015A\u1E64\u015C\u1E60\u0160\u1E66\u1E62\u1E68\u0218\u015E\u2C7E\uA7A8\uA784]/g},
			{replacement: 'SS', pattern: /[\u1E9E]/g},
			{replacement: 'T', pattern: /[\u0054\u24C9\uFF34\u1E6A\u0164\u1E6C\u021A\u0162\u1E70\u1E6E\u0166\u01AC\u01AE\u023E\uA786]/g},
			{replacement: 'TZ', pattern: /[\uA728]/g},
			{replacement: 'U', pattern: /[\u0055\u24CA\uFF35\u00D9\u00DA\u00DB\u0168\u1E78\u016A\u1E7A\u016C\u01DB\u01D7\u01D5\u01D9\u1EE6\u016E\u0170\u01D3\u0214\u0216\u01AF\u1EEA\u1EE8\u1EEE\u1EEC\u1EF0\u1EE4\u1E72\u0172\u1E76\u1E74\u0244]/g},
			{replacement: 'UE', pattern: /[\u00DC]/g},
			{replacement: 'V', pattern: /[\u0056\u24CB\uFF36\u1E7C\u1E7E\u01B2\uA75E\u0245]/g},
			{replacement: 'VY', pattern: /[\uA760]/g},
			{replacement: 'W', pattern: /[\u0057\u24CC\uFF37\u1E80\u1E82\u0174\u1E86\u1E84\u1E88\u2C72]/g},
			{replacement: 'X', pattern: /[\u0058\u24CD\uFF38\u1E8A\u1E8C]/g},
			{replacement: 'Y', pattern: /[\u0059\u24CE\uFF39\u1EF2\u00DD\u0176\u1EF8\u0232\u1E8E\u0178\u1EF6\u1EF4\u01B3\u024E\u1EFE]/g},
			{replacement: 'Z', pattern: /[\u005A\u24CF\uFF3A\u0179\u1E90\u017B\u017D\u1E92\u1E94\u01B5\u0224\u2C7F\u2C6B\uA762]/g},
			{replacement: 'a', pattern: /[\u0061\u24D0\uFF41\u1E9A\u00E0\u00E1\u00E2\u1EA7\u1EA5\u1EAB\u1EA9\u00E3\u0101\u0103\u1EB1\u1EAF\u1EB5\u1EB3\u0227\u01E1\u01DF\u1EA3\u00E5\u01FB\u01CE\u0201\u0203\u1EA1\u1EAD\u1EB7\u1E01\u0105\u2C65\u0250]/g},
			{replacement: 'aa', pattern: /[\uA733]/g},
			{replacement: 'ae', pattern: /[\u00E6\u01FD\u01E3\u00E4]/g},
			{replacement: 'ao', pattern: /[\uA735]/g},
			{replacement: 'au', pattern: /[\uA737]/g},
			{replacement: 'av', pattern: /[\uA739\uA73B]/g},
			{replacement: 'ay', pattern: /[\uA73D]/g},
			{replacement: 'b', pattern: /[\u0062\u24D1\uFF42\u1E03\u1E05\u1E07\u0180\u0183\u0253]/g},
			{replacement: 'c', pattern: /[\u0063\u24D2\uFF43\u0107\u0109\u010B\u010D\u00E7\u1E09\u0188\u023C\uA73F\u2184]/g},
			{replacement: 'd', pattern: /[\u0064\u24D3\uFF44\u1E0B\u010F\u1E0D\u1E11\u1E13\u1E0F\u0111\u018C\u0256\u0257\uA77A]/g},
			{replacement: 'dz', pattern: /[\u01F3\u01C6]/g},
			{replacement: 'e', pattern: /[\u0065\u24D4\uFF45\u00E8\u00E9\u00EA\u1EC1\u1EBF\u1EC5\u1EC3\u1EBD\u0113\u1E15\u1E17\u0115\u0117\u00EB\u1EBB\u011B\u0205\u0207\u1EB9\u1EC7\u0229\u1E1D\u0119\u1E19\u1E1B\u0247\u025B\u01DD]/g},
			{replacement: 'f', pattern: /[\u0066\u24D5\uFF46\u1E1F\u0192\uA77C]/g},
			{replacement: 'g', pattern: /[\u0067\u24D6\uFF47\u01F5\u011D\u1E21\u011F\u0121\u01E7\u0123\u01E5\u0260\uA7A1\u1D79\uA77F]/g},
			{replacement: 'h', pattern: /[\u0068\u24D7\uFF48\u0125\u1E23\u1E27\u021F\u1E25\u1E29\u1E2B\u1E96\u0127\u2C68\u2C76\u0265]/g},
			{replacement: 'hv', pattern: /[\u0195]/g},
			{replacement: 'i', pattern: /[\u0069\u24D8\uFF49\u00EC\u00ED\u00EE\u0129\u012B\u012D\u00EF\u1E2F\u1EC9\u01D0\u0209\u020B\u1ECB\u012F\u1E2D\u0268\u0131]/g},
			{replacement: 'j', pattern: /[\u006A\u24D9\uFF4A\u0135\u01F0\u0249]/g},
			{replacement: 'k', pattern: /[\u006B\u24DA\uFF4B\u1E31\u01E9\u1E33\u0137\u1E35\u0199\u2C6A\uA741\uA743\uA745\uA7A3]/g},
			{replacement: 'l', pattern: /[\u006C\u24DB\uFF4C\u0140\u013A\u013E\u1E37\u1E39\u013C\u1E3D\u1E3B\u017F\u0142\u019A\u026B\u2C61\uA749\uA781\uA747]/g},
			{replacement: 'lj', pattern: /[\u01C9]/g},
			{replacement: 'm', pattern: /[\u006D\u24DC\uFF4D\u1E3F\u1E41\u1E43\u0271\u026F]/g},
			{replacement: 'n', pattern: /[\u006E\u24DD\uFF4E\u01F9\u0144\u00F1\u1E45\u0148\u1E47\u0146\u1E4B\u1E49\u019E\u0272\u0149\uA791\uA7A5]/g},
			{replacement: 'nj', pattern: /[\u01CC]/g},
			{replacement: 'o', pattern: /[\u006F\u24DE\uFF4F\u00F2\u00F3\u00F4\u1ED3\u1ED1\u1ED7\u1ED5\u00F5\u1E4D\u022D\u1E4F\u014D\u1E51\u1E53\u014F\u022F\u0231\u022B\u1ECF\u0151\u01D2\u020D\u020F\u01A1\u1EDD\u1EDB\u1EE1\u1EDF\u1EE3\u1ECD\u1ED9\u01EB\u01ED\u00F8\u01FF\u0254\uA74B\uA74D\u0275]/g},
			{replacement: 'oe', pattern: /[\u00F6]/g},
			{replacement: 'oi', pattern: /[\u01A3]/g},
			{replacement: 'ou', pattern: /[\u0223]/g},
			{replacement: 'oo', pattern: /[\uA74F]/g},
			{replacement: 'p', pattern: /[\u0070\u24DF\uFF50\u1E55\u1E57\u01A5\u1D7D\uA751\uA753\uA755]/g},
			{replacement: 'q', pattern: /[\u0071\u24E0\uFF51\u024B\uA757\uA759]/g},
			{replacement: 'r', pattern: /[\u0072\u24E1\uFF52\u0155\u1E59\u0159\u0211\u0213\u1E5B\u1E5D\u0157\u1E5F\u024D\u027D\uA75B\uA7A7\uA783]/g},
			{replacement: 's', pattern: /[\u0073\u24E2\uFF53\u015B\u1E65\u015D\u1E61\u0161\u1E67\u1E63\u1E69\u0219\u015F\u023F\uA7A9\uA785\u1E9B]/g},
			{replacement: 'ss', pattern: /[\u00DF]/g},
			{replacement: 't', pattern: /[\u0074\u24E3\uFF54\u1E6B\u1E97\u0165\u1E6D\u021B\u0163\u1E71\u1E6F\u0167\u01AD\u0288\u2C66\uA787]/g},
			{replacement: 'tz', pattern: /[\uA729]/g},
			{replacement: 'u', pattern: /[\u0075\u24E4\uFF55\u00F9\u00FA\u00FB\u0169\u1E79\u016B\u1E7B\u016D\u01DC\u01D8\u01D6\u01DA\u1EE7\u016F\u0171\u01D4\u0215\u0217\u01B0\u1EEB\u1EE9\u1EEF\u1EED\u1EF1\u1EE5\u1E73\u0173\u1E77\u1E75\u0289]/g},
			{replacement: 'ue', pattern: /[\u00FC]/g},
			{replacement: 'v', pattern: /[\u0076\u24E5\uFF56\u1E7D\u1E7F\u028B\uA75F\u028C]/g},
			{replacement: 'vy', pattern: /[\uA761]/g},
			{replacement: 'w', pattern: /[\u0077\u24E6\uFF57\u1E81\u1E83\u0175\u1E87\u1E85\u1E98\u1E89\u2C73]/g},
			{replacement: 'x', pattern: /[\u0078\u24E7\uFF58\u1E8B\u1E8D]/g},
			{replacement: 'y', pattern: /[\u0079\u24E8\uFF59\u1EF3\u00FD\u0177\u1EF9\u0233\u1E8F\u00FF\u1EF7\u1E99\u1EF5\u01B4\u024F\u1EFF]/g},
			{replacement: 'z', pattern: /[\u007A\u24E9\uFF5A\u017A\u1E91\u017C\u017E\u1E93\u1E95\u01B6\u0225\u0240\u2C6C\uA763]/g}
		];

		for (var i = 0; i < patterns.length; i++) {
			str = str.replace(patterns[i].pattern, patterns[i].replacement);
		}

		return str;
	}
};

WE().consts = WE().util.getDynamicVar(document, 'loadWEData', 'data-consts');
WE().session = WE().util.getDynamicVar(document, 'loadWEData', 'data-session');
//finally load language files
WE().util.loadConsts(document, 'g_l');
WE().consts.browserDetect.small = window.matchMedia("(max-device-width: 800px)").matches || window.matchMedia("(max-device-height: 600px)").matches;
