/* global tinymce */

/**
 * webEdition CMS
 *
 * $Rev: 2d17f32b37c0 $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-15 08:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var SourceDialog = {
	tinymcePlugin: window.tinyMCEPopup,
	tinymce: window.tinymce,
	editor: window.tinyMCEPopup.editor,

	codemirror: null,
	chr: 0, // Unused utf-8 character, placeholder for cursor
	isMac: /macintosh|mac os/i.test(navigator.userAgent),
	cmSettings: {
		config: {
			mode: 'application/x-httpd-php',
			//mode: 'htmlmixed',
			theme: 'default',
			lineNumbers: true,
			lineWrapping: true,
			indentUnit: 2,
			tabSize: 2,
			indentWithTabs: true,
			matchBrackets: true,
			styleActiveLine: true
		},
		indentOnInit: true
	},
	saveCursorPosition: true,

	init: function () {// Initialize (before load)
		var i, userSettings = this.editor.settings.codemirror ? this.editor.settings.codemirror : {};

		if (this.saveCursorPosition) {
			this.editor.selection.setContent('<span style="display: none;" id="mceWesourceCaret"></span>' + this.editor.selection.getContent());
		}

		// Merge config
		for (i in userSettings.config) {
			this.cmSettings.config[i] = userSettings.config[i];
		}

		if (this.cmSettings.config.theme) {
			document.documentElement.className += this.cmSettings.config.theme.replace(/(^|\s)\s*/g, " cm-s-");
		}

		this.start();
	},

	start: function () {// FIXME: move inside init()
		var indentOnInit = this.cmSettings.indentOnInit;

		// Create legend for keyboard shortcuts for find & replace:
		/*
		 var head = parent.document.querySelectorAll('.mce-foot')[0],
		 div = parent.document.createElement('div'),
		 td1 = '<td style="font-size:11px;background:#777;color:#fff;padding:0 4px">',
		 td2 = '<td style="font-size:11px;padding-right:5px">';
		 div.innerHTML = '<table cellspacing="0" cellpadding="0" style="border-spacing:4px"><tr>' + td1 + (isMac ? '&#8984;-F' : 'Ctrl-F</td>') + td2 + tinymce.translate('Start search') + '</td>' + td1 + (isMac ? '&#8984;-G' : 'Ctrl-G') + '</td>' + td2 + tinymce.translate('Find next') + '</td>' + td1 + (isMac ? '&#8984;-Alt-F' : 'Shift-Ctrl-F') + '</td>' + td2 + tinymce.translate('Find previous') + '</td></tr>' + '<tr>' + td1 + (isMac ? '&#8984;-Alt-F' : 'Shift-Ctrl-F') + '</td>' + td2 + tinymce.translate('Replace') + '</td>' + td1 + (isMac ? 'Shift-&#8984;-Alt-F' : 'Shift-Ctrl-R') +'</td>' + td2 + tinymce.translate('Replace all') + '</td></tr></table>';
		 div.style.position = 'absolute';
		 div.style.left = div.style.bottom = '5px';
		 head.appendChild(div);
		 *
		 */

		// Set CodeMirror cursor and bookmark to same position as cursor was in TinyMCE:
		var html = this.editor.getContent({source_view: true});
		// [FIX] #6 z-index issue with table panel and source code dialog
		//	editor.selection.getBookmark();

		html = html.replace(/<span\s+id="mceWesourceCaret"\sstyle="display: none;"([^>]*)><\/span>/gm, String.fromCharCode(this.chr));
		this.editor.dom.remove(this.editor.dom.select('span#mceWesourceCaret'));

		// Hide TinyMCE toolbar panels, [FIX] #6 z-index issue with table panel and source code dialog
		// https://github.com/christiaan/tinymce-codemirror/issues/6
		this.tinymce.each(this.editor.contextToolbars, function (toolbar) {
			if (toolbar.panel) {
				toolbar.panel.hide();
			}
		});

		window.CodeMirror.defineInitHook(function (inst) {
			// Move cursor to correct position:
			inst.focus();
			var cursor = inst.getSearchCursor(String.fromCharCode(this.chr), false);
			if (cursor.findNext()) {
				inst.setCursor(cursor.to());
				cursor.replace('');
			}

			// Indent all code, if so requested:
			if (indentOnInit) {
				var last = inst.lineCount();
				inst.operation(function () {
					for (var i = 0; i < last; ++i) {
						inst.indentLine(i);
					}
				});
			}

		});

		this.cmSettings.config.value = html;
		// Instantiante CodeMirror:
		this.codemirror = window.CodeMirror(document.getElementById('wesourceDiv'), this.cmSettings.config);
		this.codemirror.isDirty = false;
		this.codemirror.on('change', function (inst) {
			inst.isDirty = true;
		});
	},

	writeBack: function () {
		if(!this.codemirror.isDirty){
//			this.editor.focus();
//			return;
		}

		var cc = '&#x0;',
			doc = this.codemirror.doc,
			editor,
			content,
			matches,
			tmpDiv,
			next,
			elems,
			cursorHtml,
			tags,
			searchCursor = true;

		// set placeholder for cursor position
		if (doc.somethingSelected()) {
			doc.setCursor(doc.getCursor());
		}
		doc.replaceSelection(cc);
		content = this.codemirror.getValue();

		// remove php, script and style
		content = content.replace(/<\?[=|php].*?\?>/g, '').
			replace(/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi, '').
			replace(/<style\b[^<]*(?:(?!<\/style>)<[^<]*)*<\/style>/gi, '');

		// move cursor replacement out of tags and htmlentities
		if(((matches = content.match(/(&[#0-9a-zA-Z]{0,7}&#x0;[#0-9a-zA-Z]{0,7};)/g)) && matches[0]) ||
				((matches = content.match(/(<[^>]*&#x0;[^>]*>)/g)) && matches[0])){
			content = content.replace(matches[0], matches[0].replace(cc, '') + cc);
		}

		// tables: move cursor into td
		if(searchCursor && (((matches = content.match(/(\r\n|\n|\r)[^<]*&#x0;[^<]*<tr[^>]*>[^<]*<(td|th)[^>]*>/img )) && matches[0]) ||
				((matches = content.match(/((\r\n|\n|\r)[^<]*(&#x0;)[^<]*<(td|th)[^>]*>)/im )) && matches[0]))){
			content = content.replace(matches[0], matches[0].replace(/&#x0;/ig, '') + '&#x1EE;');
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}
		if(searchCursor && (((matches = content.match(/(<\/(td|th)>[^<]*<\/tr>[^<]*(<\/thead>|<\/tbody>|<\/tfoot>)?[^<]*<\/table>[^<]*(&#x0;))/im )) && matches[0]) ||
				((matches = content.match(/(<\/(td|th)>[^<]*<\/tr>[^<]*<\/(thead|tbody|tfoot)>[^<]*(&#x0;))/im )) && matches[0]) ||
				((matches = content.match(/(<\/(td|th)>[^<]*<\/tr>[^<]*(&#x0;))/im )) && matches[0]) ||
				((matches = content.match(/(<\/(td|th)>[^<]*(&#x0;))/im )) && matches[0]) ||
				((matches = content.match(/(<\/(td|th)>[^<]*(&#x0;))/im )) && matches[0])
			)){
			content = content.replace(matches[0], '&#x1EE;' + matches[0].replace(/&#x0;/ig, ''));
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}
		// tables: rename cursor inside td
		if(((matches = content.match(/<td[^>]*>(?:(?!(<table|<\/td>))[\s\S])*&#x0;(?:(?!<\/td>)[\s\S])*<\/td>/im)) && matches[0]) || // FIXME: this can not be correct...
				false
				){
			content = content.replace(matches[0], matches[0].replace(/&#x0;/ig, '&#x1EE;'));
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}

		// ul/ol: rename cursor inside li
		if((matches = content.match(/<li[^>]*>(?:(?!<|&#x0;)[\s\S])*(?:<(?!\/?li)[^>]*>(?:(?!<|&#x0;)[\s\S])*)*&#x0;[\s\S]*?<\/li>/im )) && matches[0]){
			content = content.replace(matches[0], matches[0].replace(/&#x0;/ig, '&#x1EE;'));
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}
		// ul/ol: set cursor into li
		if(searchCursor && (((matches = content.match(/((\r\n|\n|\r)[^<]*(&#x0;)[^<]*<(ul|ol)[^>]*>[^<]*<li[^>]*>)/im )) && matches[0]) ||
				((matches = content.match(/((\r\n|\n|\r)[^<]*(&#x0;)[^<]*<li[^>]*>)/im )) && matches[0]))){
			content = content.replace(matches[0], matches[0].replace(/&#x0;/ig, '') + '&#x1EE;');
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}
		if(searchCursor && ((matches = content.match(/(<\/li>([^<]*<\/(ul|ol)>)*[^<]*(&#x0;))/im )) && matches[0]) ||
				((matches = content.match(/(<\/(ul|ol)>[^<]*(&#x0;))/im )) && matches[0])) {
			content = content.replace(matches[0], '&#x1EE;' + matches[0].replace(/&#x0;/ig, ''));
			content.replace(/&#x0;/ig, '');
			searchCursor = false;
		}

		if(!searchCursor){
			content = content.replace(/&#x0;/ig, '');
			content = content.replace(/&#x1EE;/ig, '<span class="mceWesourceCaret lala">' + cc + '</span>');

		} else {
			content = content.replace(/&#x1EE;/ig, '');
			// important: span is moved out of table if inserted on an inapropiate position...
			content = content.replace(/&#x0;/ig, '<span class="mceWesourceCaret">' + cc + '</span>');

			tmpDiv = document.createElement('div');
			tmpDiv.innerHTML = content;

			// check for cursor lists if not yet found: move to the left of the element
			tags = ['ul', 'lo'];
			for (var i = 0; i < tags.length; i++){
				var elems = tmpDiv.getElementsByTagName(tags[i]);
				for(var j = elems.length -1; j > -1; j--){
					if(elems[j].getElementsByClassName('mceWesourceCaret')[0]){
						tmpDiv.insertBefore(elems[j].getElementsByClassName('mceWesourceCaret')[0], elems[j]);
						break;
					}
				}
			}

			if((cursorHtml = tmpDiv.getElementsByClassName('mceWesourceCaret')[0])){
				if(!((cursorHtml.previousSibling && cursorHtml.previousSibling.nodeName.toLowerCase() === '#text' && cursorHtml.previousSibling.textContent.search(/\S/) !== -1) ||
						(cursorHtml.nextSibling && cursorHtml.nextSibling.nodeName.toLowerCase() === '#text' && cursorHtml.nextSibling.textContent.search(/\S/) !== -1))){

					next = cursorHtml.nextSibling ? cursorHtml.nextSibling : cursorHtml;
					while(next.nodeName.toLowerCase() === '#text' && next.nextSibling){
						next = next.nextSibling;
					}

					switch(next.nodeName.toLowerCase()){
						case 'p':
						case 'div':

							if(!cursorHtml.previousSibling || (cursorHtml.previousSibling.nodeName.toLowerCase() === '#text' &&
									cursorHtml.previousSibling.textContent.search(/(\r\n|\n|\r)/gm) !== -1)){
								next.prepend(cursorHtml);
							} else {
								next = cursorHtml.previousSibling ? cursorHtml.previousSibling : cursorHtml;
								while(next.nodeName.toLowerCase() === '#text' && next.previousSibling){
									next = next.previousSibling;
								}
								next.appendChild(cursorHtml);
							}
							break;
						case 'ul':
						case 'ol':
							if((elems = next.getElementsByTagName('li')) && elems.length){
								elems[0].prepend(cursorHtml);
							} else {
								cursorHtml.remove();
							}
							break;
						case 'table':
							if((elems = next.getElementsByTagName('td')) && elems.length){
								elems[0].prepend(cursorHtml);
							} else {
								cursorHtml.remove();
							}
							break;
						default:
							if(cursorHtml.parentNode === tmpDiv){
								cursorHtml.remove();
							}
					}
				}
				content = tmpDiv.innerHTML;
			}
		}

		// write back
		editor = this.editor;
		editor.undoManager.transact(function () {
			editor.setContent(content);
		});
		editor.focus();

		// tiny set cursor:
		var el = editor.dom.select('.mceWesourceCaret')[0];
		if (el) {
			editor.selection.scrollIntoView(el);
			editor.selection.setCursorLocation(el, 0);
			editor.dom.remove(el);
		}
		// delete other weCaret if exist
		this.tinymce.each(editor.dom.select('.mceWesourceCaret'), function (el) {
			editor.dom.remove(el);
		});
	}
};

function keyDownReplacement(e) {
	switch (e.key) {
		case 'Esc':
			WE().util.dialogTop(window).doClose();
			break;
		case 'Enter':
			e.stopPropagation();
			// do nothing!
			break;
	}
}

function weTinyDialog_doOk() {
	SourceDialog.writeBack();
	WE().util.dialogTop(window).doClose();
}

$(function () {
	SourceDialog.init();
});